@@ -1,3 +1,4 @@
+/* $Id: ipfail.c,v 1.43 2005/08/16 15:09:45 gshi Exp $ */
 /* ipfail: IP Failover plugin for Linux-HA
  *
  * Copyright (C) 2002-2004 Kevin Dwyer <kevin@pheared.net>
@@ -38,7 +39,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -85,10 +86,10 @@ main(int argc, char **argv)
 	cl_log_enable_stderr(TRUE);
 	
 	/* Get the name of the binary for logging purposes */
-	bname = cl_strdup(argv[0]);
+	bname = ha_strdup(argv[0]);
 	cl_log_set_entity(basename(bname));
-	cl_log_set_facility(HA_LOG_FACILITY);
-	cl_inherit_logging_environment(0);
+
+	cl_log_set_facility(DEFAULT_FACILITY);
 
 	hb = ll_cluster_new("heartbeat");
 
@@ -229,7 +230,7 @@ node_walk(ll_cluster_t *hb)
 		/* Look for our partner */
 		if (!strcmp("normal", hb->llc_ops->node_type(hb, node))
 		    && strcmp(node, node_name)) {
-			strlcpy(other_node, node, sizeof(other_node));
+			strcpy(other_node, node);
 			cl_log(LOG_DEBUG, "[They are %s]", other_node);
 		}
 
@@ -529,14 +530,14 @@ send_abort_giveup(ll_cluster_t *hb)
 }
 
 void
-msg_abort_giveup(struct ha_msg *msg, void *private)
+msg_abort_giveup(const struct ha_msg *msg, void *private)
 {
         abort_giveup();
 }
 
 
 void
-msg_ipfail_join(struct ha_msg *msg, void *private)
+msg_ipfail_join(const struct ha_msg *msg, void *private)
 {
 	/* msg_ipfail_join: When another ipfail client sends a join 
 	 * message, call ask_ping_nodes() to compare ping node counts.
@@ -555,7 +556,7 @@ msg_ipfail_join(struct ha_msg *msg, void *private)
 }
 
 void
-msg_resources(struct ha_msg *msg, void *private)
+msg_resources(const struct ha_msg *msg, void *private)
 {
 	const char * orig = ha_msg_value(msg, F_ORIG);
 	const char * isstable = ha_msg_value(msg, F_ISSTABLE);
@@ -645,7 +646,7 @@ ask_ping_nodes(ll_cluster_t *hb, int num_ping)
 }
 
 void
-msg_ping_nodes(struct ha_msg *msg, void *private)
+msg_ping_nodes(const struct ha_msg *msg, void *private)
 {
 	/* msg_ping_nodes: Takes the message and heartbeat cluster as input;
 	 * returns nothing.  Callback for the num_ping_nodes message.
@@ -711,7 +712,7 @@ you_are_dead(ll_cluster_t *hb)
 }
 
 void
-i_am_dead(struct ha_msg *msg, void *private)
+i_am_dead(const struct ha_msg *msg, void *private)
 {
 	/* i_am_dead: Takes the you_are_dead message and the heartbeat cluster
 	 * as input; returns nothing.
@@ -783,13 +784,19 @@ ipfail_timeout_dispatch(gpointer user_data)
 void
 open_api(ll_cluster_t *hb)
 {
-	/* Sign in to the API */
+	/* Sign in to the API and setup the log facility */
+	int facility;
+
 	cl_log(LOG_DEBUG, "Signing in with heartbeat");
 	if (hb->llc_ops->signon(hb, "ipfail")!= HA_OK) {
 		cl_log(LOG_ERR, "Cannot sign on with heartbeat");
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		exit(1);
 	}
+	if ((facility = hb->llc_ops->get_logfacility(hb)) <= 0) {
+		facility = DEFAULT_FACILITY;
+	}
+	cl_log_set_facility(facility);
 }
 
 void
