@@ -1,3 +1,4 @@
+/* $Id: crmadmin.c,v 1.57 2005/09/12 19:32:36 andrew Exp $ */
 
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
@@ -17,8 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
-#include <ha_version.h>
+#include <portability.h>
 
 #include <sys/param.h>
 
@@ -48,6 +48,7 @@
 #ifdef HAVE_GETOPT_H
 #  include <getopt.h>
 #endif
+#include <crm/dmalloc_wrapper.h>
 
 int message_timer_id = -1;
 int message_timeout_ms = 30*1000;
@@ -59,11 +60,13 @@ char *admin_uuid = NULL;
 void usage(const char *cmd, int exit_status);
 ll_cluster_t *do_init(void);
 int do_work(ll_cluster_t * hb_cluster);
-void crmadmin_ipc_connection_destroy(gpointer user_data);
+void crmd_ipc_connection_destroy(gpointer user_data);
 
 gboolean admin_msg_callback(IPC_Channel * source_data, void *private_data);
 char *pluralSection(const char *a_section);
 crm_data_t *handleCibMod(void);
+int do_find_resource(const char *rsc, crm_data_t *xml_node);
+int do_find_resource_list(int level, crm_data_t *xml_node);
 int do_find_node_list(crm_data_t *xml_node);
 gboolean admin_message_timeout(gpointer data);
 gboolean is_node_online(crm_data_t *node_state);
@@ -102,8 +105,9 @@ char *crm_option = NULL;
 
 int operation_status = 0;
 const char *sys_to = NULL;
+const char *crm_system_name = "crmadmin";
 
-#define OPTARGS	"V?K:S:HE:Dd:i:RNqt:Bv"
+#define OPTARGS	"V?K:S:HE:DW:d:i:RNqt:B"
 
 int
 main(int argc, char **argv)
@@ -126,23 +130,26 @@ main(int argc, char **argv)
 		/* daemon options */
 		{"kill", 1, 0, 'K'},  /* stop a node */
 		{"die", 0, 0, 0},  /* kill a node, no respawn */
-		{"debug_inc", 1, 0, 'i'},
-		{"debug_dec", 1, 0, 'd'},
+		{"crm_debug_inc", 1, 0, 'i'},
+		{"crm_debug_dec", 1, 0, 'd'},
 		{"status", 1, 0, 'S'},
 		{"standby", 1, 0, 's'},
 		{"active", 1, 0, 'a'},
 		{"health", 0, 0, 'H'},
 		{"election", 0, 0, 'E'},
 		{"dc_lookup", 0, 0, 'D'},
+		{"resources", 0, 0, 'R'},
 		{"nodes", 0, 0, 'N'},
+		{"whereis", 1, 0, 'W'},
 		{"option", 1, 0, 'o'},
-		{"version", 0, 0, 'v'},
 
 		{0, 0, 0, 0}
 	};
 #endif
 
-	crm_log_init(basename(argv[0]), LOG_ERR, FALSE, TRUE, argc, argv);
+	crm_system_name = basename(argv[0]);
+	crm_log_init(crm_system_name);
+
 	if(argc < 2) {
 		usage(crm_system_name, LSB_EXIT_EINVAL);
 	}
@@ -160,12 +167,17 @@ main(int argc, char **argv)
 		switch(flag) {
 #ifdef HAVE_GETOPT_H
 			case 0:
-				if (strcasecmp("reference",
+				printf("option %s", long_options[option_index].name);
+				if (optarg)
+					printf(" with arg %s", optarg);
+				printf("\n");
+			
+				if (strcmp("reference",
 					   long_options[option_index].name) == 0) {
 					this_msg_reference =
 						crm_strdup(optarg);
 
-				} else if (strcasecmp("die",
+				} else if (strcmp("die",
 						  long_options[option_index].name) == 0) {
 					DO_RESET = TRUE;
 					crmd_operation = CRM_OP_DIE;
@@ -184,12 +196,7 @@ main(int argc, char **argv)
    digit_optind = this_option_optind;
    printf ("option %c\n", c);
 */
-
-			case 'v':
-				fprintf(stdout, "HA Version %s, CRM Version %s (CIB feature set %s) %s\n",
-					VERSION, CRM_FEATURE_SET, CIB_FEATURE_SET, HA_HG_VERSION);
-				exit(0);
-				break;
+			
 			case 'V':
 				BE_VERBOSE = TRUE;
 				admin_verbose = XML_BOOLEAN_TRUE;
@@ -212,6 +219,11 @@ main(int argc, char **argv)
 			case 'B':
 				BASH_EXPORT = TRUE;
 				break;
+			case 'W':
+				DO_RESOURCE = TRUE;
+				crm_debug_2("Option %c => %s", flag, optarg);
+				rsc_name = crm_strdup(optarg);
+				break;
 			case 'K':
 				DO_RESET = TRUE;
 				crm_debug_2("Option %c => %s", flag, optarg);
@@ -242,6 +254,9 @@ main(int argc, char **argv)
 			case 'N':
 				DO_NODE_LIST = TRUE;
 				break;
+			case 'R':
+				DO_RESOURCE_LIST = TRUE;
+				break;
 			case 'H':
 				DO_HEALTH = TRUE;
 				break;
@@ -269,15 +284,20 @@ main(int argc, char **argv)
 
 	hb_cluster = do_init();
 	if (hb_cluster != NULL) {
-		int res = 0;
-		res = do_work(hb_cluster);
-		if (res > 0) {
+		int res = do_work(hb_cluster);
+		if (res >= 0) {
 			/* wait for the reply by creating a mainloop and running it until
 			 * the callbacks are invoked...
 			 */
 			mainloop = g_main_new(FALSE);
 			expected_responses++;
-			crm_debug_2("Waiting for %d replies from the local CRM", expected_responses);
+			if(res == 0) {
+				crm_debug_2("no reply expected,"
+					    " wait for the hello message only");
+				
+			} else {
+				crm_debug_2("Waiting for reply from the local CRM");
+			}
 
 			message_timer_id = Gmain_timeout_add(
 				message_timeout_ms, admin_message_timeout, NULL);
@@ -285,7 +305,7 @@ main(int argc, char **argv)
 			g_main_run(mainloop);
 			return_to_orig_privs();
 			
-		} else if(res < 0) {
+		} else {
 			crm_err("No message to send");
 			operation_status = -1;
 		}
@@ -322,7 +342,7 @@ do_work(ll_cluster_t * hb_cluster)
 			crmd_operation = CRM_OP_PING;
 
 			if (BE_VERBOSE) {
-				expected_responses = 1;
+				expected_responses = -1;/* wait until timeout instead */
 			}
 			
 			crm_xml_add(msg_options, XML_ATTR_TIMEOUT, "0");
@@ -352,25 +372,35 @@ do_work(ll_cluster_t * hb_cluster)
 
 		dest_node = NULL;
 
-	} else if(DO_NODE_LIST) {
+	} else if(DO_RESOURCE || DO_RESOURCE_LIST || DO_NODE_LIST) {
 
 		cib_t *	the_cib = cib_new();
 		crm_data_t *output = NULL;
 		
 		enum cib_errors rc = the_cib->cmds->signon(
-			the_cib, crm_system_name, cib_command_synchronous);
+			the_cib, crm_system_name, cib_command);
 
 		if(rc != cib_ok) {
 			return -1;
-		}
 			
-		output = get_cib_copy(the_cib);
-		do_find_node_list(output);
-		
+		} else if(DO_RESOURCE) {
+			output = get_cib_copy(the_cib);
+			do_find_resource(rsc_name, output);
+
+		} else if(DO_RESOURCE_LIST) {
+			crm_data_t *rscs = NULL;
+			output = get_cib_copy(the_cib);
+			rscs = get_object_root(XML_CIB_TAG_RESOURCES, output);
+			do_find_resource_list(0, rscs);
+			
+		} else if(DO_NODE_LIST) {
+			output = get_cib_copy(the_cib);
+			do_find_node_list(output);
+		}
+
 		free_xml(output);
 		the_cib->cmds->signoff(the_cib);
-		exit(rc);
-		
+		exit(rc);		
 	} else if(DO_RESET) {
 		/* tell dest_node to initiate the shutdown proceedure
 		 *
@@ -422,11 +452,10 @@ do_work(ll_cluster_t * hb_cluster)
 	}
 
 	if(sys_to == NULL) {
-		if (dest_node != NULL) {
+		if (dest_node != NULL)
 			sys_to = CRM_SYSTEM_CRMD;
-		} else {
+		else
 			sys_to = CRM_SYSTEM_DC;				
-		}
 	}
 	
 	{
@@ -438,34 +467,37 @@ do_work(ll_cluster_t * hb_cluster)
 			ha_msg_mod(cmd, XML_ATTR_REFERENCE, this_msg_reference);
 		}
 		send_ipc_message(crmd_channel, cmd);
-		crm_msg_del(cmd);
 	}
 	
 	return ret;
+
 }
 
 void
-crmadmin_ipc_connection_destroy(gpointer user_data)
+crmd_ipc_connection_destroy(gpointer user_data)
 {
-    crm_err("Connection to CRMd was terminated");
-    if(mainloop) {
-	g_main_quit(mainloop);
-    } else {
+	crm_info("Connection to CRMd was terminated");
 	exit(1);
-    }
 }
 
 
 ll_cluster_t *
 do_init(void)
 {
+	int facility;
 	GCHSource *src = NULL;
 	ll_cluster_t *hb_cluster = NULL;
 	
 	/* change the logging facility to the one used by heartbeat daemon */
 	hb_cluster = ll_cluster_new("heartbeat");
 	
-	crm_malloc0(admin_uuid, 11);
+	crm_debug_2("Switching to Heartbeat logger");
+	if (( facility =
+	      hb_cluster->llc_ops->get_logfacility(hb_cluster)) > 0) {
+		cl_log_set_facility(facility);
+	}
+
+	crm_malloc0(admin_uuid, sizeof(char) * 11);
 	if(admin_uuid != NULL) {
 		snprintf(admin_uuid, 10, "%d", getpid());
 		admin_uuid[10] = '\0';
@@ -481,7 +513,7 @@ do_init(void)
 		send_hello_message(
 			crmd_channel, admin_uuid, crm_system_name,"0", "1");
 
-		set_IPC_Channel_dnotify(src, crmadmin_ipc_connection_destroy);
+		set_IPC_Channel_dnotify(src, crmd_ipc_connection_destroy);
 		
 		return hb_cluster;
 	} 
@@ -491,7 +523,6 @@ do_init(void)
 gboolean
 admin_msg_callback(IPC_Channel * server, void *private_data)
 {
-	int rc = 0;
 	int lpc = 0;
 	IPC_Message *msg = NULL;
 	ha_msg_input_t *new_input = NULL;
@@ -507,12 +538,10 @@ admin_msg_callback(IPC_Channel * server, void *private_data)
 	       && server->ops->is_message_pending(server) == TRUE) {
 		if(new_input != NULL) {
 			delete_ha_msg_input(new_input);
-			new_input = NULL;
 		}
-
-		rc = server->ops->recv(server, &msg);
-		if (rc != IPC_OK) {
-		    cl_perror("Receive failure (%d)", rc);
+		
+		if (server->ops->recv(server, &msg) != IPC_OK) {
+			perror("Receive failure:");
 			return !hack_return_good;
 		}
 
@@ -530,17 +559,17 @@ admin_msg_callback(IPC_Channel * server, void *private_data)
 		if (new_input->xml == NULL) {
 			crm_info("XML in IPC message was not valid... "
 				 "discarding.");
-			goto cleanup;
-			
+			continue;
+
 		} else if (validate_crm_message(
 				   new_input->msg, crm_system_name, admin_uuid,
 				   XML_ATTR_RESPONSE) == FALSE) {
-			crm_debug_2("Message was not a CRM response. Discarding.");
-			goto cleanup;
+			crm_info("Message was not a CRM response. Discarding.");
+			continue;
 		}
 
 		result = cl_get_string(new_input->msg, XML_ATTR_RESULT);
-		if(result == NULL || strcasecmp(result, "ok") == 0) {
+		if(result == NULL || strcmp(result, "ok") == 0) {
 			result = "pass";
 		} else {
 			result = "fail";
@@ -575,24 +604,19 @@ admin_msg_callback(IPC_Channel * server, void *private_data)
 			/* 31 = "test-_.xml" + an_int_as_string + '\0' */
 			filename_len = 31 + strlen(this_msg_reference);
 
-			crm_malloc0(filename, filename_len);
+			crm_malloc0(filename, sizeof(char) * filename_len);
 			if(filename != NULL) {
-				snprintf(filename, filename_len,
-					"%s-%s_%d.xml",
+				sprintf(filename, "%s-%s_%d.xml",
 					result, this_msg_reference,
 					received_responses);
 				
 				filename[filename_len - 1] = '\0';
-				if (0 > write_xml_file(
-					    new_input->xml, filename, FALSE)) {
+				if (0 > write_xml_file(new_input->xml, filename)) {
 					crm_crit("Could not save response to"
 						 " %s", filename);
 				}
 			}
 		}
-	  cleanup:
-		delete_ha_msg_input(new_input);
-		new_input = NULL;
 	}
 
 	if (server->ch_status == IPC_DISCONNECT) {
@@ -604,7 +628,8 @@ admin_msg_callback(IPC_Channel * server, void *private_data)
 		crm_debug_2(
 		       "Recieved expected number (%d) of messages from Heartbeat."
 		       "  Exiting normally.", expected_responses);
-		exit(0);
+		g_main_quit(mainloop);
+		return !hack_return_good;
 	}
 
 	message_timer_id = Gmain_timeout_add(
@@ -621,12 +646,85 @@ admin_message_timeout(gpointer data)
 		(int)message_timeout_ms/1000);
 	crm_err("No messages received in %d seconds",
 		(int)message_timeout_ms/1000);
-	operation_status = -3;
 	g_main_quit(mainloop);
 	return FALSE;
 }
 
 
+int
+do_find_resource(const char *rsc, crm_data_t *xml_node)
+{
+	int found = 0;
+	crm_data_t *nodestates = get_object_root(XML_CIB_TAG_STATUS, xml_node);
+	const char *path2[] = {
+		XML_CIB_TAG_LRM,
+		XML_LRM_TAG_RESOURCES
+	};
+
+	xml_child_iter(
+		nodestates, a_node, XML_CIB_TAG_STATE,
+		crm_data_t *rscstates = NULL;
+
+		if(is_node_online(a_node) == FALSE) {
+			crm_debug_3("Skipping offline node: %s",
+				crm_element_value(a_node, XML_ATTR_ID));
+			continue;
+		}
+		
+		rscstates = find_xml_node_nested(a_node, path2, DIMOF(path2));
+		xml_child_iter(
+			rscstates, rsc_state, XML_LRM_TAG_RESOURCE,
+			const char *id = crm_element_value(
+				rsc_state,XML_ATTR_ID);
+			const char *target = crm_element_value(
+				a_node, XML_ATTR_UNAME);
+			const char *last_op = crm_element_value(
+				rsc_state,XML_LRM_ATTR_LASTOP);
+			const char *op_code = crm_element_value(
+				rsc_state,XML_LRM_ATTR_OPSTATUS);
+			
+			crm_debug_3("checking %s:%s for %s", target, id, rsc);
+
+			if(safe_str_neq(rsc, id)){
+				crm_debug_4("no match");
+				continue;
+			}
+			
+			if(safe_str_eq("stop", last_op)) {
+				crm_debug_3("resource %s is stopped on: %s",
+					  rsc, target);
+				
+			} else if(safe_str_eq(op_code, "-1")) {
+				crm_debug_3("resource %s is pending on: %s",
+					  rsc, target);				
+
+			} else if(safe_str_neq(op_code, "0")) {
+				crm_debug_3("resource %s is failed on: %s",
+					  rsc, target);				
+
+			} else {
+				crm_debug_3("resource %s is running on: %s",
+					  rsc, target);				
+				printf("resource %s is running on: %s\n",
+				       rsc, target);
+				if(BE_SILENT) {
+					fprintf(stderr, "%s ", target);
+				}
+				found++;
+			}
+			);
+		if(BE_SILENT) {
+			fprintf(stderr, "\n");
+		}
+		);
+	
+	if(found == 0) {
+		printf("resource %s is NOT running\n", rsc);
+	}
+					
+	return found;
+}
+
 gboolean
 is_node_online(crm_data_t *node_state) 
 {
@@ -652,13 +750,55 @@ is_node_online(crm_data_t *node_state)
 	return FALSE;
 }
 
+
+int
+do_find_resource_list(int level, crm_data_t *resource_list)
+{
+	int lpc = 0;
+	int found = 0;
+	const char *name = NULL;
+	const char *type = NULL;
+	const char *class = NULL;
+
+	xml_child_iter(
+		resource_list, rsc, NULL,
+		name = crm_element_name(rsc);
+		if(safe_str_eq(name, XML_CIB_TAG_RESOURCE)) {
+			class = crm_element_value(rsc, "class");
+			type = crm_element_value(rsc, XML_ATTR_TYPE);
+			for(lpc = 0; lpc < level; lpc++) {
+				printf("\t");
+			}
+			found++;
+			printf("%s: %s (%s::%s)\n",
+			       name, ID(rsc), crm_str(class), crm_str(type));
+		} else if(safe_str_eq(name, XML_CIB_TAG_GROUP)
+			  || safe_str_eq(name, XML_CIB_TAG_INCARNATION)) {
+			for(lpc = 0; lpc < level; lpc++) {
+				printf("\t");
+			}
+			printf("%s: %s (complex)\n", name, ID(rsc));
+			do_find_resource_list(level+1, rsc);
+			found++;
+		}
+		);
+	if(found == 0) {
+		for(lpc = 0; lpc < level; lpc++) {
+			printf("\t");
+		}
+		printf("NO resources configured\n");
+	}
+					
+	return found;
+}
+
 int
 do_find_node_list(crm_data_t *xml_node)
 {
 	int found = 0;
 	crm_data_t *nodes = get_object_root(XML_CIB_TAG_NODES, xml_node);
 
-	xml_child_iter_filter(
+	xml_child_iter(
 		nodes, node, XML_CIB_TAG_NODE,	
 		if(BASH_EXPORT) {
 			printf("export %s=%s\n",
@@ -687,11 +827,10 @@ usage(const char *cmd, int exit_status)
 
 	stream = exit_status ? stderr : stdout;
 
-	fprintf(stream, "usage: %s [-?Vs] [command] [command args]\n", cmd);
+	fprintf(stream, "usage: %s [-?vs] [command] [command args]\n", cmd);
 
 	fprintf(stream, "Options\n");
 	fprintf(stream, "\t--%s (-%c)\t: this help message\n", "help", '?');
-	fprintf(stream, "\t--%s (-%c)\t: version details\n", "version", 'v');
 	fprintf(stream, "\t--%s (-%c)\t: "
 		"turn on debug info. additional instances increase verbosity\n",
 		"verbose", 'V');
@@ -707,16 +846,22 @@ usage(const char *cmd, int exit_status)
 		"shutdown the CRMd on <node>\n", "kill", 'K');
 	fprintf(stream, "\t--%s (-%c) <node>\t: "
 		"request the status of <node>\n", "status", 'S');
-#if 0
 	fprintf(stream, "\t--%s (-%c)\t\t: "
 		"request the status of all nodes\n", "health", 'H');
-#endif
 	fprintf(stream, "\t--%s (-%c) <node>\t: "
 		"initiate an election from <node>\n", "election", 'E');
 	fprintf(stream, "\t--%s (-%c)\t: "
 		"request the uname of the DC\n", "dc_lookup", 'D');
 	fprintf(stream, "\t--%s (-%c)\t\t: "
 		"request the uname of all member nodes\n", "nodes", 'N');
+	fprintf(stream, "\t--%s (-%c)\t: "
+		"request the names of all resources\n", "resources", 'R');
+	fprintf(stream, "\t--%s (-%c) <rsc>\t: "
+		"request the location of <rsc>\n", "whereis", 'W');
+	fprintf(stream, "\t--%s (-%c) <node_uuid>\t: "
+		"Tell the node to enter \"standby\" mode\n", "standby", 's');
+	fprintf(stream, "\t--%s (-%c) <node_uuid>\t: "
+		"Tell the node to exit \"standby\" mode\n", "active", 'a');
 /*	fprintf(stream, "\t--%s (-%c)\t\n", "disconnect", 'D'); */
 	fflush(stream);
 
