@@ -1,3 +1,4 @@
+/* $Id: cibmon.c,v 1.28 2005/07/03 22:15:49 alan Exp $ */
 
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
@@ -17,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 
@@ -47,6 +48,7 @@
 #  include <getopt.h>
 #endif
 #include <ha_msg.h> /* someone complaining about _ha_msg_mod not being found */
+#include <crm/dmalloc_wrapper.h>
 
 #define UPDATE_PREFIX "cib.updates:"
 
@@ -95,10 +97,13 @@ main(int argc, char **argv)
 	};
 #endif
 
-	crm_log_init(crm_system_name, LOG_INFO, FALSE, FALSE, 0, NULL);
+	crm_log_init(crm_system_name);
 
 	G_main_add_SignalHandler(
 		G_PRIORITY_HIGH, SIGTERM, cibmon_shutdown, NULL, NULL);
+
+	cl_set_corerootdir(HA_COREDIR);	    
+	cl_cdtocoredir();
 	
 	while (1) {
 #ifdef HAVE_GETOPT_H
@@ -134,7 +139,7 @@ main(int argc, char **argv)
 				usage(crm_system_name, LSB_EXIT_OK);
 				break;
 			case 'm':
-				max_failures = crm_parse_int(optarg, "30");
+				max_failures = crm_atoi(optarg, "30");
 				break;
 			case 'a':
 				pre_notify = TRUE;
@@ -439,11 +444,11 @@ cibmon_diff(const char *event, HA_Message *msg)
 
 	if(rc < cib_ok) {
 		log_level = LOG_WARNING;
-		do_crm_log(log_level, "[%s] %s ABORTED: %s",
+		crm_log_maybe(log_level, "[%s] %s ABORTED: %s",
 			      event, op, cib_error2string(rc));
 		
 	} else {
-		do_crm_log(log_level, "[%s] %s confirmed", event, op);
+		crm_log_maybe(log_level, "[%s] %s confirmed", event, op);
 	}
 
 	log_cib_diff(log_level, diff, op);
