@@ -1,3 +1,4 @@
+/* $Id: primatives.c,v 1.27 2005/09/12 11:00:19 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 #include <stdio.h>
@@ -36,6 +37,7 @@
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
 
+#include <crm/dmalloc_wrapper.h>
 
 
 /*
@@ -316,7 +318,7 @@ delete_cib_object(crm_data_t *parent, crm_data_t *delete_spec)
 	}
 	object_id = crm_element_value(delete_spec, XML_ATTR_ID);
 
-	crm_debug_3("Processing: <%s id=%s>",
+	crm_debug_2("Processing: <%s id=%s>",
 		    crm_str(object_name), crm_str(object_id));
 	
 	if(delete_spec == NULL) {
@@ -337,7 +339,7 @@ delete_cib_object(crm_data_t *parent, crm_data_t *delete_spec)
 		; /* nothing */
 		
 	} else if(equiv_node == NULL) {
-		result = cib_ok;
+		result = cib_NOTEXISTS;
 
 	} else if(xml_has_children(delete_spec) == FALSE) {
 		/*  only leaves are deleted */
@@ -348,12 +350,12 @@ delete_cib_object(crm_data_t *parent, crm_data_t *delete_spec)
 	} else {
 
 		xml_child_iter(
-			delete_spec, child, 
+			delete_spec, child, NULL,
 
 			int tmp_result = delete_cib_object(equiv_node, child);
 			
 			/*  only the first error is likely to be interesting */
-			if(tmp_result != cib_ok && result == cib_ok) {
+			if(result == cib_ok) {
 				result = tmp_result;
 			}
 			);
@@ -375,10 +377,7 @@ add_cib_object(crm_data_t *parent, crm_data_t *new_obj)
 	}
 	object_id = crm_element_value(new_obj, XML_ATTR_ID);
 
-	crm_debug_3("Processing: <%s id=%s>",
-		    crm_str(object_name), crm_str(object_id));
-	
-	if(new_obj == NULL || object_name == NULL) {
+	if(new_obj == NULL) {
 		result = cib_NOOBJECT;
 
 	} else if(parent == NULL) {
@@ -398,8 +397,9 @@ add_cib_object(crm_data_t *parent, crm_data_t *new_obj)
 	} else if(equiv_node != NULL) {
 		result = cib_EXISTS;
 
-	} else {
-		result = update_cib_object(parent, new_obj);
+	} else if(add_node_copy(parent, new_obj) == NULL) {
+		result = cib_NODECOPY;
+		
 	}
 
 	return result;
@@ -427,9 +427,6 @@ update_cib_object(crm_data_t *parent, crm_data_t *update)
 	CRM_DEV_ASSERT(object_name != NULL);
 	if(crm_assert_failed) { return cib_NOOBJECT; }
 
-	crm_debug_3("Processing: <%s id=%s>",
-		    crm_str(object_name), crm_str(object_id));
-	
 	if(object_id == NULL) {
 		/*  placeholder object */
 		target = find_xml_node(parent, object_name, FALSE);
@@ -439,53 +436,50 @@ update_cib_object(crm_data_t *parent, crm_data_t *update)
 	}
 
 	if(target == NULL) {
-		target = create_xml_node(parent, object_name);
+		target = add_node_copy(parent, update);
+		crm_debug_2("Added  <%s id=%s>",
+			    crm_str(object_name), crm_str(object_id));
+
+		CRM_DEV_ASSERT(target != NULL);
+		if(crm_assert_failed) { return cib_NODECOPY; }
+
+		return cib_ok;
+		
 	} 
 
 	crm_debug_2("Found node <%s id=%s> to update",
 		    crm_str(object_name), crm_str(object_id));
 	
 	replace = crm_element_value(update, XML_CIB_ATTR_REPLACE);
+	
 	if(replace != NULL) {
-	    crm_data_t *remove = NULL;
-	    int last = 0, lpc = 0, len = 0;
-
-	    len = strlen(replace);
-	    while(lpc <= len) {
-		if(replace[lpc] == ',' || replace[lpc] == 0) {
-		    char *replace_item = NULL;
-		    if ( last == lpc ) {
-			/* nothing to do */
-			last = lpc+1;
-			goto incr;
-		    }
-
-		    crm_malloc0(replace_item, lpc - last + 1);
-		    strncpy(replace_item, replace+last, lpc-last);
-		    
-		    remove = find_xml_node(target, replace_item, FALSE);
-		    if(remove != NULL) {
+		crm_data_t *remove = find_xml_node(target, replace, FALSE);
+		if(remove != NULL) {
 			crm_debug_3("Replacing node <%s> in <%s>",
-				    replace_item, crm_element_name(target));
+				    replace, crm_element_name(target));
 			zap_xml_from_parent(target, remove);
-		    }
-		    crm_free(replace_item);
-		    last = lpc+1;
 		}
-	      incr:
-		lpc++;
-	    }
-	    xml_remove_prop(update, XML_CIB_ATTR_REPLACE);
-	    xml_remove_prop(target, XML_CIB_ATTR_REPLACE);
+		xml_remove_prop(update, XML_CIB_ATTR_REPLACE);
+		xml_remove_prop(target, XML_CIB_ATTR_REPLACE);
 	}
 	
-	copy_in_properties(target, update);
+	if(safe_str_eq(XML_CIB_TAG_STATE, object_name)){
+		update_node_state(target, update);
+		
+	} else {
+		copy_in_properties(target, update);
+	}
 
+	CRM_DEV_ASSERT(cl_is_allocated(object_name));
+	if(object_id != NULL) {
+		CRM_DEV_ASSERT(cl_is_allocated(object_id));
+	}
+	
 	crm_debug_3("Processing children of <%s id=%s>",
 		    crm_str(object_name), crm_str(object_id));
 	
 	xml_child_iter(
-		update, a_child,  
+		update, a_child, NULL, 
 		int tmp_result = 0;
 		crm_debug_3("Updating child <%s id=%s>",
 			    crm_element_name(a_child), ID(a_child));
@@ -509,3 +503,64 @@ update_cib_object(crm_data_t *parent, crm_data_t *update)
 	return result;
 }
 
+void
+update_node_state(crm_data_t *target, crm_data_t *update)
+{
+	const char *source	= NULL;
+	gboolean any_updates    = FALSE;
+	gboolean clear_stonith  = FALSE;
+	gboolean clear_shutdown = FALSE;
+
+	xml_prop_iter(
+		update, local_prop_name, local_prop_value,
+		
+		if(local_prop_name == NULL) {
+			/*  error */
+			
+		} else if(strcmp(local_prop_name, XML_ATTR_ID) == 0) {
+			
+		} else if(strcmp(local_prop_name, XML_ATTR_TSTAMP) == 0) {
+
+		} else if(strcmp(local_prop_name, XML_CIB_ATTR_CLEAR_SHUTDOWN) == 0) {
+			clear_shutdown = TRUE;
+			
+		} else if(strcmp(local_prop_name, XML_CIB_ATTR_CLEAR_STONITH) == 0) {
+			clear_stonith = TRUE;
+			clear_shutdown = TRUE;			
+			
+		} else if(strcmp(local_prop_name, XML_CIB_ATTR_SOURCE) == 0) {
+			source = local_prop_value;
+			
+		} else {
+			any_updates = TRUE;
+			crm_xml_add(target, local_prop_name, local_prop_value);
+		}
+		);
+	
+	xml_remove_prop(target, XML_CIB_ATTR_CLEAR_SHUTDOWN);
+	if(clear_shutdown) {
+		/*  unset XML_CIB_ATTR_SHUTDOWN  */
+		if(crm_element_value(target, XML_CIB_ATTR_SHUTDOWN) != NULL) {
+			crm_debug_2("Clearing %s", XML_CIB_ATTR_SHUTDOWN);
+			xml_remove_prop(target, XML_CIB_ATTR_SHUTDOWN);
+			any_updates = TRUE;
+		}
+	}
+
+	xml_remove_prop(target, XML_CIB_ATTR_CLEAR_STONITH);
+	if(clear_stonith) {
+		/*  unset XML_CIB_ATTR_STONITH */
+		if(crm_element_value(target, XML_CIB_ATTR_STONITH) != NULL) {
+			crm_debug_2("Clearing %s", XML_CIB_ATTR_STONITH);
+			xml_remove_prop(target, XML_CIB_ATTR_STONITH);
+			any_updates = TRUE;
+		}
+	}
+	
+	if(any_updates) {
+		set_node_tstamp(target);
+		crm_xml_add(target, XML_CIB_ATTR_SOURCE, source);
+	}
+	
+}
+
