@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 #include <crm/crm.h>
@@ -48,6 +48,7 @@
 #include <crm/cib.h>
 #include <crmd.h>
 
+#include <crm/dmalloc_wrapper.h>
 
 #define CLIENT_EXIT_WAIT 30
 
@@ -56,20 +57,23 @@ void do_pe_invoke_callback(const HA_Message *msg, int call_id, int rc,
 			   crm_data_t *output, void *user_data);
 
 /*	 A_PE_START, A_PE_STOP, A_TE_RESTART	*/
-void
+enum crmd_fsa_input
 do_pe_control(long long action,
 	      enum crmd_fsa_cause cause,
 	      enum crmd_fsa_state cur_state,
 	      enum crmd_fsa_input current_input,
 	      fsa_data_t *msg_data)
 {
+	enum crmd_fsa_input result = I_NULL;
 	struct crm_subsystem_s *this_subsys = pe_subsystem;
 
 	long long stop_actions = A_PE_STOP;
 	long long start_actions = A_PE_START;
 	
 	if(action & stop_actions) {
-		stop_subsystem(this_subsys, FALSE);
+		if(stop_subsystem(this_subsys) == FALSE) {
+			register_fsa_error(C_FSA_INTERNAL, I_FAIL, NULL);
+		}
 	}
 
 	if(action & start_actions) {
@@ -82,13 +86,15 @@ do_pe_control(long long action,
 			       this_subsys->name);
 		}
 	}
+	
+	return result;
 }
 
 int fsa_pe_query = 0;
 char *fsa_pe_ref = NULL;
 
 /*	 A_PE_INVOKE	*/
-void
+enum crmd_fsa_input
 do_pe_invoke(long long action,
 	     enum crmd_fsa_cause cause,
 	     enum crmd_fsa_state cur_state,
@@ -96,9 +102,32 @@ do_pe_invoke(long long action,
 	     fsa_data_t *msg_data)
 {
 	if(is_set(fsa_input_register, R_PE_CONNECTED) == FALSE){
+		if(pe_subsystem->pid > 0) {
+			int pid_status = -1;
+			int rc = waitpid(
+				pe_subsystem->pid, &pid_status, WNOHANG);
+
+			if(rc > 0 && WIFEXITED(pid_status)) {
+				clear_bit_inplace(fsa_input_register,
+						  pe_subsystem->flag_connected);
+	
+				if(is_set(fsa_input_register,
+					  pe_subsystem->flag_required)) {
+					/* this wasnt supposed to happen */
+					crm_err("%s[%d] terminated during start",
+						pe_subsystem->name,
+						pe_subsystem->pid);
+					register_fsa_error(
+						C_FSA_INTERNAL, I_ERROR, NULL);
+				}
+				pe_subsystem->pid = -1;
+				return I_NULL;
+			}
+		} 
+		
 		crm_info("Waiting for the PE to connect");
 		crmd_fsa_stall(NULL);
-		return;		
+		return I_NULL;		
 	}
 
 	if(is_set(fsa_input_register, R_HAVE_CIB) == FALSE) {
@@ -107,7 +136,7 @@ do_pe_invoke(long long action,
 
 		/* start the join from scratch */
 		register_fsa_input_before(C_FSA_INTERNAL, I_ELECTION, NULL);
-		return;		
+		return I_NULL;		
 	}
 	
 	crm_debug("Requesting the current CIB: %s",fsa_state2string(fsa_state));
@@ -119,6 +148,8 @@ do_pe_invoke(long long action,
 			pe_subsystem->name);
 		register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
 	}
+	
+	return I_NULL;		
 }
 
 void
@@ -128,17 +159,8 @@ do_pe_invoke_callback(const HA_Message *msg, int call_id, int rc,
 	HA_Message *cmd = NULL;
 	int ccm_transition_id = -1;
 	gboolean cib_has_quorum = FALSE;
-	crm_data_t *local_cib = NULL;
-	
-#if CRM_DEPRECATED_SINCE_2_0_4
-	if(safe_str_eq(crm_element_name(output), XML_TAG_CIB)) {
-		local_cib = output;
-	} else {
-		local_cib = find_xml_node(output, XML_TAG_CIB, TRUE);
-	}
-#else
-	local_cib = output;
-#endif
+	crm_data_t *local_cib = find_xml_node(output, XML_TAG_CIB, TRUE);
+
 	if(call_id != fsa_pe_query) {
 		crm_debug_2("Skipping superceeded CIB query: %d (current=%d)",
 			    call_id, fsa_pe_query);
@@ -167,7 +189,7 @@ do_pe_invoke_callback(const HA_Message *msg, int call_id, int rc,
 	cib_has_quorum = crm_is_true(
 		crm_element_value(local_cib, XML_ATTR_HAVE_QUORUM));
 
-	ccm_transition_id = crm_parse_int(
+	ccm_transition_id = crm_atoi(
 		crm_element_value(local_cib, XML_ATTR_CCM_TRANSITION), "-1");
 
 	if(ccm_transition_id < (int)fsa_membership_copy->id) {
