@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 #include <crm/crm.h>
@@ -48,6 +48,7 @@
 #include <crm/cib.h>
 #include <crmd.h>
 
+#include <crm/dmalloc_wrapper.h>
 
 static void
 crmdManagedChildRegistered(ProcTrack* p)
@@ -63,22 +64,16 @@ crmdManagedChildDied(
 {
 	struct crm_subsystem_s *the_subsystem = p->privatedata;
 	
-	crm_info("Process %s:[%d] exited (signal=%d, exitcode=%d)",
-		 the_subsystem->name, the_subsystem->pid, signo, exitcode);
-	
 	the_subsystem->pid = -1;
-	the_subsystem->ipc = NULL;	
+	the_subsystem->ipc = NULL;
 	clear_bit_inplace(fsa_input_register, the_subsystem->flag_connected);
 
-	crm_debug_3("Triggering FSA: %s", __FUNCTION__);
-	G_main_set_trigger(fsa_source);
-
 	if(is_set(fsa_input_register, the_subsystem->flag_required)) {
 		/* this wasnt supposed to happen */
 		crm_err("The %s subsystem terminated unexpectedly",
 			the_subsystem->name);
 		
-		register_fsa_input_before(C_FSA_INTERNAL, I_ERROR, NULL);
+		register_fsa_input_before(C_IPC_MESSAGE, I_ERROR, NULL);
 	}
 
 	p->privatedata = NULL;
@@ -98,41 +93,36 @@ static ProcTrack_ops crmd_managed_child_ops = {
 };
 
 gboolean
-stop_subsystem(struct crm_subsystem_s *the_subsystem, gboolean force_quit)
+stop_subsystem(struct crm_subsystem_s*	the_subsystem)
 {
-	int quit_signal = SIGTERM;
+	IPC_Channel *client_channel = the_subsystem->ipc;
 	crm_debug_2("Stopping sub-system \"%s\"", the_subsystem->name);
 	clear_bit_inplace(fsa_input_register, the_subsystem->flag_required);
 	
 	if (the_subsystem->pid <= 0) {
 		crm_debug_2("Client %s not running", the_subsystem->name);
-		return FALSE;
-		
-	}
 
-	if(is_set(fsa_input_register, the_subsystem->flag_connected) == FALSE) {
+	} else if(FALSE == is_set(
+			  fsa_input_register, the_subsystem->flag_connected)) {
 		/* running but not yet connected */
-		crm_debug("Stopping %s before it had connected",
-			  the_subsystem->name);
-	}
-/*
-	if(force_quit && the_subsystem->sent_kill == FALSE) {
-		quit_signal = SIGKILL;
-
-	} else if(force_quit) {
-		crm_debug("Already sent -KILL to %s: [%d]",
-			  the_subsystem->name, the_subsystem->pid);
-	}
-*/
-	errno = 0;
-	if(CL_KILL(the_subsystem->pid, quit_signal) == 0) {
-		crm_info("Sent -TERM to %s: [%d]",
-			 the_subsystem->name, the_subsystem->pid);
-		the_subsystem->sent_kill = TRUE;
+		crm_warn("Stopping %s before it had connected",
+			 the_subsystem->name);
+		
+		CL_KILL(the_subsystem->pid, -SIGKILL);
+		the_subsystem->pid = -1;
+		
+	} else if(client_channel == NULL
+		  || client_channel->ops->get_chan_status(
+			  client_channel) != IPC_CONNECT) {
+		crm_err("Client %s has already quit", the_subsystem->name);
 		
 	} else {
-		cl_perror("Sent -TERM to %s: [%d]",
-			  the_subsystem->name, the_subsystem->pid);
+		HA_Message *quit = create_request(
+			CRM_OP_QUIT, NULL, NULL, the_subsystem->name,
+			AM_I_DC?CRM_SYSTEM_DC:CRM_SYSTEM_CRMD, NULL);
+	
+		crm_info("Sending quit message to %s.", the_subsystem->name);
+		send_ipc_message(client_channel, quit);
 	}
 	
 	return TRUE;
@@ -148,6 +138,7 @@ start_subsystem(struct crm_subsystem_s*	the_subsystem)
 	unsigned int	j;
 	struct rlimit	oflimits;
 	const char 	*devnull = "/dev/null";
+	char *args = NULL;
 
 	crm_info("Starting sub-system \"%s\"", the_subsystem->name);
 	set_bit_inplace(fsa_input_register, the_subsystem->flag_required);
@@ -185,21 +176,15 @@ start_subsystem(struct crm_subsystem_s*	the_subsystem)
 		default:	/* Parent */
 			NewTrackedProc(pid, 0, PT_LOGNORMAL,
 				       the_subsystem, &crmd_managed_child_ops);
-			crm_debug_2("Client %s is has pid: %d",
-				    the_subsystem->name, pid);
 			the_subsystem->pid = pid;
 			return TRUE;
 
 		case 0:		/* Child */
-			/* create a new process group to avoid
-			 * being interupted by heartbeat
-			 */
-			setpgid(0, 0);
 			break;
 	}
 
-	crm_debug("Executing \"%s (%s)\" (pid %d)",
-		  the_subsystem->command, the_subsystem->name, (int) getpid());
+	crm_debug("Executing \"%s %s\" (pid %d)",
+		  the_subsystem->command, the_subsystem->args, (int) getpid());
 
 	/* A precautionary measure */
 	getrlimit(RLIMIT_NOFILE, &oflimits);
@@ -211,25 +196,22 @@ start_subsystem(struct crm_subsystem_s*	the_subsystem)
 	(void)open(devnull, O_WRONLY);	/* Stdout: fd 1 */
 	(void)open(devnull, O_WRONLY);	/* Stderr: fd 2 */
 	
-	if(getenv("HA_VALGRIND_ENABLED") != NULL) {
-		char *opts[] = { crm_strdup(VALGRIND_BIN),
- 				 crm_strdup("--show-reachable=yes"),
-				 crm_strdup("--leak-check=full"),
-				 crm_strdup("--time-stamp=yes"),
-				 crm_strdup("--suppressions="VALGRIND_SUPP),
-/* 				 crm_strdup("--gen-suppressions=all"), */
-				 crm_strdup(VALGRIND_LOG),
-				 crm_strdup(the_subsystem->command),
-				 NULL
+	if(the_subsystem->args != NULL) {
+		args = crm_strdup(the_subsystem->args);
+	}
+	{
+		char* const start_args[] = {
+			crm_strdup(the_subsystem->command),
+			args,
+			NULL
 		};
-		(void)execvp(VALGRIND_BIN, opts);
-	} else {
-		char *opts[] = { crm_strdup(the_subsystem->command), NULL };
-		(void)execvp(the_subsystem->command, opts);
+		
+		(void)execvp(the_subsystem->command, start_args);
 	}
 	
 	/* Should not happen */
-	cl_perror("FATAL: Cannot exec %s", the_subsystem->command);
+	cl_perror("FATAL: Cannot exec %s %s",
+		  the_subsystem->command, crm_str(the_subsystem->args));
 
 	exit(100); /* Suppress respawning */
 	return TRUE; /* never reached */
