@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 #include <crm/crm.h>
@@ -47,17 +47,19 @@
 #include <crm/cib.h>
 #include <crmd.h>
 
+#include <crm/dmalloc_wrapper.h>
 
 struct crm_subsystem_s *te_subsystem  = NULL;
 
 /*	 A_TE_START, A_TE_STOP, A_TE_RESTART	*/
-void
+enum crmd_fsa_input
 do_te_control(long long action,
 	      enum crmd_fsa_cause cause,
 	      enum crmd_fsa_state cur_state,
 	      enum crmd_fsa_input current_input,
 	      fsa_data_t *msg_data)
 {
+	enum crmd_fsa_input result = I_NULL;
 	struct crm_subsystem_s *this_subsys = te_subsystem;
 	
 	long long stop_actions = A_TE_STOP;
@@ -70,7 +72,9 @@ do_te_control(long long action,
 /* 		} */
 	
 	if(action & stop_actions) {
-		stop_subsystem(this_subsys, FALSE);
+		if(stop_subsystem(this_subsys) == FALSE) {
+			register_fsa_error(C_FSA_INTERNAL, I_FAIL, NULL);
+		}
 	}
 
 	if(action & start_actions) {
@@ -83,67 +87,83 @@ do_te_control(long long action,
 				 this_subsys->name);
 		}
 	}
+
+	return result;
 }
 
 /*	 A_TE_INVOKE, A_TE_CANCEL	*/
-void
+enum crmd_fsa_input
 do_te_invoke(long long action,
 	     enum crmd_fsa_cause cause,
 	     enum crmd_fsa_state cur_state,
 	     enum crmd_fsa_input current_input,
 	     fsa_data_t *msg_data)
 {
+	enum crmd_fsa_input ret = I_NULL;
 	HA_Message *cmd = NULL;
 	
 	
 	if(AM_I_DC == FALSE) {
 		crm_debug("Not DC: No need to invoke the TE (anymore): %s",
 			  fsa_action2string(action));
-		return;
+		return I_NULL;
 		
-	} else if(fsa_state != S_TRANSITION_ENGINE && (action & A_TE_INVOKE)) {
+	} else if(fsa_state != S_TRANSITION_ENGINE && action ^ A_TE_CANCEL) {
 		crm_debug("No need to invoke the TE (%s) in state %s",
 			  fsa_action2string(action),
 			  fsa_state2string(fsa_state));
-		return;
-
-	} else if(!is_set(fsa_input_register, te_subsystem->flag_required)) {
-		crm_err("Ignoring action %s in state: %s"
-			" - We dont want the TE anymore",
-			fsa_action2string(action), fsa_state2string(cur_state));
-		return;
+		return I_NULL;
 		
+	} else if(is_set(fsa_input_register, R_SHUTDOWN)) {
+		crm_err("No point invoking the TE, we're shutting down");
+		return I_NULL;
+
 	} else if(is_set(fsa_input_register, R_TE_CONNECTED) == FALSE) {
+		if(te_subsystem->pid > 0) {
+			int pid_status = -1;
+			int rc = waitpid(
+				te_subsystem->pid, &pid_status, WNOHANG);
+
+			if(rc > 0 && WIFEXITED(pid_status)) {
+				clear_bit_inplace(fsa_input_register,
+						  te_subsystem->flag_connected);
+	
+				if(is_set(fsa_input_register,
+					  te_subsystem->flag_required)) {
+					/* this wasnt supposed to happen */
+					crm_err("%s[%d] terminated during start",
+						te_subsystem->name,
+						te_subsystem->pid);
+					register_fsa_error(
+						C_FSA_INTERNAL, I_ERROR, NULL);
+				}
+				te_subsystem->pid = -1;
+				return I_NULL;
+			}
+		} 
+
 		crm_info("Waiting for the TE to connect before action %s",
 			fsa_action2string(action));
-		
+
 		if(action & A_TE_INVOKE) {
 			register_fsa_input(
 				msg_data->fsa_cause, msg_data->fsa_input,
 				msg_data->data);
 		}
-
 		crmd_fsa_stall(NULL);
-		return;
+		return I_NULL;
 	}
 
 	if(action & A_TE_INVOKE) {
 		ha_msg_input_t *input = fsa_typed_data(fsa_dt_ha_msg);
-		const char *graph_file = cl_get_string(input->msg, F_CRM_TGRAPH);
-		const char *graph_input = cl_get_string(input->msg, F_CRM_TGRAPH_INPUT);
+		if(input->xml != NULL) {
 
-		if(graph_file != NULL || input->xml != NULL) {			
 			crm_debug("Starting a transition");
 			set_bit_inplace(fsa_input_register, R_IN_TRANSITION);
 			
 			cmd = create_request(
 				CRM_OP_TRANSITION, input->xml, NULL,
 				CRM_SYSTEM_TENGINE, CRM_SYSTEM_DC, NULL);
-
-			ha_msg_add(cmd, F_CRM_TGRAPH_INPUT, graph_input);
-			if(graph_file) {
-				ha_msg_add(cmd, F_CRM_TGRAPH, graph_file);
-			}
 			
 			send_request(cmd, NULL);
 
@@ -166,6 +186,8 @@ do_te_invoke(long long action,
 		
 		send_request(cmd, NULL);
 	}
+
+	return ret;
 }
 
 
