@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 #include <crm/crm.h>
@@ -36,6 +36,7 @@
 #include <crmd_messages.h>
 #include <crmd_utils.h>
 
+#include <crm/dmalloc_wrapper.h>
 
 void copy_ccm_node(oc_node_t a_node, oc_node_t *a_node_copy);
 
@@ -43,7 +44,7 @@ void copy_ccm_node(oc_node_t a_node, oc_node_t *a_node_copy);
  *	A_FINALIZE_TIMER_STOP, A_FINALIZE_TIMER_START
  *	A_INTEGRATE_TIMER_STOP, A_INTEGRATE_TIMER_START
  */
-void
+enum crmd_fsa_input
 do_timer_control(long long action,
 		   enum crmd_fsa_cause cause,
 		   enum crmd_fsa_state cur_state,
@@ -51,6 +52,7 @@ do_timer_control(long long action,
 		   fsa_data_t *msg_data)
 {
 	gboolean timer_op_ok = TRUE;
+	enum crmd_fsa_input result = I_NULL;
 	
 	if(action & A_DC_TIMER_STOP) {
 		timer_op_ok = crm_timer_stop(election_trigger);
@@ -69,8 +71,8 @@ do_timer_control(long long action,
 	if(action & A_DC_TIMER_START && timer_op_ok) {
 		crm_timer_start(election_trigger);
 		if(AM_I_DC) {
-		    /* there can be only one */
-		    register_fsa_input(cause, I_ELECTION, NULL);
+			/* there can be only one */
+			result = I_ELECTION;
 		}
 		
 	} else if(action & A_FINALIZE_TIMER_START) {
@@ -82,34 +84,8 @@ do_timer_control(long long action,
 /* 	} else if(action & A_ELECTION_TIMEOUT_START) { */
 /* 		crm_timer_start(election_timeout); */
 	}
-}
-
-static const char *
-get_timer_desc(fsa_timer_t *timer) 
-{
-	if(timer == election_trigger) {
-		return "Election Trigger";
-
- 	} else if(timer == election_timeout) {
-		return "Election Timeout";
-		
- 	} else if(timer == shutdown_escalation_timer) {
-		return "Shutdown Escalation";
-
- 	} else if(timer == integration_timer) {
-		return "Integration Timer";
-
- 	} else if(timer == finalization_timer) {
-		return "Finalization Timer";
-
- 	} else if(timer == wait_timer) {
-		return "Wait Timer";
-
- 	} else if(timer == recheck_timer) {
-		return "PEngine Recheck Timer";
-		
-	}	
-	return "Unknown Timer";
+	
+	return I_NULL;
 }
 
 gboolean
@@ -117,18 +93,21 @@ crm_timer_popped(gpointer data)
 {
 	fsa_timer_t *timer = (fsa_timer_t *)data;
 
-	if(timer == wait_timer
-	   || timer == recheck_timer
-	   || timer == finalization_timer
-	   || timer == election_trigger) {
-		crm_info("%s (%s) just popped!",
-			get_timer_desc(timer),
+	if(timer == election_trigger) {
+		crm_info("Election Trigger (%s) just popped!",
+			 fsa_input2string(timer->fsa_input));
+
+ 	} else if(timer == election_timeout) {
+		crm_info("Election Timeout (%s) just popped!",
+			 fsa_input2string(timer->fsa_input));
+
+ 	} else if(timer == shutdown_escalation_timer) {
+		crm_err("Shutdown Escalation Timer (%s) just popped!",
 			fsa_input2string(timer->fsa_input));
 		
 	} else {
-		crm_err("%s (%s) just popped!",
-			get_timer_desc(timer),
-			fsa_input2string(timer->fsa_input));
+		crm_info("Timer %s just popped!",
+			 fsa_input2string(timer->fsa_input));
 	}
 
 	if(timer->repeat == FALSE) {
@@ -154,8 +133,6 @@ crm_timer_popped(gpointer data)
 	} else if(timer->fsa_input != I_NULL) {
 		register_fsa_input(C_TIMER_POPPED, timer->fsa_input, NULL);
 	}
-	
-	crm_debug_3("Triggering FSA: %s", __FUNCTION__);
 	G_main_set_trigger(fsa_source);
 	
 	return TRUE;
@@ -164,12 +141,22 @@ crm_timer_popped(gpointer data)
 gboolean
 crm_timer_start(fsa_timer_t *timer)
 {
-	const char *timer_desc = get_timer_desc(timer);
+	const char *timer_desc = NULL;
+	if(timer == election_trigger) {
+		timer_desc = "Election Trigger";
 
-	if(timer->source_id == 0 && timer->period_ms > 0) {
+ 	} else if(timer == election_timeout) {
+		timer_desc = "Election Timeout";
+		
+	} else {
+		timer_desc = "Timer";
+	}
+
+	if((timer->source_id == (guint)-1 || timer->source_id == (guint)-2)
+	   && timer->period_ms > 0) {
 		timer->source_id = Gmain_timeout_add(
 			timer->period_ms, timer->callback, (void*)timer);
-		CRM_ASSERT(timer->source_id != 0);
+
 		crm_debug("Started %s (%s:%dms), src=%d",
 			  timer_desc, fsa_input2string(timer->fsa_input),
 			  timer->period_ms, timer->source_id);
@@ -192,28 +179,91 @@ crm_timer_start(fsa_timer_t *timer)
 gboolean
 crm_timer_stop(fsa_timer_t *timer)
 {
-	const char *timer_desc = get_timer_desc(timer);
+	const char *timer_desc = NULL;
+	if(timer == election_trigger) {
+		timer_desc = "Election Trigger";
+
+ 	} else if(timer == election_timeout) {
+		timer_desc = "Election Timeout";
+		
+ 	} else if(timer == wait_timer) {
+		timer_desc = "Stall Timeout";
+		
+	} else {
+		timer_desc = "Timer";
+	}
 
 	if(timer == NULL) {
 		crm_err("Attempted to stop NULL timer");
 		return FALSE;
 		
-	} else if(timer->source_id != 0) {
+	} else if(timer->source_id != (guint)-1
+		  && timer->source_id != (guint)-2) {
 		crm_debug("Stopping %s (%s:%dms), src=%d",
 			  timer_desc, fsa_input2string(timer->fsa_input),
 			  timer->period_ms, timer->source_id);
 		Gmain_timeout_remove(timer->source_id);
-		timer->source_id = 0;
+		timer->source_id = -2;
 		
 	} else {
 		crm_debug_2("%s (%s:%dms) already stopped",
 			  timer_desc, fsa_input2string(timer->fsa_input),
 			  timer->period_ms);
+		timer->source_id = -2;
 		return FALSE;
 	}
 	return TRUE;
 }
 
+
+long long
+toggle_bit(long long action_list, long long action)
+{
+	crm_debug_5("Toggling bit %.16llx", action);
+	action_list ^= action;
+	crm_debug_5("Result %.16llx", action_list & action);
+	return action_list;
+}
+
+long long
+clear_bit(long long action_list, long long action)
+{
+	unsigned int	level = LOG_DEBUG_5;
+	crm_log_maybe(level, "Clearing bit\t%.16llx", action);
+	
+	/* ensure its set */
+	action_list |= action;
+
+	/* then toggle */
+	action_list = action_list ^ action;
+
+	return action_list;
+}
+
+long long
+set_bit(long long action_list, long long action)
+{
+	unsigned int	level = LOG_DEBUG_5;
+	crm_log_maybe(level, "Setting bit\t%.16llx", action);
+	action_list |= action;
+	return action_list;
+}
+
+
+gboolean
+is_set(long long action_list, long long action)
+{
+	crm_debug_5("Checking bit\t%.16llx in %.16llx", action, action_list);
+	return ((action_list & action) == action);
+}
+
+gboolean
+is_set_any(long long action_list, long long action)
+{
+	crm_debug_5("Checking bit\t%.16llx in %.16llx", action, action_list);
+	return ((action_list & action) != 0);
+}
+
 const char *
 fsa_input2string(enum crmd_fsa_input input)
 {
@@ -527,9 +577,6 @@ fsa_action2string(long long action)
 		case A_ELECTION_VOTE:
 			actionAsText = "A_ELECTION_VOTE";
 			break;
-		case A_ELECTION_CHECK:
-			actionAsText = "A_ELECTION_CHECK";
-			break;
 		case A_CL_JOIN_ANNOUNCE:
 			actionAsText = "A_CL_JOIN_ANNOUNCE";
 			break;
@@ -626,9 +673,6 @@ fsa_action2string(long long action)
 		case A_TE_STOP:
 			actionAsText = "A_TE_STOP";
 			break;
-		case A_TE_HALT:
-			actionAsText = "A_TE_HALT";
-			break;
 		case A_TE_CANCEL:
 			actionAsText = "A_TE_CANCEL";
 			break;
@@ -682,100 +726,100 @@ fsa_dump_inputs(int log_level, const char *text, long long input_register)
 	}
 
 	if(is_set(input_register, R_THE_DC)) {
-		do_crm_log(log_level,
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
 			   "%s %.16llx (R_THE_DC)", text, R_THE_DC);
 	}
 	if(is_set(input_register, R_STARTING)) {
-		do_crm_log(log_level, "%s %.16llx (R_STARTING)",
-			      text, R_STARTING);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_STARTING)", text, R_STARTING);
 	}
 	if(is_set(input_register, R_SHUTDOWN)) {
-		do_crm_log(log_level, "%s %.16llx (R_SHUTDOWN)",
-			      text, R_SHUTDOWN);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_SHUTDOWN)", text, R_SHUTDOWN);
 	}
 	if(is_set(input_register, R_STAYDOWN)) {
-		do_crm_log(log_level, "%s %.16llx (R_STAYDOWN)",
-			      text, R_STAYDOWN);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_STAYDOWN)", text, R_STAYDOWN);
 	}
 	if(is_set(input_register, R_JOIN_OK)) {
-		do_crm_log(log_level, "%s %.16llx (R_JOIN_OK)",
-			      text, R_JOIN_OK);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_JOIN_OK)", text, R_JOIN_OK);
 	}
-	if(is_set(input_register, R_READ_CONFIG)) {
-		do_crm_log(log_level, "%s %.16llx (R_READ_CONFIG)",
-			      text, R_READ_CONFIG);
+	if(is_set(input_register, R_HAVE_RES)) {
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_HAVE_RES)", text, R_HAVE_RES);
 	}
 	if(is_set(input_register, R_INVOKE_PE)) {
-		do_crm_log(log_level, "%s %.16llx (R_INVOKE_PE)",
-			      text, R_INVOKE_PE);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_INVOKE_PE)", text, R_INVOKE_PE);
 	}
 	if(is_set(input_register, R_CIB_CONNECTED)) {
-		do_crm_log(log_level, "%s %.16llx (R_CIB_CONNECTED)",
-			      text, R_CIB_CONNECTED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_CIB_CONNECTED)", text, R_CIB_CONNECTED);
 	}
 	if(is_set(input_register, R_PE_CONNECTED)) {
-		do_crm_log(log_level, "%s %.16llx (R_PE_CONNECTED)",
-			      text, R_PE_CONNECTED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_PE_CONNECTED)", text, R_PE_CONNECTED);
 	}
 	if(is_set(input_register, R_TE_CONNECTED)) {
-		do_crm_log(log_level, "%s %.16llx (R_TE_CONNECTED)",
-			      text, R_TE_CONNECTED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_TE_CONNECTED)", text, R_TE_CONNECTED);
 	}
 	if(is_set(input_register, R_LRM_CONNECTED)) {
-		do_crm_log(log_level, "%s %.16llx (R_LRM_CONNECTED)",
-			      text, R_LRM_CONNECTED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_LRM_CONNECTED)", text, R_LRM_CONNECTED);
 	}
 	if(is_set(input_register, R_CIB_REQUIRED)) {
-		do_crm_log(log_level, "%s %.16llx (R_CIB_REQUIRED)",
-			      text, R_CIB_REQUIRED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_CIB_REQUIRED)", text, R_CIB_REQUIRED);
 	}
 	if(is_set(input_register, R_PE_REQUIRED)) {
-		do_crm_log(log_level, "%s %.16llx (R_PE_REQUIRED)",
-			      text, R_PE_REQUIRED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_PE_REQUIRED)", text, R_PE_REQUIRED);
 	}
 	if(is_set(input_register, R_TE_REQUIRED)) {
-		do_crm_log(log_level, "%s %.16llx (R_TE_REQUIRED)",
-			      text, R_TE_REQUIRED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_TE_REQUIRED)", text, R_TE_REQUIRED);
 	}
 	if(is_set(input_register, R_REQ_PEND)) {
-		do_crm_log(log_level, "%s %.16llx (R_REQ_PEND)",
-			      text, R_REQ_PEND);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_REQ_PEND)", text, R_REQ_PEND);
 	}
 	if(is_set(input_register, R_PE_PEND)) {
-		do_crm_log(log_level, "%s %.16llx (R_PE_PEND)",
-			      text, R_PE_PEND);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_PE_PEND)", text, R_PE_PEND);
 	}
 	if(is_set(input_register, R_TE_PEND)) {
-		do_crm_log(log_level, "%s %.16llx (R_TE_PEND)",
-			      text, R_TE_PEND);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_TE_PEND)", text, R_TE_PEND);
 	}
 	if(is_set(input_register, R_RESP_PEND)) {
-		do_crm_log(log_level, "%s %.16llx (R_RESP_PEND)",
-			      text, R_RESP_PEND);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_RESP_PEND)", text, R_RESP_PEND);
 	}
 	if(is_set(input_register, R_CIB_DONE)) {
-		do_crm_log(log_level, "%s %.16llx (R_CIB_DONE)",
-			      text, R_CIB_DONE);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_CIB_DONE)", text, R_CIB_DONE);
 	}
 	if(is_set(input_register, R_HAVE_CIB)) {
-		do_crm_log(log_level, "%s %.16llx (R_HAVE_CIB)",
-			      text, R_HAVE_CIB);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_HAVE_CIB)", text, R_HAVE_CIB);
 	}
 	if(is_set(input_register, R_CIB_ASKED)) {
-		do_crm_log(log_level, "%s %.16llx (R_CIB_ASKED)",
-			      text, R_CIB_ASKED);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_CIB_ASKED)", text, R_CIB_ASKED);
 	}
 	if(is_set(input_register, R_CCM_DATA)) {
-		do_crm_log(log_level, "%s %.16llx (R_CCM_DATA)",
-			      text, R_CCM_DATA);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_CCM_DATA)", text, R_CCM_DATA);
 	}	
 	if(is_set(input_register, R_PEER_DATA)) {
-		do_crm_log(log_level, "%s %.16llx (R_PEER_DATA)",
-			      text, R_PEER_DATA);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_PEER_DATA)", text, R_PEER_DATA);
 	}
 	if(is_set(input_register, R_IN_RECOVERY)) {
-		do_crm_log(log_level, "%s %.16llx (R_IN_RECOVERY)",
-			      text, R_IN_RECOVERY);
+		do_crm_log(log_level, __FILE__, __FUNCTION__,
+			   "%s %.16llx (R_IN_RECOVERY)", text, R_IN_RECOVERY);
 	}
 }
 
@@ -785,272 +829,267 @@ fsa_dump_actions(long long action, const char *text)
 	int log_level = LOG_DEBUG_3;
 	
 	if(is_set(action, A_READCONFIG)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_READCONFIG) %s", A_READCONFIG, text);
 	}
 	if(is_set(action, A_STARTUP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_STARTUP) %s", A_STARTUP, text);
 	}
 	if(is_set(action, A_STARTED)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_STARTED) %s", A_STARTED, text);
 	}
 	if(is_set(action, A_HA_CONNECT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CONNECT) %s", A_HA_CONNECT, text);
 	}
 	if(is_set(action, A_HA_DISCONNECT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DISCONNECT) %s",
 			  A_HA_DISCONNECT, text);
 	}
 	if(is_set(action, A_LRM_CONNECT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_LRM_CONNECT) %s",
 			  A_LRM_CONNECT, text);
 	}
 	if(is_set(action, A_LRM_EVENT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_LRM_EVENT) %s",
 			  A_LRM_EVENT, text);
 	}
 	if(is_set(action, A_LRM_INVOKE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_LRM_INVOKE) %s",
 			  A_LRM_INVOKE, text);
 	}
 	if(is_set(action, A_LRM_DISCONNECT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_LRM_DISCONNECT) %s",
 			  A_LRM_DISCONNECT, text);
 	}
 	if(is_set(action, A_DC_TIMER_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_TIMER_STOP) %s",
 			  A_DC_TIMER_STOP, text);
 	}
 	if(is_set(action, A_DC_TIMER_START)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_TIMER_START) %s",
 			  A_DC_TIMER_START, text);
 	}
 	if(is_set(action, A_INTEGRATE_TIMER_START)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_INTEGRATE_TIMER_START) %s",
 			  A_INTEGRATE_TIMER_START, text);
 	}
 	if(is_set(action, A_INTEGRATE_TIMER_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_INTEGRATE_TIMER_STOP) %s",
 			  A_INTEGRATE_TIMER_STOP, text);
 	}
 	if(is_set(action, A_FINALIZE_TIMER_START)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_FINALIZE_TIMER_START) %s",
 			  A_FINALIZE_TIMER_START, text);
 	}
 	if(is_set(action, A_FINALIZE_TIMER_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_FINALIZE_TIMER_STOP) %s",
 			  A_FINALIZE_TIMER_STOP, text);
 	}
 	if(is_set(action, A_ELECTION_COUNT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_ELECTION_COUNT) %s",
 			  A_ELECTION_COUNT, text);
 	}
 	if(is_set(action, A_ELECTION_VOTE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_ELECTION_VOTE) %s",
 			  A_ELECTION_VOTE, text);
 	}
-	if(is_set(action, A_ELECTION_CHECK)) {
-		do_crm_log(log_level, 
-			   "Action %.16llx (A_ELECTION_CHECK) %s",
-			  A_ELECTION_CHECK, text);
-	}
 	if(is_set(action, A_CL_JOIN_ANNOUNCE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CL_JOIN_ANNOUNCE) %s",
 			  A_CL_JOIN_ANNOUNCE, text);
 	}
 	if(is_set(action, A_CL_JOIN_REQUEST)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CL_JOIN_REQUEST) %s",
 			  A_CL_JOIN_REQUEST, text);
 	}
 	if(is_set(action, A_CL_JOIN_RESULT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CL_JOIN_RESULT) %s",
 			  A_CL_JOIN_RESULT, text);
 	}
 	if(is_set(action, A_DC_JOIN_OFFER_ALL)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_JOIN_OFFER_ALL) %s",
 			  A_DC_JOIN_OFFER_ALL, text);
 	}
 	if(is_set(action, A_DC_JOIN_OFFER_ONE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_JOIN_OFFER_ONE) %s",
 			  A_DC_JOIN_OFFER_ONE, text);
 	}
 	if(is_set(action, A_DC_JOIN_PROCESS_REQ)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_JOIN_PROCESS_REQ) %s",
 			  A_DC_JOIN_PROCESS_REQ, text);
 	}
 	if(is_set(action, A_DC_JOIN_PROCESS_ACK)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_JOIN_PROCESS_ACK) %s",
 			  A_DC_JOIN_PROCESS_ACK, text);
 	}
 	if(is_set(action, A_DC_JOIN_FINALIZE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_JOIN_FINALIZE) %s",
 			  A_DC_JOIN_FINALIZE, text);
 	}
 	if(is_set(action, A_MSG_PROCESS)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_MSG_PROCESS) %s",
 			  A_MSG_PROCESS, text);
 	}
 	if(is_set(action, A_MSG_ROUTE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_MSG_ROUTE) %s",
 			  A_MSG_ROUTE, text);
 	}
 	if(is_set(action, A_RECOVER)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_RECOVER) %s",
 			  A_RECOVER, text);
 	}
 	if(is_set(action, A_DC_RELEASE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_RELEASE) %s",
 			  A_DC_RELEASE, text);
 	}
 	if(is_set(action, A_DC_RELEASED)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_RELEASED) %s",
 			  A_DC_RELEASED, text);
 	}
 	if(is_set(action, A_DC_TAKEOVER)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_DC_TAKEOVER) %s",
 			  A_DC_TAKEOVER, text);
 	}
 	if(is_set(action, A_SHUTDOWN)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_SHUTDOWN) %s", A_SHUTDOWN, text);
 	}
 	if(is_set(action, A_SHUTDOWN_REQ)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_SHUTDOWN_REQ) %s",
 			  A_SHUTDOWN_REQ, text);
 	}
 	if(is_set(action, A_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_STOP  ) %s", A_STOP  , text);
 	}
 	if(is_set(action, A_EXIT_0)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_EXIT_0) %s", A_EXIT_0, text);
 	}
 	if(is_set(action, A_EXIT_1)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_EXIT_1) %s", A_EXIT_1, text);
 	}
 	if(is_set(action, A_CCM_CONNECT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CCM_CONNECT) %s",
 			  A_CCM_CONNECT, text);
 	}
 	if(is_set(action, A_CCM_DISCONNECT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CCM_DISCONNECT) %s",
 			  A_CCM_DISCONNECT, text);
 	}
 	if(is_set(action, A_CCM_EVENT)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CCM_EVENT) %s",
 			  A_CCM_EVENT, text);
 	}
 	if(is_set(action, A_CCM_UPDATE_CACHE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CCM_UPDATE_CACHE) %s",
 			  A_CCM_UPDATE_CACHE, text);
 	}
 	if(is_set(action, A_CIB_BUMPGEN)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CIB_BUMPGEN) %s",
 			  A_CIB_BUMPGEN, text);
 	}
 	if(is_set(action, A_CIB_INVOKE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CIB_INVOKE) %s",
 			  A_CIB_INVOKE, text);
 	}
 	if(is_set(action, A_CIB_START)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CIB_START) %s",
 			  A_CIB_START, text);
 	}
 	if(is_set(action, A_CIB_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_CIB_STOP) %s", A_CIB_STOP, text);
 	}
 	if(is_set(action, A_TE_INVOKE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_TE_INVOKE) %s", A_TE_INVOKE, text);
 	}
 	if(is_set(action, A_TE_START)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_TE_START) %s",
 			  A_TE_START, text);
 	}
 	if(is_set(action, A_TE_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_TE_STOP) %s", A_TE_STOP, text);
 	}
 	if(is_set(action, A_TE_CANCEL)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_TE_CANCEL) %s",
 			  A_TE_CANCEL, text);
 	}
 	if(is_set(action, A_PE_INVOKE)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_PE_INVOKE) %s",
 			  A_PE_INVOKE, text);
 	}
 	if(is_set(action, A_PE_START)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_PE_START) %s", A_PE_START, text);
 	}
 	if(is_set(action, A_PE_STOP)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_PE_STOP) %s", A_PE_STOP, text);
 	}
 	if(is_set(action, A_NODE_BLOCK)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_NODE_BLOCK) %s",
 			  A_NODE_BLOCK, text);
 	}
 	if(is_set(action, A_UPDATE_NODESTATUS)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_UPDATE_NODESTATUS) %s",
 			  A_UPDATE_NODESTATUS, text);
 	}
 	if(is_set(action, A_LOG)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_LOG   ) %s", A_LOG, text);
 	}
 	if(is_set(action, A_ERROR)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_ERROR ) %s", A_ERROR, text);
 	}
 	if(is_set(action, A_WARN)) {
-		do_crm_log(log_level, 
+		do_crm_log(log_level, __FILE__, __FUNCTION__, 
 			   "Action %.16llx (A_WARN  ) %s", A_WARN, text);
 	}
 }
@@ -1074,14 +1113,15 @@ create_node_entry(const char *uuid, const char *uname, const char *type)
 	
 	crm_xml_add(tmp1, XML_ATTR_UNAME, uname);
 	crm_xml_add(tmp1, XML_ATTR_TYPE, type);
-
-	fsa_cib_anon_update(XML_CIB_TAG_NODES, tmp1,
-			    cib_scope_local|cib_quorum_override);
-
+	
+	update_local_cib(create_cib_fragment(tmp1, NULL));
 	free_xml(tmp1);
 	
 }
 
+
+
+
 struct crmd_ccm_data_s *
 copy_ccm_data(const struct crmd_ccm_data_s *ccm_input) 
 {
@@ -1097,47 +1137,6 @@ copy_ccm_data(const struct crmd_ccm_data_s *ccm_input)
 	return ccm_input_copy;
 }
 
-void
-delete_ccm_data(struct crmd_ccm_data_s *ccm_input) 
-{
-	int lpc, offset = 0;
-	oc_node_t *a_node = NULL;
-	oc_ev_membership_t *oc_in = NULL;
-	if(ccm_input == NULL) {
-		return;
-	}
-	
-	oc_in = ccm_input->oc;
-	if(oc_in == NULL) {
-		goto bail;
-	}
-	
-	offset = oc_in->m_memb_idx;
-	for(lpc = 0; lpc < oc_in->m_n_member; lpc++) {
-		a_node = &oc_in->m_array[lpc+offset];
-		crm_free(a_node->node_uname);
-		a_node->node_uname = NULL;
-	}
-
-	offset = oc_in->m_in_idx;
-	for(lpc = 0; lpc < oc_in->m_n_in; lpc++) {
-		a_node = &oc_in->m_array[lpc+offset];
-		crm_free(a_node->node_uname);
-		a_node->node_uname = NULL;
-	}
-
-	offset = oc_in->m_out_idx;
-	for(lpc = 0; lpc < oc_in->m_n_out; lpc++) {
-		a_node = &oc_in->m_array[lpc+offset];
-		crm_free(a_node->node_uname);
-		a_node->node_uname = NULL;
-	}
-
-  bail:
- 	crm_free(ccm_input->oc);
- 	crm_free(ccm_input);
-}
-
 oc_ev_membership_t *
 copy_ccm_oc_data(const oc_ev_membership_t *oc_in) 
 {
@@ -1196,7 +1195,8 @@ copy_ccm_oc_data(const oc_ev_membership_t *oc_in)
 		oc_node_t a_node      = oc_in->m_array[lpc+offset];
 		oc_node_t *a_node_copy = &(oc_copy->m_array[lpc+offset]);
 		crm_debug_3("Copying ccm member node %d", lpc);
-		copy_ccm_node(a_node, a_node_copy);		
+		copy_ccm_node(a_node, a_node_copy);
+		
 	}
 
 	offset = oc_in->m_in_idx;
@@ -1205,6 +1205,7 @@ copy_ccm_oc_data(const oc_ev_membership_t *oc_in)
 		oc_node_t *a_node_copy = &(oc_copy->m_array[lpc+offset]);
 		crm_debug_3("Copying ccm new node %d", lpc);
 		copy_ccm_node(a_node, a_node_copy);
+		
 	}
 
 	offset = oc_in->m_out_idx;
@@ -1226,99 +1227,43 @@ copy_ccm_node(oc_node_t a_node, oc_node_t *a_node_copy)
 		  a_node.node_id, a_node.node_born_on,
 		  a_node.node_uname);
 	
-	if(a_node_copy->node_uname != NULL) {
-		crm_debug_2("%p (%s) already copied", a_node_copy, a_node_copy->node_uname);
-		return;
-	}
-
 	a_node_copy->node_id      = a_node.node_id;
-	a_node_copy->node_born_on = a_node.node_born_on;
+	a_node_copy->node_born_on = a_node.node_born_on;	
 	a_node_copy->node_uname   = NULL;
 	
 	if(a_node.node_uname != NULL) {
-		a_node_copy->node_uname = crm_strdup(a_node.node_uname);
+			a_node_copy->node_uname =
+				crm_strdup(a_node.node_uname);
 	} else {
-		crm_err("Node Id %d had a NULL uname!", a_node.node_id);
+		crm_err("Node Id %d had a NULL uname!",
+			a_node.node_id);
 	}
+	
 	crm_debug_3("Copied ccm node: id=%d, born=%d, uname=%s",
 		  a_node_copy->node_id, a_node_copy->node_born_on,
 		  a_node_copy->node_uname);
 }
 
-static gboolean
-ghash_node_clfree(gpointer key, gpointer value, gpointer user_data)
-{
-	/* value->node_uname is free'd as "key" */
-	if(key != NULL) {
-		crm_free(key);
-	}
-	if(value != NULL) {
-		crm_free(value);
-	}
-	return TRUE;
-}
-
-void
-free_ccm_cache(oc_node_list_t *tmp) 
-{
-	/* Free the old copy */
-	if(tmp == NULL) {
-		return;
-	}
-	
-	if(tmp->members != NULL) {
-		g_hash_table_foreach_remove(
-			tmp->members, ghash_node_clfree, NULL);
-	}
-	if(tmp->new_members != NULL) {
-		g_hash_table_foreach_remove(
-			tmp->new_members, ghash_node_clfree, NULL);
-	}
-	if(tmp->dead_members != NULL) {
-		g_hash_table_foreach_remove(
-			tmp->dead_members, ghash_node_clfree, NULL);
-	}
-	
-	crm_free(tmp);
-	crm_debug_3("Free'd old copies");
-}
-
 crm_data_t*
 create_node_state(
-	const char *uname, const char *ha_state, const char *ccm_state,
+	const char *uuid, const char *uname,
+	const char *ha_state, const char *ccm_state,
 	const char *crmd_state, const char *join_state, const char *exp_state,
-	gboolean clear_shutdown, const char *src)
+	const char *src)
 {
 	crm_data_t *node_state = create_xml_node(NULL, XML_CIB_TAG_STATE);
 
 	crm_debug_2("%s Creating node state entry for %s", src, uname);
 	set_uuid(fsa_cluster_conn, node_state, XML_ATTR_UUID, uname);
 
-	if(ID(node_state) == NULL) {
-		crm_debug("Node %s is not a cluster member", uname);
-		free_xml(node_state);
-		return NULL;
-	}
-	
 	crm_xml_add(node_state, XML_ATTR_UNAME,         uname);
 	crm_xml_add(node_state, XML_CIB_ATTR_HASTATE,   ha_state);
 	crm_xml_add(node_state, XML_CIB_ATTR_INCCM,     ccm_state);
 	crm_xml_add(node_state, XML_CIB_ATTR_CRMDSTATE, crmd_state);
 	crm_xml_add(node_state, XML_CIB_ATTR_JOINSTATE, join_state);
 	crm_xml_add(node_state, XML_CIB_ATTR_EXPSTATE,  exp_state);
-	crm_xml_add(node_state, XML_ATTR_ORIGIN, src);
+	crm_xml_add(node_state, "origin", src);
 
-	if(clear_shutdown) {
-		crm_xml_add(node_state, XML_CIB_ATTR_SHUTDOWN,  "0");
-#if CRM_DEPRECATED_SINCE_2_0_3
-		crm_xml_add(node_state, "clear_shutdown",  "true");
-#endif
-/* 		crm_xml_add(node_state, */
-/* 			    XML_CIB_ATTR_REPLACE, XML_TAG_TRANSIENT_NODEATTRS); */
-	}
-	
-		
-	
 	crm_log_xml_debug_3(node_state, "created");
 
 	return node_state;
@@ -1342,89 +1287,41 @@ process_client_disconnect(crmd_client_t *curr_client)
 {
 	struct crm_subsystem_s *the_subsystem = NULL;
 
-	CRM_CHECK(curr_client != NULL, return);
+	CRM_DEV_ASSERT(curr_client != NULL);
+	if(crm_assert_failed) { return; }
+	
 	crm_debug_2("received HUP from %s", curr_client->table_key);
 		
 	if (curr_client->sub_sys == NULL) {
-		crm_debug_2("Client hadn't registered with us yet");
+		crm_debug("Client hadn't registered with us yet");
 		
-	} else if (strcasecmp(CRM_SYSTEM_PENGINE, curr_client->sub_sys) == 0) {
+	} else if (strcmp(CRM_SYSTEM_PENGINE, curr_client->sub_sys) == 0) {
 		the_subsystem = pe_subsystem;
 		
-	} else if (strcasecmp(CRM_SYSTEM_TENGINE, curr_client->sub_sys) == 0) {
+	} else if (strcmp(CRM_SYSTEM_TENGINE, curr_client->sub_sys) == 0) {
 		the_subsystem = te_subsystem;
 			
-	} else if (strcasecmp(CRM_SYSTEM_CIB, curr_client->sub_sys) == 0){
+	} else if (strcmp(CRM_SYSTEM_CIB, curr_client->sub_sys) == 0){
 		the_subsystem = cib_subsystem;
 	}
 		
 	if(the_subsystem != NULL) {
 		the_subsystem->ipc = NULL;
-		the_subsystem->client = NULL;		
-		crm_info("Received HUP from %s:[%d]",
-			 the_subsystem->name, the_subsystem->pid);
 		
-	} else {
-		/* else that was a transient client */
-		crm_debug_2("Received HUP from transient client");
-	}
+	} /* else that was a transient client */
 	
 	if (curr_client->table_key != NULL) {
 		/*
-		 * Key is destroyed below as:
+		 * Key is destroyed below:
 		 *	curr_client->table_key
 		 * Value is cleaned up by:
-		 *      crmd_ipc_connection_destroy
-		 *   which will also call:
 		 *	G_main_del_IPC_Channel
 		 */
 		g_hash_table_remove(ipc_clients, curr_client->table_key);
 	}
-}
-
-void update_dc(HA_Message *msg, gboolean assert_same)
-{
-	char *last_dc = fsa_our_dc;
-	const char *dc_version = NULL;
-	const char *welcome_from = NULL;
-
-	if(msg != NULL) {
-		dc_version = cl_get_string(msg, F_CRM_VERSION);
-		welcome_from = cl_get_string(msg, F_CRM_HOST_FROM);
-		
-		CRM_CHECK(dc_version != NULL, return);
-		CRM_CHECK(welcome_from != NULL, return);
-
-		if(AM_I_DC && safe_str_neq(welcome_from, fsa_our_uname)) {
-		    crm_warn("Not updating DC to %s (%s): we are also a DC",
-			     welcome_from, dc_version);
-		    return;
-		}
-
-		if(assert_same) {
-			CRM_CHECK(fsa_our_dc != NULL, ;);
-			CRM_CHECK(safe_str_eq(fsa_our_dc, welcome_from), ;);
-		}
-	}
-
-	crm_free(fsa_our_dc_version);
-	fsa_our_dc_version = NULL;
-	fsa_our_dc = NULL;
-
-	if(welcome_from != NULL) {
-		fsa_our_dc = crm_strdup(welcome_from);
-	}
-	if(dc_version != NULL) {
-		fsa_our_dc_version = crm_strdup(dc_version);
-	}
-
-	if(fsa_our_dc != NULL) {
-	    crm_info("Set DC to %s (%s)",
-		     crm_str(fsa_our_dc), crm_str(fsa_our_dc_version));
-
-	} else if(last_dc != NULL) {
-	    crm_info("Unset DC %s", crm_str(last_dc));
-	}
 	
-	crm_free(last_dc);
+	crm_free(curr_client->table_key);
+	crm_free(curr_client->sub_sys);
+	crm_free(curr_client->uuid);
+	crm_free(curr_client);
 }
