@@ -1,3 +1,4 @@
+/* $Id: main.c,v 1.18 2005/05/20 09:58:43 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <crm/crm.h>
 #include <stdio.h>
@@ -28,18 +29,13 @@
 #include <fcntl.h>
 
 #include <hb_api.h>
-#include <heartbeat.h>
 #include <clplumbing/uids.h>
 #include <clplumbing/coredumps.h>
-#include <clplumbing/cl_misc.h>
 
 #include <crm/common/ipc.h>
 #include <crm/common/ctrl.h>
-#include <crm/pengine/common.h>
 
-#if HAVE_LIBXML2
-#  include <libxml/parser.h>
-#endif
+#include <crm/dmalloc_wrapper.h>
 
 #define SYS_NAME CRM_SYSTEM_PENGINE
 #define OPTARGS	"hVc"
@@ -49,23 +45,20 @@ GMainLoop*  mainloop = NULL;
 const char* crm_system_name = SYS_NAME;
 
 void usage(const char* cmd, int exit_status);
-int pe_init(void);
+int init_start(void);
 gboolean pengine_shutdown(int nsig, gpointer unused);
 extern gboolean process_pe_message(crm_data_t * msg, IPC_Channel *sender);
-extern unsigned int pengine_input_loglevel;
 
 int
 main(int argc, char ** argv)
 {
-	int flag;
-	int argerr = 0;
-	char *param_val = NULL;
 	gboolean allow_cores = TRUE;
-	const char *param_name = NULL;
+	int	argerr = 0;
+	int flag;
     
-	crm_log_init(crm_system_name, LOG_INFO, TRUE, FALSE, 0, NULL);
- 	G_main_add_SignalHandler(
- 		G_PRIORITY_HIGH, SIGTERM, pengine_shutdown, NULL, NULL);
+	crm_log_init(crm_system_name);
+	G_main_add_SignalHandler(
+		G_PRIORITY_HIGH, SIGTERM, pengine_shutdown, NULL, NULL);
 
 	while ((flag = getopt(argc, argv, OPTARGS)) != EOF) {
 		switch(flag) {
@@ -83,12 +76,7 @@ main(int argc, char ** argv)
 				break;
 		}
 	}
-
-	if(argc - optind == 1 && safe_str_eq("metadata", argv[optind])) {
-		pe_metadata();
-		return 0;
-	}
-	
+    
 	if (optind > argc) {
 		++argerr;
 	}
@@ -96,28 +84,15 @@ main(int argc, char ** argv)
 	if (argerr) {
 		usage(crm_system_name,LSB_EXIT_GENERIC);
 	}
-
-	param_name = ENV_PREFIX "" KEY_LOG_PENGINE_INPUTS;
-	param_val = getenv(param_name);
-	crm_debug("%s = %s", param_name, param_val);
-	pengine_input_loglevel = crm_log_level;
-	if(param_val != NULL) {
-		int do_log = 0;
-		cl_str_to_boolean(param_val, &do_log);
-		if(do_log == FALSE) {
-			pengine_input_loglevel = crm_log_level + 1;
-		}
-		param_val = NULL;
-	}
-	
+    
 	/* read local config file */
 	crm_debug_4("do start");
-	return pe_init();
+	return init_start();
 }
 
 
 int
-pe_init(void)
+init_start(void)
 {
 	IPC_Channel *crm_ch = NULL;
 
@@ -138,10 +113,6 @@ pe_init(void)
 		g_main_run(mainloop);
 		return_to_orig_privs();
 
-#if HAVE_LIBXML2
-		xmlCleanupParser();
-#endif
-		
 		crm_info("Exiting %s", crm_system_name);
 		return 0;
 	}
@@ -173,7 +144,20 @@ usage(const char* cmd, int exit_status)
 gboolean
 pengine_shutdown(int nsig, gpointer unused)
 {
-	crm_info("Exiting PEngine (SIGTERM)");
-	exit(LSB_EXIT_OK);
+#if 0
+	static int shuttingdown = 0;
+  
+	if (!shuttingdown) {
+		shuttingdown = 1;
+	}
+	if (mainloop != NULL && g_main_is_running(mainloop)) {
+		g_main_quit(mainloop);
+	}else{
+		exit(LSB_EXIT_OK);
+	}
+	return TRUE;
+#else
+	return FALSE;
+#endif
 }
 
