@@ -1,4 +1,4 @@
-/* $Id: hb_resource.c,v 1.48.2.18 2005/10/27 17:32:24 blaschke Exp $ */
+/* $Id: hb_resource.c,v 1.79 2005/07/29 06:55:37 sunjd Exp $ */
 /*
  * hb_resource: Linux-HA heartbeat resource management code
  *
@@ -6,26 +6,26 @@
  *				<lclaudio@conectiva.com.br>
  * Copyright (C) 1999-2002 Alan Robertson <alanr@unix.sh>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #include <portability.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -34,7 +34,7 @@
 #include <hb_resource.h>
 #include <heartbeat_private.h>
 #include <hb_api_core.h>
-#include <setproctitle.h>
+#include <clplumbing/setproctitle.h>
 #include <clplumbing/cl_signal.h>
 #include <clplumbing/Gmain_timeout.h>
 #include <clplumbing/realtime.h>
@@ -149,7 +149,7 @@ void	Initiate_Reset(Stonith* s, const char * nodename, gboolean doreset);
 static int FilterNotifications(const char * msgtype);
 static int countbystatus(const char * status, int matchornot);
 static gboolean hb_rsc_isstable(void);
-static void PerformAutoFailback(void);
+static void shutdown_if_needed(void);
 
 ProcTrack_ops hb_rsc_RscMgmtProcessTrackOps = {
 	RscMgmtProcessDied,
@@ -170,11 +170,6 @@ static ProcTrack_ops StonithStatProcessTrackOps = {
 };
 
 
-void
-init_resource_module(void)
-{
-	hb_register_comm_up_callback(comm_up_resource_action);
-}
 
 static void
 HBDoMsg_T_STARTING_or_RESOURCES(const char * type, struct node_info * fromnode
@@ -230,6 +225,68 @@ HBDoMsg_default(const char * type, struct node_info * fromnode
 	QueueRemoteRscReq(PerformQueuedNotifyWorld, msg);
 }
 
+/* Received a "SHUTDONE" message from someone... */
+static void 
+HBDoMsg_T_SHUTDONE(const char * type, struct node_info * fromnode
+,	TIME_T msgtime, seqno_t seqno, const char * iface, struct ha_msg * msg)
+{
+	if (heartbeat_comm_state == COMM_LINKSUP) {
+		process_resources(type, msg, fromnode);
+	}
+	heartbeat_monitor(msg, KEEPIT, iface);
+	if (fromnode == curnode) {
+		if (ANYDEBUG) {
+			cl_log(LOG_DEBUG
+			,	"Received T_SHUTDONE from us.");
+		}
+		if (ANYDEBUG) {
+			cl_log(LOG_DEBUG
+			,	"Calling hb_mcp_final_shutdown"
+			" in a second.");
+		}
+		/* Trigger next phase of final shutdown process in a second */
+		Gmain_timeout_add(1000, hb_mcp_final_shutdown, NULL); /* phase 0 - normal */
+	}else{
+		fromnode->has_resources = FALSE;
+		other_is_stable = 0;
+		other_holds_resources= HB_NO_RSC;
+
+		cl_log(LOG_INFO
+		,	"Received shutdown notice from '%s'."
+		,	fromnode->nodename);
+		takeover_from_node(fromnode->nodename);
+	}
+}
+
+void
+init_resource_module(void)
+{
+	hb_register_msg_callback(T_SHUTDONE, HBDoMsg_T_SHUTDONE);
+	hb_register_comm_up_callback(comm_up_resource_action);
+}
+
+#ifndef WCOREDUMP
+#	define	WCOREDUMP(rc)	0
+#endif
+
+static const char *
+rctomsg(int waitrc)
+{
+	static char	retval[64];
+
+	if (WIFSIGNALED(waitrc)) {
+		snprintf(retval, sizeof(retval)
+		,	"killed by signal %d%s"
+		,	WTERMSIG(waitrc)
+		,	WCOREDUMP(waitrc) ? " (core dumped)" : "");
+	}else{
+		snprintf(retval, sizeof(retval)
+		,	"exited with return code %d"
+		,	WEXITSTATUS(waitrc));
+	}
+	return	retval;
+}
+
 static const char *	rsc_msg[] =	{HB_NO_RESOURCES, HB_LOCAL_RESOURCES
 ,	HB_FOREIGN_RESOURCES, HB_ALL_RESOURCES};
 
@@ -282,31 +339,22 @@ FilterNotifications(const char * msgtype)
 	return rc;
 }
 
-static gboolean
-AutoFailbackProc(gpointer dummy)
-{
-	PerformAutoFailback();
-	return FALSE;
-}
-
 static void
 PerformAutoFailback(void)
 {
 	if (ANYDEBUG) {
 		cl_log(LOG_DEBUG, "Calling PerformAutoFailback()");
 	}
-	if (shutdown_in_progress
-	||	(procinfo->i_hold_resources & HB_FOREIGN_RSC) == 0
+	if ((procinfo->i_hold_resources & HB_FOREIGN_RSC) == 0
 	||	!rsc_needs_failback || !auto_failback) {
 		rsc_needs_failback = FALSE;
-		hb_shutdown_if_needed();
+		shutdown_if_needed();
 		return;
 	}
 
-	if (going_standby != NOT 
-	||	!other_is_stable || resourcestate != HB_R_STABLE) {
-		cl_log(LOG_DEBUG, "Auto failback delayed.");
-		Gmain_timeout_add(1*1000, AutoFailbackProc, NULL);
+	if (going_standby != NOT) {
+		cl_log(LOG_ERR, "Auto failback ignored.");
+		rsc_needs_failback = FALSE;
 		return;
 	}
 	if (ANYDEBUG) {
@@ -414,6 +462,10 @@ notify_world(struct ha_msg * msg, const char * ostatus)
 					setenv(HANICEFAILBACK, "yes", 1);
 				}
 				
+				/*should we use logging daemon or not in script*/
+				setenv(HALOGD, cl_log_get_uselogd()?
+				       "yes":"no", 1);
+				
 				if (ANYDEBUG) {
 					cl_log(LOG_DEBUG
 					,	"notify_world: Running %s %s"
@@ -456,6 +508,8 @@ hb_rsc_recover_dead_resources(struct node_info* hip)
 	gboolean	need_stonith = TRUE;
 	struct ha_msg *	hmsg;
 	char		timestamp[16];
+	standby_running = zero_longclock;
+	going_standby	= NOT;
 
 	
 	if ((hmsg = ha_msg_new(6)) == NULL) {
@@ -482,28 +536,16 @@ hb_rsc_recover_dead_resources(struct node_info* hip)
 			return;
 		}
 	}
-	else if (going_standby != NOT) {
-		cl_log(LOG_INFO, "Cancelling pending standby operation");
-		going_standby = NOT;
-		standby_running = zero_longclock;
-
-		if ((!other_is_stable)
-		&&	((procinfo->i_hold_resources & HB_ALL_RSC) == HB_ALL_RSC)) {
-			other_is_stable = TRUE;
-			if (ANYDEBUG) {
-				cl_log(LOG_DEBUG
-				,	"hb_rsc_recover_dead_resources:"
-				" other now stable");
-			}
-		}
-		hb_shutdown_if_needed();
-	}
 	
 	/*deliver this message to clients*/
 	heartbeat_monitor(hmsg, KEEPIT, "<internal>");
 	ha_msg_del(hmsg);
 	hmsg = NULL;
 
+	if (!DoManageResources) {
+		return;
+	}
+
 	if (hip->nodetype == PINGNODE_I) {
 		takeover_from_node(hip->nodename);
 		return;
@@ -567,15 +609,6 @@ hb_rsc_recover_dead_resources(struct node_info* hip)
 		,	hip->nodename);
 		send_stonith_msg(hip->nodename, T_STONITH_UNNEEDED);
 		if (nice_failback) {
-			if ((procinfo->i_hold_resources & HB_ALL_RSC) == HB_ALL_RSC) {
-				other_is_stable = TRUE;
-				if (ANYDEBUG) {
-					cl_log(LOG_DEBUG
-					,	"hb_rsc_recover_dead_resources:"
-					" other now stable");
-				}
-				return;
-			}
 			/* These might happen due to timing weirdnesses */
 			if (! (procinfo->i_hold_resources & HB_LOCAL_RSC)){
 				req_our_resources(TRUE);
@@ -774,7 +807,7 @@ process_resources(const char * type, struct ha_msg* msg
 	enum hb_rsc_state	newrstate = resourcestate;
 	static int			first_time = 1;
 
-	hb_shutdown_if_needed();
+	shutdown_if_needed();
 	if (!DoManageResources || !nice_failback) {
 		return;
 	}
@@ -1051,7 +1084,7 @@ process_resources(const char * type, struct ha_msg* msg
 	}
 
 	AuditResources();
-	hb_shutdown_if_needed();
+	shutdown_if_needed();
 }
 
 void
@@ -1219,7 +1252,7 @@ takeover_from_node(const char * nodename)
 	if (shutdown_in_progress) {
 		cl_log(LOG_INFO
 		,	"Resource takeover cancelled - shutdown in progress.");
-		hb_shutdown_if_needed();
+		shutdown_if_needed();
 		return;
 	}else if (hip->nodetype != PINGNODE_I) {
 		cl_log(LOG_INFO
@@ -1413,17 +1446,23 @@ req_our_resources(int getthemanyway)
 			cl_log(LOG_DEBUG, "req_our_resources()"
 			": running [%s]",	getcmd);
 		}
+		/*should we use logging daemon or not in script*/
+		setenv(HALOGD, cl_log_get_uselogd()?
+		       "yes":"no", 1);				
+
 		if ((rc=system(getcmd)) != 0) {
-			cl_perror("%s returned %d", getcmd, rc);
+			cl_perror("%s %s", getcmd, rctomsg(rc));
 			finalrc=HA_FAIL;
 		}
 	}
-	rc=pclose(rkeys);
+	if ((rc = pclose(rkeys)) != 0) {
+		cl_log(LOG_ERR, "pclose(%s) %s", cmd, rctomsg(rc));
+	}
 	rkeys = NULL;
 	if (rc < 0 && errno != ECHILD) {
-		cl_perror("pclose(%s) returned %d", cmd, rc);
+		cl_perror("pclose(%s) [%s?]", cmd, rctomsg(rc));
 	}else if (rc > 0) {
-		cl_log(LOG_ERR, "[%s] exited with 0x%x", cmd, rc);
+		cl_log(LOG_ERR, "[%s] %s", cmd, rctomsg(rc));
 	}
 
 	if (rsc_count == 0) {
@@ -1727,7 +1766,7 @@ ask_for_resources(struct ha_msg *msg)
 	if (ANYDEBUG) {
 		cl_log(LOG_INFO, "New standby state: %d", going_standby);
 	}
-	hb_shutdown_if_needed();
+	shutdown_if_needed();
 }
 
 static int
@@ -1881,12 +1920,19 @@ go_standby(enum standby who, int resourceset) /* Which resources to give up */
 		}
 		sprintf(cmd, HALIB "/ResourceManager %s %s"
 		,	actioncmds[action], buf);
+
+		/*should we use logging daemon or not in script*/
+		setenv(HALOGD, cl_log_get_uselogd()?
+		       "yes":"no", 1);
+		
 		if ((rc=system(cmd)) != 0) {
-			cl_log(LOG_ERR, "%s returned %d", cmd, rc);
+			cl_log(LOG_ERR, "%s %s", cmd, rctomsg(rc));
 			finalrc=HA_FAIL;
 		}
 	}
-	pclose(rkeys);
+	if ((rc = pclose(rkeys)) != 0) {
+		cl_log(LOG_ERR, "pclose(%s) %s", cmd, rctomsg(rc));
+	}
 	cl_log(LOG_INFO, "%s HA resource %s completed (standby)."
 	,	rsc_msg[actresources]
 	,	action == ACTION_ACQUIRE ? "acquisition" : "release");
@@ -1896,8 +1942,8 @@ go_standby(enum standby who, int resourceset) /* Which resources to give up */
 
 }
 
-void
-hb_shutdown_if_needed(void)
+static void
+shutdown_if_needed(void)
 {
 	if (rsc_needs_shutdown) {
 		hb_giveup_resources();
@@ -1928,10 +1974,17 @@ hb_giveup_resources(void)
 	pid_t		pid;
 	struct ha_msg *	m;
 	static int	resource_shutdown_in_progress = FALSE;
+	
+	if (!DoManageResources){
+		if (!shutdown_in_progress) {
+			hb_initiate_shutdown(FALSE);
+		}
+		return;
+	}
 
 	if (!hb_rsc_isstable()) {
 		/* Try again later... */
-		/* (through hb_shutdown_if_needed()) */
+		/* (through shutdown_if_needed()) */
 		if (!rsc_needs_shutdown) {
 			cl_log(LOG_WARNING
 			,	"Shutdown delayed until current"
@@ -2012,19 +2065,20 @@ hb_giveup_resources(void)
 		if (buf[strlen(buf)-1] == '\n') {
 			buf[strlen(buf)-1] = EOS;
 		}
+
+		/*should we use logging daemon or not in script*/
+		setenv(HALOGD, cl_log_get_uselogd()?
+		       "yes":"no", 1);
+		
 		sprintf(cmd, HALIB "/ResourceManager givegroup %s", buf);
 		if ((rc=system(cmd)) != 0) {
-			cl_log(LOG_ERR, "%s returned %d", cmd, rc);
+			cl_log(LOG_ERR, "%s %s", cmd, rctomsg(rc));
 			finalrc=HA_FAIL;
 		}
 	}
-	pclose(rkeys);
-
-	/* Kill all our managed children... */
-	ForEachProc(&ManagedChildTrackOps, hb_kill_tracked_process
-	,	GINT_TO_POINTER(SIGTERM));
-	ForEachProc(&hb_rsc_RscMgmtProcessTrackOps, hb_kill_tracked_process
-	,	GINT_TO_POINTER(SIGKILL));
+	if ((rc = pclose(rkeys)) != 0) {
+		cl_log(LOG_ERR, "pclose(%s) %s", cmd, rctomsg(rc));
+	}
 
 	cl_log(LOG_INFO, "All HA resources relinquished.");
 
@@ -2087,17 +2141,17 @@ Initiate_Reset(Stonith* s, const char * nodename, gboolean doreset)
 		cl_log(LOG_INFO
 		,	"Resetting node %s with [%s]"
 		,	nodename
-		,	s->s_ops->getinfo(s, ST_DEVICEID));
+		,	stonith_get_info(s, ST_DEVICEID));
 	}else{
 		cl_log(LOG_INFO
 		,	"Checking status of STONITH device [%s]"
-		,	s->s_ops->getinfo(s, ST_DEVICEID));
+		,	stonith_get_info(s, ST_DEVICEID));
 	}
 
 	if (doreset) {
-		rc = s->s_ops->reset_req(s, ST_GENERIC_RESET, nodename);
+		rc = stonith_req_reset(s, ST_GENERIC_RESET, nodename);
 	}else{
-		rc = s->s_ops->status(s);
+		rc = stonith_get_status(s);
 	}
 	switch (rc) {
 
@@ -2113,7 +2167,7 @@ Initiate_Reset(Stonith* s, const char * nodename, gboolean doreset)
 	case S_BADHOST:
 		cl_log(LOG_ERR
 		,	"Device %s cannot reset host %s."
-		,	s->s_ops->getinfo(s, ST_DEVICEID)
+		,	stonith_get_info(s, ST_DEVICEID)
 		,	nodename);
 		exitcode = 100;
 		result = T_STONITH_BADHOST;
@@ -2124,7 +2178,7 @@ Initiate_Reset(Stonith* s, const char * nodename, gboolean doreset)
 			cl_log(LOG_ERR, "Host %s not reset!", nodename);
 		}else{
 			cl_log(LOG_ERR, "STONITH device %s not operational!"
-			,	s->s_ops->getinfo(s, ST_DEVICEID));
+			,	stonith_get_info(s, ST_DEVICEID));
 		}
 		exitcode = 1;
 		result = T_STONITH_BAD;
@@ -2178,7 +2232,7 @@ RscMgmtProcessDied(ProcTrack* p, int status, int signo, int exitcode
 
 	p->privatedata = NULL;
 	StartNextRemoteRscReq();
-	hb_shutdown_if_needed();
+	shutdown_if_needed();
 }
 
 static const char *
@@ -2249,14 +2303,14 @@ QueueRemoteRscReq(RemoteRscReqFunc func, struct ha_msg* msg)
 		cl_log(LOG_DEBUG
 		,	"Queueing remote resource request (hook = 0x%p) %s"
 		,	(void *)hook, fp);
-		cl_log_message(msg);
+		cl_log_message(LOG_DEBUG, msg);
 	}
 
 	if (fp == NULL || !FilterNotifications(fp)) {
 		if (DEBUGDETAILS) {
 			cl_log(LOG_DEBUG
 			,	"%s: child process unneeded.", fp);
-			cl_log_message(msg);
+			cl_log_message(LOG_DEBUG, msg);
 		}
 		g_hook_free(&RemoteRscReqQueue, hook);
 		return;
@@ -2290,7 +2344,7 @@ StartNextRemoteRscReq(void)
 	hook = g_hook_first_valid(&RemoteRscReqQueue, FALSE);
 	if (hook == NULL) {
 		ResourceMgmt_child_count = 0;
-		hb_shutdown_if_needed();
+		shutdown_if_needed();
 		return;
 	}
 
@@ -2324,7 +2378,7 @@ PerformQueuedNotifyWorld(GHook* hook)
 	 */
 	if (DEBUGDETAILS) {
 		cl_log(LOG_DEBUG, "PerformQueuedNotifyWorld() msg follows");
-		cl_log_message(m);
+		cl_log_message(LOG_DEBUG, m);
 	}
 	notify_world(m, curnode->status);
 	/* "m" is automatically destroyed when "hook" is */
@@ -2400,100 +2454,123 @@ StonithStatProcessName(ProcTrack* p)
 
 /*
  * $Log: hb_resource.c,v $
- * Revision 1.48.2.18  2005/10/27 17:32:24  blaschke
+ * Revision 1.79  2005/07/29 06:55:37  sunjd
+ * bug668: license update
  *
- * Bug 927 - Shutdown if needed after standby is cancelled.
+ * Revision 1.78  2005/06/01 04:36:26  msoffen
+ * Added sys/wait.h
  *
- * Revision 1.48.2.17  2005/10/26 15:34:09  blaschke
+ * Revision 1.77  2005/05/28 03:57:12  alan
+ * Made exit codes more informative in resource management code.
  *
- * Bug 835 - Instead of ignoring auto failback if going standby, delay auto
- * failback one second if going standby or either node is unstable
+ * Revision 1.76  2005/04/20 23:45:17  gshi
+ * change the path for setproctitle.h
  *
- * Revision 1.48.2.16  2005/10/11 15:38:29  blaschke
+ * Revision 1.75  2005/04/13 18:04:46  gshi
+ * bug 442:
  *
- * Bug 559 Part II - Don't cancel standby op because ping node died
+ * Enable logging daemon  by default
+ * use static variables in cl_log and export interfaces to get/set variables
  *
- * Revision 1.48.2.15  2005/10/10 17:12:19  blaschke
+ * Revision 1.74  2005/03/28 15:41:49  alan
+ * BUG 49: Some of the STONITH logic (and other things) shouldn't be
+ * executed if heartbeat isn't managing resources.
  *
- * Bug 559 - Stop nodes waiting for other to become stable even after other
- * is dead.
+ * Revision 1.73  2005/03/03 16:22:07  andrew
+ * Fix from gshi for bug #302
+ *  - seems to work well
  *
- * Revision 1.48.2.14  2005/08/30 17:45:59  davidlee
- * Declarations must precede statements
+ * Revision 1.72  2005/02/28 21:23:30  gshi
+ * giveup_resources() servers as two purposes in 1.x series:
+ * 1) give up all resources
+ * 2) shutdown heartbeat
  *
- * Revision 1.48.2.13  2004/11/11 22:13:53  gshi
+ * since in V2 all resources are managed in crm/lrm, this function
+ * should not do any thing with the first purpose,therefore it should
+ * start shutdown phase if it is in version 2
  *
- * de dead notice should be called after the node is really decared dead
+ * Revision 1.71  2005/01/18 20:33:03  andrew
+ * Appologies for the top-level commit, one change necessitated another which
+ *   exposed some bugs... etc etc
+ *
+ * Remove redundant usage of XML in the CRM
+ * - switch to "struct ha_msg" aka. HA_Message for everything except data
+ * Make sure the expected type of all FSA input data is verified before processing
+ * Fix a number of bugs including
+ * - looking in the wrong place for the API result data in the CIB API
+ *   (hideous that this actually worked).
+ * - not overwriting error codes when sending the result to the client in the CIB API
+ *   (this lead to some error cases being treated as successes later in the code)
+ * Add PID to log messages sent to files (not to syslog)
+ * Add a log level to calls for cl_log_message()
+ * - convert existing calls, sorry if I got the level wrong
+ * Add some checks in cl_msg.c code to prevent NULL pointer exceptions
+ * - usually when NULL is passed to strlen() or similar
+ *
+ * Revision 1.70  2005/01/03 18:12:10  alan
+ * Stonith version 2.
+ * Some compatibility with old versions is still missing.
+ * Basically, you can't (yet) use files for config information.
+ * But, we'll fix that :-).
+ * Right now ssh, null and baytech all work.
+ *
+ * Revision 1.69  2004/11/17 21:00:14  msoffen
+ * Moved initialization of standby_running and going_standby to compile on BSD
+ *
+ * Revision 1.68  2004/11/16 05:47:23  zhenh
+ * 1.Make the ordering shutdown work. 2.Move HBDoMsg_T_SHUTDONE() to hb_resource.c
+ *
+ * Revision 1.67  2004/11/11 20:27:03  gshi
+ * node dead notice should be called after the node is really decared dead
  * instead of after receving a shutdown message from it
  *
- * Revision 1.48.2.12  2004/10/20 15:29:25  alan
- * Fixed a bug related to repeat STONITHs failing because we freed some
- * data we shouldn't have.
+ * Revision 1.66  2004/11/08 20:48:36  gshi
+ * implemented logging daemon
  *
- * Revision 1.48.2.11  2004/09/11 20:52:33  alan
- * Brought back changes from HEAD
- * 1) Added apphbd option
- * 2) Restructuring of packet handling
- * 3) Various BEAM fixes
+ * The logging daemon is to double-buffer log messages to protect us from blocking
+ * writes to syslog / logfiles.
  *
- * Revision 1.48.2.10  2004/09/07 16:56:25  alan
- * Changed some of the hb_resource code to not be active when resource management
- * is shut off.
+ * Revision 1.65  2004/10/20 15:31:00  alan
+ * Fixed a bug in STONITH retries where it frees something
+ * prematurely.
  *
- * Revision 1.48.2.9  2004/09/07 15:12:29  alan
- * Fixed some holes in making sure things stop correctly, and also
- * that audits don't complain, and that nothing gets acquired while
- * shutting down.  Changes are as follows:
+ * Revision 1.64  2004/09/10 22:47:40  alan
+ * BEAM FIXES:  various minor fixes related to running out of memory.
  *
- * Fixed code so that it would not try and take over when the state was
- * STABLE or SHUTDOWN when it received a node death indication.
+ * Revision 1.63  2004/09/10 07:23:50  alan
+ * BEAM fixes:  Fixed a couple of resource leaks, and a couple of use-after-free errors.
  *
- * Changed code so that it won't try and take something over that we
- * already know we own (in most cases).
+ * Revision 1.62  2004/09/10 01:12:23  alan
+ * BEAM CHANGES: Fixed a couple of very minor bugs, and cleaned up some BEAM warnings.
  *
- * When we receive a T_SHUTDONE message from our peer we now take over
- * all the resources they had *immediately* without waiting for them
- * to be declared dead.  This was needed to make an audit error go away.
- * Otherwise in this interval between the other node giving up all resources
- * and it being declared dead, we noticed that some resources weren't being
- * served, and considered this an error.  We use the go_standby() code
- * to do this.  The messages are a bit odd, but it works nicely.
- * Fixing the messages so it said T_SHUTDOWN request instead of go_standby()
- * request would be nice.  This also shortens the interval during which
- * we don't have resources running during a graceful shutdown of one
- * of the nodes.
+ * Revision 1.61  2004/09/08 22:19:42  alan
+ * Bringing forward changes from the 1.2.x branch.
  *
- * Ignore req_our_resources() and standby messages during shutdown.
+ * Revision 1.60  2004/09/08 18:57:30  gshi
+ * fixed a bug: api_test cannot receive messages
+ * the default handler gets all messages
+ * we need to check if any client need that message in the default handler
  *
- * Revision 1.48.2.8  2004/09/06 22:43:08  alan
- * Changed hb_resource so it will mark the other side as stable
- * when it says it's given up all resources.
+ * Revision 1.59  2004/08/14 14:42:22  alan
+ * Put in lots of comments about resource-work tieins in heartbeat.c, plus put in
+ * a short term workaround for dealing with one particular case:
+ * req_our_resources().
  *
- * Revision 1.48.2.7  2004/09/06 17:15:27  alan
- * Changed CPUlimit code to think in terms of 60 second minimums, and also
- * upped CPU limit for heartbeat.
- * Changed heartbeat to not take over resources when there aren't any
- * to take over (hopefully)
+ * Revision 1.58  2004/08/10 04:55:24  alan
+ * Completed first pass of -M flag reorganization.
+ * It passes BasicSanityCheck.
+ * But, I haven't actually tried -M yet ;-)
  *
- * Revision 1.48.2.6  2004/09/03 01:42:34  alan
- * Removed a slightly dangerous piece of debugging code...
+ * Revision 1.57  2004/07/02 03:34:21  sunjd
+ * Fix cannot stop two nodes simultaneously, pls help to test more
  *
- * Revision 1.48.2.5  2004/09/03 00:56:39  alan
- * Added some debugging code to hb_resource.c for the really weird thing
- * I've been seeing where fgets whacks out...
+ * Revision 1.56  2004/05/17 15:12:08  lars
+ * Reverting over-eager approach to disabling old resource manager code.
  *
- * Revision 1.48.2.4  2004/08/27 19:51:07  alan
- * Put in some better messages to help debug the issue with FIFO writes crapping out,
- * and also to neaten things up for child processes.
- *
- * Revision 1.48.2.3  2004/07/13 07:41:16  sunjd
- * fix simultaneous stop issue
- *
- * Revision 1.48.2.2  2004/04/29 01:08:58  alan
- * Put in Gshi's fix for restarting:
- * send a stability message back to the sender upon receiving that kind of message.
- * This is to fix a bug that after a node re-read config file and restart
- * the other node will think that node is unstable.
+ * Revision 1.55  2004/05/15 09:28:08  andrew
+ * Disable ALL legacy resource management iff configured with --enable-crm
+ * Possibly I have been a little over-zealous but likely the feature(s)
+ *  would need to be re-written to use the new design anyway.
  *
  * Revision 1.54  2004/04/27 18:29:03  gshi
  * send a stability message back to the sender
