@@ -1,3 +1,4 @@
+/* $Id: hb_signal.c,v 1.17 2005/08/15 04:54:58 sunjd Exp $ */
 /*
  * hb_signal.c: signal handling routines to be used by Heartbeat
  *
@@ -22,7 +23,7 @@
  */
 
 #define _USE_BSD
-#include <lha_internal.h>
+#include <portability.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -190,25 +191,25 @@ __hb_signal_debug_action(int sig)
 
 	switch(sig) {
 		case SIGUSR1:
-			++debug_level;
+			++debug;
 			break;
 
 		case SIGUSR2:
-			if (debug_level > 0) {
-				--debug_level;
+			if (debug > 0) {
+				--debug;
 			}else{
-				debug_level=0;
+				debug=0;
 			}
 			break;
 	}
 
- 	PILSetDebugLevel(PluginLoadingSystem, NULL, NULL , debug_level);
+ 	PILSetDebugLevel(PluginLoadingSystem, NULL, NULL , debug);
 	{
 		static char cdebug[8];
-		snprintf(cdebug, sizeof(debug_level), "%d", debug_level);
+		snprintf(cdebug, sizeof(debug), "%d", debug);
 		setenv(HADEBUGVAL, cdebug, TRUE);
 	}
-	if (debug_level <= 0) {
+	if (debug <= 0) {
 		unsetenv(HADEBUGVAL);
 	}
 }
@@ -245,14 +246,14 @@ hb_signal_debug_usr2_action(void)
 static void
 __parent_hb_signal_debug_action(int sig)
 {
-	int	olddebug = debug_level;
+	int	olddebug = debug;
 
 	__hb_signal_debug_action(sig);
 	hb_signal_signal_all(sig);
 
-	ha_log(LOG_DEBUG, "debug now set to %d [pid %d]", debug_level
+	ha_log(LOG_DEBUG, "debug now set to %d [pid %d]", debug
 	,	(int) getpid());
-	if (debug_level == 1 && olddebug == 0) {
+	if (debug == 1 && olddebug == 0) {
 		hb_versioninfo();
 		hb_dump_proc_stats(curproc);
 	}
@@ -662,7 +663,7 @@ hb_signal_set_master_control_process(sigset_t *set)
 		return(-1);
 	}
 	
-	set_sigchld_proctrack(G_PRIORITY_HIGH,DEFAULT_MAXDISPATCHTIME);
+	set_sigchld_proctrack(G_PRIORITY_HIGH);
 	hb_signal_process_pending_set_mask_set(use_set);
 
 	return(0);
