@@ -1,3 +1,4 @@
+/* $Id: send_arp.c,v 1.22 2005/09/21 10:30:16 andrew Exp $ */
 /* 
  * send_arp
  * 
@@ -27,7 +28,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <limits.h>
 #include <libnet.h>
@@ -50,35 +54,10 @@ static int send_arp(LTYPE* l, u_long ip, u_char *device, u_char mac[6]
 ,	u_char *broadcast, u_char *netmask, u_short arptype);
 
 static char print_usage[]={
-"send_arp: sends out custom ARP packet.\n"
-"  usage: send_arp [-i repeatinterval-ms] [-r repeatcount] [-p pidfile] \\\n"
-"              device src_ip_addr src_hw_addr broadcast_ip_addr netmask\n"
-"\n"
-"  where:\n"
-"    repeatinterval-ms: timing, in milliseconds of sending arp packets\n"
-"      For each ARP announcement requested, a pair of ARP packets is sent,\n"
-"      an ARP request, and an ARP reply. This is becuse some systems\n"
-"      ignore one or the other, and this combination gives the greatest\n"
-"      chance of success.\n"
-"\n"
-"      Each time an ARP is sent, if another ARP will be sent then\n"
-"      the code sleeps for half of repeatinterval-ms.\n"
-"\n"
-"    repeatcount: how many pairs of ARP packets to send.\n"
-"                 See above for why pairs are sent\n"
-"\n"
-"    pidfile: pid file to use\n"
-"\n"
-"    device: netowrk interace to use\n"
-"\n"
-"    src_ip_addr: source ip address\n"
-"\n"
-"    src_hw_addr: source hardware address.\n"
-"                 If \"auto\" then the address of device\n"
-"\n"
-"    broadcast_ip_addr: ignored\n"
-"\n"
-"    netmask: ignored\n"
+	"send_arp: sends out custom ARP packet. packetfactory.net\n"
+	"\tusage: send_arp [-i repeatinterval-ms] [-r repeatcount]"
+	" [-p pidfile] device src_ip_addr src_hw_addr broadcast_ip_addr netmask\n"
+	"\tIf src_hw_addr is \"auto\" then the address of device will be used"
 };
 
 static const char * SENDARPNAME = "send_arp";
@@ -138,7 +117,8 @@ main(int argc, char *argv[])
         cl_log_set_entity(SENDARPNAME);
         cl_log_enable_stderr(TRUE);
         cl_log_set_facility(LOG_USER);
-	cl_inherit_logging_environment(0);
+        /* Use logd if it's enabled by heartbeat */
+        cl_inherit_use_logd(ENV_PREFIX ""KEY_LOGDAEMON, 0);
 
 	while ((flag = getopt(argc, argv, "i:r:p:")) != EOF) {
 		switch(flag) {
@@ -152,13 +132,13 @@ main(int argc, char *argv[])
 		case 'p':	pidfilename= optarg;
 				break;
 
-		default:	fprintf(stderr, "%s\n\n", print_usage);
+		default:	fprintf(stderr, "usage: %s\n\n", print_usage);
 				return 1;
 				break;
 		}
 	}
 	if (argc-optind != 5) {
-		fprintf(stderr, "%s\n\n", print_usage);
+		fprintf(stderr, "usage: %s\n\n", print_usage);
 		return 1;
 	}
 
@@ -191,11 +171,7 @@ main(int argc, char *argv[])
 	}
 
 #if defined(HAVE_LIBNET_1_0_API)
-#ifdef ON_DARWIN
-	if ((ip = libnet_name_resolve((unsigned char*)ipaddr, 1)) == -1UL) {
-#else
 	if ((ip = libnet_name_resolve(ipaddr, 1)) == -1UL) {
-#endif
 		cl_log(LOG_ERR, "Cannot resolve IP address [%s]", ipaddr);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
@@ -210,11 +186,15 @@ main(int argc, char *argv[])
 	}
 #elif defined(HAVE_LIBNET_1_1_API)
 	if ((l=libnet_init(LIBNET_LINK, device, errbuf)) == NULL) {
-		cl_log(LOG_ERR, "libnet_init failure on %s: %s", device, errbuf);
+		cl_log(LOG_ERR, "libnet_init failure on %s", device);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
 	}
+#ifdef ON_DARWIN
+ 	if ((signed)(ip = libnet_name2addr4(l, (unsigned char *)ipaddr, 1)) == -1) {
+#else
 	if ((signed)(ip = libnet_name2addr4(l, ipaddr, 1)) == -1) {
+#endif
 		cl_log(LOG_ERR, "Cannot resolve IP address [%s]", ipaddr);
 		unlink(pidfilename);
 		return EXIT_FAILURE;
@@ -293,15 +273,20 @@ get_hw_addr(char *device, u_char mac[6])
 	struct libnet_link_int  *network;
 	char                    err_buf[LIBNET_ERRBUF_SIZE];
 
+	/* Get around bad prototype for libnet_error() */
+	char errmess1 [] = "libnet_open_link_interface: %s\n";
+	char errmess2 [] = "libnet_get_hwaddr: %s\n";
+
+
 	network = libnet_open_link_interface(device, err_buf);
 	if (!network) {
-		fprintf(stderr, "libnet_open_link_interface: %s\n", err_buf);
+		libnet_error(LIBNET_ERR_FATAL, errmess1, err_buf);
 		return -1;
 	}
 
 	mac_address = libnet_get_hwaddr(network, device, err_buf);
 	if (!mac_address) {
-		fprintf(stderr, "libnet_get_hwaddr: %s\n", err_buf);
+		libnet_error(LIBNET_ERR_FATAL, errmess2, err_buf);
 		return -1;
 	}
 
@@ -395,7 +380,7 @@ send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr,
 	/* Convert ASCII Mac Address to 6 Hex Digits. */
 
 	/* Ethernet header */
-	if (get_hw_addr((char*)device, device_mac) < 0) {
+	if (get_hw_addr(device, device_mac) < 0) {
 		cl_log(LOG_ERR, "Cannot find mac address for %s",
 				device);
 		return -1;
@@ -439,7 +424,7 @@ send_arp(struct libnet_link_int *l, u_long ip, u_char *device, u_char *macaddr,
 		return -1;
 	}
 
-	n = libnet_write_link_layer(l, (char*)device, buf, LIBNET_ARP_H + LIBNET_ETH_H);
+	n = libnet_write_link_layer(l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H);
 	if (n == -1) {
 		cl_log(LOG_ERR, "libnet_build_ethernet failed:");
 	}
@@ -689,3 +674,191 @@ write_pid_file(const char *pidfilename)
 }
 
 
+/*
+ * $Log: send_arp.c,v $
+ * Revision 1.22  2005/09/21 10:30:16  andrew
+ * Darwin wants an unsigned char* for ipaddr
+ *
+ * Revision 1.21  2005/08/29 01:44:57  sunjd
+ * bug799: move rsctmp to /var/run/heartbeat
+ *
+ * Revision 1.20  2005/08/08 06:00:05  sunjd
+ * bug458:use cl_log() instead of syslog
+ *
+ * Revision 1.19  2005/07/29 07:05:00  sunjd
+ * bug668: license update
+ *
+ * Revision 1.18  2005/06/24 14:58:06  alan
+ * Removed an unneeded cast that was causing troubles in FC4.
+ *
+ * Revision 1.17  2005/06/20 13:03:20  andrew
+ * Signedness on Darwin
+ *
+ * Revision 1.16  2005/06/08 08:15:19  sunjd
+ * Make GCC4 happy. unsigned<->signed
+ *
+ * Revision 1.15  2005/04/10 05:34:02  alan
+ * BUG 459: Fixed a directory-creation race condition in send_arp.
+ *
+ * Revision 1.14  2005/03/04 15:27:04  alan
+ * Fixed a signed/unsigned bug in send_arp.c
+ *
+ * Revision 1.13  2004/09/10 02:03:00  alan
+ * BEAM FIX:  A variable was declared unsigned that should have been declared signed.
+ * 	This is the result of a previous size_t fix that should have used ssize_t instead.
+ *
+ * Revision 1.12  2004/06/15 01:49:04  horms
+ * Changes to make gratuitous ARP Packets RFC2002 (4.6) compliant
+ *
+ * Revision 1.11  2004/04/27 11:55:54  horms
+ * Slightly better error checking
+ *
+ * Revision 1.10  2004/04/27 09:49:32  horms
+ * Fix for pid code for FreeBSD (and others)
+ *
+ * Revision 1.9  2004/03/19 15:07:43  alan
+ * Put in a fix provided by Michael Dipper <md@LF.net>
+ * A '/' is missing from the pidfile pathname for send_arp.
+ *
+ * Revision 1.8  2004/02/17 22:11:57  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.7  2004/01/22 01:53:35  alan
+ * Changed send_arp to exit(0) when killed by SIGTERM.
+ *
+ * Revision 1.6  2003/12/11 22:58:04  alan
+ * Put in a patch from Werner Schulthei for allowing old-style MAC addrs
+ * in send_arp.
+ *
+ * Fixed some signed/unsigned warning issues in the code.
+ *
+ * Revision 1.5  2003/10/31 16:15:20  msoffen
+ * Added limit.h to define MAXINT value.
+ *
+ * Revision 1.4  2003/10/29 11:40:50  horms
+ * Send arp  creates a driectory to store its pid file in on-the-fly. IPaddr does likewise for state files.
+ *
+ * Revision 1.3  2003/10/23 07:36:29  horms
+ * added pid file to sendarp
+ *
+ * Revision 1.2  2003/10/15 17:03:23  horms
+ * Merged  get_hw_addr into and send_arp
+ *
+ * Revision 1.1  2003/10/15 09:52:12  horms
+ * Move utilities that link against libnet (sendarp and get_hw_addr)
+ * to their own directory so we no longer have to come up with
+ * creative ways to selectively mangle CFLAGS.
+ *
+ * Revision 1.24  2003/09/26 06:02:13  alan
+ * Fixed an undefined variable warning.
+ *
+ * Revision 1.23  2003/07/12 17:02:59  alan
+ * Hopefully last fix for the changes made to allow user to specify arp intervals, etc.
+ *
+ * Revision 1.22  2003/07/12 16:19:54  alan
+ * Fixed a bug in the new send_arp options and their invocation...
+ *
+ * Revision 1.21  2003/04/15 18:56:33  msoffen
+ * Removed printf("\n") that served no purpose anymore (used to print .\n).
+ *
+ * Revision 1.20  2003/04/15 11:07:05  horms
+ * errors go to stderr, not stdout
+ *
+ * Revision 1.19  2003/03/21 17:38:31  alan
+ * Put in a patch by Thiago Rondon <thiago@nl.linux.org> to fix a minor
+ * compile error in send_arp.c, which only affects the 1.1 libnet API code.
+ *
+ * Revision 1.18  2003/02/17 18:51:03  alan
+ * Minor typo correction for #error line in send_arp.c
+ *
+ * Revision 1.17  2003/02/17 16:30:46  msoffen
+ * Made it error out if libnet isn't defined at all (no 1.0 or 1.1 version).
+ *
+ * Revision 1.16  2003/02/17 15:31:50  alan
+ * Fixed a nasty bug where we don't pass the interface to the libnet libraries
+ * correctly.  Thanks to Steve Snodgrass for finding it.
+ *
+ * Revision 1.15  2003/02/07 08:37:17  horms
+ * Removed inclusion of portability.h from .h files
+ * so that it does not need to be installed.
+ *
+ * Revision 1.14  2003/02/05 09:06:33  horms
+ * Lars put a lot of work into making sure that portability.h
+ * is included first, everywhere. However this broke a few
+ * things when building against heartbeat headers that
+ * have been installed (usually somewhere under /usr/include or
+ * /usr/local/include).
+ *
+ * This patch should resolve this problem without undoing all of
+ * Lars's hard work.
+ *
+ * As an asside: I think that portability.h is a virus that has
+ * infected all of heartbeat's code and now must also infect all
+ * code that builds against heartbeat. I wish that it didn't need
+ * to be included all over the place. Especially in headers to
+ * be installed on the system. However, I respect Lars's opinion
+ * that this is the best way to resolve some weird build problems
+ * in the current tree.
+ *
+ * Revision 1.13  2003/01/31 10:02:09  lars
+ * Various small code cleanups:
+ * - Lots of "signed vs unsigned" comparison fixes
+ * - time_t globally replaced with TIME_T
+ * - All seqnos moved to "seqno_t", which defaults to unsigned long
+ * - DIMOF() definition centralized to portability.h and typecast to int
+ * - EOS define moved to portability.h
+ * - dropped inclusion of signal.h from stonith.h, so that sigignore is
+ *   properly defined
+ *
+ * Revision 1.12  2002/09/12 14:06:18  msoffen
+ * Removed a write to stderr of ".", really served no purpose and always ran.
+ * It was a carryover from the old send_arp.c.
+ *
+ * Revision 1.11  2002/09/05 06:12:42  alan
+ * Put in code to recover a bug fix from Japan, plus
+ * make the code hopefully work with both the old and new libnet APIs.
+ *
+ * Revision 1.10  2002/08/16 14:18:41  msoffen
+ * Changes to get IP takeover working properly on OpenBSD
+ * Changed how *BSD deletes an alias.
+ * Create get_hw_addr (uses libnet package).
+ *
+ * Revision 1.9  2002/07/08 04:14:12  alan
+ * Updated comments in the front of various files.
+ * Removed Matt's Solaris fix (which seems to be illegal on Linux).
+ *
+ * Revision 1.8  2002/06/06 04:43:40  alan
+ * Got rid of a warning (error) about an unused RCS version string.
+ *
+ * Revision 1.7  2002/05/28 18:25:48  msoffen
+ * Changes to replace send_arp with a libnet based version.  This works accross
+ * all operating systems we currently "support" (Linux, FreeBSD, Solaris).
+ *
+ * Revision 1.6  2001/10/24 00:21:58  alan
+ * Fix to send both a broadcast ARP request as well as the ARP response we
+ * were already sending.  All the interesting research work for this fix was
+ * done by Masaki Hasegawa <masaki-h@pp.iij4u.or.jp> and his colleagues.
+ *
+ * Revision 1.5  2001/06/07 21:29:44  alan
+ * Put in various portability changes to compile on Solaris w/o warnings.
+ * The symptoms came courtesy of David Lee.
+ *
+ * Revision 1.4  2000/12/04 20:33:17  alan
+ * OpenBSD fixes from Frank DENIS aka Jedi/Sector One <j@c9x.org>
+ *
+ * Revision 1.3  1999/10/05 06:17:29  alanr
+ * Fixed various uninitialized variables
+ *
+ * Revision 1.2  1999/09/30 18:34:27  alanr
+ * Matt Soffen's FreeBSD changes
+ *
+ * Revision 1.1.1.1  1999/09/23 15:31:24  alanr
+ * High-Availability Linux
+ *
+ * Revision 1.4  1999/09/08 03:46:27  alanr
+ * Changed things so they work when rearranged to match the FHS :-)
+ *
+ * Revision 1.3  1999/08/17 03:49:09  alanr
+ * *** empty log message ***
+ *
+ */
