@@ -1,3 +1,4 @@
+/* $Id: ipc.h,v 1.52 2005/07/25 20:43:12 gshi Exp $ */
 /*
  * ipc.h IPC abstraction data structures.
  *
@@ -32,6 +33,14 @@
 #include <sys/types.h>
 #include <sys/poll.h>
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
 #ifdef IPC_TIME_DEBUG
 #include <clplumbing/longclock.h>
 #define MAXIPCTIME 3000
@@ -65,8 +74,6 @@
 #define IPC_FAIL 1
 #define IPC_BROKEN 2
 #define IPC_INTR 3
-#define IPC_TIMEOUT 4
-
 /*
  *	IPC:  Sockets-like Interprocess Communication Abstraction
  *
@@ -130,7 +137,6 @@ typedef void(*flow_callback_t)(IPC_Channel*, void*);
 /* channel structure.*/
 struct IPC_CHANNEL{
 	int		ch_status;	/* identify the status of channel.*/
-	int		refcount;	/* reference count */
 	pid_t		farside_pid;	/* far side pid */
 	void*		ch_private;	/* channel private data. */
 					/* (may contain conn. info.) */
@@ -152,9 +158,6 @@ struct IPC_CHANNEL{
 	/* is the send blocking or nonblocking*/
 	gboolean	should_send_block;
 	
-	/* if send would block, should an error be returned or not */
-	gboolean	should_block_fail;
-	
 /*  There are two queues in channel. One is for sending and the other
  *  is for receiving. 
  *  Those two queues are channel's internal queues. They should not be 
@@ -583,9 +586,7 @@ struct IPC_OPS{
 	 */
 	
 	int (*get_conntype)(struct IPC_CHANNEL* ch);
-
-	int (*disconnect)(struct IPC_CHANNEL* ch);
-		
+	
 };
 
 
@@ -705,11 +706,7 @@ struct SOCKET_MSG_HEAD{
 };
 
 
-/* MAXMSG is the maximum final message size on the wire. */
-#define	MAXMSG		(256*1024)
-/* MAXUNCOMPRESSED is the maximum, raw data size prior to compression. */
-/* 1:8 compression ratio is to be expected on data such as xml */
-#define	MAXUNCOMPRESSED	(2048*1024)
+#define	MAXDATASIZE	65536
 #define HEADMAGIC	0xabcd
 #define POOL_SIZE (4*1024)
 struct ipc_bufpool{
