@@ -1,3 +1,4 @@
+/* $Id: xml.h,v 1.29 2005/09/12 11:00:19 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -18,6 +19,7 @@
 #ifndef CRM_COMMON_XML__H
 #define CRM_COMMON_XML__H
 
+#include <portability.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -30,11 +32,7 @@
 #include <clplumbing/cl_log.h> 
 
 /* #define USE_LIBXML 1 */
-#if CRM_DEV_BUILD
-#  define XML_PARANOIA_CHECKS 1
-#else
-#  define XML_PARANOIA_CHECKS 0
-#endif
+#define XML_PARANOIA_CHECKS 1
 
 #ifdef USE_LIBXML
 #  include <libxml/tree.h> 
@@ -45,17 +43,9 @@
 
 extern gboolean add_message_xml(
 	HA_Message *msg, const char *field, const crm_data_t *xml);
-extern crm_data_t *get_message_xml(HA_Message *msg, const char *field);
+extern crm_data_t *get_message_xml(const HA_Message *msg, const char *field);
 extern GHashTable *xml2list(crm_data_t *parent);
-#if CRM_DEPRECATED_SINCE_2_0_3
-extern GHashTable *xml2list_202(crm_data_t *parent);
-#endif
-extern void hash2nvpair(gpointer key, gpointer value, gpointer user_data);
-extern void hash2field(gpointer key, gpointer value, gpointer user_data);
-extern void hash2metafield(gpointer key, gpointer value, gpointer user_data);
-
-extern gboolean do_id_check(crm_data_t *xml_obj, GHashTable *id_hash,
-			    gboolean silent_add, gboolean silent_rename);
+extern void do_id_check(crm_data_t *xml_obj, GHashTable *id_hash);
 
 /*
  * Replacement function for xmlCopyPropList which at the very least,
@@ -98,12 +88,24 @@ extern const char *get_xml_attr_nested(crm_data_t *parent,
 				       const char **node_path, int length,
 				       const char *attr_name, gboolean error);
 
-#define free_xml(a_node) do {				\
-		if(a_node != NULL) {			\
-			crm_validate_data(a_node);	\
-			ha_msg_del(a_node);		\
-		}					\
-	} while(0)
+/*
+ * Free the XML "stuff" associated with a_node
+ *
+ * If a_node is part of another XML blob, barf.
+ *   (Should be using free_xml_from_parent)
+ *
+ * Otherwise, free everything recursivly
+ *
+ * Wont barf on NULL.
+ *
+ */
+extern void free_xml_fn(crm_data_t *a_node);
+#if 1
+#  define free_xml(xml_obj) free_xml_fn(xml_obj); xml_obj = NULL
+#else
+#  define free_xml(xml_obj) xml_obj = NULL
+#endif
+
 
 void free_xml_from_parent(crm_data_t *parent, crm_data_t *a_node);
 #define zap_xml_from_parent(parent, xml_obj) free_xml_from_parent(parent, xml_obj); xml_obj = NULL
@@ -144,12 +146,7 @@ extern void unlink_xml_node(crm_data_t *node);
 /*
  * Set a timestamp attribute on a_node
  */
-extern void add_xml_tstamp(crm_data_t *a_node);
-
-/*
- * 
- */
-extern void purge_diff_markers(crm_data_t *a_node);
+extern void set_node_tstamp(crm_data_t *a_node);
 
 /*
  * Returns a deep copy of src_node
@@ -163,22 +160,19 @@ extern crm_data_t *copy_xml(const crm_data_t *src_node);
 extern crm_data_t *add_node_copy(
 	crm_data_t *new_parent, const crm_data_t *xml_node);
 
-extern int add_node_nocopy(crm_data_t *parent, const char *name, crm_data_t *child);
-
 
 /*
  * XML I/O Functions
  *
  * Whitespace between tags is discarded.
  */
-extern crm_data_t *file2xml(FILE *input, gboolean compressed);
+extern crm_data_t *file2xml(FILE *input);
 
 extern crm_data_t *stdin2xml(void);
 
 extern crm_data_t *string2xml(const char *input);
 
-extern int write_xml_file(
-	crm_data_t *xml_node, const char *filename, gboolean compress);
+extern int write_xml_file(crm_data_t *xml_node, const char *filename);
 
 extern char *dump_xml_formatted(const crm_data_t *msg);
 
@@ -210,22 +204,17 @@ extern crm_data_t *find_entity(
 	crm_data_t *parent, const char *node_name, const char *id);
 
 extern crm_data_t *subtract_xml_object(
-	crm_data_t *left, crm_data_t *right, const char *marker);
+	crm_data_t *left, crm_data_t *right, gboolean suppress);
 
 extern int add_xml_object(
 	crm_data_t *parent, crm_data_t *target, const crm_data_t *update);
 
 extern void xml_remove_prop(crm_data_t *obj, const char *name);
 
-extern gboolean replace_xml_child(
-	crm_data_t *parent, crm_data_t *child, crm_data_t *update, gboolean delete_only);
+extern void crm_set_element_parent(crm_data_t *data, crm_data_t *parent);
 
-extern gboolean update_xml_child(crm_data_t *child, crm_data_t *to_update);
-
-extern int find_xml_children(
-	crm_data_t **children, crm_data_t *root,
-	const char *tag, const char *field, const char *value,
-	gboolean search_matches);
+extern gboolean delete_xml_child(
+	crm_data_t *parent, crm_data_t *child, crm_data_t *to_delete);
 
 /*
  *
@@ -235,60 +224,29 @@ extern char *crm_element_value_copy(const crm_data_t *data, const char *name);
 
 extern const char *crm_element_name(const crm_data_t *data);
 
-extern void xml_validate(const crm_data_t *root);
+extern void crm_validate_data(const crm_data_t *root);
 
-extern gboolean xml_has_children(const crm_data_t *root);	 		
+extern void crm_update_parents(crm_data_t *root);
 
-extern char *calculate_xml_digest(crm_data_t *local_cib, gboolean sort, gboolean do_filter);
+extern gboolean xml_has_children(crm_data_t *root);	 		
 
-extern gboolean validate_with_dtd(
-	crm_data_t *xml_blob, gboolean to_logs, const char *dtd_file);
-
-#if XML_PARANOIA_CHECKS
-#  define crm_validate_data(obj) xml_validate(obj)
-#else
-#  define crm_validate_data(obj) CRM_DEV_ASSERT(obj != NULL)
-#endif
-
-#define xml_child_iter(parent, child, loop_code)			\
+#   define xmlGetNodePath(data) crm_element_value(data, XML_ATTR_TAGNAME)
+#   define xml_child_iter(parent, child, filter, loop_code)		\
 	if(parent != NULL) {						\
 		int __counter = 0;					\
 		crm_data_t *child = NULL;				\
 		crm_validate_data(parent);				\
 		for (__counter = 0; __counter < parent->nfields; __counter++) { \
-			if(parent->types[__counter] != FT_STRUCT	\
-			   && parent->types[__counter] != FT_UNCOMPRESS) { \
+			if(parent->types[__counter] != FT_STRUCT) {		\
 				continue;				\
 			}						\
 			child = parent->values[__counter];		\
 			if(child == NULL) {				\
 				crm_debug_4("Skipping %s == NULL",	\
 					  parent->names[__counter]);	\
-			} else {					\
-				loop_code;				\
-			}						\
-		}							\
-	} else {							\
-		crm_debug_4("Parent of loop was NULL");			\
-	}
-
-#define xml_child_iter_filter(parent, child, filter, loop_code)		\
-	if(parent != NULL) {						\
-		int __counter = 0;					\
-		crm_data_t *child = NULL;				\
-		crm_validate_data(parent);				\
-		for (__counter = 0; __counter < parent->nfields; __counter++) { \
-			if(parent->types[__counter] != FT_STRUCT	\
-			   && parent->types[__counter] != FT_UNCOMPRESS) { \
-				continue;				\
-			}						\
-			child = parent->values[__counter];		\
-			if(child == NULL) {				\
-				crm_debug_4("Skipping %s == NULL",	\
-					  parent->names[__counter]);	\
-			} else if(filter == NULL/*constant condition*/	\
+			} else if(filter == NULL			\
 				  || safe_str_eq(filter, parent->names[__counter])) { \
-				loop_code;				\
+				loop_code;					\
 			} else {					\
 				crm_debug_4("Skipping <%s../>",		\
 					  parent->names[__counter]);	\
@@ -306,13 +264,14 @@ extern gboolean validate_with_dtd(
 		crm_debug_5("Searching %d fields", parent->nfields);	\
 		for (__counter = 0; __counter < parent->nfields; __counter++) { \
 			crm_debug_5("Searching field %d", __counter);	\
-			prop_name = parent->names[__counter];		\
 			if(parent->types[__counter] != FT_STRING) {	\
 				continue;				\
-			} else if(prop_name[0] == '_'			\
-				  && prop_name[1] == '_') {		\
+			} else if(safe_str_eq(parent->names[__counter], F_XML_TAGNAME)) { \
+				continue;				\
+			} else if(safe_str_eq(parent->names[__counter], F_XML_PARENT)) { \
 				continue;				\
 			}						\
+			prop_name = parent->names[__counter];		\
 			prop_value = parent->values[__counter];		\
 			code;						\
 		}							\
