@@ -1,3 +1,4 @@
+/* $Id: crm.h,v 1.76 2005/09/02 12:31:44 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -19,34 +20,31 @@
 #define CRM__H
 
 #include <stdlib.h>
+#include <ha_config.h>
 #include <glib.h>
 #undef MIN
 #undef MAX
 
 #include <string.h>
+#include <portability.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/cl_malloc.h>
 #ifdef MCHECK
 #include <mcheck.h>
 #endif
 
+#include <config.h>
+
 #ifndef CRM_DEV_BUILD
 #  define CRM_DEV_BUILD 0
 #endif
 
-#define CRM_DEPRECATED_SINCE_2_0_1 0
-#define CRM_DEPRECATED_SINCE_2_0_2 0
-#define CRM_DEPRECATED_SINCE_2_0_3 0
+#define CRM_DEPRECATED_SINCE_2_0_1 1
+#define CRM_DEPRECATED_SINCE_2_0_2 1
+#define CRM_DEPRECATED_SINCE_2_0_3 1
 #define CRM_DEPRECATED_SINCE_2_0_4 1
-#define CRM_DEPRECATED_SINCE_2_0_5 1
-#define CRM_DEPRECATED_SINCE_2_0_6 1
-#define CRM_DEPRECATED_SINCE_2_0_7 1
-#define CRM_DEPRECATED_SINCE_2_0_8 1
 #define CRM_DEPRECATED_SINCE_2_1_0 1
 
-#define CRM_META			"CRM_meta"
-#define crm_meta_name(field) CRM_META"_"field
-
 #define ipc_call_diff_max_ms 5000
 #define action_diff_warn_ms  5000
 #define action_diff_max_ms   20000
@@ -56,30 +54,26 @@
 #include <crm/common/util.h>
 
 #define CRM_ASSERT(expr) if((expr) == FALSE) {				\
-	crm_abort(__FILE__, __PRETTY_FUNCTION__, __LINE__, #expr, TRUE, FALSE); \
-    }
+		do_crm_log(LOG_CRIT, __FILE__, __PRETTY_FUNCTION__,	\
+			   "Triggered dev assert at %s:%d : %s",	\
+			   __FILE__, __LINE__, #expr);			\
+		abort();						\
+	}
 
 extern gboolean crm_assert_failed;
 
-#define CRM_DEV_ASSERT(expr)						\
-	crm_assert_failed = FALSE;					\
+#define CRM_DEV_ASSERT(expr) crm_assert_failed = FALSE;			\
 	if((expr) == FALSE) {						\
 		crm_assert_failed = TRUE;				\
-		crm_abort(__FILE__,__PRETTY_FUNCTION__,__LINE__, #expr, FALSE, TRUE); \
+		do_crm_log(CRM_DEV_BUILD?LOG_CRIT:LOG_ERR,		\
+			   __FILE__, __PRETTY_FUNCTION__,		\
+			   "Triggered dev assert at %s:%d : %s",	\
+			   __FILE__, __LINE__, #expr);			\
+		if(CRM_DEV_BUILD) {					\
+			abort();					\
+		}							\
 	}
 
-extern const char *crm_system_name;
-
-#define CRM_CHECK(expr, failure_action) if((expr) == FALSE) {		\
-	crm_abort(__FILE__,__PRETTY_FUNCTION__,__LINE__, #expr, FALSE, TRUE); \
-	failure_action;							\
-    }
-
-#define CRM_CHECK_AND_STORE(expr, failure_action) if((expr) == FALSE) {	\
-	crm_abort(__FILE__,__PRETTY_FUNCTION__,__LINE__, #expr, TRUE, TRUE); \
-	failure_action;							\
-    }
-
 /* Clean these up at some point, some probably should be runtime options */
 #define WORKING_DIR	HA_VARLIBDIR"/heartbeat/crm"
 #define CRM_SOCK_DIR	HA_VARRUNDIR"/heartbeat/crm"
@@ -90,7 +84,7 @@ extern const char *crm_system_name;
 #define CIB_FILENAME	WORKING_DIR"/cib.xml"
 #define CIB_BACKUP	WORKING_DIR"/cib_backup.xml"
 
-#define CRM_FEATURE_SET	"2.0"
+#define CRM_VERSION	"1.0"
 
 #define MSG_LOG			1
 #define DOT_FSA_ACTIONS		1
@@ -124,7 +118,6 @@ extern const char *crm_system_name;
 #define CRM_OP_RETRIVE_CIB	"retrieve_cib"
 #define CRM_OP_PING		"ping"
 #define CRM_OP_VOTE		"vote"
-#define CRM_OP_NOVOTE		"no-vote"
 #define CRM_OP_HELLO		"hello"
 #define CRM_OP_HBEAT		"dc_beat"
 #define CRM_OP_PECALC		"pe_calc"
@@ -145,12 +138,6 @@ extern const char *crm_system_name;
 #define CRM_OP_DEBUG_UP		"debug_inc"
 #define CRM_OP_DEBUG_DOWN	"debug_dec"
 #define CRM_OP_INVOKE_LRM	"lrm_invoke"
-#define CRM_OP_LRM_REFRESH	"lrm_refresh"
-#define CRM_OP_LRM_QUERY	"lrm_query"
-#define CRM_OP_LRM_DELETE	"lrm_delete"
-#define CRM_OP_LRM_FAIL		"lrm_fail"
-#define CRM_OP_PROBED		"probe_complete"
-#define CRM_OP_REPROBE		"probe_again"
 
 #define CRMD_STATE_ACTIVE	"member"
 #define CRMD_STATE_INACTIVE	"down"
@@ -159,43 +146,37 @@ extern const char *crm_system_name;
 #define CRMD_JOINSTATE_PENDING	"pending"
 #define CRMD_JOINSTATE_MEMBER	CRMD_STATE_ACTIVE
 
-#define CRMD_ACTION_DELETE		"delete"
-#define CRMD_ACTION_CANCEL		"cancel"
-
-#define CRMD_ACTION_MIGRATE		"migrate_to"
-#define CRMD_ACTION_MIGRATED		"migrate_from"
-
 #define CRMD_ACTION_START		"start"
 #define CRMD_ACTION_STARTED		"running"
+#define CRMD_ACTION_START_FAIL		"start_failed"
+#define CRMD_ACTION_START_PENDING	"starting"
 
 #define CRMD_ACTION_STOP		"stop"
 #define CRMD_ACTION_STOPPED		"stopped"
-
-#define CRMD_ACTION_PROMOTE		"promote"
-#define CRMD_ACTION_PROMOTED		"promoted"
-#define CRMD_ACTION_DEMOTE		"demote"
-#define CRMD_ACTION_DEMOTED		"demoted"
+#define CRMD_ACTION_STOP_FAIL		"stop_failed"
+#define CRMD_ACTION_STOP_PENDING	"stopping"
 
 #define CRMD_ACTION_NOTIFY		"notify"
 #define CRMD_ACTION_NOTIFIED		"notified"
 
-#define CRMD_ACTION_STATUS		"monitor"
+#define CRMD_ACTION_MON			"monitor"
+#define CRMD_ACTION_MON_PENDING		CRMD_ACTION_STARTED
+#define CRMD_ACTION_MON_OK		CRMD_ACTION_STARTED
+#define CRMD_ACTION_MON_FAIL		"monitor_failed"
+/* #define CRMD_ACTION_GENERIC		"pending" */
+#define CRMD_ACTION_GENERIC_PENDING	"pending"
+#define CRMD_ACTION_GENERIC_OK		"complete"
+#define CRMD_ACTION_GENERIC_FAIL	"pending_failed"
 
 typedef GList* GListPtr;
-#define slist_destroy(child_type, child, parent, a)			\
-	{		 						\
-		GListPtr __crm_iter_head = parent;			\
-		child_type *child = NULL;				\
-		while(__crm_iter_head != NULL) {			\
-			child = __crm_iter_head->data;			\
-			__crm_iter_head = __crm_iter_head->next;	\
-			{ a; }						\
-		}							\
-		g_list_free(parent);					\
-	}
+
+#define crm_atoi(text, default) atoi(text?text:default)
+
+extern gboolean safe_str_eq(const char *a, const char *b);
+extern gboolean safe_str_neq(const char *a, const char *b);
 
 #define slist_iter(child, child_type, parent, counter, a)		\
-	{		 						\
+	{								\
 		GListPtr __crm_iter_head = parent;			\
 		child_type *child = NULL;				\
 		int counter = 0;					\
@@ -214,44 +195,51 @@ typedef GList* GListPtr;
 
 #define LOG_MSG  LOG_DEBUG_3
 
-/*
- * Throughout the macros below, note the leading, pre-comma, space in the
- * various ' , ##args' occurences to aid portability across versions of 'gcc'.
- *	http://gcc.gnu.org/onlinedocs/cpp/Variadic-Macros.html#Variadic-Macros
- */
-#define do_crm_log(level, fmt, args...) do {				\
-		if(crm_log_level < (level)) {				\
-			continue;					\
-		} else if((level) > LOG_DEBUG) {			\
-			cl_log(LOG_DEBUG, "debug%d: %s: " fmt,		\
-			       level-LOG_INFO, __PRETTY_FUNCTION__ , ##args); \
-		} else {						\
-			cl_log(level, "%s: " fmt,			\
-			       __PRETTY_FUNCTION__ , ##args);		\
-		}							\
-	} while(0)
+#define crm_crit(w...)    do_crm_log(LOG_CRIT,    __FILE__, __FUNCTION__, w)
+#define crm_err(w...)     do_crm_log(LOG_ERR,     __FILE__, __FUNCTION__, w)
+#define crm_warn(w...)    do_crm_log(LOG_WARNING, __FILE__, __FUNCTION__, w)
+#define crm_notice(w...)  do_crm_log(LOG_NOTICE,  __FILE__, __FUNCTION__, w)
+#define crm_info(w...)    do_crm_log(LOG_INFO,    __FILE__, __FUNCTION__, w)
+#define crm_log_maybe(level, fmt...) if(crm_log_level >= level) {	\
+		do_crm_log(level, __FILE__, __FUNCTION__, fmt);		\
+	}
 
-#define crm_crit(fmt, args...)    do_crm_log(LOG_CRIT,    fmt , ##args)
-#define crm_err(fmt, args...)     do_crm_log(LOG_ERR,     fmt , ##args)
-#define crm_warn(fmt, args...)    do_crm_log(LOG_WARNING, fmt , ##args)
-#define crm_notice(fmt, args...)  do_crm_log(LOG_NOTICE,  fmt , ##args)
-#define crm_info(fmt, args...)    do_crm_log(LOG_INFO,    fmt , ##args)
-#define crm_debug(fmt, args...)   do_crm_log(LOG_DEBUG,   fmt , ##args)
-#define crm_debug_2(fmt, args...) do_crm_log(LOG_DEBUG_2, fmt , ##args)
-#define crm_debug_3(fmt, args...) do_crm_log(LOG_DEBUG_3, fmt , ##args)
-#define crm_debug_4(fmt, args...) do_crm_log(LOG_DEBUG_4, fmt , ##args)
-#define crm_debug_5(fmt, args...) do_crm_log(LOG_DEBUG_5, fmt , ##args)
-#define crm_debug_6(fmt, args...) do_crm_log(LOG_DEBUG_6, fmt , ##args)
+#define crm_debug(fmt...)   crm_log_maybe(LOG_DEBUG, fmt)
+#define crm_debug_2(fmt...) crm_log_maybe(LOG_DEBUG_2, fmt)
+
+/* If this is not a developmental build, give the compiler every chance to
+ * optimize these away
+ */
+#if CRM_DEV_BUILD
+#  define crm_debug_3(fmt...) crm_log_maybe(LOG_DEBUG_3, fmt)
+#  define crm_debug_4(fmt...) crm_log_maybe(LOG_DEBUG_4, fmt)
+#  define crm_debug_5(fmt...) crm_log_maybe(LOG_DEBUG_5, fmt)
+#  define crm_debug_6(fmt...) crm_log_maybe(LOG_DEBUG_6, fmt)
+#else
+#  define crm_debug_3(w...) if(0) { do_crm_log(LOG_DEBUG, NULL, NULL, w); }
+#  define crm_debug_4(w...) if(0) { do_crm_log(LOG_DEBUG, NULL, NULL, w); }
+#  define crm_debug_5(w...) if(0) { do_crm_log(LOG_DEBUG, NULL, NULL, w); }
+#  define crm_debug_6(w...) if(0) { do_crm_log(LOG_DEBUG, NULL, NULL, w); }
+#endif
 
 extern void crm_log_message_adv(
 	int level, const char *alt_debugfile, const HA_Message *msg);
 
-#define crm_log_message(level, msg) if(crm_log_level >= (level)) {	\
+#define crm_log_message(level, msg) if(crm_log_level >= level) {	\
 		crm_log_message_adv(level, NULL, msg);			\
 	}
 
-#define crm_log_xml(level, text, xml)   if(crm_log_level >= (level)) {	\
-		print_xml_formatted(level,  __PRETTY_FUNCTION__, xml, text); \
+#define crm_do_action(level, actions) if(crm_log_level >= level) {	\
+		actions;						\
+	}
+#define crm_action_info(x)    crm_do_action(LOG_INFO,    x)
+#define crm_action_debug(x)   crm_do_action(LOG_DEBUG,   x)
+#define crm_action_debug_2(x) crm_do_action(LOG_DEBUG_2, x)
+#define crm_action_debug_3(x) crm_do_action(LOG_DEBUG_3, x)
+#define crm_action_debug_4(x) crm_do_action(LOG_DEBUG_4, x)
+
+#define crm_log_xml(level, text, xml)   if(crm_log_level >= level) {  \
+		print_xml_formatted(level,  __FUNCTION__, xml, text); \
 	}
 #define crm_log_xml_crit(xml, text)    crm_log_xml(LOG_CRIT,    text, xml)
 #define crm_log_xml_err(xml, text)     crm_log_xml(LOG_ERR,     text, xml)
@@ -266,47 +254,55 @@ extern void crm_log_message_adv(
 
 #define crm_str(x)    (const char*)(x?x:"<null>")
 
-#if CRM_DEV_BUILD
-#    define crm_malloc0(malloc_obj, length) do {			\
-		if(malloc_obj) {					\
+#if CRM_USE_MALLOC
+#  define crm_malloc0(new_obj,length)					\
+	{								\
+		new_obj = malloc(length);				\
+		if(new_obj == NULL) {					\
+			crm_crit("Out of memory... exiting");		\
+			exit(1);					\
+		} else {						\
+			memset(new_obj, 0, length);			\
+		}							\
+	}
+#  define crm_free(x) if(x) { free(x); x=NULL; }
+#  define crm_is_allocated(obj) obj?TRUE:FALSE
+#else
+#  if CRM_DEV_BUILD
+#    define crm_malloc0(new_obj,length)					\
+	{								\
+		if(new_obj) {						\
 			crm_err("Potential memory leak:"		\
 				" %s at %s:%d not NULL before alloc.",	\
-				#malloc_obj, __FILE__, __LINE__);	\
+				#new_obj, __FILE__, __LINE__);		\
+			abort();					\
 		}							\
-		malloc_obj = cl_malloc(length);				\
-		CRM_ASSERT(malloc_obj != NULL);				\
-		memset(malloc_obj, 0, length);				\
-	} while(0)
-/* it's not a memory leak to already have an object to realloc, that's
- * the usual case, however if it does have a value, it must have been
- * allocated by the same allocator!
- */ 
-#    define crm_realloc(realloc_obj, length) do {			\
-		if (realloc_obj != NULL) {				\
-			CRM_ASSERT(cl_is_allocated(realloc_obj) == 1);	\
+		new_obj = cl_malloc(length);				\
+		if(new_obj == NULL) {					\
+			crm_crit("Out of memory... exiting");		\
+			abort();					\
 		}							\
-		realloc_obj = cl_realloc(realloc_obj, length);		\
-		CRM_ASSERT(realloc_obj != NULL);			\
-	} while(0)
-#    define crm_free(free_obj) if(free_obj) {			\
-		CRM_ASSERT(cl_is_allocated(free_obj) == 1);	\
-		cl_free(free_obj);				\
-		free_obj=NULL;					\
+		memset(new_obj, 0, length);				\
 	}
 #else
-#    define crm_malloc0(malloc_obj, length) do {			\
-		malloc_obj = cl_malloc(length);				\
-		CRM_ASSERT(malloc_obj != NULL);				\
-		memset(malloc_obj, 0, length);				\
-	} while(0)
-#    define crm_realloc(realloc_obj, length) do {			\
-		realloc_obj = cl_realloc(realloc_obj, length);		\
-		CRM_ASSERT(realloc_obj != NULL);			\
-	} while(0)
-	
-#    define crm_free(free_obj) if(free_obj) { cl_free(free_obj); free_obj=NULL; }
+#    define crm_malloc0(new_obj,length)					\
+	{								\
+		new_obj = cl_malloc(length);				\
+		if(new_obj == NULL) {					\
+			crm_crit("Out of memory... exiting");		\
+			abort();					\
+		}							\
+		memset(new_obj, 0, length);				\
+	}
+#  endif
+#  define crm_free(x) if(x) {				\
+		CRM_ASSERT(cl_is_allocated(x) == 1);	\
+		cl_free(x);				\
+		x=NULL;				\
+	}
+#  define crm_is_allocated(obj) cl_is_allocated(obj)
 #endif
 
 #define crm_msg_del(msg) if(msg != NULL) { ha_msg_del(msg); msg = NULL; }
-#define crm_strdup(str) crm_strdup_fn(str, __FILE__, __PRETTY_FUNCTION__, __LINE__)
+
 #endif
