@@ -1,4 +1,4 @@
-/* $Id: hb_api.h,v 1.12.2.4 2004/10/16 05:22:46 alan Exp $ */
+/* $Id: hb_api.h,v 1.35 2005/08/05 19:40:13 gshi Exp $ */
 /*
  * Client-side Low-level clustering API for heartbeat.
  *
@@ -44,10 +44,17 @@
 
 #ifndef __HB_API_H
 #	define __HB_API_H 1
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif 
+
 #include <ha_msg.h>
 #include <clplumbing/ipc.h>
 
-#define	LLC_PROTOCOL_VERSION	1
+#define	LLC_PROTOCOL_VERSION	2
+
+#include <clplumbing/cl_uuid.h>
 
 typedef void (*llc_msg_callback_t) (const struct ha_msg* msg
 ,	void* private_data);
@@ -69,8 +76,8 @@ typedef struct ll_cluster {
 }ll_cluster_t;
 
 struct llc_ops {
-	int		(*signon) (ll_cluster_t*, const char * service);
-	int		(*signoff) (ll_cluster_t*);
+	int		(*signon) (ll_cluster_t*, const char * clientid);
+	int		(*signoff) (ll_cluster_t*, gboolean destroy_channel);
 	int		(*delete) (ll_cluster_t*);
 	
 /*
@@ -177,6 +184,25 @@ struct llc_ops {
 	const char*	(*client_status)(ll_cluster_t*, const char *host,
 			const char *clientid, int timeout);
 
+/*
+ *	get_uuid_by_name:
+ *			return the uuid for the node which has the given name
+ */
+	
+	int		(*get_uuid_by_name)(ll_cluster_t*,
+					    const char*,
+					    cl_uuid_t*);
+/*
+ *	get_name_by_uuid:
+ *			return the name for the node which has the given uuid
+ */
+	
+	int		(*get_name_by_uuid)(ll_cluster_t*,
+					    cl_uuid_t*,
+					    char*,
+					    size_t);
+	
+	
 /*************************************************************************
  * Intracluster messaging
  *************************************************************************/
@@ -193,6 +219,15 @@ struct llc_ops {
 ,			struct ha_msg* msg
 ,			const char * nodename);
 
+/*
+ *	sendnodemsg_byuuid: 
+ *			Send the given message to the given node in cluster.
+ */
+
+	int		(*sendnodemsg_byuuid)(ll_cluster_t*,
+					      struct ha_msg* msg,
+					      cl_uuid_t*);
+
 /*
  *	send_ordered_clustermsg: Send ordered message to all cluster members.
  */
@@ -320,7 +355,32 @@ struct llc_ops {
  */
 	const char * (*get_resources)(ll_cluster_t *);
 
+	/*
+	 * chan_is_connected()
+	 *
+	 * Return true if the channel is connected
+	 */
+	
+	gboolean (*chan_is_connected)(ll_cluster_t *);
 
+	
+	/* Set the send queue length in heartbeat side
+	   for the channel. This function can be used
+	   to set a large send queue if the client will
+	   receive slowly */
+	
+	int	(*set_sendq_len)(ll_cluster_t* lcl, int length);
+	     
+	/* set the send blocking mode
+	 * TRUE indicate blocking, i.e if the send queue is full
+	 * the function will block there until there are slots available
+	 * or the IPC is disconnected
+	 * FALSE indicates the function will return immediately even 
+	 * if there is no slot available
+	 */	
+	int (*set_send_block_mode)(ll_cluster_t*, gboolean);
+	
+	     
 	const char * (*errmsg)(ll_cluster_t*);
 };
 
@@ -344,6 +404,8 @@ struct llc_ops {
 #define KEY_STONITH	"stonith"
 #define KEY_STONITHHOST "stonith_host"
 #define KEY_CLIENT_CHILD "respawn"
+#define KEY_COMPRESSION "compression"
+#define KEY_COMPRESSION_THRESHOLD "compression_threshold"
 #define KEY_RT_PRIO	"rtprio"
 #define KEY_GEN_METH	"hbgenmethod"
 #define KEY_REALTIME	"realtime"
@@ -351,12 +413,13 @@ struct llc_ops {
 #define KEY_NORMALPOLL	"normalpoll"
 #define KEY_APIPERM	"apiauth"
 #define KEY_MSGFMT      "msgfmt"
+#define KEY_LOGDAEMON   "use_logd"
+#define KEY_CONNINTVAL	"conn_logd_time"
 #define KEY_BADPACK	"log_badpack"
 #define KEY_REGAPPHBD	"use_apphbd"
 #define KEY_COREDUMP	"coredumps"
 #define KEY_COREROOTDIR	"coreroot"
-
-
+#define KEY_REL2	"crm"
 
 ll_cluster_t*	ll_cluster_new(const char * llctype);
 #endif /* __HB_API_H */
