@@ -1,3 +1,4 @@
+/* $Id: hb_api_core.h,v 1.14 2005/09/09 17:21:18 gshi Exp $ */
 /*
  * hb_api_core_h: Internal definitions and functions for the heartbeat API
  *
@@ -29,26 +30,21 @@
 #ifndef _HB_API_CORE_H
 #	define _HB_API_CORE_H 1
 
+#include <config.h>
 #include <sys/types.h>
 #include <glib.h>
 #include <clplumbing/GSource.h>
 #include <ha_msg.h>
 
 /* Dispatch priorities for various kinds of events */
-#define	PRI_SENDSTATUS		(G_PRIORITY_HIGH-5)
-#define	PRI_SENDPKT		(PRI_SENDSTATUS+1)
-#define	PRI_READPKT		(PRI_SENDPKT+1)
-#define	PRI_FIFOMSG		(PRI_READPKT+1)
-
-#define PRI_CHECKSIGS		(G_PRIORITY_DEFAULT)
-#define PRI_FREEMSG		(PRI_CHECKSIGS+1)
-#define	PRI_CLIENTMSG		(PRI_FREEMSG+1)
-
-#define	PRI_APIREGISTER		(G_PRIORITY_LOW)
-#define	PRI_RANDOM		(PRI_APIREGISTER+1)
-#define	PRI_AUDITCLIENT		(PRI_RANDOM+1)
-#define	PRI_WRITECACHE		(PRI_AUDITCLIENT+1)
-#define	PRI_DUMPSTATS		(PRI_WRITECACHE+20)
+#define	PRI_SENDSTATUS		G_PRIORITY_HIGH
+#define	PRI_DUMPSTATS		G_PRIORITY_LOW
+#define	PRI_AUDITCLIENT		G_PRIORITY_LOW
+#define	PRI_APIREGISTER		(G_PRIORITY_LOW-1)
+#define	PRI_CLUSTERMSG		G_PRIORITY_DEFAULT
+#define	PRI_FIFOMSG		PRI_CLUSTERMSG-1
+#define PRI_FREEMSG		G_PRIORITY_DEFAULT
+#define	PRI_CLIENTMSG		PRI_FIFOMSG
 
 void process_registerevent(IPC_Channel* chan,  gpointer user_data);
 
@@ -111,10 +107,7 @@ typedef struct client_process {
 #	define	F_NODENAME	"node"
 #define	API_NODELIST_END	"nodelist-end"
 #define	API_NODESTATUS		"nodestatus"
-#define	API_NODEWEIGHT		"nodeweight"
-#define	API_NODESITE		"nodesite"
 #define	API_NODETYPE		"nodetype"
-#define	API_NUMNODES		"numnodes"
 
 #define	API_IFLIST		"iflist"
 #	define	F_IFNAME	"ifname"
@@ -137,11 +130,11 @@ typedef struct client_process {
 #define	API_BADREQ		"badreq"
 #define	API_MORE		"ok/more"
 
-#define	API_FIFO_DIR	HA_VARLIBHBDIR "/api"
+#define	API_FIFO_DIR	VAR_LIB_D "/api"
 #define	API_FIFO_LEN	(sizeof(API_FIFO_DIR)+32)
 
 #define	NAMEDCLIENTDIR	API_FIFO_DIR
-#define	CASUALCLIENTDIR	HA_VARLIBHBDIR "/casual"
+#define	CASUALCLIENTDIR	VAR_LIB_D "/casual"
 
 #define	REQ_SUFFIX	".req"
 #define	RSP_SUFFIX	".rsp"
