@@ -1,4 +1,4 @@
-/* $Id: heartbeat.h,v 1.39.2.6 2005/08/19 16:18:15 sunjd Exp $ */
+/* $Id: heartbeat.h,v 1.70 2005/09/07 15:38:38 msoffen Exp $ */
 /*
  * heartbeat.h: core definitions for the Linux-HA heartbeat program
  *
@@ -36,6 +36,10 @@
 #	define	FLUSH(fd)	tcflush(fd, TCIOFLUSH)
 #endif
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <limits.h>
 #include <syslog.h>
 #include <netdb.h>
@@ -52,6 +56,7 @@
 #include <clplumbing/cl_log.h>
 #include <clplumbing/longclock.h>
 #include <clplumbing/ipc.h>
+#include <clplumbing/proctrack.h>
 #include <clplumbing/cl_malloc.h>
 #define index FooIndex
 #define time FooTime
@@ -85,18 +90,18 @@
 
 #define	FIFOMODE	0600
 #define	RQSTDELAY	10
-
+#define	ACK_MSG_DIV	10
 #ifndef HA_D
-#	define	HA_D		"/etc/ha.d"
+#	define	HA_D		HA_RC_DIR
 #endif
 #ifndef VAR_RUN_D
 #	define	VAR_RUN_D	HA_VARRUNDIR
 #endif
 #ifndef VAR_LOG_D
-#	define	VAR_LOG_D	"/var/log"
+#	define	VAR_LOG_D	HA_VARLOGDIR
 #endif
 #ifndef HALIB
-#	define HALIB		"/usr/lib/heartbeat"
+#	define HALIB		HA_LIBDIR
 #endif
 #ifndef HA_MODULE_D
 #	define HA_MODULE_D	HALIB "/modules"
@@ -105,15 +110,11 @@
 #	define HA_PLUGIN_D	HALIB "/plugins"
 #endif
 #ifndef TTY_LOCK_D
-#	if defined(linux)
-#		define	TTY_LOCK_D	"/var/lock"
-#	else
-#		define	TTY_LOCK_D	"/var/spool/lock"
-#	endif
+#	define	TTY_LOCK_D	HA_LOCKDIR
 #endif
 
 #ifndef RSC_TMPDIR
-#	define	RSC_TMPDIR	VAR_LIB_D "/rsctmp"
+#	define	RSC_TMPDIR	HA_VARRUNDIR "/heartbeat/rsctmp"
 #endif
 
 /* #define HA_DEBUG */
@@ -156,7 +157,7 @@
 #define HANICEFAILBACK	"HA_NICEFAILBACK" /* "yes" when nice_failback is on */
 #define HADONTASK	"HA_DONTASK"	/* "yes" when no other nodes "active" ...*/
 #define HADEBUGVAL	"HA_DEBUG"	/* current debug value (if nonzero) */
-
+#define HALOGD		"HA_LOGD"	/* whether we use logging daemon or not */
 
 #define	DEFAULTBAUD	B19200	/* Default serial link speed */
 #define	DEFAULTBAUDRATE	19200	/* Default serial link speed as int */
@@ -199,7 +200,7 @@
 #define HA_FUNCS	HA_D "/shellfuncs"
 
 #define	RC_ARG0		"harc"
-
+#define ENV_PREFIX	"HA_"
 
 
 /* Which side of a pipe is which? */
@@ -211,17 +212,28 @@
 #define	FD_STDOUT	1
 #define	FD_STDERR	2
 
+#define PROTOCOL_VERSION	1
+
 typedef unsigned long seqno_t;
 
-#define	MAXMISSING	16
+#define	MAXMSGHIST	1000
+#define	MAXMISSING	MAXMSGHIST
+
 #define	NOSEQUENCE	0xffffffffUL
 struct seqtrack {
 	longclock_t	last_rexmit_req;
 	int		nmissing;
 	seqno_t		generation;	/* Heartbeat generation # */
 	seqno_t		last_seq;
+	seqno_t		first_missing_seq; /* the smallest missing seq number*/
+	GList*		client_status_msg_queue; /*client status message queue*/
 	seqno_t		seqmissing[MAXMISSING];
 	const char *	last_iface;
+	seqno_t		ack_trigger; /*whenever a message received 
+				      *with seq % ACK_MSG_DIV == ack_trigger
+				      *we send back an ACK
+				    */
+	seqno_t		ackseq; /* ACKed seq*/
 };
 
 struct link {
@@ -241,6 +253,7 @@ struct link {
 struct node_info {
 	int		nodetype;
 	char		nodename[HOSTLENG];	/* Host name from config file */
+	cl_uuid_t	uuid;
 	char		status[STATUSLENG];	/* Status from heartbeat */
 	struct link	links[MAXMEDIA];
 	int		nlinks;
@@ -280,6 +293,7 @@ struct sys_config {
         int    		use_dbgfile;            /* Flag to use the debug file*/
 	int		rereadauth;		/* 1 if we need to reread auth file */
 	seqno_t		generation;	/* Heartbeat generation # */
+	cl_uuid_t	uuid;		/* uuid for this node*/
 	int		authnum;
 	Stonith*	stonith;	/* Stonith method: WE NEED A LIST TO SUPPORT MULTIPLE STONITH DEVICES PER NODE -EZA */
 	struct HBauth_info* authmethod;	/* auth_config[authnum] */
@@ -287,11 +301,7 @@ struct sys_config {
 	struct HBauth_info  auth_config[MAXAUTH];
 	GList*		client_list;
 			/* List data: struct client_child */
-			/* These all show up in client_children */
-			/* when they're spawned (and have a pid) */
-	GHashTable*	client_children;/* Indexed by pid */
-			/* associated data: struct client_child */
-			/* They appear here after being spawned */
+	GList*		last_client;/* Last in client_list */
 };
 
 
@@ -309,7 +319,6 @@ struct hb_media {
 
 int parse_authfile(void);
 
-#define	MAXMSGHIST	1000
 struct msg_xmit_hist {
 	struct ha_msg*	msgq[MAXMSGHIST];
 	seqno_t		seqnos[MAXMSGHIST];
@@ -317,6 +326,8 @@ struct msg_xmit_hist {
 	int		lastmsg;
 	seqno_t		hiseq;
 	seqno_t		lowseq; /* one less than min actually present */
+	seqno_t		ackseq;
+	struct node_info* lowest_acknode;
 };
 
 /*
@@ -326,14 +337,15 @@ struct msg_xmit_hist {
  *	Normally, if they they die, we restart them.
  */
 struct client_child {
-	pid_t	pid;		/* Process id of child process */
-	int	respawn;	/* Respawn it if it dies? */
-	uid_t	u_runas;	/* Which user to run as? */
-	gid_t	g_runas;	/* Which group id to run as? */
-	int	respawncount;	/* Last time we respawned this command */
-	int	shortrcount;	/* How many times has it respawned too fast? */
-	char*	command;	/* What command to run? */
-	char*	path;		/* Path (argv[0])? */
+	pid_t		pid;		/* Process id of child process */
+	ProcTrack*	proctrack;	/* Process tracking structure */
+	int		respawn;	/* Respawn it if it dies? */
+	uid_t		u_runas;	/* Which user to run as? */
+	gid_t		g_runas;	/* Which group id to run as? */
+	int		respawncount;	/* Last time we respawned */
+	int		shortrcount;	/* Count of fast respawns */
+	char*		command;	/* What command to run? */
+	char*		path;		/* Path (argv[0])? */
 };
 
 int api_remove_client_pid(pid_t c_pid, const char * reason);
@@ -357,7 +369,7 @@ extern char *			localnodename;
 
 /* Generally useful exportable HA heartbeat routines... */
 extern void		ha_assert(const char *s, int line, const char * file);
-
+gboolean		heartbeat_on_congestion(void);
 extern int		send_cluster_msg(struct ha_msg*msg);
 extern void		cleanexit(int exitcode);
 extern void		check_auth_change(struct sys_config *);
@@ -373,6 +385,17 @@ struct link *  iface_lookup_node(const char *);
 int	add_node(const char * value, int nodetype);
 void	SetParameterValue(const char * name, const char * value);
 
+gint		uuid_equal(gconstpointer v, gconstpointer v2);
+guint		uuid_hash(gconstpointer key);
+void		add_nametable(const char* nodename, struct node_info* value);
+void		add_uuidtable(cl_uuid_t*, struct node_info* value);
+const char *	uuid2nodename(cl_uuid_t* uuid);
+int		nodename2uuid(const char* nodename, cl_uuid_t*);
+int		inittable(void);
+void		update_tables(const char* nodename, cl_uuid_t* uuid);
+struct node_info* lookup_tables(const char* nodename, cl_uuid_t* uuid);
+void		cleanuptable(void);
+int		GetUUID(cl_uuid_t* uuid);
 
 #ifndef HA_HAVE_SETENV
 int setenv(const char *name, const char * value, int why);
