@@ -52,8 +52,8 @@
  * "rc" is the return code of an opertioan. it's value is in following enum 
  * which is defined in "raexec.h"
   * enum UNIFORM_RET_EXECRA {
- *	EXECRA_EXEC_UNKNOWN_ERROR = -2,
- *	EXECRA_NO_RA = -1,
+ *	EXECRA_EXEC_UNKNOWN_ERROR = 152,
+ *	EXECRA_NO_RA = 151,
  *	EXECRA_OK = 0,
  *	EXECRA_UNKNOWN_ERROR = 1,
  *	EXECRA_INVALID_PARAM = 2,
@@ -73,6 +73,8 @@
 #ifndef __LRM_API_H
 #define __LRM_API_H 1
 
+#include <portability.h>
+
 #include <glib.h>
 #include <lrm/raexec.h>
 #include <clplumbing/GSource.h>
@@ -183,11 +185,8 @@ struct rsc_ops
  *
  *callid:	the call id returned by perform_op()
  *
- *return:	HA_OK for success, HA_FAIL for failure op not found
- *				or other failure
- *			NB: the client always gets a notification over callback
- *				even for operations which were idle (of course, if
- *				the request has been accepted for processing)
+ *return:	if the operation has been stopped then return HA_OK
+ *		else return HA_FAIL
  */
 	int (*cancel_op) (lrm_rsc_t*, int call_id);
 
@@ -196,8 +195,6 @@ struct rsc_ops
  *		and return them as cancelled.
  *
  *return:	HA_OK for success, HA_FAIL for failure
- *		NB: op is not flushed unless it is idle;
- *       	in that case this call will block
  */
 	int (*flush_ops) (lrm_rsc_t*);
 
@@ -256,15 +253,6 @@ struct lrm_ops
 	int		(*set_lrm_callback) (ll_lrm_t*,
 			lrm_op_done_callback_t op_done_callback_func);
 
-/*
-	int		(*set_parameters)(ll_lrm_t*, const GHashTable* option);
-
-	GHashTable*     (*get_all_parameters)(ll_lrm_t*);
-
-	char * 		(*get_parameter)(ll_lrm_t *, const char * paramname);
-
-	char *		(*get_parameter_description)(ll_lrm_t*);
-*/
 
 /*
  *get_rsc_class_supported:
@@ -366,8 +354,6 @@ struct lrm_ops
  *delete_rsc:	delete the resource by the rsc_id
  *
  *return:	HA_OK for success, HA_FAIL for failure
- *		NB: resource removal is delayed until all operations are
- *		removed; the client, however, gets the reply immediately
  */
 	int	(*delete_rsc)(ll_lrm_t*, const char* rsc_id);
 
