@@ -1,3 +1,4 @@
+/* $Id: cl_log.c,v 1.64 2005/08/16 15:08:20 gshi Exp $ */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,14 +15,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <syslog.h>
 #include <time.h>
-#include <sys/utsname.h>
 #include <clplumbing/ipc.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/loggingdaemon.h>
@@ -36,8 +36,6 @@
 #include <sys/stat.h>
 #include <clplumbing/GSource.h>
 #include <clplumbing/cl_misc.h>
-#include <clplumbing/cl_syslog.h>
-#include <ha_msg.h>
 
 #ifndef MAXLINE
 #	define MAXLINE	512
@@ -56,16 +54,14 @@
 #  endif 
 #endif
 
+#define	cl_malloc	malloc
+#define	cl_free		free
 #define	DFLT_ENTITY	"cluster"
 #define NULLTIME 	0
 #define QUEUE_SATURATION_FUZZ 10
 
+static char	log_entity[MAXENTITY];
 static IPC_Channel*	logging_daemon_chan = NULL;
-static gboolean		syslogformatfile = FALSE;
-/*
- * If true, then output messages more or less like this...
- * Jul 14 21:45:18 beam logd: [1056]: info: setting log file to /dev/null
- */
 
 int LogToDaemon(int priority, const char * buf, int bstrlen, gboolean use_pri_str);
 
@@ -75,22 +71,19 @@ IPC_Message* ChildLogIPCMessage(int priority, const char *buf, int bstrlen,
 void	FreeChildLogIPCMessage(IPC_Message* msg);
 gboolean send_dropped_message(gboolean use_pri_str, IPC_Channel *chan);
 
-const char * prio2str(int priority);
 static int		use_logging_daemon =  FALSE;
 static int		conn_logd_time = 0;
-static char		cl_log_entity[MAXENTITY]= DFLT_ENTITY;
-static char		common_log_entity[MAXENTITY]= DFLT_ENTITY;
 static int		cl_log_facility = LOG_USER;
+static const char *	cl_log_entity = DFLT_ENTITY;
 
 static void		cl_opensyslog(void);
 static int		syslog_enabled = 0;
 static int		stderr_enabled = 0;
 static const char*	logfile_name = NULL;
 static const char*	debugfile_name = NULL;
-static int		cl_process_pid = -1;
-int			debug_level = 0;
-static GDestroyNotify	destroy_logging_channel_callback;
-static void		(*create_logging_channel_callback)(IPC_Channel* chan);
+static int cl_process_pid = -1;
+static GDestroyNotify destroy_logging_channel_callback;
+static void (*create_logging_channel_callback)(IPC_Channel* chan);
 static gboolean		logging_chan_in_main_loop = FALSE;
 
 /***********************
@@ -103,18 +96,6 @@ IPC_Channel* get_log_chan(void){
 }
 /*************************/
 
-/**************************
- * check if the fd is in use for logging
- **************************/
-int
-cl_log_is_logd_fd(int fd)
-{
-	return logging_daemon_chan && (
-		fd == logging_daemon_chan->ops->get_send_select_fd(logging_daemon_chan)
-		||
-		fd == logging_daemon_chan->ops->get_recv_select_fd(logging_daemon_chan)
-		);
-}
 
 void
 cl_log_enable_stderr(int truefalse)
@@ -127,11 +108,6 @@ cl_log_set_uselogd(int truefalse)
 {
 	use_logging_daemon = truefalse;
 }
-void
-cl_log_enable_syslog_filefmt(int truefalse)
-{
-	syslogformatfile = (gboolean)truefalse;
-}
 
 gboolean
 cl_log_get_uselogd(void)
@@ -154,102 +130,31 @@ cl_log_set_logdtime(int logdtime)
 	return;
 }
 
-
-#define ENVPRE		"HA_"
-
-#define ENV_HADEBUGVAL	"HA_debug"
-#define ENV_LOGFENV	"HA_logfile"	/* well-formed log file :-) */
-#define ENV_DEBUGFENV	"HA_debugfile"	/* Debug log file */
-#define ENV_LOGFACILITY	"HA_logfacility"/* Facility to use for logger */
-#define ENV_SYSLOGFMT	"HA_syslogmsgfmt"/* TRUE if we should use syslog message formatting */
-#define ENV_LOGDAEMON	"HA_use_logd"
-#define	ENV_CONNINTVAL	"HA_conn_logd_time"
-#define TRADITIONAL_COMPRESSION "HA_traditional_compression"
-#define COMPRESSION	 "HA_compression"
-
-void
-inherit_compress(void)
+gboolean
+cl_inherit_use_logd(const char* param_name, int sendq_length) 
 {
-	char* inherit_env = NULL;
+	char*		param_val;
+	gboolean	truefalse = FALSE;
 	
-	inherit_env = getenv(TRADITIONAL_COMPRESSION);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		gboolean value;
-		
-		if (cl_str_to_boolean(inherit_env, &value)!= HA_OK){
-			cl_log(LOG_ERR, "inherit traditional_compression failed");
-		}else{
-			cl_set_traditional_compression(value);
-		}
+	param_val = getenv(param_name);
+	
+	if(param_val != NULL) {
+		cl_str_to_boolean(param_val, &truefalse);
+		cl_log_set_uselogd(truefalse) ;
 	}
 	
-}
-
-void
-cl_inherit_logging_environment(int logqueuemax)
-{
-	char * inherit_env = NULL;
-
-	/* Donnot need to free the return pointer from getenv */
-	inherit_env = getenv(ENV_HADEBUGVAL);
-	if (inherit_env != NULL && atoi(inherit_env) != 0 ) {
-		debug_level = atoi(inherit_env);
-		inherit_env = NULL;
-	}
-
-	inherit_env = getenv(ENV_LOGFENV);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		cl_log_set_logfile(inherit_env);
-		inherit_env = NULL;
-	}
-
-	inherit_env = getenv(ENV_DEBUGFENV);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		cl_log_set_debugfile(inherit_env);
-		inherit_env = NULL;
-	}
-
-	inherit_env = getenv(ENV_LOGFACILITY);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		int facility = -1;
-		facility = cl_syslogfac_str2int(inherit_env);
-		if ( facility >= 0 ) {
-			cl_log_set_facility(facility);
+	if (truefalse){
+		if (sendq_length > 0){
+			cl_set_logging_wqueue_maxlen(sendq_length);
 		}
-		inherit_env = NULL;
+		cl_log(LOG_INFO, "Enable using logging daemon");
+	}else {
+		cl_log(LOG_INFO, "Disable using logging daemon");
 	}
-
-	inherit_env = getenv(ENV_SYSLOGFMT);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		gboolean truefalse;
-		if (cl_str_to_boolean(inherit_env, &truefalse) == HA_OK) {
-			cl_log_enable_syslog_filefmt(truefalse);
-		}
-	}
-
-	inherit_env = getenv(ENV_LOGDAEMON);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		gboolean	uselogd;
-		cl_str_to_boolean(inherit_env, &uselogd);
-		cl_log_set_uselogd(uselogd);
-		if (uselogd) {
-			if (logqueuemax > 0) {
-				cl_set_logging_wqueue_maxlen(logqueuemax);
-			}
-		}
-	}
-
-	inherit_env = getenv(ENV_CONNINTVAL);
-	if (inherit_env != NULL && *inherit_env != EOS) {
-		int logdtime;
-		logdtime = cl_get_msec(inherit_env);
-		cl_log_set_logdtime(logdtime);
-	}
-
-	inherit_compress();
-	return;
-}
-
+	
+	return truefalse;
+	
+}     
 
 static void
 add_logging_channel_mainloop(IPC_Channel* chan)
@@ -360,8 +265,6 @@ cl_log_test_logd(void)
 	
 }
 
-/* FIXME: This is way too ugly to bear */
-
 void
 cl_log_set_facility(int facility)
 {
@@ -382,8 +285,7 @@ cl_log_set_entity(const char *	entity)
 	if (entity == NULL) {
 		entity = DFLT_ENTITY;
 	}
-	strncpy(cl_log_entity, entity, MAXENTITY);
-	cl_log_entity[MAXENTITY-1] = '\0';
+	cl_log_entity = cl_strdup(entity);
 	if (syslog_enabled) {
 		syslog_enabled = 0;
 		cl_opensyslog();
@@ -393,26 +295,20 @@ cl_log_set_entity(const char *	entity)
 void
 cl_log_set_logfile(const char *	path)
 {
-    if(path != NULL && strcasecmp("/dev/null", path) == 0) {
-	path = NULL;
-    }
 	logfile_name = path;
 }
 void
 cl_log_set_debugfile(const char * path)
 {
-    if(path != NULL && strcasecmp("/dev/null", path) == 0) {
-	path = NULL;
-    }
-    debugfile_name = path;
+	debugfile_name = path;
 }
 
 
-/* 
- * This function sets two callback functions.
+/* This function set two callback functions.
  * One for creating a channel and 
  * the other for destroying a channel*
  */
+
 int
 cl_log_set_logd_channel_source( void (*create_callback)(IPC_Channel* chan),
 				GDestroyNotify destroy_callback)
@@ -421,13 +317,13 @@ cl_log_set_logd_channel_source( void (*create_callback)(IPC_Channel* chan),
 	
 	if (destroy_callback == NULL){
 		destroy_logging_channel_callback = remove_logging_channel_mainloop;
-	}else{		
+	}else {		
 		destroy_logging_channel_callback = destroy_callback;
 	}
 	
 	if (create_callback == NULL){
 		create_logging_channel_callback = add_logging_channel_mainloop;	
-	}else{
+	}else {
 		create_logging_channel_callback = create_callback;	
 	}
 	
@@ -439,71 +335,6 @@ cl_log_set_logd_channel_source( void (*create_callback)(IPC_Channel* chan),
 	return 0;
 }
 
-const char *
-prio2str(int priority)
-{
-	static const char *log_prio[8] = {
-		"EMERG",
-		"ALERT",
-		"CRIT",
-		"ERROR",
-		"WARN",
-		"notice",
-		"info",
-		"debug"
-	};
-	int		logpri;
-
-	logpri =  LOG_PRI(priority);
-
-	return (logpri < 0 || logpri >= DIMOF(log_prio)) ?
-		"(undef)" : log_prio[logpri];
-}
-
-/* print log line to a FILE *f */
-#define print_logline(fp,entity,entity_pid,ts,pristr,buf) { \
-			fprintf(fp, "%s[%d]: %s ",entity,entity_pid,ha_timestamp(ts)); \
-			if (pristr) \
-				fprintf(fp,"%s: %s\n",pristr,buf); \
-			else \
-				fprintf(fp,"%s\n",buf); \
-		}
-
-static char * syslog_timestamp(TIME_T t);
-
-static void
-append_log( const char * fname, const char * entity, int entity_pid
-,	TIME_T timestamp, const char * pristr, const char * msg)
-{
-	FILE *			fp = fopen(fname,"a");
-	static int		got_uname = FALSE;
-	static struct utsname	un;
-
-	if (!fp) {
-		syslog(LOG_ERR, "Cannot append to %s: %s", fname
-		,	strerror(errno)); 
-		return;
-	}
-	if (!syslogformatfile) {
-		print_logline(fp, entity, entity_pid, timestamp, pristr, msg);
-		fclose(fp);
-		return;
-	}
-	if (!got_uname) {
-		uname(&un);
-	}
-	/*
-	 * Jul 14 21:45:18 beam logd: [1056]: info: setting log file to /dev/null
-	 */
-	fprintf(fp, "%s %s %s: [%d]: %s%s%s\n"
-	,	syslog_timestamp(timestamp)
-	,	un.nodename, entity, entity_pid
-	,	(pristr ? pristr : "")
-	,	(pristr ? ": " : "")
-	,	msg);
-	fclose(fp);
-}
-
 /*
  * This function can cost us realtime unless use_logging_daemon
  * is enabled.  Then we log everything through a child process using
@@ -515,29 +346,49 @@ void
 cl_direct_log(int priority, const char* buf, gboolean use_priority_str,
 	      const char* entity, int entity_pid, TIME_T ts)
 {
+	FILE *		fp = NULL;
+	int		logpri;
 	const char *	pristr;
 	int	needprivs = !cl_have_full_privs();
 
+	static const char *log_prio[8] = {
+		"EMERG",
+		"ALERT",
+		"CRIT",
+		"ERROR",
+		"WARN",
+		"notice",
+		"info",
+		"debug"
+	};
+	
 	if (entity == NULL){
 		entity =cl_log_entity;
 	}
 	
-	pristr = use_priority_str ? prio2str(priority) : NULL;
-
+	if (use_priority_str){
+		logpri =  LOG_PRI(priority);
+		
+		if (logpri < 0 || logpri >= DIMOF(log_prio)) {
+			pristr = "(undef)";
+		}else{
+			pristr = log_prio[logpri];
+		}
+	}else{
+		pristr = NULL;
+	}
+	
 	if (needprivs) {
 		return_to_orig_privs();
 	}
 	
 	if (syslog_enabled) {
-		if (entity) {
-			strncpy(common_log_entity, entity, MAXENTITY);
-		} else {
-			strncpy(common_log_entity, DFLT_ENTITY,MAXENTITY);
+		if(entity){
+			strncpy(log_entity, entity, MAXENTITY);
+		}else{
+			strncpy(log_entity, DFLT_ENTITY,MAXENTITY);
 		}
-
-		common_log_entity[MAXENTITY-1] = '\0';
-
-		if (pristr) {
+		if (pristr){
 			syslog(priority, "[%d]: %s: %s%c",
 			       entity_pid, pristr,  buf, 0);
 		}else {
@@ -545,14 +396,80 @@ cl_direct_log(int priority, const char* buf, gboolean use_priority_str,
 		}
 	}
 
+	if (stderr_enabled) {
+		if (pristr){
+			fprintf(stderr, "%s[%d]: %s %s: %s\n"
+				,	(entity ? entity : DFLT_ENTITY)
+				,       entity_pid
+				,	ha_timestamp(ts)
+				,	pristr,  buf);
+		}else {
+			fprintf(stderr, "%s[%d]: %s %s\n"
+				,	(entity ? entity : DFLT_ENTITY)
+				,       entity_pid
+				,	ha_timestamp(ts)
+				,	buf);
+			
+		}
+		
+	}
+	
 	if (debugfile_name != NULL) {
-		append_log(debugfile_name,entity,entity_pid,ts,pristr,buf);
-	}
 
-	if (priority != LOG_DEBUG && logfile_name != NULL) {
-		append_log(logfile_name,entity,entity_pid,ts,pristr,buf);
-	}
+		fp = fopen(debugfile_name, "a");
+		if (fp != NULL) {
+			if (pristr){
+				fprintf(fp, "%s[%d]: %s %s: %s\n"
+					,	(entity ? entity : DFLT_ENTITY)
+					,       entity_pid
+					,	ha_timestamp(ts)
+					,	pristr,  buf);
+			}else{
+				fprintf(fp, "%s[%d]: %s %s\n"
+					,	(entity ? entity : DFLT_ENTITY)
+					,       entity_pid
+					,	ha_timestamp(ts)
+					,	buf);
+				
+			}
+			
+			fclose(fp);
+		} else {
+			fprintf(stderr, "Cannot open %s: %s\n",
+				debugfile_name, strerror(errno));
+			syslog(LOG_ERR, "Cannot open %s: %s\n",
+				debugfile_name, strerror(errno));
+		}
 
+	}
+	
+	if (priority != LOG_DEBUG && logfile_name != NULL) { 
+		
+		fp = fopen(logfile_name, "a");
+		if (fp != NULL) {
+			if (pristr){
+				fprintf(fp, "%s[%d]: %s %s: %s\n"
+				,	(entity ? entity : DFLT_ENTITY)
+				,       entity_pid
+				,	ha_timestamp(ts)
+				,	pristr,  buf);
+			}else {
+				fprintf(fp, "%s[%d]: %s %s\n"
+					,	(entity ? entity : DFLT_ENTITY)
+					,       entity_pid
+					,	ha_timestamp(ts)
+					,	buf);	
+			}
+			
+				fclose(fp);
+		} else {
+			fprintf(stderr, "Cannot open %s: %s\n",
+				logfile_name, strerror(errno));
+			syslog(LOG_ERR, "Cannot open %s: %s\n",
+				logfile_name, strerror(errno));
+		}
+	}
+	
 	if (needprivs) {
 		return_to_dropped_privs();
 	}
@@ -577,7 +494,21 @@ cl_log(int priority, const char * fmt, ...)
 {
 	va_list		ap;
 	char		buf[MAXLINE];
+	int		logpri = LOG_PRI(priority);
 	ssize_t		nbytes;
+	const char *	pristr;
+	int	needprivs = !cl_have_full_privs();
+
+	static const char *log_prio[8] = {
+		"EMERG",
+		"ALERT",
+		"CRIT",
+		"ERROR",
+		"WARN",
+		"notice",
+		"info",
+		"debug"
+	};
 
 	cl_process_pid = (int)getpid();
 
@@ -591,15 +522,20 @@ cl_log(int priority, const char * fmt, ...)
 	if (nbytes >= (ssize_t)sizeof(buf)){
 		nbytes =  sizeof(buf) -1 ;
 	}
-
-	if (stderr_enabled) {
-		print_logline(stderr, cl_log_entity,cl_process_pid,
-			NULLTIME, prio2str(priority), buf);
+	
+	if (logpri < 0 || logpri >= DIMOF(log_prio)) {
+		pristr = "(undef)";
+	}else{
+		pristr = log_prio[logpri];
 	}
 
-	if (use_logging_daemon && cl_log_depth <= 1) {
-		LogToLoggingDaemon(priority, buf, nbytes, TRUE);
-	}else{
+	if (needprivs) {
+		return_to_orig_privs();
+	}
+	
+	if ( use_logging_daemon && cl_log_depth <= 1) {
+		LogToLoggingDaemon(priority, buf, nbytes + 1, TRUE);
+	}else {
 		/* this may cause blocking... maybe should make it optional? */ 
 		cl_direct_log(priority, buf, TRUE, NULL, cl_process_pid, NULLTIME);
 	}
@@ -645,38 +581,6 @@ cl_glib_msg_handler(const gchar *log_domain,	GLogLevelFlags log_level
 
 	cl_log(ha_level, "glib: %s", message);
 }
-static char *
-syslog_timestamp(TIME_T t)
-{
-	static char		ts[64];
-	struct tm*		ttm;
-	TIME_T			now;
-	time_t			nowtt;
-	static const char*	monthstr [12] = {
-		"Jan", "Feb", "Mar",
-		"Apr", "May", "Jun",
-		"Jul", "Aug", "Sep",
-		"Oct", "Nov", "Dec"
-	};
-	
-	/* Work around various weridnesses in different OSes and time_t definitions */
-	if (t == 0){
-		now = time(NULL);
-	}else{
-		now = t;
-	}
-
-	nowtt = (time_t)now;
-	ttm = localtime(&nowtt);
-
-	snprintf(ts, sizeof(ts), "%3s %02d %02d:%02d:%02d"
-	,	monthstr[ttm->tm_mon], ttm->tm_mday
-	,	ttm->tm_hour, ttm->tm_min, ttm->tm_sec);
-	return(ts);
-}
-
-
-
 char *
 ha_timestamp(TIME_T t)
 {
@@ -686,7 +590,7 @@ ha_timestamp(TIME_T t)
 	time_t		nowtt;
 	
 	/* Work around various weridnesses in different OSes and time_t definitions */
-	if (t == 0){
+	if(t == 0){
 		now = time(NULL);
 	}else{
 		now = t;
@@ -739,8 +643,8 @@ cl_set_logging_wqueue_maxlen(int qlen)
 
 int
 LogToDaemon(int priority, const char * buf, 
-	    int bufstrlen, gboolean use_pri_str)
-{
+	    int bufstrlen, gboolean use_pri_str){
+
 	int rc;
 	
 	cl_log_depth++;
@@ -754,15 +658,6 @@ LogToDaemon(int priority, const char * buf,
 
 static int		drop_msg_num = 0;
 
-void
-cl_flush_logs(void) 
-{
-	if(logging_daemon_chan == NULL) {
-		return;
-	}
-	logging_daemon_chan->ops->waitout(logging_daemon_chan);
-}
-
 int
 LogToLoggingDaemon(int priority, const char * buf, 
 		   int bufstrlen, gboolean use_pri_str)
@@ -789,7 +684,7 @@ LogToLoggingDaemon(int priority, const char * buf,
 			priority, buf, TRUE, NULL, cl_process_pid, NULLTIME);
 		return HA_FAIL;
 	}
-
+	
 	msg = ChildLogIPCMessage(priority, buf, bufstrlen, use_pri_str, chan);	
 	if (msg == NULL) {
 		drop_msg_num++;
@@ -828,8 +723,8 @@ LogToLoggingDaemon(int priority, const char * buf,
 				chan->ops->destroy(chan);
 			}
 			logging_daemon_chan = NULL;
-			cl_direct_log(priority, buf, TRUE, NULL, cl_process_pid, NULLTIME);
-
+			cl_direct_log(priority, buf, TRUE, NULL, cl_process_pid, NULLTIME);			
+			
 			if (drop_msg_num > 0){
 				/* Direct logging here is ok since we're
 				 *    switching to that for everything
@@ -927,14 +822,14 @@ ChildLogIPCMessage(int priority, const char *buf, int bufstrlen,
 	memset(ret, 0, sizeof(IPC_Message));
 	
 	/* Compute msg len: including room for the EOS byte */
-	msglen = sizeof(LogDaemonMsg)+bufstrlen + 1;
-	bodybuf = cl_malloc(msglen + ch->msgpad);
+	msglen = sizeof(LogDaemonMsg)+bufstrlen;
+	bodybuf = cl_malloc(msglen + ch->msgpad + 1);
 	if (bodybuf == NULL) {
 		cl_free(ret);
 		return NULL;
 	}
 	
-	memset(bodybuf, 0, msglen + ch->msgpad);
+	memset(bodybuf, 0, msglen + ch->msgpad + 1);
 	memset(&logbuf, 0, sizeof(logbuf));
 	logbuf.msgtype = LD_LOGIT;
 	logbuf.facility = cl_log_facility;
@@ -942,7 +837,7 @@ ChildLogIPCMessage(int priority, const char *buf, int bufstrlen,
 	logbuf.use_pri_str = use_prio_str;
 	logbuf.entity_pid = getpid();
 	logbuf.timestamp = time(NULL);
-	if (*cl_log_entity){
+	if (cl_log_entity){
 		strncpy(logbuf.entity,cl_log_entity,MAXENTITY);
 	}else {
 		strncpy(logbuf.entity,DFLT_ENTITY,MAXENTITY);
@@ -989,12 +884,12 @@ FreeChildLogIPCMessage(IPC_Message* msg)
 static void
 cl_opensyslog(void)
 {
-	if (*cl_log_entity == '\0' || cl_log_facility < 0) {
+	if (cl_log_entity == NULL || cl_log_facility < 0) {
 		return;
 	}
 	syslog_enabled = 1;
-	strncpy(common_log_entity, cl_log_entity, MAXENTITY);
-	openlog(common_log_entity, LOG_CONS, cl_log_facility);
+	openlog(log_entity, LOG_CONS, cl_log_facility);
+
 }
 
 /* What a horrible substitute for a low-overhead event log!! - FIXME!! */
@@ -1110,33 +1005,3 @@ DumpCircularBuffer(int nsig, gpointer user_data)
 	cl_log(LOG_INFO, "Mark: End dump of buffer %s", buffer->name);
 	return TRUE;
 }
-
-
-void
-cl_log_args(int argc, char **argv)
-{
-	int lpc = 0;
-	int len = 0;
-	int existing_len = 0;
-	char *arg_string = NULL;
-
-	if(argc == 0 || argv == NULL) {
-	    return;
-	}
-	
-	for(;lpc < argc; lpc++) {
-		if(argv[lpc] == NULL) {
-			break;
-		}
-		
-		len = 2 + strlen(argv[lpc]); /* +1 space, +1 EOS */
-		if(arg_string) {
-			existing_len = strlen(arg_string);
-		}
-
-		arg_string = cl_realloc(arg_string, len + existing_len);
-		sprintf(arg_string + existing_len, "%s ", argv[lpc]);
-	}
-	cl_log(LOG_INFO, "Invoked: %s", arg_string);
-	cl_free(arg_string);
-}
