@@ -19,7 +19,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <heartbeat.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -33,7 +33,6 @@
 #include <clplumbing/netstring.h>
 #include <clplumbing/base64.h>
 #include <assert.h>
-#include <ctype.h>
 
 /*
  * Avoid sprintf.  Use snprintf instead, even if you count your bytes.
@@ -49,7 +48,7 @@ int compose_netstring(char*, const char*, const char*, size_t, size_t*);
 int is_auth_netstring(const char*, size_t, const char*, size_t);
 char* msg2netstring(const struct ha_msg*, size_t*);
 int process_netstring_nvpair(struct ha_msg* m, const char* nvpair, int nvlen);
-extern int	bytes_for_int(int x);
+
 extern const char *	FT_strings[];
 
 static int (*authmethod)(int whichauth
@@ -68,31 +67,6 @@ cl_set_authentication_computation_method(int (*method)(int whichauth
 	authmethod = method;
 }
 
-int cl_parse_int(const char *sp, const char *smax, int* len);
-
-int
-cl_parse_int(const char *sp, const char *smax, int* len) 
-{
-	char ch = 0;
-	int offset = 0;
-	*len = 0;
-
-	errno = 0;
-	for( ; sp+offset < smax; offset++) {
-		ch = sp[offset] - '0';
-		if(ch > 9) { /* ch >= 0 is implied by the data type*/
-			break;
-		}
-		*len *= 10;
-		*len += ch;
-	}
-	
-	if(offset == 0) {
-		cl_log(LOG_ERR,
-		       "cl_parse_int: Couldn't parse an int from: %.5s", sp);
-	} 
-	return offset;
-}
 
 int
 compose_netstring(char * s, const char * smax, const char* data,
@@ -101,10 +75,10 @@ compose_netstring(char * s, const char * smax, const char* data,
 
 	char *	sp = s;
 
-	/* 2 == ":" + "," */
-	if (s + len + 2 + bytes_for_int(len) > smax) {
-		cl_log(LOG_ERR,
-		       "netstring pointer out of boundary(compose_netstring)");
+	/* 3 == ":" + "," + at least one digit number */
+	if (s + len + 3 > smax) {
+		cl_log(LOG_ERR
+		,	"netstring pointer out of boundary(compose_netstring)");
 		return(HA_FAIL);
 	}
 
@@ -143,9 +117,8 @@ msg2netstring_buf(const struct ha_msg *m, char *s,
 
 	for (i=0; i < m->nfields; i++) {
 		size_t flen;
-		int	tmplen;
 		
-		/* some of these functions in its turn invoke us again */
+		
 		ret = fieldtypefuncs[m->types[i]].tonetstring(sp, 
 							      smax,
 							      m->names[i],
@@ -161,23 +134,11 @@ msg2netstring_buf(const struct ha_msg *m, char *s,
 			return ret;
 		}
 		
-		tmplen = netstring_extra(fieldtypefuncs[m->types[i]].netstringlen(m->nlens[i],
-										  m->vlens[i],
-										  m->values[i]));
-		
-		if (flen != tmplen ){
-			cl_log(LOG_ERR,"netstring len discrepency: actual usage is %d bytes"
-			       "it should use %d", (int)flen, tmplen);			       
-		}
 		sp +=flen;
 		
 	}
 	
-	if (sp + strlen(MSG_END_NETSTRING) > smax){
-		cl_log(LOG_ERR, "%s: out of boundary for MSG_END_NETSTRING",
-		       __FUNCTION__);
-		return HA_FAIL;
-	}
+	
 	strcpy(sp, MSG_END_NETSTRING);
 	sp += sizeof(MSG_END_NETSTRING) -1;
 	
@@ -209,59 +170,46 @@ msg2netstring_ll(const struct ha_msg *m, size_t * slen, int need_auth)
 	int	len;
 	char*	s;
 	int	authnum;
-	char	authtoken[MAXLINE];
-	char	authstring[MAXLINE];
+	char	authtoken[MAXMSG];
+	char	authstring[MAXMSG];
 	char*	sp;
 	size_t	payload_len;
-	char*   smax;
 
 	len= get_netstringlen_auth(m) + 1;
 	
-	/* use MAXUNCOMPRESSED for the in memory size check */
-	if (len >= MAXUNCOMPRESSED){
-		cl_log(LOG_ERR, "%s: msg is too large; len=%d,"
-		       " MAX msg allowed=%d", __FUNCTION__, len, MAXUNCOMPRESSED);
+	if (len >= MAXMSG){
+		cl_log(LOG_ERR, "msg2netstring: msg is too large"
+		       "len =%d,MAX msg allowed=%d", len, MAXMSG);
 		return NULL;
 	}
 
-	s = cl_calloc(1, len);
+	s = ha_calloc(1, len);
 	if (!s){
-		cl_log(LOG_ERR, "%s: no memory for netstring", __FUNCTION__);
+		cl_log(LOG_ERR, "msg2netstring: no memory for netstring");
 		return(NULL);
 	}
 
-	smax = s + len;
-
 	if (msg2netstring_buf(m, s, len, &payload_len) != HA_OK){
-		cl_log(LOG_ERR, "%s:  msg2netstring_buf() failed", __FUNCTION__);
-		cl_free(s);
+		cl_log(LOG_ERR, "msg2netstring: msg2netstring_buf() failed");
+		ha_free(s);
 		return(NULL);
 	}
 	
 	sp = s + payload_len;
 	
 	if ( need_auth && authmethod){
-		int auth_strlen;
-
 		authnum = authmethod(-1, s, payload_len, authtoken,sizeof(authtoken));
 		if (authnum < 0){
 			cl_log(LOG_WARNING
 			       ,	"Cannot compute message authentication!");
-			cl_free(s);
 			return(NULL);
 		}
 		
 		sprintf(authstring, "%d %s", authnum, authtoken);
-		auth_strlen = strlen(authstring);
-		if (sp  + 2 + auth_strlen + bytes_for_int(auth_strlen)  >= smax){
-			cl_log(LOG_ERR, "%s: out of boundary for auth", __FUNCTION__);
-			cl_free(s);
-			return NULL;
-		}
 		sp += sprintf(sp, "%ld:%s,", (long)strlen(authstring), authstring);	
 		
 	}
-	*slen = sp - s;
+	*slen = sp - s + 1;
 
 	return(s);
 }
@@ -269,20 +217,12 @@ msg2netstring_ll(const struct ha_msg *m, size_t * slen, int need_auth)
 char *
 msg2netstring(const struct ha_msg *m, size_t * slen)
 {
-	char* ret;
-	ret = msg2netstring_ll(m, slen, TRUE);
-	
-	return ret;
-	
+	return msg2netstring_ll(m, slen, TRUE);
 }
 char *
 msg2netstring_noauth(const struct ha_msg *m, size_t * slen)
 {
-	char * ret;
-	
-	ret =  msg2netstring_ll(m, slen, FALSE);
-	
-	return ret;
+	return msg2netstring_ll(m, slen, FALSE);
 }
 
 
@@ -294,25 +234,25 @@ static int
 peel_netstring(const char * s, const char * smax, int* len,
 	       const char ** data, int* parselen )
 {
-	int offset = 0;
 	const char *	sp = s;
 
 	if (sp >= smax){
 		return(HA_FAIL);
 	}
 
-	offset = cl_parse_int(sp, smax, len);
-	if (*len < 0 || offset <= 0){
-		cl_log(LOG_ERR, "peel_netstring: Couldn't parse an int starting at: %.5s", sp);
+	if (sscanf(sp,"%d", len) != 1){
+		return(HA_FAIL);
+	}
+
+	if (*len < 0){
 		return(HA_FAIL);
 	}
 
-	sp = sp+offset;
 	while (*sp != ':' && sp < smax) {
 		sp ++;
 	}
 
-	if (sp >= smax) {
+	if (sp >= smax ){
 		return(HA_FAIL);
 	}
 
@@ -321,9 +261,6 @@ peel_netstring(const char * s, const char * smax, int* len,
 	*data = sp;
 	
 	sp += (*len);
-	if (sp >= smax) {
-		return(HA_FAIL);
-	}
 	if (*sp != ','){
 		return(HA_FAIL);
 	}
@@ -351,18 +288,17 @@ process_netstring_nvpair(struct ha_msg* m, const char* nvpair, int nvlen)
 
 	assert(*nvpair == '(');
 	nvpair++;
-
-	type = nvpair[0] - '0';
+	if (sscanf(nvpair, "%d", &type) != 1){
+		cl_log(LOG_ERR, " sscaning for type failed in %s", 
+		       __FUNCTION__);
+		return HA_FAIL;
+	}
+	
 	nvpair++;
-
-	/* if this condition is no longer true, change the above to:
-	 *   nvpair += cl_parse_int(nvpair, nvpair+nvlen, &type)
-	 */
-	assert(type >= 0 && type < 10);
-
 	assert(*nvpair == ')');
 	nvpair++;
 	
+	
 	if ((nlen = strcspn(nvpair, EQUAL)) <= 0
 	    ||	nvpair[nlen] != '=') {
 		if (!cl_msg_quiet_fmterr) {
@@ -479,7 +415,7 @@ netstring2msg(const char* s, size_t length, int needauth)
 	
 	msg = netstring2msg_rec(s, length, &slen);
 	
-	if (needauth == FALSE || !authmethod){
+	if (needauth == FALSE){
 		goto out;
 	} 
 	
@@ -490,6 +426,7 @@ netstring2msg(const char* s, size_t length, int needauth)
 		       "peel_netstring() error in getting auth string");		
 		cl_log(LOG_ERR, "sp=%s", sp);
 		cl_log(LOG_ERR, "s=%s", s);
+		
 		ha_msg_del(msg);
 		return(NULL);
 	}
@@ -524,9 +461,9 @@ is_auth_netstring(const char * datap, size_t datalen,
 		  const char * authstring, size_t authlen)
 {
 
-	char	authstr[MAXLINE];	/* A copy of authstring */
+	char	authstr[MAXMSG];	/* A copy of authstring */
 	int	authwhich;
-	char	authtoken[MAXLINE];
+	char	authtoken[MAXMSG];
 
 
 	/*
@@ -535,7 +472,7 @@ is_auth_netstring(const char * datap, size_t datalen,
 	if (!authmethod) {
 		return TRUE;
 	}
-	strncpy(authstr, authstring, MAXLINE);
+	strncpy(authstr, authstring, MAXMSG);
 	authstr[authlen] = 0;
 	if (sscanf(authstr, "%d %s", &authwhich, authtoken) != 2) {
 		if (!cl_msg_quiet_fmterr) {
@@ -545,8 +482,8 @@ is_auth_netstring(const char * datap, size_t datalen,
 		return(0);
 	}
 
-	memset(authstr, 0, MAXLINE);
-	if (authmethod(authwhich, datap, datalen, authstr, MAXLINE)
+	memset(authstr, 0, MAXMSG);
+	if (authmethod(authwhich, datap, datalen, authstr, MAXMSG)
 	    !=	authwhich) {
 	  
 		if (!cl_msg_quiet_fmterr) {
