@@ -1,12 +1,9 @@
+/* $Id: ipcsocket.c,v 1.160 2005/07/25 20:43:12 gshi Exp $ */
 /*
  * ipcsocket unix domain socket implementation of IPC abstraction.
  *
  * Copyright (c) 2002 Xiaoxiang Liu <xiliu@ncsa.uiuc.edu>
  *
- * Stream support (c) 2004,2006 David Lee <t.d.lee@durham.ac.uk>
- *	Note: many of the variable/function names "*socket*" should be
- *	interpreted as having a more generic "ipc-channel-type" meaning.
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -23,7 +20,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <clplumbing/ipc.h>
 #include <clplumbing/cl_log.h>
@@ -57,52 +54,10 @@
 #ifdef HAVE_SYS_UCRED_H
 #	include <sys/ucred.h>
 #endif
-
-/* For 'getpeerucred()' (Solaris 10 upwards) */
-#ifdef HAVE_UCRED_H
-#	include <ucred.h>
-#endif
-
-#ifdef HAVE_SYS_SOCKET_H
-# include <sys/socket.h>
-#endif
-
-/*
- * Normally use "socket" code.  But on some OSes alternatives may be
- * preferred (or necessary).
- */
-#define HB_IPC_SOCKET	1
-#define HB_IPC_STREAM	2
-/* #define HB_IPC_ANOTHER	3 */
-
-#ifndef HB_IPC_METHOD
-# if defined(SO_PEERCRED) || defined(HAVE_GETPEEREID) \
-	|| defined(SCM_CREDS) || defined(HAVE_GETPEERUCRED)
-#  define HB_IPC_METHOD	HB_IPC_SOCKET
-# elif defined(HAVE_STROPTS_H)
-#  define HB_IPC_METHOD	HB_IPC_STREAM
-# else
-#  error.  Surely we have sockets or streams...
-# endif
-#endif
-
-
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-
-# include <sys/poll.h>
-# include <netinet/in.h>
-# include <sys/un.h>
-
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-
-# include <stropts.h>
-
-#else
-
-# error "IPC type invalid"
-
-#endif
-
+#include <sys/socket.h>
+#include <sys/poll.h>
+#include <netinet/in.h>
+#include <sys/un.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -111,20 +66,19 @@
 #ifndef UNIX_PATH_MAX
 #	define UNIX_PATH_MAX 108
 #endif
+#define MAX_LISTEN_NUM 10
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
+#ifndef SUN_LEN
+#    define SUN_LEN(ptr) ((size_t) (offsetof (sockaddr_un, sun_path) + strlen ((ptr)->sun_path))
+#endif
 
-# define MAX_LISTEN_NUM 10
+#ifndef MSG_NOSIGNAL
+#define		MSG_NOSIGNAL	0
+#endif
 
-# ifndef MSG_NOSIGNAL
-#  define		MSG_NOSIGNAL	0
-# endif
-
-# ifndef AF_LOCAL
-#  define         AF_LOCAL AF_UNIX
-# endif
-
-#endif /* HB_IPC_METHOD */
+#ifndef AF_LOCAL
+#define         AF_LOCAL AF_UNIX
+#endif
 
 /***********************************************************************
  *
@@ -138,10 +92,6 @@
 #	define USE_GETPEEREID
 #elif defined(SCM_CREDS)
 #	define	USE_SCM_CREDS
-#elif HAVE_GETPEERUCRED		/* e.g. Solaris 10 upwards */
-#	define USE_GETPEERUCRED
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-#	define USE_STREAM_CREDS
 #else
 #	define	USE_DUMMY_CREDS
 /* This will make it compile, but attempts to authenticate
@@ -149,27 +99,12 @@
  */
 #endif
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-
-# ifdef USE_BINDSTAT_CREDS
-# ifndef SUN_LEN
-#    define SUN_LEN(ptr) ((size_t) (offsetof (sockaddr_un, sun_path) + strlen ((ptr)->sun_path))
-# endif
-# endif
-
-#endif /* HB_IPC_METHOD */
-
 /* wait connection private data. */
 struct SOCKET_WAIT_CONN_PRIVATE{
   /* the path name wich the connection will be built on. */
   char path_name[UNIX_PATH_MAX];
-#if HB_IPC_METHOD == HB_IPC_SOCKET
   /* the domain socket. */
   int s;
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-  /* the streams pipe */
-  int pipefds[2];
-#endif
 };
 
 /* channel private data. */
@@ -181,15 +116,10 @@ struct SOCKET_CH_PRIVATE{
   /* the size of expecting data for below buffered message buf_msg */
   int remaining_data;
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
   /* The address of our peer - used by USE_BINDSTAT_CREDS version of
    *   socket_verify_auth()
    */
   struct sockaddr_un *peer_addr;
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-  uid_t farside_uid;
-  gid_t farside_gid;
-#endif
 		
   /* the buf used to save unfinished message */
   struct IPC_MESSAGE *buf_msg;
@@ -210,7 +140,6 @@ struct IPC_Stats {
 };
 
 struct IPC_Stats	SocketIPCStats = {0,0,0,0};
-extern int	debug_level;
 
 /* unix domain socket implementations of IPC functions. */
 
@@ -224,25 +153,15 @@ static struct IPC_MESSAGE* socket_message_new(struct IPC_CHANNEL*ch
 
 struct IPC_WAIT_CONNECTION *socket_wait_conn_new(GHashTable* ch_attrs);
 
-/* *** FIXME: This is also declared in 'ocf_ipc.c'. */
 struct IPC_CHANNEL* socket_client_channel_new(GHashTable *attrs);
 
-static struct IPC_CHANNEL* socket_server_channel_new(int sockfd);
-
-static struct IPC_CHANNEL * channel_new(int sockfd, int conntype, const char *pathname);
-static int client_channel_new_auth(int sockfd);
-
-typedef void (*DelProc)(IPC_Message*);
-
-static struct IPC_MESSAGE * ipcmsg_new(struct IPC_CHANNEL* ch,
-  const void* data, int len, void* private, DelProc d);
+struct IPC_CHANNEL* socket_server_channel_new(int sockfd);
 
 static pid_t socket_get_farside_pid(int sockfd);
 
 extern int (*ipc_pollfunc_ptr)(struct pollfd *, nfds_t, int);
 
 static int socket_resume_io_read(struct IPC_CHANNEL *ch, int*, gboolean read1anyway);
-
 static struct IPC_OPS socket_ops;
 static gboolean ipc_time_debug_flag = TRUE;
 
@@ -440,7 +359,7 @@ socket_chan_audit(const struct IPC_CHANNEL* ch)
 		cl_log(LOG_CRIT, "Negative remaining_data");
 		badch = TRUE;
 	}
-	if (chp->remaining_data < 0 || chp->remaining_data > MAXMSG) {
+	if (chp->remaining_data < 0 || chp->remaining_data > MAXDATASIZE) {
 		cl_log(LOG_CRIT, "Excessive/bad remaining_data");
 		badch = TRUE;
 	}
@@ -621,38 +540,9 @@ socket_destroy_wait_conn(struct IPC_WAIT_CONNECTION * wait_conn)
 	struct SOCKET_WAIT_CONN_PRIVATE * wc = wait_conn->ch_private;
 
 	if (wc != NULL) {
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-		if (wc->s >= 0) {
-			if (debug_level > 1) {
-				cl_log(LOG_DEBUG
-				,	"%s: closing socket %d"
-				,	__FUNCTION__, wc->s);
-			}
-			close(wc->s);
-			cl_poll_ignore(wc->s);
-			unlink(wc->path_name);
-			wc->s = -1;
-		}
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-		cl_poll_ignore(wc->pipefds[0]);
-		if (wc->pipefds[0] >= 0) {
-			if (debug_level > 1) {
-				cl_log(LOG_DEBUG
-				,	"%s: closing pipe[0] %d"
-				,	__FUNCTION__, wc->pipefds[0]);
-			}
-			wc->pipefds[0] = -1;
-		}
-		if (wc->pipefds[1] >= 0) {
-			if (debug_level > 1) {
-				cl_log(LOG_DEBUG
-				,	"%s: closing pipe[1] %d"
-				,	__FUNCTION__, wc->pipefds[1]);
-			}
-			wc->pipefds[0] = -1;
-		}
+		close(wc->s);
+		cl_poll_ignore(wc->s);
 		unlink(wc->path_name);
-#endif
 		g_free(wc);
 	}
 	g_free((void*) wait_conn);
@@ -664,11 +554,7 @@ socket_wait_selectfd(struct IPC_WAIT_CONNECTION *wait_conn)
 {
 	struct SOCKET_WAIT_CONN_PRIVATE * wc = wait_conn->ch_private;
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 	return (wc == NULL ? -1 : wc->s);
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-	return (wc == NULL ? -1 : wc->pipefds[0]);
-#endif
 
 }
 
@@ -677,50 +563,36 @@ static struct IPC_CHANNEL*
 socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 ,	struct IPC_AUTH *auth_info)
 {
+	/* make peer_addr a pointer so it can be used by the
+	 *   USE_BINDSTAT_CREDS implementation of socket_verify_auth()
+	 */
+	struct sockaddr_un *			peer_addr;
 	struct IPC_CHANNEL *			ch = NULL;
+	socklen_t				sin_size;
 	int					s;
 	int					new_sock;
 	struct SOCKET_WAIT_CONN_PRIVATE*	conn_private;
 	struct SOCKET_CH_PRIVATE *		ch_private ;
 	int auth_result = IPC_FAIL;
-	int					saveerrno=errno;
 	gboolean was_error = FALSE;
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-	/* make peer_addr a pointer so it can be used by the
-	 *   USE_BINDSTAT_CREDS implementation of socket_verify_auth()
-	 */
-	struct sockaddr_un *			peer_addr = NULL;
-	socklen_t				sin_size;
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-	struct strrecvfd strrecvfd;
-#endif
 	
+	peer_addr = g_new(struct sockaddr_un, 1);
+
 	/* get select fd */
 
 	s = wait_conn->ops->get_select_fd(wait_conn); 
 	if (s < 0) {
 		cl_log(LOG_ERR, "get_select_fd: invalid fd");
+		g_free(peer_addr);
+		peer_addr = NULL;
 		return NULL;
 	}
 
 	/* Get client connection. */
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-	peer_addr = g_new(struct sockaddr_un, 1);
 	sin_size = sizeof(struct sockaddr_un);
-	new_sock = accept(s, (struct sockaddr *)peer_addr, &sin_size);
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-	if (ioctl(s, I_RECVFD, &strrecvfd) == -1) {
-		new_sock = -1;
-	}
-	else {
-		new_sock = strrecvfd.fd;
-	}
-#endif
-	saveerrno=errno;
-	if (new_sock == -1){
+	if ((new_sock = accept(s, (struct sockaddr *)peer_addr, &sin_size)) == -1){
 		if (errno != EAGAIN && errno != EWOULDBLOCK) {
-			cl_perror("socket_accept_connection: accept(sock=%d)"
-			,	s);
+			cl_perror("socket_accept_connection: accept");
 		}
 		was_error = TRUE;
 		
@@ -737,12 +609,7 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 			strncpy(ch_private->path_name,conn_private->path_name
 			,		sizeof(conn_private->path_name));
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 			ch_private->peer_addr = peer_addr;
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-			ch_private->farside_uid = strrecvfd.uid;
-			ch_private->farside_gid = strrecvfd.gid;
-#endif
 		}
 	}
 
@@ -753,15 +620,12 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 			ch->ch_status = IPC_CONNECT;
 			ch->farside_pid = socket_get_farside_pid(new_sock);
 			return ch;
+
 		}
-		saveerrno=errno;
 	}
   
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 	g_free(peer_addr);
 	peer_addr = NULL;
-#endif
-	errno=saveerrno;
 	return NULL;
 
 }
@@ -769,7 +633,7 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 
 
 /* 
- * Called by socket_destroy(). Disconnect the connection 
+ * Called by socket_destory(). Disconnect the connection 
  * and set ch_status to IPC_DISCONNECT. 
  *
  * parameters :
@@ -786,43 +650,22 @@ socket_disconnect(struct IPC_CHANNEL* ch)
 	struct SOCKET_CH_PRIVATE* conn_info;
 
 	conn_info = (struct SOCKET_CH_PRIVATE*) ch->ch_private;
-	if (debug_level > 1) {
-		cl_log(LOG_DEBUG
-		,	"%s(sock=%d, ch=0x%lx){"
-		,	__FUNCTION__
-		,	conn_info->s, (unsigned long)ch);
-	}
 #if 0
 	if (ch->ch_status != IPC_DISCONNECT) {
   		cl_log(LOG_INFO, "forced disconnect for fd %d", conn_info->s);
 	}
 #endif
-	if (ch->ch_status == IPC_CONNECT) {
-		socket_resume_io(ch);		
-	}
-	
-	if (conn_info->s >= 0) {
-		if (debug_level > 1) {
-			cl_log(LOG_DEBUG
-			,	"%s: closing socket %d"
-			,	__FUNCTION__, conn_info->s);
-		}
-		close(conn_info->s);
-		cl_poll_ignore(conn_info->s);
-		conn_info->s = -1;
-	}
+	close(conn_info->s);
+	cl_poll_ignore(conn_info->s);
+	conn_info->s = -1;
 	ch->ch_status = IPC_DISCONNECT;
-	if (debug_level > 1) {
-		cl_log(LOG_DEBUG, "}/*%s(sock=%d, ch=0x%lx)*/"
-		,	__FUNCTION__, conn_info->s, (unsigned long)ch);
-	}
 	return IPC_OK;
 }
 
 
 
 /* 
- * destroy a ipc queue and clean all memory space assigned to this queue.
+ * destory a ipc queue and clean all memory space assigned to this queue.
  * parameters:
  *      q  (IN) the pointer to the queue which should be destroied.
  *
@@ -843,17 +686,9 @@ socket_destroy_queue(struct IPC_QUEUE * q)
 static void
 socket_destroy_channel(struct IPC_CHANNEL * ch)
 {
-	--ch->refcount;
-	if (ch->refcount > 0) {
-		return;
-	}
 	if (ch->ch_status == IPC_CONNECT){
 		socket_resume_io(ch);		
 	}
-	if (debug_level > 1) {
-		cl_log(LOG_DEBUG, "socket_destroy(ch=0x%lx){"
-		,	(unsigned long)ch);
-	}
 	socket_disconnect(ch);
 	socket_destroy_queue(ch->send_queue);
 	socket_destroy_queue(ch->recv_queue);
@@ -863,22 +698,16 @@ socket_destroy_channel(struct IPC_CHANNEL * ch)
 	}
 
 	if (ch->ch_private != NULL) {
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 		struct SOCKET_CH_PRIVATE *priv = (struct SOCKET_CH_PRIVATE *)
 			ch->ch_private;
 		if(priv->peer_addr != NULL) {
 			unlink(priv->peer_addr->sun_path);
 			g_free((void*)(priv->peer_addr));
 		}
-#endif
     		g_free((void*)(ch->ch_private));		
 	}
 	memset(ch, 0xff, sizeof(*ch));
 	g_free((void*)ch);
-	if (debug_level > 1) {
-		cl_log(LOG_DEBUG, "}/*socket_destroy(ch=0x%lx)*/"
-		,	(unsigned long)ch);
-	}
 }
 
 static int
@@ -932,14 +761,10 @@ static int
 socket_initiate_connection(struct IPC_CHANNEL * ch)
 {
 	struct SOCKET_CH_PRIVATE* conn_info;  
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 	struct sockaddr_un peer_addr; /* connector's address information */
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-#endif
   
 	conn_info = (struct SOCKET_CH_PRIVATE*) ch->ch_private;
   
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 	/* Prepare the socket */
 	memset(&peer_addr, 0, sizeof(peer_addr));
 	peer_addr.sun_family = AF_LOCAL;    /* host byte order */ 
@@ -954,9 +779,6 @@ socket_initiate_connection(struct IPC_CHANNEL * ch)
 	, 	sizeof(struct sockaddr_un)) == -1) {
 		return IPC_FAIL;
 	}
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-
-#endif
 
 	ch->ch_status = IPC_CONNECT;
 	ch->farside_pid = socket_get_farside_pid(conn_info->s);
@@ -1016,7 +838,7 @@ socket_send(struct IPC_CHANNEL * ch, struct IPC_MESSAGE* msg)
 	int diff;
 	struct IPC_MESSAGE* newmsg;
 	
-	if (msg->msg_len < 0 || msg->msg_len > MAXMSG) {
+	if (msg->msg_len < 0 || msg->msg_len > MAXDATASIZE) {
 		cl_log(LOG_ERR, "socket_send: "
 		       "invalid message");		       
 		return IPC_FAIL;
@@ -1030,14 +852,10 @@ socket_send(struct IPC_CHANNEL * ch, struct IPC_MESSAGE* msg)
 
 	if ( !ch->should_send_block &&
 	    ch->send_queue->current_qlen >= ch->send_queue->max_qlen) {
-		cl_log(LOG_WARNING, "send queue maximum length(%d) exceeded",
-		       (int)ch->send_queue->max_qlen);
+		/*cl_log(LOG_WARNING, "send queue maximum length(%d) exceeded",
+		  ch->send_queue->max_qlen );*/
 
-		if (ch->should_block_fail) {
-			return IPC_FAIL;
-		} else {
-			return IPC_OK;
-		}
+		return IPC_FAIL;
 	}
 	
 	while (ch->send_queue->current_qlen >= ch->send_queue->max_qlen){
@@ -1376,10 +1194,6 @@ socket_resume_io_read(struct IPC_CHANNEL *ch, int* nbytes, gboolean read1anyway)
 		int				msg_len;
 		int				len;
 
-#if HB_IPC_METHOD == HB_IPC_STREAM
-		struct strbuf d;
-		int flags, rc;
-#endif
 
 		CHANAUDIT(ch);
 		++debug_loopcount;
@@ -1391,16 +1205,7 @@ socket_resume_io_read(struct IPC_CHANNEL *ch, int* nbytes, gboolean read1anyway)
 		
 		/* Now try to receive some data */
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
 		msg_len = recv(conn_info->s, msg_begin, len, MSG_DONTWAIT);
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-		d.maxlen = len;
-		d.len = 0;
-		d.buf = msg_begin;
-		flags = 0;
-		rc = getmsg(conn_info->s, NULL, &d, &flags);
-		msg_len = (rc < 0) ? rc : d.len;
-#endif
 		SocketIPCStats.last_recv_rc = msg_len;
 		SocketIPCStats.last_recv_errno = errno;
 		++SocketIPCStats.recv_count;
@@ -1489,7 +1294,7 @@ socket_resume_io_write(struct IPC_CHANNEL *ch, int* nmsg)
 
 		GList *				element;
 		struct IPC_MESSAGE *		msg;
-		struct SOCKET_MSG_HEAD		head;
+		struct SOCKET_MSG_HEAD*		head;
                 struct IPC_MESSAGE* 		oldmsg = NULL;
 		int				sendrc = 0;
                 struct IPC_MESSAGE* 		newmsg;
@@ -1529,9 +1334,9 @@ socket_resume_io_write(struct IPC_CHANNEL *ch, int* nmsg)
 			msg = newmsg;
 		}
 		
-                head.msg_len = msg->msg_len;
-		head.magic = HEADMAGIC;
-		memcpy(msg->msg_buf, &head, sizeof(struct SOCKET_MSG_HEAD));
+		head = (struct SOCKET_MSG_HEAD*) msg->msg_buf;
+                head->msg_len = msg->msg_len;
+		head->magic = HEADMAGIC;
 		
 		if (ch->bytes_remaining == 0){
 			/*we start to send a new message*/
@@ -1552,23 +1357,11 @@ socket_resume_io_write(struct IPC_CHANNEL *ch, int* nmsg)
 		sendrc = 0;
 		
                 do {
-#if HB_IPC_METHOD == HB_IPC_STREAM
-			struct strbuf d;
-			int msglen, putmsgrc;
-#endif
-
                         CHANAUDIT(ch);
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-			sendrc = send(conn_info->s, p
-			,       bytes_remaining, (MSG_DONTWAIT|MSG_NOSIGNAL));
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-			d.maxlen = 0;
-			d.len = msglen = bytes_remaining;
-			d.buf = p;
-			putmsgrc = putmsg(conn_info->s, NULL, &d, 0);
-			sendrc = putmsgrc == 0 ? msglen : -1; 
-#endif
+			
+                       sendrc = send(conn_info->s, p
+                       ,       bytes_remaining, (MSG_DONTWAIT|MSG_NOSIGNAL));
                         SocketIPCStats.last_send_rc = sendrc;
                         SocketIPCStats.last_send_errno = errno;
                         ++SocketIPCStats.send_count;
@@ -1772,6 +1565,9 @@ static IPC_Message*
 socket_new_ipcmsg(IPC_Channel* ch, const void* data, int len, void* private)
 {
 	IPC_Message*	hdr;
+	char*	copy = NULL;
+	char*	buf;
+	char*	body;
 	
 	if (ch == NULL || len < 0){
 		cl_log(LOG_ERR, "socket_new_ipcmsg:"
@@ -1785,24 +1581,8 @@ socket_new_ipcmsg(IPC_Channel* ch, const void* data, int len, void* private)
 		return NULL;
 	}
 
-	hdr = ipcmsg_new(ch, data, len, private, socket_del_ipcmsg);
 	
-	if (hdr) ipcmsg_count_allocated ++;
-
-	return hdr;
-}
-
-static
-struct IPC_MESSAGE *
-ipcmsg_new(struct IPC_CHANNEL * ch, const void* data, int len, void* private,
-	DelProc delproc)
-{
-	struct IPC_MESSAGE * hdr;
-	char*	copy = NULL;
-	char*	buf;
-	char*	body;
-	
-	if ((hdr = g_new(struct IPC_MESSAGE, 1))  == NULL) {
+	if ((hdr = (IPC_Message*)g_malloc(sizeof(*hdr)))  == NULL) {
 		return NULL;
 	}
 	
@@ -1828,9 +1608,11 @@ ipcmsg_new(struct IPC_CHANNEL * ch, const void* data, int len, void* private,
 	hdr->msg_buf = buf;
 	hdr->msg_body = body;
 	hdr->msg_ch = ch;
-	hdr->msg_done = delproc;
+	hdr->msg_done = socket_del_ipcmsg;
 	hdr->msg_private = private;
 	
+	ipcmsg_count_allocated ++;
+	
 	return hdr;
 }
 
@@ -1892,10 +1674,6 @@ socket_queue_new(void)
  *     attrs = g_hash_table_new(g_str_hash, g_str_equal); 
  *     g_hash_table_insert(attrs, PATH_ATTR, path_name);   
  *     here PATH_ATTR is defined as "path". 
- *
- * NOTE :
- *   The streams implementation uses "Streams Programming Guide", Solaris 8,
- *   as its guide (sample code near end of "Configuration" chapter 11).
  */
 struct IPC_WAIT_CONNECTION *
 socket_wait_conn_new(GHashTable *ch_attrs)
@@ -1903,15 +1681,11 @@ socket_wait_conn_new(GHashTable *ch_attrs)
   struct IPC_WAIT_CONNECTION * temp_ch;
   char *path_name;
   char *mode_attr;
+  struct sockaddr_un my_addr;
   int s, flags;
   struct SOCKET_WAIT_CONN_PRIVATE *wait_private;
   mode_t s_mode;
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-  struct sockaddr_un my_addr;
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-  int pipefds[2];
-#endif
-
+  
   path_name = (char *) g_hash_table_lookup(ch_attrs, IPC_PATH_ATTR);
   mode_attr = (char *) g_hash_table_lookup(ch_attrs, IPC_MODE_ATTR);
 
@@ -1924,7 +1698,6 @@ socket_wait_conn_new(GHashTable *ch_attrs)
     return NULL;
   }
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
   /* prepare the unix domain socket */
   if ((s = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1) {
     cl_perror("socket_wait_conn_new: socket() failure");
@@ -1932,8 +1705,7 @@ socket_wait_conn_new(GHashTable *ch_attrs)
   }
 
   if (unlink(path_name) < 0 && errno != ENOENT) {
-	  cl_perror("socket_wait_conn_new: unlink failure(%s)",
-		    path_name);
+    cl_perror("socket_wait_conn_new: unlink failure");
   }
   memset(&my_addr, 0, sizeof(my_addr));
   my_addr.sun_family = AF_LOCAL;         /* host byte order */
@@ -1951,37 +1723,6 @@ socket_wait_conn_new(GHashTable *ch_attrs)
     close(s);
     return NULL;
   }
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-  /* Set up the communication channel the clients will use to us (server) */
-  if (pipe(pipefds) == -1) {
-    cl_perror("pipe() failure");
-    return NULL;
-  }
-
-  /* Let clients have unique connections to us */
-  if (ioctl(pipefds[1], I_PUSH, "connld") == -1) {
-    cl_perror("ioctl(%d, I_PUSH, \"connld\") failure", pipefds[1]);
-    return NULL;
-  }
-
-  if (unlink(path_name) < 0 && errno != ENOENT) {
-	  cl_perror("socket_wait_conn_new: unlink failure(%s)",
-		    path_name);
-  }
-
-  if (mkfifo(path_name, s_mode) == -1) {
-    cl_perror("socket_wait_conn_new: mkfifo(%s, ...) failure", path_name);
-    return NULL;
-  }
-
-  if (fattach(pipefds[1], path_name) == -1) {
-    cl_perror("socket_wait_conn_new: fattach(..., %s) failure", path_name);
-    return NULL;
-  }
-
-  /* the pseudo-socket is the other part of the pipe */
-  s = pipefds[0];
-#endif
 
   /* Change the permission of the socket */
   if (chmod(path_name,s_mode) < 0){
@@ -1991,18 +1732,13 @@ socket_wait_conn_new(GHashTable *ch_attrs)
     return NULL;
   }
 
-#if HB_IPC_METHOD == HB_IPC_SOCKET
   /* listen to the socket */
   if (listen(s, MAX_LISTEN_NUM) == -1) {
     cl_perror("socket_wait_conn_new: listen(MAX_LISTEN_NUM)");
     close(s);
     return NULL;
   }
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-
-#endif
-
-  flags = fcntl(s, F_GETFL);
+  flags = fcntl(s, F_GETFL, O_NONBLOCK);
   if (flags == -1) {
     cl_perror("socket_wait_conn_new: cannot read file descriptor flags");
     close(s);
@@ -2016,12 +1752,7 @@ socket_wait_conn_new(GHashTable *ch_attrs)
   }
   
   wait_private =  g_new(struct SOCKET_WAIT_CONN_PRIVATE, 1);
-#if HB_IPC_METHOD == HB_IPC_SOCKET
   wait_private->s = s;
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-  wait_private->pipefds[0] = pipefds[0];
-  wait_private->pipefds[1] = pipefds[1];
-#endif
   strncpy(wait_private->path_name, path_name, sizeof(wait_private->path_name));
   temp_ch = g_new(struct IPC_WAIT_CONNECTION, 1);
   temp_ch->ch_private = (void *) wait_private;
@@ -2044,8 +1775,15 @@ socket_wait_conn_new(GHashTable *ch_attrs)
 
 struct IPC_CHANNEL * 
 socket_client_channel_new(GHashTable *ch_attrs) {
+  struct IPC_CHANNEL * temp_ch;
+  struct SOCKET_CH_PRIVATE* conn_info;
   char *path_name;
-  int sockfd;
+  int sockfd, flags;
+#ifdef USE_BINDSTAT_CREDS
+  char rand_id[16];
+  char uuid_str_tmp[40];
+  struct sockaddr_un sock_addr;
+#endif
 
   /*
    * I don't really understand why the client and the server use different
@@ -2070,40 +1808,33 @@ socket_client_channel_new(GHashTable *ch_attrs) {
    */
 
   path_name = (char *) g_hash_table_lookup(ch_attrs, IPC_PATH_ATTR);
-  if (path_name == NULL) {
-	return NULL;
-  }
-
-#if HB_IPC_METHOD == HB_IPC_SOCKET
+  if (path_name != NULL) { 
+	  if (strlen(path_name) >= sizeof(conn_info->path_name)) {
+	    return NULL;
+    }
     /* prepare the socket */
     if ((sockfd = socket(AF_LOCAL, SOCK_STREAM, 0)) == -1) {
       cl_perror("socket_client_channel_new: socket");
       return NULL;
     }
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-    sockfd = open(path_name, O_RDWR|O_NONBLOCK);
-    if (sockfd == -1) {
-      cl_perror("socket_client_channel_new: open(%s, ...) failure", path_name);
-      return NULL;
-    }
-#endif
+  }else{
+    return NULL;
+  }
   
-	if (client_channel_new_auth(sockfd) < 0) {
-		close(sockfd);
-		return NULL;
-	}
-
+  temp_ch = g_new(struct IPC_CHANNEL, 1);
+  if (temp_ch == NULL){
+	  cl_log(LOG_ERR, "socket_server_channel_new:"
+		 " allocating memory for channel failed");
+	  return NULL;	  
+  }
+  memset(temp_ch, 0, sizeof(struct IPC_CHANNEL));
+  
+  
+  
+  conn_info = g_new(struct SOCKET_CH_PRIVATE, 1);
+  conn_info->peer_addr = NULL;
   
-  return channel_new(sockfd, IPC_CLIENT, path_name);
-}
-
-static
-int client_channel_new_auth(int sockfd) {
 #ifdef USE_BINDSTAT_CREDS
-  char rand_id[16];
-  char uuid_str_tmp[40];
-  struct sockaddr_un sock_addr;
-
   /* Prepare the socket */
   memset(&sock_addr, 0, sizeof(sock_addr));
   sock_addr.sun_family = AF_UNIX;
@@ -2113,71 +1844,99 @@ int client_channel_new_auth(int sockfd) {
   uuid_unparse(rand_id, uuid_str_tmp);
   
   snprintf(sock_addr.sun_path, sizeof(sock_addr.sun_path),
-	   "%s/%s", HA_VARLIBHBDIR, uuid_str_tmp);
+	   "%s/%s/%s", HA_VARLIBDIR, PACKAGE, uuid_str_tmp);
   
   unlink(sock_addr.sun_path);
   if(bind(sockfd, (struct sockaddr*)&sock_addr, SUN_LEN(&sock_addr)) < 0) {
 	  perror("Client bind() failure");
-	  return 0;
+	  close(sockfd);
+	  g_free(conn_info); conn_info = NULL;
+	  g_free(temp_ch);
+	  return NULL;
   }
 #endif
+  
+  flags = fcntl(sockfd, F_GETFL, O_NONBLOCK);
+  if (flags == -1) {
+	  cl_perror("socket_client_channel_new: cannot read file descriptor flags");
+	  g_free(conn_info); conn_info = NULL;
+	  g_free(temp_ch);
+	  close(sockfd);
+    return NULL;
+  }
+  flags |= O_NONBLOCK;
+  if (fcntl(sockfd, F_SETFL, flags) < 0) {
+    cl_perror("socket_client_channel_new: cannot set O_NONBLOCK");
+    close(sockfd);
+    g_free(conn_info);
+    g_free(temp_ch);
+    return NULL;
+  }
 
-  return 0;
+  conn_info->s = sockfd;
+  conn_info->remaining_data = 0;
+  conn_info->buf_msg = NULL;
+  
+  strncpy(conn_info->path_name, path_name, sizeof(conn_info->path_name));
+  temp_ch->ch_status = IPC_DISCONNECT;
+#ifdef DEBUG
+  cl_log(LOG_INFO, "Initializing client socket %d to DISCONNECT", sockfd);
+#endif
+  temp_ch->ch_private = (void*) conn_info;
+  temp_ch->ops = (struct IPC_OPS *)&socket_ops;
+  temp_ch->msgpad = sizeof(struct SOCKET_MSG_HEAD);
+  temp_ch->bytes_remaining = 0;
+  temp_ch->should_send_block = FALSE;
+  temp_ch->send_queue = socket_queue_new();
+  temp_ch->recv_queue = socket_queue_new();
+  temp_ch->pool = NULL;
+  temp_ch->high_flow_mark = temp_ch->send_queue->max_qlen;
+  temp_ch->low_flow_mark = -1;
+  temp_ch->conntype = IPC_CLIENT;
+  return temp_ch;
+  
 }
 
-static
 struct IPC_CHANNEL * 
-socket_server_channel_new(int sockfd) {
-	return channel_new(sockfd, IPC_SERVER, "?");
-}
-
-static
-struct IPC_CHANNEL * 
-channel_new(int sockfd, int conntype, const char *path_name) {
+socket_server_channel_new(int sockfd){
   struct IPC_CHANNEL * temp_ch;
   struct SOCKET_CH_PRIVATE* conn_info;
   int flags;
   
-  if (path_name == NULL || strlen(path_name) >= sizeof(conn_info->path_name)) {
-	return NULL;
-  }
   
   temp_ch = g_new(struct IPC_CHANNEL, 1); 
   if (temp_ch == NULL){
-	  cl_log(LOG_ERR, "channel_new: allocating memory for channel failed");
+	  cl_log(LOG_ERR, "socket_server_channel_new:"
+		 " allocating memory for channel failed");
 	  return NULL;	  
   }
   memset(temp_ch, 0, sizeof(struct IPC_CHANNEL));
   
   conn_info = g_new(struct SOCKET_CH_PRIVATE, 1);
   
-  flags = fcntl(sockfd, F_GETFL);
+  flags = fcntl(sockfd, F_GETFL, O_NONBLOCK);
   if (flags == -1) {
-	  cl_perror("channel_new: cannot read file descriptor flags");
+	  cl_perror("socket_server_channel_new: cannot read file descriptor flags");
 	  g_free(conn_info); conn_info = NULL;
 	  g_free(temp_ch);
-	  if (conntype == IPC_CLIENT) close(sockfd);
 	  return NULL;
   }
   flags |= O_NONBLOCK;
   if (fcntl(sockfd, F_SETFL, flags) < 0) {
-	  cl_perror("channel_new: cannot set O_NONBLOCK");
+	  cl_perror("socket_server_channel_new: cannot set O_NONBLOCK");
 	  g_free(conn_info); conn_info = NULL;
 	  g_free(temp_ch);
-	  if (conntype == IPC_CLIENT) close(sockfd);
-	  return NULL;
+    return NULL;
   }
 
   conn_info->s = sockfd;
   conn_info->remaining_data = 0;
   conn_info->buf_msg = NULL;
-#if HB_IPC_METHOD == HB_IPC_SOCKET
   conn_info->peer_addr = NULL;
-#endif
-  strncpy(conn_info->path_name, path_name, sizeof(conn_info->path_name));
+  strcpy(conn_info->path_name, "?");
 
 #ifdef DEBUG
-  cl_log(LOG_INFO, "Initializing socket %d to DISCONNECT", sockfd);
+  cl_log(LOG_INFO, "Initializing server socket %d to DISCONNECT", sockfd);
 #endif
   temp_ch->ch_status = IPC_DISCONNECT;
   temp_ch->ch_private = (void*) conn_info;
@@ -2185,15 +1944,12 @@ channel_new(int sockfd, int conntype, const char *path_name) {
   temp_ch->msgpad = sizeof(struct SOCKET_MSG_HEAD);
   temp_ch->bytes_remaining = 0;
   temp_ch->should_send_block = FALSE;
-  temp_ch->should_block_fail = TRUE;
   temp_ch->send_queue = socket_queue_new();
   temp_ch->recv_queue = socket_queue_new();
   temp_ch->pool = NULL;
   temp_ch->high_flow_mark = temp_ch->send_queue->max_qlen;
   temp_ch->low_flow_mark = -1;
-  temp_ch->conntype = conntype;
-  temp_ch->refcount = 0;
-
+  temp_ch->conntype = IPC_SERVER;
   return temp_ch;
   
 }
@@ -2209,34 +1965,10 @@ ipc_channel_pair(IPC_Channel* channels[2])
 	int	sockets[2];
 	int	rc;
 	int	j;
-	const char *pname;
-
-#if HB_IPC_METHOD == HB_IPC_SOCKET
-	pname = "[socketpair]";
 
 	if ((rc = socketpair(AF_LOCAL, SOCK_STREAM, 0, sockets)) < 0) {
 		return IPC_FAIL;
 	}
-#elif HB_IPC_METHOD == HB_IPC_STREAM
-	pname = "[pipe]";
-
-	if ((rc = pipe(sockets)) < 0) {
-		return IPC_FAIL;
-	}
-	rc = 0;
-	for (j=0; j < 2; ++j) {
-		if (fcntl(sockets[j], F_SETFL, O_NONBLOCK) < 0) {
-			cl_perror("ipc_channel_pair: cannot set O_NONBLOCK");
-			rc = -1;
-		}
-	}
-	if (rc < 0) {
-		close(sockets[0]);
-		close(sockets[1]);
-		return IPC_FAIL;
-	}
-#endif
-
 	if ((channels[0] = socket_server_channel_new(sockets[0])) == NULL) {
 		close(sockets[0]);
 		close(sockets[1]);
@@ -2254,7 +1986,7 @@ ipc_channel_pair(IPC_Channel* channels[2])
 		channels[j]->conntype = IPC_PEER;
 		/* Valid, but not terribly meaningful */
 		channels[j]->farside_pid = getpid();
-  		strncpy(p->path_name, pname, sizeof(p->path_name));
+  		strncpy(p->path_name, "[socketpair]", sizeof(p->path_name));
 	}
 	
 	return IPC_OK;
@@ -2296,13 +2028,32 @@ socket_free_message(struct IPC_MESSAGE * msg) {
  *       the pointer to the new message or NULL if the message can't be created.
  */
 
+
 static struct IPC_MESSAGE*
 socket_message_new(struct IPC_CHANNEL *ch, int msg_len)
 {
-	return ipcmsg_new(ch, NULL, msg_len, NULL, socket_free_message);
+  struct IPC_MESSAGE * temp_msg;
+
+  temp_msg = g_new(struct IPC_MESSAGE, 1);
+  memset(temp_msg, 0, sizeof(struct IPC_MESSAGE));
+  if (msg_len != 0){
+	  temp_msg->msg_buf = g_malloc(msg_len + ch->msgpad);
+	  temp_msg->msg_body = (char*)temp_msg->msg_buf + ch->msgpad;
+  }else{
+	  temp_msg->msg_buf = temp_msg->msg_body = NULL;
+  }
+  
+  temp_msg->msg_len = msg_len;
+  temp_msg->msg_private = NULL;
+  temp_msg->msg_ch = ch;
+  temp_msg->msg_done = socket_free_message;
+
+  return temp_msg;
 }
 
 
+
+
 /***********************************************************************
  *
  * IPC authentication schemes...  More machine dependent than
@@ -2339,7 +2090,7 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
 	||	(size_t)n != sizeof(cred)) {
 		return IPC_FAIL;
 	}
-#if 0
+#if 1
 	cl_log(LOG_DEBUG, "SO_PEERCRED returned [%d, (%ld:%ld)]"
 	,	cred.pid, (long)cred.uid, (long)cred.uid);
 	cl_log(LOG_DEBUG, "Verifying authentication: cred.uid=%d cred.gid=%d"
@@ -2365,7 +2116,6 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
 
 /* get farside pid for our peer process */
 
-static
 pid_t
 socket_get_farside_pid(int sockfd)
 {
@@ -2431,7 +2181,6 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
 	return ret;
 }
 
-static
 pid_t
 socket_get_farside_pid(int sock)
 {
@@ -2488,7 +2237,6 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
 #	define crEgid	sc_egid
 #	define crngrp	sc_ngroups
 #	define crgrps	sc_groups
-#	undef EXTRASPACE
 #	define EXTRASPACE	SOCKCREDSIZE(ngroups)
 
 #elif HAVE_STRUCT_CRED
@@ -2581,7 +2329,6 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
  * information.
  */
 
-static
 pid_t
 socket_get_farside_pid(int sock)
 {
@@ -2594,8 +2341,8 @@ socket_get_farside_pid(int sock)
 /***********************************************************************
  * Bind/Stat VERSION... (Supported on OSX/Darwin and 4.3+BSD at least...)
  *
- * This is for use on systems such as OSX-Darwin where
- *   none of the other options is available.
+ * This is for use on systems such as OSX-Darwin and maybe Solaris where
+ *   none of the other options are available.
  *
  * This implementation has been adapted from "Advanced Programming
  *   in the Unix Environment", Section 15.5.2, by W. Richard Stevens.
@@ -2670,7 +2417,6 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
 }
 
 
-static
 pid_t
 socket_get_farside_pid(int sock)
 {
@@ -2678,118 +2424,15 @@ socket_get_farside_pid(int sock)
 }
 #endif /* Bind/stat version */
 
-/***********************************************************************
- * USE_STREAM_CREDS VERSION... (e.g. Solaris pre-10)
- ***********************************************************************/
-#ifdef USE_STREAM_CREDS
-static int 
-socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
-{
-	struct SOCKET_CH_PRIVATE *conn_info;
-
-	if (ch == NULL || ch->ch_private == NULL) {
-		return IPC_FAIL;
-	}
-
-	conn_info = (struct SOCKET_CH_PRIVATE *) ch->ch_private;
-
-	if (auth_info == NULL
-	  || (auth_info->uid == NULL && auth_info->gid == NULL)) {
-		return IPC_OK;	/* no restriction for authentication */
-	}
-
-	/* verify the credential information. */
-	if (	auth_info->uid
-	&&	(g_hash_table_lookup(auth_info->uid,
-		  GUINT_TO_POINTER((guint)conn_info->farside_uid)) != NULL)) {
-		return IPC_OK;
-	}else if (auth_info->gid
-	&&	(g_hash_table_lookup(auth_info->gid,
-		  GUINT_TO_POINTER((guint)conn_info->farside_gid)) != NULL)) {
-		return IPC_OK;
-	}
-	return IPC_FAIL;
-}
-
-static
-pid_t
-socket_get_farside_pid(int sock)
-{
-	return -1;
-}
-#endif
-
-/***********************************************************************
- * GETPEERUCRED VERSION... (e.g. Solaris 10 upwards)
- ***********************************************************************/
-
-#ifdef USE_GETPEERUCRED
-/* verify the authentication information. */
-static int 
-socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
-{
-	struct SOCKET_CH_PRIVATE *conn_info;
-	ucred_t *ucred = NULL;
-	int rc = IPC_FAIL;
-
-	if (ch == NULL || ch->ch_private == NULL) {
-		return IPC_FAIL;
-	}
-
-	conn_info = (struct SOCKET_CH_PRIVATE *) ch->ch_private;
-
-	if (auth_info == NULL
-	  || (auth_info->uid == NULL && auth_info->gid == NULL)) {
-		return IPC_OK;	/* no restriction for authentication */
-	}
-
-	if (getpeerucred(conn_info->s, &ucred) < 0) {
-		cl_perror("getpeereid() failure");
-		return IPC_FAIL;
-	}
-
-	/* Check credentials against authorization information */
-
-	if (auth_info->uid
-	  && (g_hash_table_lookup(auth_info->uid,
-		  GUINT_TO_POINTER((guint)ucred_geteuid(ucred))) != NULL)) {
-		rc = IPC_OK;
-	}else if (auth_info->gid
-	  && (g_hash_table_lookup(auth_info->gid,
-		  GUINT_TO_POINTER((guint)ucred_getegid(ucred))) != NULL)) {
-		rc = IPC_OK;
-  	}
-
-	ucred_free(ucred);
-
-	return rc;
-}
-
-static
-pid_t
-socket_get_farside_pid(int sockfd)
-{
-	ucred_t *ucred = NULL;
-	pid_t pid;
-
-	if (getpeerucred(sockfd, &ucred) < 0) {
-		cl_perror("getpeereid() failure");
-		return IPC_FAIL;
-	}
-
-	pid = ucred_getpid(ucred);
-
-	ucred_free(ucred);
-
-	return pid;
-}
-#endif
-
 /***********************************************************************
  * DUMMY VERSION... (other systems...)
  *
+ * I'm afraid Solaris falls into this category :-(
  * Other options that seem to be out there include
  * SCM_CREDENTIALS and LOCAL_CREDS
+ * Or maybe something called doors for Solaris
+ * Unfortunately, it looks like Doors is tied to threads :-(
+ * Can the streams credentials code be used with local domain sockets?
  * There are some kludgy things you can do with SCM_RIGHTS
  * to pass an fd which could only be opened by the user id to
  * validate the user id, but I don't know of a similar kludge which
@@ -2808,7 +2451,6 @@ socket_verify_auth(struct IPC_CHANNEL* ch, struct IPC_AUTH * auth_info)
 	return IPC_FAIL;
 }
 
-static
 pid_t
 socket_get_farside_pid(int sock)
 {
@@ -2842,5 +2484,4 @@ static struct IPC_OPS socket_ops = {
 	is_sendq_full:		socket_is_sendq_full,
 	is_recvq_full:		socket_is_recvq_full,
 	get_conntype:		socket_get_conntype,
-	disconnect:		socket_disconnect,
 };
