@@ -1,3 +1,4 @@
+/* $Id: ipctest.c,v 1.42 2005/08/18 05:09:40 alan Exp $ */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,17 +17,15 @@
 
 #undef _GNU_SOURCE  /* in case it was defined on the command line */
 #define _GNU_SOURCE
-#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <stdio.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-/* libgen.h: for 'basename()' on Solaris */
-#include <libgen.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/cl_poll.h>
@@ -35,7 +34,6 @@
 #include <clplumbing/cl_malloc.h>
 
 #define	MAXERRORS	1000
-#define	MAXERRORS_RECV	10
 
 typedef int (*TestFunc_t)(IPC_Channel*chan, int count);
 
@@ -57,24 +55,6 @@ static void checkifblocked(IPC_Channel* channel);
 static int (*PollFunc)(struct pollfd * fds, unsigned int, int)
 =	(int (*)(struct pollfd * fds, unsigned int, int))  poll;
 static gboolean checkmsg(IPC_Message* rmsg, const char * who, int rcount);
-
-const char *procname;
-
-int iter_def = 10000;	/* number of iterations */
-int verbosity = 0;	/* verbosity level */
-
-/*
- * The ipc interface can be invoked as either:
- * 1. pair (pipe/socketpair);
- * 2. separate connect/accept (like server with multiple independent clients).
- *
- * If number of clients is given as 0, the "pair" mechanism is used,
- * otherwise the client/server mechanism.
- */
-/* *** CLIENTS_MAX currently 1 while coding *** */
-#define CLIENTS_MAX 1	/* max. number of independent clients */
-int clients_def = 0;	/* number of independent clients */
-
 static int
 channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 {
@@ -82,20 +62,12 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 	int		rc  = 0;
 	int		waitstat = 0;
 
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d: main process",
-		  procname, (int)getpid(), __LINE__);
-	}
 	switch (fork()) {
 		case -1:
 			cl_perror("can't fork");
 			exit(1);
 			break;
 		default: /* Parent */
-			if (verbosity >= 1) {
-				cl_log(LOG_DEBUG, "%s[%d]%d: main waiting...",
-				  procname, (int)getpid(), __LINE__);
-			}
 			while (wait(&waitstat) > 0) {
 				if (WIFEXITED(waitstat)) {
 					rc += WEXITSTATUS(waitstat);
@@ -103,10 +75,6 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 					rc += 1;
 				}
 			}
-			if (verbosity >= 1) {
-				cl_log(LOG_DEBUG, "%s[%d]%d: main ended rc: %d",
-				  procname, (int)getpid(), __LINE__, rc);
-			}
 			if (rc > 127) {
 				rc = 127;
 			}
@@ -131,15 +99,7 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 		case 0:		/* echo "client" Child */
 			channels[1]->ops->destroy(channels[1]);
 			channels[1] = NULL;
-			if (verbosity >= 1) {
-				cl_log(LOG_DEBUG, "%s[%d]%d: client starting...",
-				  procname, (int)getpid(), __LINE__);
-			}
 			rc = clientfunc(channels[0], count);
-			if (verbosity >= 1) {
-				cl_log(LOG_DEBUG, "%s[%d]%d: client ended rc:%d",
-				  procname, (int)getpid(), __LINE__, rc);
-			}
 			exit (rc > 127 ? 127 : rc);
 			break;
 
@@ -148,10 +108,6 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 	}
 	channels[0]->ops->destroy(channels[0]);
 	channels[0] = NULL;
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d: server starting...",
-		  procname, (int)getpid(), __LINE__);
-	}
 	rc = serverfunc(channels[1], count);
 	wait(&waitstat);
 	if (WIFEXITED(waitstat)) {
@@ -159,200 +115,35 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 	}else{
 		rc += 1;
 	}
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d: server ended rc:%d",
-		  procname, (int)getpid(), __LINE__, rc);
-	}
 	return(rc);
 }
 
-/* server with many clients */
-static int
-clientserver(TestFunc_t clientfunc, TestFunc_t serverfunc, int count, int clients)
+#if 0
+/*
+ *	FIX commpath to non/tmp before enabling this code.
+ *
+ */
+static void
+clientserverpair(IPC_Channel* channels[2])
 {
-	IPC_Channel* channel;
-	int rc  = 0;
-	int waitstat = 0;
-	struct IPC_WAIT_CONNECTION *wconn;
-	char path[] = IPC_PATH_ATTR;
-	char commpath[] = "/tmp/foobar";	/* *** CHECK/FIX: Is this OK? */
-	GHashTable * wattrs;
-	int i;
-	pid_t pid;
+	char			path[] = IPC_PATH_ATTR;
+	char			commpath[] = "/tmp/foobar"
+	GHashTable *		wattrs;
+	IPC_WAIT_CONNECTION*	wconn;
 
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d: main process",
-		  procname, (int)getpid(), __LINE__);
-	}
-
-	switch (fork()) {
-		case -1:
-			cl_perror("can't fork");
-			exit(1);
-			break;
-		default: /* Parent */
-			if (verbosity >= 1) {
-				cl_log(LOG_DEBUG, "%s[%d]%d: main waiting...",
-				  procname, (int)getpid(), __LINE__);
-			}
-			while ((pid = wait(&waitstat)) > 0) {
-				if (WIFEXITED(waitstat)) {
-					rc += WEXITSTATUS(waitstat);
-				}else{
-					rc += 1;
-				}
-			}
-			if (verbosity >= 1) {
-				cl_log(LOG_DEBUG, "%s[%d]%d: main ended rc: %d",
-				  procname, (int)getpid(), __LINE__, rc);
-			}
-			if (rc > 127) {
-				rc = 127;
-			}
-			exit(rc);
-			break;
-		case 0: /* Child */
-			break;
-	}
-
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d:",
-		  procname, (int)getpid(), __LINE__);
-	}
-
-	/* set up a server */
 	wattrs = g_hash_table_new(g_str_hash, g_str_equal);
-	if (! wattrs) {
-		cl_perror("g_hash_table_new() failed");
-		exit(1);
-	}
+
 	g_hash_table_insert(wattrs, path, commpath);
 
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d:",
-		  procname, (int)getpid(), __LINE__);
-	}
+	wconn = ipc_wait_conn_constructor(IPC_ANYTYPE, wconnattrs);
 
-	wconn = ipc_wait_conn_constructor(IPC_ANYTYPE, wattrs);
-	if (! wconn) {
-		cl_perror("could not establish server");
+	if (wconn == NULL) {
+		cl_perror("Can't create wait connection");
 		exit(1);
 	}
 
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d:",
-		  procname, (int)getpid(), __LINE__);
-	}
-
-	/* spawn the clients */
-	for (i = 1; i <= clients; i++) {
-		if (verbosity >= 1) {
-			cl_log(LOG_DEBUG, "%s[%d]%d: fork client %d of %d",
-			  procname, (int)getpid(), __LINE__, i, clients);
-		}
-		switch (fork()) {
-			case -1:
-				cl_perror("can't fork");
-				exit(1);
-				break;
-
-			case 0:	/* echo "client" Child */
-				if (verbosity >= 1) {
-					cl_log(LOG_DEBUG, "%s[%d]%d: client %d starting...",
-					  procname, (int)getpid(), __LINE__, i);
-				}
-				channel = ipc_channel_constructor(IPC_ANYTYPE, wattrs);
-				if (channel == NULL) {
-					cl_perror("client: channel creation failed");
-					exit(1);
-				}
-
-				rc = channel->ops->initiate_connection(channel);
-				if (rc != IPC_OK) {
-					cl_perror("channel[1] failed to connect");
-					exit(1);
-				}
-				checksock(channel);
-				rc = clientfunc(channel, count);
-				if (verbosity >= 1) {
-					cl_log(LOG_DEBUG, "%s[%d]%d: client %d ended rc:%d",
-					  procname, (int)getpid(), __LINE__, rc, i);
-				}
-				exit (rc > 127 ? 127 : rc);
-				break;
-
-			default:
-				break;
-		}
-	}
-
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d: server starting...",
-		  procname, (int)getpid(), __LINE__);
-	}
-	/* accept on server */
-	/* ***
-	 * Two problems (or more) here:
-	 * 1. What to do if no incoming call pending?
-	 *    At present, fudge by sleeping a little so client gets started.
-	 * 2. How to handle multiple clients?
-	 *    Would need to be able to await both new connections and
-	 *    data on existing connections.
-	 *    At present, fudge CLIENTS_MAX as 1.
-	 * ***
-	 */
-	sleep(1); /* *** */
-	channel = wconn->ops->accept_connection(wconn, NULL);
-	if (channel == NULL) {
-		cl_perror("server: acceptance failed");
-	}
-
-	checksock(channel);
-
-	rc = serverfunc(channel, count);
-
-	/* server finished: tidy up */
-	wconn->ops->destroy(wconn);
-
-	if (verbosity >= 1) {
-		cl_log(LOG_DEBUG, "%s[%d]%d: server ended rc:%d",
-		  procname, (int)getpid(), __LINE__, rc);
-	}
-
-	/* reap the clients */
-	for (i = 1; i <= clients; i++) {
-		pid_t pid;
-
-		pid = wait(&waitstat);
-		if (verbosity >= 1) {
-			cl_log(LOG_DEBUG, "%s[%d]%d: client %d reaped:%d",
-			  procname, (int)getpid(), __LINE__,
-			  (int) pid, WIFEXITED(waitstat));
-		}
-		if (WIFEXITED(waitstat)) {
-			rc += WEXITSTATUS(waitstat);
-		}else{
-			rc += 1;
-		}
-	}
-
-	return(rc);
 }
-
-static void
-echomsgbody(void * body, int n, int niter, size_t * len)
-{
-	char *str = body;
-	int l;
-
-	l = snprintf(str, n-1, "String-%d", niter);
-	if (l < (n-1)) {
-		memset(&str[l], 'a', (n - (l+1)));
-	}
-	str[n-1] = '\0';
-	*len = n;
-}
-
+#endif
 static void
 checkifblocked(IPC_Channel* chan)
 {
@@ -367,103 +158,41 @@ extern long	SeqNums[32];
 #endif
 
 static int
-transport_tests(int iterations, int clients)
+transport_tests(int iterations)
 {
 	int	rc = 0;
 
 #ifdef CHEAT_CHECKS
 	memset(SeqNums, 0, sizeof(SeqNums));
 #endif
-	rc += (clients <= 0)
-	  ? channelpair(echoclient, echoserver, iterations)
-	  : clientserver(echoclient, echoserver, iterations, clients);
+	rc += channelpair(echoclient, echoserver, iterations);
 
 #ifdef CHEAT_CHECKS
 	memset(SeqNums, 0, sizeof(SeqNums));
 #endif
-	rc += (clients <= 0)
-	  ? channelpair(asyn_echoclient, asyn_echoserver, iterations)
-	  : clientserver(asyn_echoclient, asyn_echoserver, iterations, clients);
+	rc += channelpair(asyn_echoclient, asyn_echoserver, iterations);
 
 #ifdef CHEAT_CHECKS
 	memset(SeqNums, 0, sizeof(SeqNums));
 #endif
-	rc += (clients <= 0)
-	  ? channelpair(mainloop_client, mainloop_server, iterations)
-	  : clientserver(mainloop_client, mainloop_server, iterations, clients);
+	rc += channelpair(mainloop_client, mainloop_server, iterations);
 
 	return rc;
 }
 
-int data_size = 10;
-
 int
 main(int argc, char ** argv)
 {
-	int argflag, argerrs;
-	int iterations;
-	int clients;
 	int	rc = 0;
 
-	/*
-	 * Check and process arguments.
-	 *	-v: verbose
-	 *	-i: number of iterations
-	 *	-c: number of clients (invokes client/server mechanism)
-	 *	-s: data-size
-	 */
-	procname = basename(argv[0]);
-
-	argerrs = 0;
-	iterations = iter_def;
-	clients = clients_def;
-	while ((argflag = getopt(argc, argv, "i:vuc:s:")) != EOF) {
-		switch (argflag) {
-		case 'i':	/* iterations */
-			iterations = atoi(optarg);
-			break;
-		case 'v':	/* verbosity */
-			verbosity++;
-			break;
-		case 'c':	/* number of clients */
-			clients = atoi(optarg);
-			if (clients < 1 || clients > CLIENTS_MAX) {
-				fprintf(stderr, "number of clients out of range"
-				  "(1 to %d)\n", CLIENTS_MAX);
-				argerrs++;
-			}
-			break;
-		case 's':	/* data size */
-			data_size = atoi(optarg);
-			if (data_size < 0) {
-				fprintf(stderr, "data size must be >=0\n");
-				argerrs++;
-			}
-			break;
-		default:
-			argerrs++;
-			break;
-		}
-	}
-	if (argerrs) {
-		fprintf(stderr,
-		  "Usage: %s [-v] [-i iterations] [-c clients] [-s size]\n"
-			"\t-v : verbose\n"
-			"\t-i : iterations (default %d)\n"
-			"\t-c : number of clients (default %d; nonzero invokes client/server)\n"
-			"\t-s : data size (default 10 bytes)\n",
-		  procname, iter_def, clients_def);
-		exit(1);
-	}
-
 	cl_malloc_forced_for_glib();
 
-	cl_log_set_entity(procname);
+	cl_log_set_entity("ipctest");
 	cl_log_enable_stderr(TRUE);
 
 
 
-	rc += transport_tests(iterations, clients);
+	rc += transport_tests(10000);
 
 #if 0
 	/* Broken for the moment - need to fix it long term */
@@ -472,7 +201,7 @@ main(int argc, char ** argv)
 	g_main_set_poll_func(cl_glibpoll);
 	ipc_set_pollfunc(cl_poll);
 
-	rc += transport_tests(5 * iterations, clients);
+	rc += transport_tests(50000);
 #endif
 	
 	cl_log(LOG_INFO, "TOTAL errors: %d", rc);
@@ -484,10 +213,6 @@ static int
 checksock(IPC_Channel* channel)
 {
 
-	if (!channel) {
-		cl_log(LOG_ERR, "Channel null");
-		return 1;
-	}
 	if (!IPC_ISRCONN(channel)) {
 		cl_log(LOG_ERR, "Channel status is %d"
 		", not IPC_CONNECT", channel->ch_status);
@@ -557,13 +282,12 @@ EOFcheck(IPC_Channel* chan)
 static int
 echoserver(IPC_Channel* wchan, int repcount)
 {
-	char	*str;
+	char	str[256];
 	int	j;
 	int	errcount = 0;
 	IPC_Message	wmsg;
 	IPC_Message*	rmsg = NULL;
 	
-	str = malloc(data_size);
 	
 	memset(&wmsg, 0, sizeof(wmsg));
 	wmsg.msg_private = NULL;
@@ -576,8 +300,8 @@ echoserver(IPC_Channel* wchan, int repcount)
 	for (j=1; j <= repcount
 	;++j, rmsg != NULL && (rmsg->msg_done(rmsg),1)) {
 		int	rc;
-
-		echomsgbody(str, data_size, j, &(wmsg.msg_len));
+		snprintf(str, sizeof(str)-1, "String-%d", j);
+		wmsg.msg_len = strlen(str)+1;
 		if ((rc = wchan->ops->send(wchan, &wmsg)) != IPC_OK) {
 			cl_log(LOG_ERR
 			,	"echotest: send failed %d rc iter %d"
@@ -643,8 +367,6 @@ echoserver(IPC_Channel* wchan, int repcount)
 #endif
 	wchan->ops->destroy(wchan); wchan = NULL;
 
-	free(str);
-
 	return errcount;
 }
 static int
@@ -681,12 +403,6 @@ echoclient(IPC_Channel* rchan, int repcount)
 			,	rc, j, errno);
 			cl_perror("recv");
 			++errcount;
-			if (errcount > MAXERRORS_RECV) {
-				cl_log(LOG_ERR,
-				  "echoclient: errcount excessive: %d: abandoning",
-				  errcount);
-				exit(1);
-			}
 			--j;
 			rmsg=NULL;
 			continue;
@@ -714,6 +430,14 @@ echoclient(IPC_Channel* rchan, int repcount)
 	return errcount;
 }
 
+static void
+echomsgbody(void * body, int niter, size_t * len)
+{
+	char *	str = body;
+	sprintf(str, "String-%d", niter);
+	*len = strlen(str)+1;
+}
+
 void dump_ipc_info(IPC_Channel* chan);
 
 static int
@@ -837,8 +561,8 @@ asyn_echoserver(IPC_Channel* wchan, int repcount)
 				if (wrcount > repcount) {
 					break;
 				}
-				wmsg = wchan->ops->new_ipcmsg(wchan, NULL, data_size, NULL);
-				echomsgbody(wmsg->msg_body, data_size, wrcount, &wmsg->msg_len);
+				wmsg = wchan->ops->new_ipcmsg(wchan, NULL, 32, NULL);
+				echomsgbody(wmsg->msg_body, wrcount, &wmsg->msg_len);
 				if ((rc = wchan->ops->send(wchan, wmsg)) != IPC_OK){
 					
 					cl_log(LOG_INFO, "channel sstatus in echo server is %d",
@@ -1117,8 +841,8 @@ s_send_msg(gpointer data)
 	
 	++i->wcount;
 	
-	wmsg = i->chan->ops->new_ipcmsg(i->chan, NULL, data_size, NULL);
-	echomsgbody(wmsg->msg_body, data_size, i->wcount, &wmsg->msg_len);
+	wmsg = i->chan->ops->new_ipcmsg(i->chan, NULL, 32, NULL);
+	echomsgbody(wmsg->msg_body, i->wcount, &wmsg->msg_len);
 	
 	/*cl_log(LOG_INFO, "s_send_msg: sending out %d", i->wcount);*/
 	
@@ -1205,12 +929,10 @@ s_rcv_msg(IPC_Channel* chan, gpointer data)
 static gboolean
 checkmsg(IPC_Message* rmsg, const char * who, int rcount)
 {
-	char		*str;
+	char		str[256];
 	size_t		len;
-	
-	str = malloc(data_size);
 
-	echomsgbody(str, data_size, rcount, &len);
+	echomsgbody(str, rcount, &len);
 
 	if (rmsg->msg_len != len) {
 		cl_log(LOG_ERR
@@ -1251,9 +973,6 @@ checkmsg(IPC_Message* rmsg, const char * who, int rcount)
 		,	who, rcount);
 #endif
 	}
-
-	free(str);
-
 	return TRUE;
 }
 
