@@ -1,4 +1,4 @@
-/* $Id: ipctransientserver.c,v 1.5.2.3 2004/09/15 07:27:19 alan Exp $ */
+/* $Id: ipctransientserver.c,v 1.13 2004/11/22 19:03:01 gshi Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,6 +16,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+#undef _GNU_SOURCE  /* in case it was defined on the command line */
+#define _GNU_SOURCE
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -91,13 +93,14 @@ init_server_ipc_comms(const char *child,
 	
     local_sock_len += strlen(child);
     local_sock_len += strlen(WORKING_DIR);
-
-    if ((commpath = (char*)malloc(sizeof(char)*local_sock_len)) == NULL) {
-		cl_perror("[Server] Can't malloc.");
-		exit(1);
-     }
     
-    snprintf(commpath, local_sock_len, WORKING_DIR "/%s", child);
+    commpath = (char*)malloc(sizeof(char)*local_sock_len);
+    if (commpath == NULL){
+	    cl_log(LOG_ERR, "init_server_ipc_comms:"
+		   " allocating memory failed");
+	    exit(1);
+    }
+    sprintf(commpath, WORKING_DIR "/%s", child);
     commpath[local_sock_len - 1] = '\0';
 
     g_hash_table_insert(attrs, path, commpath);
@@ -105,8 +108,8 @@ init_server_ipc_comms(const char *child,
     mask = umask(0);
     wait_ch = ipc_wait_conn_constructor(IPC_ANYTYPE, attrs);
     if (wait_ch == NULL){
-		cl_perror("[Server] Can't create wait channel of type %s", IPC_ANYTYPE);
-		exit(1);
+	    cl_perror("[Server] Can't create wait channel of type %s", IPC_ANYTYPE);
+	    exit(1);
     }
     mask = umask(mask);
     g_hash_table_destroy(attrs);
@@ -241,15 +244,22 @@ create_simple_message(char *text, IPC_Channel *ch)
 	}
 
     ack_msg = (IPC_Message *)malloc(sizeof(IPC_Message));
-	copy_text = strdup(text);
+    if (ack_msg == NULL){
+	    cl_log(LOG_ERR, "create_simple_message:"
+		   "allocating memory for IPC_Message failed");		
+	    return NULL;
+    }
     
-    if (ack_msg) {
-		ack_msg->msg_private = NULL;
-		ack_msg->msg_done    = NULL;
-		ack_msg->msg_body    = copy_text;
-		ack_msg->msg_ch      = ch;
-		ack_msg->msg_len = strlen(text)+1;
-	}
+    memset(ack_msg, 0, sizeof(IPC_Message));
     
-	return ack_msg;
+    copy_text = strdup(text);
+    
+    ack_msg->msg_private = NULL;
+    ack_msg->msg_done    = NULL;
+    ack_msg->msg_body    = copy_text;
+    ack_msg->msg_ch      = ch;
+
+    ack_msg->msg_len = strlen(text)+1;
+    
+    return ack_msg;
 }
