@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <lha_internal.h>
+#include <portability.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -68,15 +68,13 @@ int cib_client_set_master(cib_t *cib, int call_options);
 
 int cib_client_bump_epoch(cib_t *cib, int call_options);
 int cib_client_create(cib_t *cib, const char *section, crm_data_t *data,
-		      crm_data_t **output_data, int call_options);
+	      crm_data_t **output_data, int call_options);
 int cib_client_modify(cib_t *cib, const char *section, crm_data_t *data,
-		      crm_data_t **output_data, int call_options);
-int cib_client_update(cib_t *cib, const char *section, crm_data_t *data,
-		      crm_data_t **output_data, int call_options);
+	      crm_data_t **output_data, int call_options);
 int cib_client_replace(cib_t *cib, const char *section, crm_data_t *data,
-		       crm_data_t **output_data, int call_options);
+	       crm_data_t **output_data, int call_options);
 int cib_client_delete(cib_t *cib, const char *section, crm_data_t *data,
-		      crm_data_t **output_data, int call_options);
+	      crm_data_t **output_data, int call_options);
 int cib_client_delete_absolute(
 	cib_t *cib, const char *section, crm_data_t *data,
 	crm_data_t **output_data, int call_options);
@@ -95,7 +93,6 @@ int cib_client_del_notify_callback(
 gint ciblib_GCompareFunc(gconstpointer a, gconstpointer b);
 
 extern cib_t *cib_native_new(cib_t *cib);
-extern void cib_native_delete(cib_t *cib);
 
 static enum cib_variant configured_variant = cib_native;
 
@@ -133,6 +130,7 @@ cib_new(void)
 
 	/* the rest will get filled in by the variant constructor */
 	crm_malloc0(new_cib->cmds, sizeof(cib_api_operations_t));
+	memset(new_cib->cmds, 0, sizeof(cib_api_operations_t));
 
 	new_cib->cmds->set_op_callback     = cib_client_set_op_callback;
 	new_cib->cmds->add_notify_callback = cib_client_add_notify_callback;
@@ -155,7 +153,6 @@ cib_new(void)
 
 	new_cib->cmds->create  = cib_client_create;
 	new_cib->cmds->modify  = cib_client_modify;
-	new_cib->cmds->update  = cib_client_update;
 	new_cib->cmds->replace = cib_client_replace;
 	new_cib->cmds->delete  = cib_client_delete;
 	new_cib->cmds->erase   = cib_client_erase;
@@ -165,30 +162,12 @@ cib_new(void)
 	
 	cib_native_new(new_cib);
 	if(verify_cib_cmds(new_cib) == FALSE) {
-		cib_delete(new_cib);
 		return NULL;
 	}
 	
 	return new_cib;
 }
 
-void
-cib_delete(cib_t *cib)
-{
-	GList *list = cib->notify_list;
-	while(list != NULL) {
-		cib_notify_client_t *client = g_list_nth_data(list, 0);
-		list = g_list_remove(list, client);
-		crm_free(client);
-	}
-	
-	cib_native_delete(cib);
-	g_hash_table_destroy(cib_op_callback_table);
-	crm_free(cib->cmds);
-	crm_free(cib);
-}
-
-
 int
 cib_client_set_op_callback(
 	cib_t *cib, void (*callback)(const struct ha_msg *msg, int call_id,
@@ -377,21 +356,6 @@ int cib_client_modify(cib_t *cib, const char *section, crm_data_t *data,
 		return cib_variant;
 	} 
 
-	return cib->cmds->variant_op(cib, CIB_OP_MODIFY, NULL, section,
-				     data, output_data, call_options);
-}
-
-int cib_client_update(cib_t *cib, const char *section, crm_data_t *data,
-		      crm_data_t **output_data, int call_options) 
-{
-	if(cib == NULL) {
-		return cib_missing;
-	} else if(cib->state == cib_disconnected) {
-		return cib_not_connected;
-	} else if(cib->cmds->variant_op == NULL) {
-		return cib_variant;
-	} 
-
 	return cib->cmds->variant_op(cib, CIB_OP_UPDATE, NULL, section,
 				     data, output_data, call_options);
 }
@@ -495,8 +459,7 @@ int cib_client_add_notify_callback(
 	
 	if(list_item != NULL) {
 		crm_warn("Callback already present");
-		crm_free(new_client);
-		
+
 	} else {
 		cib->notify_list = g_list_append(
 			cib->notify_list, new_client);
@@ -602,9 +565,6 @@ remove_cib_op_callback(int call_id, gboolean all_callbacks)
 int
 num_cib_op_callbacks(void)
 {
-	if(cib_op_callback_table == NULL) {
-		return 0;
-	}
 	return g_hash_table_size(cib_op_callback_table);
 }
 
@@ -617,43 +577,43 @@ cib_pluralSection(const char *a_section)
 	if (a_section == NULL) {
 		a_section_parent = crm_strdup("all");
 
-	} else if(strcasecmp(a_section, XML_TAG_CIB) == 0) {
+	} else if(strcmp(a_section, XML_TAG_CIB) == 0) {
 		a_section_parent = crm_strdup("all");
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_NODE) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_NODE) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_NODES);
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_STATE) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_STATE) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_STATUS);
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_CONSTRAINT) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_CONSTRAINT) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_CONSTRAINTS);
 		
-	} else if(strcasecmp(a_section, XML_CONS_TAG_RSC_LOCATION) == 0) {
+	} else if(strcmp(a_section, XML_CONS_TAG_RSC_LOCATION) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_CONSTRAINTS);
 		
-	} else if(strcasecmp(a_section, XML_CONS_TAG_RSC_DEPEND) == 0) {
+	} else if(strcmp(a_section, XML_CONS_TAG_RSC_DEPEND) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_CONSTRAINTS);
 		
-	} else if(strcasecmp(a_section, XML_CONS_TAG_RSC_ORDER) == 0) {
+	} else if(strcmp(a_section, XML_CONS_TAG_RSC_ORDER) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_CONSTRAINTS);
 		
-	} else if(strcasecmp(a_section, "resource") == 0) {
+	} else if(strcmp(a_section, "resource") == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_RESOURCES);
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_RESOURCE) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_RESOURCE) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_RESOURCES);
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_GROUP) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_GROUP) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_RESOURCES);
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_INCARNATION) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_INCARNATION) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_RESOURCES);
 		
-	} else if(strcasecmp(a_section, XML_CIB_TAG_NVPAIR) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_NVPAIR) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_CRMCONFIG);
 
-	} else if(strcasecmp(a_section, XML_TAG_ATTR_SETS) == 0) {
+	} else if(strcmp(a_section, XML_TAG_ATTR_SETS) == 0) {
 		a_section_parent = crm_strdup(XML_CIB_TAG_CRMCONFIG);
 
 	} else {
@@ -671,21 +631,9 @@ cib_error2string(enum cib_errors return_code)
 {
 	const char *error_msg = NULL;
 	switch(return_code) {
-		case cib_bad_permissions:
-			error_msg = "bad permissions for the on-disk configuration. shutdown heartbeat and repair.";
-			break;
-		case cib_bad_digest:
-			error_msg = "the on-disk configuration was manually altered. shutdown heartbeat and repair.";
-			break;
-		case cib_bad_config:
-			error_msg = "the on-disk configuration is not valid";
-			break;
 		case cib_msg_field_add:
 			error_msg = "failed adding field to cib message";
 			break;			
-		case cib_id_check:
-			error_msg = "missing id or id-collision detected";
-			break;			
 		case cib_operation:
 			error_msg = "invalid operation";
 			break;
@@ -753,7 +701,7 @@ cib_error2string(enum cib_errors return_code)
 			error_msg = "The object already exists";
 			break;
 		case cib_NOTEXISTS:
-			error_msg = "The object/attribute does not exist";
+			error_msg = "The object does not exist";
 			break;
 		case CIBRES_CORRUPT:
 			error_msg = "The CIB is corrupt";
@@ -818,18 +766,9 @@ cib_error2string(enum cib_errors return_code)
 		case cib_diff_resync:
 			error_msg = "Application of an update diff failed, requesting a full refresh";
 			break;
-		case cib_bad_section:
-			error_msg = "Invalid CIB section specified";
-			break;
 		case cib_old_data:
 			error_msg = "Update was older than existing configuration";
 			break;
-		case cib_dtd_validation:
-			error_msg = "Update does not conform to the DTD in "HA_NOARCHDATAHBDIR"/crm.dtd";
-			break;
-		case cib_invalid_argument:
-			error_msg = "Invalid argument";
-			break;
 	}
 			
 	if(error_msg == NULL) {
@@ -877,22 +816,22 @@ cib_op2string(enum cib_update_op operation)
 int
 cib_section2enum(const char *a_section) 
 {
-	if(a_section == NULL || strcasecmp(a_section, "all") == 0) {
+	if(a_section == NULL || strcmp(a_section, "all") == 0) {
 		return cib_section_all;
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_NODES) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_NODES) == 0) {
 		return cib_section_nodes;
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_STATUS) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_STATUS) == 0) {
 		return cib_section_status;
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_CONSTRAINTS) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_CONSTRAINTS) == 0) {
 		return cib_section_constraints;
 		
-	} else if(strcasecmp(a_section, XML_CIB_TAG_RESOURCES) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_RESOURCES) == 0) {
 		return cib_section_resources;
 
-	} else if(strcasecmp(a_section, XML_CIB_TAG_CRMCONFIG) == 0) {
+	} else if(strcmp(a_section, XML_CIB_TAG_CRMCONFIG) == 0) {
 		return cib_section_crmconfig;
 
 	}
@@ -925,8 +864,8 @@ cib_compare_generation(crm_data_t *left, crm_data_t *right)
 			elem_r = crm_element_value(right, attributes[lpc]);
 		}
 	
-		if(elem_l != NULL) { int_elem_l = crm_parse_int(elem_l, NULL); }
-		if(elem_r != NULL) { int_elem_r = crm_parse_int(elem_r, NULL); }
+		if(elem_l != NULL) { int_elem_l = atoi(elem_l); }
+		if(elem_r != NULL) { int_elem_r = atoi(elem_r); }
 
 		if(int_elem_l < int_elem_r) {
 			crm_debug_2("%s (%s < %s)", attributes[lpc],
@@ -943,13 +882,12 @@ cib_compare_generation(crm_data_t *left, crm_data_t *right)
 	return 0;
 }
 
+
 crm_data_t*
 get_cib_copy(cib_t *cib)
 {
 	crm_data_t *xml_cib;
-#if CRM_DEPRECATED_SINCE_2_0_4
 	crm_data_t *xml_cib_copy;
-#endif
 	int options = cib_scope_local|cib_sync_call;
 	if(cib->cmds->query(cib, NULL, &xml_cib, options) != cib_ok) {
 		crm_err("Couldnt retrieve the CIB");
@@ -959,19 +897,11 @@ get_cib_copy(cib_t *cib)
 		return NULL;
 	}
 
-	if(safe_str_eq(crm_element_name(xml_cib), XML_TAG_CIB)) {
-		return xml_cib;
-		
-#if CRM_DEPRECATED_SINCE_2_0_4
-	} else {
-		xml_cib_copy = copy_xml(
-			find_xml_node(xml_cib, XML_TAG_CIB, TRUE));
-		free_xml(xml_cib);
-		return xml_cib_copy;
-#endif
-	}
+	xml_cib_copy = copy_xml(
+		find_xml_node(xml_cib, XML_TAG_CIB, TRUE));
 	free_xml(xml_cib);
-	return NULL;
+	
+	return xml_cib_copy;
 }
 
 crm_data_t*
@@ -1007,20 +937,20 @@ apply_cib_diff(crm_data_t *old, crm_data_t *diff, crm_data_t **new)
 	int diff_del_epoch  = 0;
 	int diff_del_admin_epoch = 0;
 
-	CRM_CHECK(diff != NULL, return FALSE);
-	CRM_CHECK(old != NULL, return FALSE);
+	CRM_DEV_ASSERT(diff != NULL);
+	CRM_DEV_ASSERT(old != NULL);
 	
 	value = crm_element_value(old, XML_ATTR_GENERATION_ADMIN);
-	this_admin_epoch = crm_parse_int(value, "0");
+	this_admin_epoch = atoi(value?value:"0");
 	crm_debug_3("%s=%d (%s)", XML_ATTR_GENERATION_ADMIN,
 		  this_admin_epoch, value);
 	
 	value = crm_element_value(old, XML_ATTR_GENERATION);
-	this_epoch = crm_parse_int(value, "0");
+	this_epoch = atoi(value?value:"0");
 	crm_debug_3("%s=%d (%s)", XML_ATTR_GENERATION, this_epoch, value);
 	
 	value = crm_element_value(old, XML_ATTR_NUMUPDATES);
-	this_updates = crm_parse_int(value, "0");
+	this_updates = atoi(value?value:"0");
 	crm_debug_3("%s=%d (%s)", XML_ATTR_NUMUPDATES, this_updates, value);
 	
 	cib_diff_version_details(
@@ -1056,6 +986,7 @@ apply_cib_diff(crm_data_t *old, crm_data_t *diff, crm_data_t **new)
 			cl_msg_remove(tmp, XML_ATTR_GENERATION_ADMIN);
 			cl_msg_remove(tmp, XML_ATTR_GENERATION);
 			cl_msg_remove(tmp, XML_ATTR_NUMUPDATES);
+			CRM_DEV_ASSERT(tmp->nfields == len - 3);
 		}
 		
 		tmp = find_xml_node(diff_copy, "diff-added", TRUE);
@@ -1064,6 +995,7 @@ apply_cib_diff(crm_data_t *old, crm_data_t *diff, crm_data_t **new)
 			cl_msg_remove(tmp, XML_ATTR_GENERATION_ADMIN);
 			cl_msg_remove(tmp, XML_ATTR_GENERATION);
 			cl_msg_remove(tmp, XML_ATTR_NUMUPDATES);
+			CRM_DEV_ASSERT(tmp->nfields == len - 3);
 		}
 		
 		result = apply_xml_diff(old, diff_copy, new);
@@ -1077,92 +1009,15 @@ apply_cib_diff(crm_data_t *old, crm_data_t *diff, crm_data_t **new)
 	return result;
 }
 
-gboolean xml_has_child(crm_data_t *data, const char *name);
-
-gboolean
-xml_has_child(crm_data_t *data, const char *name) 
-{
-	xml_child_iter_filter(data, child, name,
-		return TRUE;
-		);
-	return FALSE;
-}
-
-gboolean
-cib_config_changed(crm_data_t *old_cib, crm_data_t *new_cib, crm_data_t **result)
-{
-	gboolean config_changes = FALSE;
-	const char *tag = NULL;
-	crm_data_t *diff = NULL;
-	crm_data_t *dest = NULL;
-
-	if(result) {
-		*result = NULL;
-	}
-
-	diff = diff_xml_object(old_cib, new_cib, FALSE);
-	if(diff == NULL) {
-		return FALSE;
-	}
-
-	tag = "diff-removed";
-	dest = find_xml_node(diff, tag, FALSE);
-	if(dest) {
-		dest = find_xml_node(dest, "cib", FALSE);
-		
-	}
-
-	if(dest) {
-		if(xml_has_child(dest, "status")) {
-			cl_msg_remove(dest, "status");
-		}
-		if(xml_has_children(dest)) {
-			config_changes = TRUE;
-		}
-	}
-
-	tag = "diff-added";
-	dest = find_xml_node(diff, tag, FALSE);
-	if(dest) {
-		dest = find_xml_node(dest, "cib", FALSE);
-	}
-
-	if(dest) {
-		if(xml_has_child(dest, "status")) {
-			cl_msg_remove(dest, "status");
-		}
-
-		xml_prop_iter(dest, name, value,
-			      if(safe_str_neq(name, XML_ATTR_NUMUPDATES)) {
-				  config_changes = TRUE;
-				  break;
-			      }
-		    );
-		
-		if(xml_has_children(dest)) {
-			config_changes = TRUE;
-		}
-	}
-
-	
-	if(result) {
-		*result = diff;
-	} else {
-		free_xml(diff);
-	}
-	
-	return config_changes;
-}
-
 crm_data_t *
 diff_cib_object(crm_data_t *old_cib, crm_data_t *new_cib, gboolean suppress)
 {
+	crm_data_t *diff = diff_xml_object(old_cib, new_cib, suppress);
+
 	crm_data_t *dest = NULL;
 	crm_data_t *src = NULL;
 	const char *name = NULL;
 	const char *value = NULL;
-
-	crm_data_t *diff = diff_xml_object(old_cib, new_cib, suppress);
 	
 	/* add complete version information */
 	src = old_cib;
@@ -1237,13 +1092,13 @@ log_cib_diff(int log_level, crm_data_t *diff, const char *function)
 		&del_admin_epoch, &del_epoch, &del_updates);
 
 	if(add_updates != del_updates) {
-		do_crm_log(log_level, "%s: Diff: --- %d.%d.%d", function,
+		do_crm_log(log_level, NULL, function, "Diff: --- %d.%d.%d",
 			   del_admin_epoch, del_epoch, del_updates);
-		do_crm_log(log_level, "%s: Diff: +++ %d.%d.%d", function,
+		do_crm_log(log_level, NULL, function, "Diff: +++ %d.%d.%d",
 			   add_admin_epoch, add_epoch, add_updates);
 	} else if(diff != NULL) {
-		do_crm_log(log_level,
-			   "%s: Local-only Change: %d.%d.%d", function,
+		do_crm_log(log_level, NULL, function,
+			   "Local-only Change: %d.%d.%d",
 			   add_admin_epoch, add_epoch, add_updates);
 	}
 	
@@ -1263,13 +1118,13 @@ cib_version_details(
 		
 	} else {
 		value = crm_element_value(cib, XML_ATTR_GENERATION_ADMIN);
-		*admin_epoch = crm_parse_int(value, "-1");
+		*admin_epoch = atoi(value?value:"-1");
 
 		value  = crm_element_value(cib, XML_ATTR_GENERATION);
-		*epoch = crm_parse_int(value, "-1");
+		*epoch = atoi(value?value:"-1");
 
 		value = crm_element_value(cib, XML_ATTR_NUMUPDATES);
-		*updates = crm_parse_int(value, "-1");
+		*updates = atoi(value?value:"-1");
 	}
 	return TRUE;	
 }
@@ -1308,12 +1163,12 @@ get_object_root(const char *object_type, crm_data_t *the_root)
 
 	if(object_type == NULL
 	   || strlen(object_type) == 0
-	   || safe_str_eq(XML_CIB_TAG_SECTION_ALL, object_type)
+	   || safe_str_eq("all", object_type)
 	   || safe_str_eq(XML_TAG_CIB, object_type)) {
 		/* get the whole cib */
 		return the_root;
 
-	} else if(strcasecmp(object_type, XML_CIB_TAG_STATUS) == 0) {
+	} else if(strcmp(object_type, XML_CIB_TAG_STATUS) == 0) {
 		/* these live in a different place */
 		tmp_node = find_xml_node(the_root, XML_CIB_TAG_STATUS, FALSE);
 
@@ -1355,62 +1210,81 @@ get_crm_option(crm_data_t *cib, const char *name, gboolean do_warn)
 	if(safe_str_eq(value, "")) {
 		value = NULL;
 	}
-	
 	return value;
+	
 }
 
+
 crm_data_t*
 create_cib_fragment_adv(
-	crm_data_t *update, const char *update_section, const char *source)
+	crm_data_t *update, const char *section, const char *source)
 {
 	crm_data_t *cib = NULL;
 	gboolean whole_cib = FALSE;
+	crm_data_t *fragment = create_xml_node(NULL, XML_TAG_FRAGMENT);
 	crm_data_t *object_root  = NULL;
+	char *auto_section = NULL;
 	const char *update_name = NULL;
-	char *local_section = NULL;
-
-/* 	crm_debug("Creating a blank fragment: %s", update_section); */
+	if(update != NULL) {
+		update_name = crm_element_name(update);
+	}
+	auto_section = cib_pluralSection(update_name);
 	
-	if(update == NULL && update_section == NULL) {
+	if(update == NULL && section == NULL) {
 		crm_debug_3("Creating a blank fragment");
 		update = createEmptyCib();
-		crm_xml_add(cib, XML_ATTR_ORIGIN, source);
-		return update;
+		section = auto_section;
 
 	} else if(update == NULL) {
 		crm_err("No update to create a fragment for");
+		cib = createEmptyCib();
+		return NULL;
+		
+	} else if(section == NULL) {
+		section = auto_section;
+
+	} else if(strcmp(auto_section, section) != 0) {
+		crm_err("Values for update (tag=%s) and section (%s)"
+			" were not consistent", crm_element_name(update), section);
+		crm_free(auto_section);
 		return NULL;
 		
-	} else if(update_section == NULL) {
-		local_section = cib_pluralSection(update_name);
-		update_section = local_section;
 	}
 
-	if(safe_str_eq(crm_element_name(update), XML_TAG_CIB)) {
+	if(safe_str_eq(section, "all")
+	   && safe_str_eq(crm_element_name(update), XML_TAG_CIB)) {
 		whole_cib = TRUE;
 	}
 	
+	crm_xml_add(fragment, XML_ATTR_SECTION, section);
+
 	if(whole_cib == FALSE) {
 		cib = createEmptyCib();
-		crm_xml_add(cib, XML_ATTR_ORIGIN, source);
-		object_root = get_object_root(update_section, cib);
+		crm_xml_add(cib, "debug_source", source);
+		object_root = get_object_root(section, cib);
 		add_node_copy(object_root, update);
+		add_node_copy(fragment, cib);
+		free_xml(cib);
+		cib = find_xml_node(fragment, XML_TAG_CIB, TRUE);
 
 	} else {
-		cib = copy_xml(update);
-		crm_xml_add(cib, XML_ATTR_ORIGIN, source);
+		add_node_copy(fragment, update);
+		cib = find_xml_node(fragment, XML_TAG_CIB, TRUE);
+		crm_xml_add(cib, "debug_source", source);
 	}
+	
+	crm_free(auto_section);
 
-	crm_free(local_section);
 	crm_debug_3("Verifying created fragment");
 	if(verifyCibXml(cib) == FALSE) {
 		crm_err("Fragment creation failed");
-		crm_log_xml_err(cib, "[src]");		
-		free_xml(cib);
-		cib = NULL;
+		crm_log_xml_err(update, "[src]");
+		crm_log_xml_err(fragment, "[created]");
+		free_xml(fragment);
+		fragment = NULL;
 	}
 	
-	return cib;
+	return fragment;
 }
 
 /*
@@ -1427,8 +1301,13 @@ createEmptyCib(void)
 	config = create_xml_node(cib_root, XML_CIB_TAG_CONFIGURATION);
 	status = create_xml_node(cib_root, XML_CIB_TAG_STATUS);
 
+	set_node_tstamp(cib_root);
+	set_node_tstamp(config);
+	set_node_tstamp(status);
+	
 /* 	crm_xml_add(cib_root, "version", "1"); */
 	crm_xml_add(cib_root, "generated", XML_BOOLEAN_TRUE);
+	crm_xml_add(cib_root, XML_ATTR_CIB_REVISION, cib_feature_revision_s);
 
 	create_xml_node(config, XML_CIB_TAG_CRMCONFIG);
 	create_xml_node(config, XML_CIB_TAG_NODES);
@@ -1449,32 +1328,28 @@ createEmptyCib(void)
 gboolean
 verifyCibXml(crm_data_t *cib)
 {
-	int lpc = 0;
 	gboolean is_valid = TRUE;
 	crm_data_t *tmp_node = NULL;
-
-	const char *sections[] = {
-		XML_CIB_TAG_NODES,
-		XML_CIB_TAG_RESOURCES,
-		XML_CIB_TAG_CONSTRAINTS,
-		XML_CIB_TAG_STATUS,
-		XML_CIB_TAG_CRMCONFIG
-	};
 	
 	if (cib == NULL) {
 		crm_warn("CIB was empty.");
 		return FALSE;
 	}
+	
+	tmp_node = get_object_root(XML_CIB_TAG_NODES, cib);
+	if (tmp_node == NULL) { is_valid = FALSE; }
 
-	/* basic tests... are the standard section all there */
-	for(lpc = 0; lpc < DIMOF(sections); lpc++) {
-		tmp_node = get_object_root(sections[lpc], cib);
-		if (tmp_node == NULL) {
-			crm_warn("Section %s is not present in the CIB",
-				 sections[lpc]);
-			is_valid = FALSE;
-		}
-	}
+	tmp_node = get_object_root(XML_CIB_TAG_RESOURCES, cib);
+	if (tmp_node == NULL) { is_valid = FALSE; }
+
+	tmp_node = get_object_root(XML_CIB_TAG_CONSTRAINTS, cib);
+	if (tmp_node == NULL) { is_valid = FALSE; }
+
+	tmp_node = get_object_root(XML_CIB_TAG_STATUS, cib);
+	if (tmp_node == NULL) { is_valid = FALSE; }
+
+	tmp_node = get_object_root(XML_CIB_TAG_CRMCONFIG, cib);
+	if (tmp_node == NULL) { is_valid = FALSE; }
 
 	/* more integrity tests */
 
