@@ -1,3 +1,4 @@
+/* $Id: ipc.c,v 1.13 2005/09/11 20:56:56 andrew Exp $ */
 /* 
  * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -16,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <sys/param.h>
 
@@ -45,6 +46,7 @@
 #include <ha_msg.h>
 
 
+#include <crm/dmalloc_wrapper.h>
 
 gboolean 
 send_ha_message(ll_cluster_t *hb_conn, HA_Message *msg, const char *node, gboolean force_ordered)
@@ -59,67 +61,55 @@ send_ha_message(ll_cluster_t *hb_conn, HA_Message *msg, const char *node, gboole
 		crm_err("No heartbeat connection specified");
 		all_is_good = FALSE;
 
-	} else if(hb_conn->llc_ops->chan_is_connected(hb_conn) == FALSE) {
+	} else if(hb_conn->llc_ops->chan_is_connected(hb_conn) != HA_OK) {
 		crm_err("Not connected to Heartbeat");
 		all_is_good = FALSE;
 		
+	} else if(get_stringlen(msg) >= MAXMSG) {
+		crm_err("Message is too large to send");
+		all_is_good = FALSE;
+
 	} else if(node != NULL) {
 		if(hb_conn->llc_ops->send_ordered_nodemsg(
 			   hb_conn, msg, node) != HA_OK) {
+			IPC_Channel *ipc = hb_conn->llc_ops->ipcchan(hb_conn);
 			all_is_good = FALSE;
 			crm_err("Send failed");
-			
+			CRM_DEV_ASSERT(ipc->send_queue->current_qlen < ipc->send_queue->max_qlen);
 		} else {
 			crm_debug_2("Message sent...");
 		}
-
 	} else if(force_ordered) {
 		if(hb_conn->llc_ops->send_ordered_clustermsg(hb_conn, msg) != HA_OK) {
+			IPC_Channel *ipc = hb_conn->llc_ops->ipcchan(hb_conn);
 			all_is_good = FALSE;
 			crm_err("Broadcast Send failed");
+			CRM_DEV_ASSERT(ipc->send_queue->current_qlen < ipc->send_queue->max_qlen);
 		} else {
 			crm_debug_2("Broadcast message sent...");
 		}
 	} else {
 		if(hb_conn->llc_ops->sendclustermsg(hb_conn, msg) != HA_OK) {
+			IPC_Channel *ipc = hb_conn->llc_ops->ipcchan(hb_conn);
 			all_is_good = FALSE;
 			crm_err("Broadcast Send failed");
-
+			CRM_DEV_ASSERT(ipc->send_queue->current_qlen < ipc->send_queue->max_qlen);
 		} else {
 			crm_debug_2("Broadcast message sent...");
 		}
 	}
-
-	if(all_is_good == FALSE && hb_conn != NULL) {
-		IPC_Channel *ipc = NULL;
-		IPC_Queue *send_q = NULL;
-		
-		if(hb_conn->llc_ops->chan_is_connected(hb_conn) != HA_OK) {
-			ipc = hb_conn->llc_ops->ipcchan(hb_conn);
-		}
-		if(ipc != NULL) {
-/* 			ipc->ops->resume_io(ipc); */
-			send_q = ipc->send_queue;
-		}
-		if(send_q != NULL) {
-			CRM_CHECK(send_q->current_qlen < send_q->max_qlen, ;);
-		}
-	}
 	
 	crm_log_message_adv(all_is_good?LOG_MSG:LOG_WARNING,"HA[outbound]",msg);
 	return all_is_good;
 }
 
+#define ipc_log(fmt...) do_crm_log(server?LOG_WARNING:LOG_ERR, __FILE__, __FUNCTION__, fmt)
+
 /* frees msg */
 gboolean 
-send_ipc_message(IPC_Channel *ipc_client, HA_Message *msg)
+crm_send_ipc_message(IPC_Channel *ipc_client, HA_Message *msg, gboolean server)
 {
 	gboolean all_is_good = TRUE;
-	int fail_level = LOG_WARNING;
-
-	if(ipc_client != NULL && ipc_client->conntype == IPC_CLIENT) {
-		fail_level = LOG_ERR;
-	}
 
 	if (msg == NULL) {
 		crm_err("cant send NULL message");
@@ -130,32 +120,28 @@ send_ipc_message(IPC_Channel *ipc_client, HA_Message *msg)
 		all_is_good = FALSE;
 
 	} else if(ipc_client->ops->get_chan_status(ipc_client) != IPC_CONNECT) {
-		do_crm_log(fail_level, "IPC Channel to %d is not connected",
-			      (int)ipc_client->farside_pid);
+		ipc_log("IPC Channel is not connected");
+		all_is_good = FALSE;
+
+	} else if(get_stringlen(msg) >= MAXMSG) {
+		crm_err("Message is too large to send");
 		all_is_good = FALSE;
 	}
 
 	if(all_is_good && msg2ipcchan(msg, ipc_client) != HA_OK) {
-		do_crm_log(fail_level, "Could not send IPC message to %d",
-			(int)ipc_client->farside_pid);
+		ipc_log("Could not send IPC, message");
 		all_is_good = FALSE;
 
 		if(ipc_client->ops->get_chan_status(ipc_client) != IPC_CONNECT) {
-			do_crm_log(fail_level,
-				      "IPC Channel to %d is no longer connected",
-				      (int)ipc_client->farside_pid);
+			ipc_log("IPC Channel is no longer connected");
 
-		} else if(ipc_client->conntype == IPC_CLIENT) {
-			if(ipc_client->send_queue->current_qlen >= ipc_client->send_queue->max_qlen) {
-				crm_err("Send queue to %d (size=%d) full.",
-					ipc_client->farside_pid,
-					(int)ipc_client->send_queue->max_qlen);
-			}
+		} else if(server == FALSE) {
+			CRM_DEV_ASSERT(ipc_client->send_queue->current_qlen < ipc_client->send_queue->max_qlen);
 		}
-	}
-/* 	ipc_client->ops->resume_io(ipc_client); */
-	
+	}	
+
 	crm_log_message_adv(all_is_good?LOG_MSG:LOG_WARNING,"IPC[outbound]",msg);
+	crm_msg_del(msg);
 	
 	return all_is_good;
 }
@@ -247,10 +233,11 @@ init_client_ipc_comms_nodispatch(const char *channel_name)
 	char *commpath = NULL;
 	int local_socket_len = 2; /* 2 = '/' + '\0' */
 
+	
 	local_socket_len += strlen(channel_name);
 	local_socket_len += strlen(CRM_SOCK_DIR);
 
-	crm_malloc0(commpath, local_socket_len);
+	crm_malloc0(commpath, sizeof(char)*local_socket_len);
 	if(commpath != NULL) {
 		sprintf(commpath, CRM_SOCK_DIR "/%s", channel_name);
 		commpath[local_socket_len - 1] = '\0';
@@ -265,23 +252,20 @@ init_client_ipc_comms_nodispatch(const char *channel_name)
 
 	if (ch == NULL) {
 		crm_err("Could not access channel on: %s", commpath);
-		crm_free(commpath);
 		return NULL;
 		
 	} else if (ch->ops->initiate_connection(ch) != IPC_OK) {
 		crm_debug("Could not init comms on: %s", commpath);
 		ch->ops->destroy(ch);
-		crm_free(commpath);
 		return NULL;
 	}
 
-	ch->ops->set_recv_qlen(ch, 512);
-	ch->ops->set_send_qlen(ch, 512);
- 	ch->should_send_block = TRUE;
+	ch->ops->set_recv_qlen(ch, 100);
+	ch->ops->set_send_qlen(ch, 100);
+/* 	ch->should_send_block = TRUE; */
 
 	crm_debug_3("Processing of %s complete", commpath);
 
-	crm_free(commpath);
 	return ch;
 }
 
@@ -319,30 +303,32 @@ gboolean
 subsystem_msg_dispatch(IPC_Channel *sender, void *user_data)
 {
 	int lpc = 0;
-	HA_Message *msg = NULL;
+	IPC_Message *msg = NULL;
 	ha_msg_input_t *new_input = NULL;
 	gboolean all_is_well = TRUE;
 	const char *sys_to;
 	const char *task;
 
-	while(IPC_ISRCONN(sender)) {
+	while(sender->ops->is_message_pending(sender)) {
 		gboolean process = FALSE;
-		if(sender->ops->is_message_pending(sender) == 0) {
+		if (sender->ch_status == IPC_DISCONNECT) {
+			/* The message which was pending for us is that
+			 * the IPC status is now IPC_DISCONNECT */
+			crm_debug("Channel is disconnected");
 			break;
 		}
-
-		msg = msgfromIPC_noauth(sender);
+		if (sender->ops->recv(sender, &msg) != IPC_OK) {
+			perror("Receive failure:");
+			return !all_is_well;
+		}
 		if (msg == NULL) {
-			crm_err("No message from %d this time",
-				sender->farside_pid);
+			crm_err("No message this time");
 			continue;
 		}
 
 		lpc++;
-		new_input = new_ha_msg_input(msg);
-		crm_msg_del(msg);
-		msg = NULL;
-		
+		new_input = new_ipc_msg_input(msg);
+		msg->msg_done(msg);
 		crm_log_message(LOG_MSG, new_input->msg);
 
 		sys_to = cl_get_string(new_input->msg, F_CRM_SYS_TO);
@@ -397,17 +383,17 @@ subsystem_msg_dispatch(IPC_Channel *sender, void *user_data)
 		}
 		
 		delete_ha_msg_input(new_input);
-		new_input = NULL;
-
-		if(sender->ch_status == IPC_CONNECT) {
-			break;
-		}
+		msg = NULL;
 	}
 
+	/* clean up after a break */
+	if(msg != NULL) {
+		msg->msg_done(msg);
+	}
+	
 	crm_debug_2("Processed %d messages", lpc);
 	if (sender->ch_status != IPC_CONNECT) {
-		crm_err("The server %d has left us: Shutting down...NOW",
-			sender->farside_pid);
+		crm_err("The server has left us: Shutting down...NOW");
 
 		exit(1); /* shutdown properly later */
 		
