@@ -1,3 +1,4 @@
+/* $Id: iso8601.c,v 1.11 2005/08/27 17:13:17 andrew Exp $ */
 /* 
  * Copyright (C) 2005 Andrew Beekhof <andrew@beekhof.net>
  * 
@@ -28,7 +29,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <crm/crm.h>
 #include <time.h>
 #include <ctype.h>
@@ -61,13 +62,13 @@ void normalize_time(ha_time_t *a_time);
 #define GMTOFF(tm) (timezone)
 #endif
 
-char *
-date_to_string(ha_time_t *date_time, int flags) 
+
+void
+log_date(int log_level, const char *prefix, ha_time_t *date_time, int flags) 
 {
 	char *date_s = NULL;
 	char *time_s = NULL;
 	char *offset_s = NULL;
-	char *result_s = NULL;
 	ha_time_t *dt = NULL;
 	
 	if(flags & ha_log_local) {
@@ -77,13 +78,11 @@ date_to_string(ha_time_t *date_time, int flags)
 		dt = date_time->normalized;
 	}
 
-	CRM_CHECK(dt != NULL, return NULL);
+	CRM_DEV_ASSERT(dt != NULL); if(crm_assert_failed) { return; }
 	
 	if(flags & ha_log_date) {
-		crm_malloc0(date_s, 32);
+		crm_malloc0(date_s, sizeof(char)*(32));
 		if(date_s == NULL) {
-			return NULL;
-			
 		} else if(flags & ha_date_weeks) {
 			snprintf(date_s, 31, "%d-W%.2d-%d",
 				 dt->weekyears, dt->weeks, dt->weekdays);
@@ -98,9 +97,9 @@ date_to_string(ha_time_t *date_time, int flags)
 	}
 	if(flags & ha_log_time) {
 		int offset = 0;
-		crm_malloc0(time_s, 32);
+		crm_malloc0(time_s, sizeof(char)*(32));
 		if(time_s == NULL) {
-			return NULL;
+			return;
 		} 
 
 		snprintf(time_s, 31, "%.2d:%.2d:%.2d",
@@ -110,7 +109,7 @@ date_to_string(ha_time_t *date_time, int flags)
 			offset =(dt->offset->hours * 100) + dt->offset->minutes;
 		}
 		
-		crm_malloc0(offset_s, 32);
+		crm_malloc0(offset_s, sizeof(char)*(32));
 		if((flags & ha_log_local) == 0 || offset == 0) {
 			snprintf(offset_s, 31, "Z");
 
@@ -127,29 +126,14 @@ date_to_string(ha_time_t *date_time, int flags)
 				 offset>0?"+":"-", hr, mins);
 		}
 	}
-
-	crm_malloc0(result_s, 100);
-
-	snprintf(result_s, 100, "%s%s%s%s",
-		 date_s?date_s:"", (date_s!=NULL&&time_s!=NULL)?" ":"",
-		 time_s?time_s:"", offset_s?offset_s:"");
+	crm_log_maybe(log_level, "%s%s%s%s%s%s",
+		      prefix?prefix:"", prefix?": ":"",
+		      date_s?date_s:"", (date_s!=NULL&&time_s!=NULL)?" ":"",
+		      time_s?time_s:"", offset_s?offset_s:"");
 
 	crm_free(date_s);
 	crm_free(time_s);
 	crm_free(offset_s);
-
-	return result_s;
-}
-
-void
-log_date(int log_level, const char *prefix, ha_time_t *date_time, int flags) 
-{
-	char *date_s = date_to_string(date_time, flags);
-	do_crm_log(log_level, "%s%s%s",
-		      prefix?prefix:"", prefix?": ":"",
-		      date_s?date_s:"__invalid_date__");
-
-	crm_free(date_s);
 }
 
 void
@@ -170,7 +154,7 @@ parse_time_offset(char **offset_str)
 		
 	} else if((*offset_str)[0] == '+'
 		  || (*offset_str)[0] == '-'
-		  || isdigit((int) (*offset_str)[0])) {
+		  || isdigit((*offset_str)[0])) {
 		gboolean negate = FALSE;
 		if((*offset_str)[0] == '-') {
 			negate = TRUE;
@@ -210,8 +194,8 @@ parse_time(char **time_str, ha_time_t *a_time, gboolean with_offset)
 		new_time = new_ha_date(FALSE);
 	}
 
-	CRM_CHECK(new_time != NULL, return NULL);
-	CRM_CHECK(new_time->has != NULL, free_ha_date(new_time); return NULL);
+	CRM_DEV_ASSERT(new_time != NULL);
+	CRM_DEV_ASSERT(new_time->has != NULL);
 
 	crm_debug_4("Get hours...");
 	if(parse_int(time_str, 2, 24, &new_time->hours)) {
@@ -230,7 +214,7 @@ parse_time(char **time_str, ha_time_t *a_time, gboolean with_offset)
 	   
 	if(with_offset) {
 		crm_debug_4("Get offset...");
-		while(isspace((int) (*time_str)[0])) {
+		while(isspace((*time_str)[0])) {
 			(*time_str)++;
 		}
 
@@ -243,8 +227,8 @@ parse_time(char **time_str, ha_time_t *a_time, gboolean with_offset)
 void
 normalize_time(ha_time_t *a_time)
 {
-	CRM_CHECK(a_time != NULL, return);
-	CRM_CHECK(a_time->has != NULL, return);
+	CRM_DEV_ASSERT(a_time != NULL);
+	CRM_DEV_ASSERT(a_time->has != NULL);
 
 	if(a_time->normalized == NULL) {
 		crm_malloc0(a_time->normalized, sizeof(ha_time_t));
@@ -265,7 +249,7 @@ normalize_time(ha_time_t *a_time)
 			sub_seconds(a_time->normalized,a_time->offset->seconds);
 		}
 	}
-	CRM_CHECK(is_date_sane(a_time), return);
+	CRM_DEV_ASSERT(is_date_sane(a_time));
 }
 
 
@@ -276,21 +260,12 @@ parse_date(char **date_str)
 	gboolean is_done = FALSE;
 	gboolean converted = FALSE;
 	ha_time_t *new_time = NULL;
+	crm_malloc0(new_time, sizeof(ha_time_t));
+	crm_malloc0(new_time->has, sizeof(ha_has_time_t));
 
-	CRM_CHECK(date_str != NULL, return NULL);
-	CRM_CHECK(strlen(*date_str) > 0, return NULL);
+	CRM_DEV_ASSERT(date_str != NULL);
+	CRM_DEV_ASSERT(strlen(*date_str) > 0);
 	
-	if((*date_str)[0] == 'T' || (*date_str)[2] == ':') {
-	    /* Just a time supplied - Infer current date */
-	    new_time = new_ha_date(TRUE);
-	    parse_time(date_str, new_time, TRUE);
-	    is_done = TRUE;
-
-	} else {
-	    crm_malloc0(new_time, sizeof(ha_time_t));
-	    crm_malloc0(new_time->has, sizeof(ha_has_time_t));
-	}
-
 	while(is_done == FALSE) {
 		char ch = (*date_str)[0];
 		crm_debug_5("Switching on ch=%c (len=%d)",
@@ -307,7 +282,7 @@ parse_date(char **date_str)
 			break;
 			
 		} else if(ch == 'W') {
-			CRM_CHECK(new_time->has->weeks == FALSE, ;);
+			CRM_DEV_ASSERT(new_time->has->weeks == FALSE);
 			(*date_str)++;
 			if(parse_int(date_str, 2, 53, &new_time->weeks)){
 				new_time->has->weeks = TRUE;
@@ -356,7 +331,7 @@ parse_date(char **date_str)
 			parse_time(date_str, new_time, TRUE);
 			is_done = TRUE;
 
-		} else if(isdigit((int) ch)) {
+		} else if(isdigit(ch)) {
 			if(new_time->has->years == FALSE
 			   && parse_int(date_str, 4, 9999, &new_time->years)) {
 				new_time->has->years = TRUE;
@@ -402,7 +377,7 @@ parse_date(char **date_str)
 	
 	log_date(LOG_DEBUG_3, "Unpacked", new_time, ha_log_date|ha_log_time);
 
-	CRM_CHECK(is_date_sane(new_time), return NULL);
+	CRM_DEV_ASSERT(is_date_sane(new_time));
 	
 	return new_time;
 }
@@ -412,16 +387,16 @@ parse_time_duration(char **interval_str)
 {
 	gboolean is_time = FALSE;
 	ha_time_t *diff = NULL;
-
-	CRM_CHECK(interval_str != NULL, goto bail);
-	CRM_CHECK(strlen(*interval_str) > 0, goto bail);
-	CRM_CHECK((*interval_str)[0] == 'P', goto bail);
-	(*interval_str)++;
-
 	crm_malloc0(diff, sizeof(ha_time_t));
 	crm_malloc0(diff->has, sizeof(ha_has_time_t));
+
+	CRM_DEV_ASSERT(interval_str != NULL);
+	CRM_DEV_ASSERT(strlen(*interval_str) > 0);
 	
-	while(isspace((int) (*interval_str)[0]) == FALSE) {
+	CRM_DEV_ASSERT((*interval_str)[0] == 'P');
+	(*interval_str)++;
+	
+	while(isspace((*interval_str)[0]) == FALSE) {
 		int an_int = 0;
 		char ch = 0;
 
@@ -476,13 +451,6 @@ parse_time_duration(char **interval_str)
 		}
 	}
 	return diff;
-
-  bail:
-	if(diff) {
-		crm_free(diff->has);
-	}
-	crm_free(diff);
-	return NULL;
 }
 
 ha_time_period_t*
@@ -491,12 +459,12 @@ parse_time_period(char **period_str)
 	gboolean invalid = FALSE;
 	const char *original = *period_str;
 	ha_time_period_t *period = NULL;
+	crm_malloc0(period, sizeof(ha_time_period_t));
 
-	CRM_CHECK(period_str != NULL, return NULL);
-	CRM_CHECK(strlen(*period_str) > 0, return NULL);
+	CRM_DEV_ASSERT(period_str != NULL);
+	CRM_DEV_ASSERT(strlen(*period_str) > 0);
 
 	tzset();
-	crm_malloc0(period, sizeof(ha_time_period_t));
 	
 	if((*period_str)[0] == 'P') {
 		period->diff = parse_time_duration(period_str);
@@ -505,7 +473,7 @@ parse_time_period(char **period_str)
 	}
 
 	if((*period_str)[0] != 0) {
-		CRM_CHECK((*period_str)[0] == '/', invalid = TRUE; goto bail);
+		CRM_DEV_ASSERT((*period_str)[0] == '/');
 		(*period_str)++;
 		
 		if((*period_str)[0] == 'P') {
@@ -524,11 +492,9 @@ parse_time_period(char **period_str)
 		
 		ha_set_timet_time(period->start, &now);
 		normalize_time(period->start);
-
 	} else {
-		invalid = TRUE;
-		CRM_CHECK((*period_str)[0] == '/', goto bail);
-		goto bail;
+		CRM_DEV_ASSERT((*period_str)[0] == '/');
+		return NULL;
 	}
 	
 	
@@ -546,7 +512,6 @@ parse_time_period(char **period_str)
 		invalid = TRUE;
 	}
 
-  bail:
 	if(invalid) {
 		crm_free(period->start);
 		crm_free(period->end);
@@ -609,20 +574,20 @@ weeks_in_year(int year)
 gboolean
 convert_from_gregorian(ha_time_t *a_date)
 {
-	CRM_CHECK(gregorian_to_ordinal(a_date), return FALSE);
-	CRM_CHECK(ordinal_to_weekdays(a_date), return FALSE);
+	CRM_DEV_ASSERT(gregorian_to_ordinal(a_date));
+	CRM_DEV_ASSERT(ordinal_to_weekdays(a_date));
 	return TRUE;
 }
 
 gboolean
 gregorian_to_ordinal(ha_time_t *a_date)
 {
-	CRM_CHECK(a_date->has->years, return FALSE);
-	CRM_CHECK(a_date->has->months, return FALSE);
-	CRM_CHECK(a_date->has->days, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->years);
+	CRM_DEV_ASSERT(a_date->has->months);
+	CRM_DEV_ASSERT(a_date->has->days);
 
-	CRM_CHECK(a_date->months > 0, return FALSE);
-	CRM_CHECK(a_date->days > 0, return FALSE);
+	CRM_DEV_ASSERT(a_date->months > 0);
+	CRM_DEV_ASSERT(a_date->days > 0);
 
 	a_date->yeardays = month2days[a_date->months-1];
 	a_date->yeardays += a_date->days;
@@ -641,22 +606,21 @@ gregorian_to_ordinal(ha_time_t *a_date)
 gboolean
 convert_from_ordinal(ha_time_t *a_date)
 {
-	CRM_CHECK(ordinal_to_gregorian(a_date), return FALSE);
-	CRM_CHECK(ordinal_to_weekdays(a_date), return FALSE);
+	CRM_DEV_ASSERT(ordinal_to_gregorian(a_date));
+	CRM_DEV_ASSERT(ordinal_to_weekdays(a_date));
 	return TRUE;
 }
 
 
 gboolean ordinal_to_gregorian(ha_time_t *a_date) 
 {
-	/* Day of the year this month ends on */
-	int m_end = 0;
+	CRM_DEV_ASSERT(a_date->has->years);
+	CRM_DEV_ASSERT(a_date->has->yeardays);
 
-	CRM_CHECK(a_date->has->years, return FALSE);
-	CRM_CHECK(a_date->has->yeardays, return FALSE);
-
-	CRM_CHECK(a_date->yeardays > 0, return FALSE);
+	CRM_DEV_ASSERT(a_date->yeardays > 0);
 	
+	a_date->days = a_date->yeardays;
+	a_date->months = 11;
 	if(is_leap_year(a_date->years) && a_date->yeardays > 366) {
 		crm_err("Year %.4d only has 366 days (supplied %.3d)",
 			a_date->years, a_date->yeardays);
@@ -668,21 +632,27 @@ gboolean ordinal_to_gregorian(ha_time_t *a_date)
 		a_date->yeardays = 365;
 	}
 	
-	a_date->days = a_date->yeardays;
-	a_date->months = 0;
-	do {
-		a_date->months++;
-		m_end += days_per_month(a_date->months, a_date->years);
-		a_date->days -= days_per_month(a_date->months-1, a_date->years);
-	
-		crm_debug_6("month %d: %d vs. %d - current day: %d",
+	while(a_date->months > 0
+	      && a_date->yeardays <= month2days[a_date->months]) {
+		crm_debug_6("month %d: %d vs. %d",
 			    a_date->months, a_date->yeardays,
-			    m_end, a_date->days);
-	} while (a_date->months < 12 && m_end < a_date->yeardays);
+			    month2days[a_date->months]);
+		(a_date->months)--;
+	}
 
-	CRM_CHECK(a_date->months > 0, return FALSE);
-	CRM_CHECK(a_date->days <= days_per_month(a_date->months, a_date->years), 
-			return FALSE);
+	a_date->days -= month2days[a_date->months];
+	(a_date->months)++;
+	
+	CRM_DEV_ASSERT(a_date->months > 0);
+
+	if(is_leap_year(a_date->years) && a_date->months > 2) {
+		(a_date->days)--;
+	}
+	if(a_date->days == 0) {
+		/* annoying underflow */
+		a_date->days = days_per_month(a_date->months, a_date->years);
+		(a_date->months)--;
+	}
 
 	a_date->has->days = TRUE;
 	a_date->has->months = TRUE;
@@ -703,9 +673,9 @@ ordinal_to_weekdays(ha_time_t *a_date)
 	int jan1 = january1(a_date->years);
 	int h = -1;
 
-	CRM_CHECK(a_date->has->years, return FALSE);
-	CRM_CHECK(a_date->has->yeardays, return FALSE);
-	CRM_CHECK(a_date->yeardays > 0, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->years);
+	CRM_DEV_ASSERT(a_date->has->yeardays);
+	CRM_DEV_ASSERT(a_date->yeardays > 0);
 	
 	h = a_date->yeardays + jan1 - 1;
  	a_date->weekdays = 1 + ((h-1) % 7);
@@ -755,13 +725,13 @@ convert_from_weekdays(ha_time_t *a_date)
 	gboolean conversion = FALSE;
 	int jan1 = january1(a_date->weekyears);
 
-	CRM_CHECK(a_date->has->weekyears, return FALSE);
-	CRM_CHECK(a_date->has->weeks, return FALSE);
-	CRM_CHECK(a_date->has->weekdays, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->weekyears);
+	CRM_DEV_ASSERT(a_date->has->weeks);
+	CRM_DEV_ASSERT(a_date->has->weekdays);
 
-	CRM_CHECK(a_date->weeks > 0, return FALSE);
-	CRM_CHECK(a_date->weekdays > 0, return FALSE);
-	CRM_CHECK(a_date->weekdays < 8, return FALSE);
+	CRM_DEV_ASSERT(a_date->weeks > 0);
+	CRM_DEV_ASSERT(a_date->weekdays > 0);
+	CRM_DEV_ASSERT(a_date->weekdays < 8);
 	
 	a_date->has->years = TRUE;
 	a_date->years = a_date->weekyears;
@@ -797,8 +767,8 @@ ha_set_time(ha_time_t *lhs, ha_time_t *rhs, gboolean offset)
 {
 	crm_debug_6("lhs=%p, rhs=%p, offset=%d", lhs, rhs, offset);
 
-	CRM_CHECK(lhs != NULL && rhs != NULL, return);
-	CRM_CHECK(lhs->has != NULL && rhs->has != NULL, return);
+	CRM_DEV_ASSERT(lhs != NULL && rhs != NULL);
+	CRM_DEV_ASSERT(lhs->has != NULL && rhs->has != NULL);
 	
 	lhs->years = rhs->years;
 	lhs->has->years = rhs->has->years;
@@ -854,7 +824,7 @@ ha_set_tm_time(ha_time_t *lhs, struct tm *rhs)
 		lhs->has->years = TRUE;
 	}
 
-	if(rhs->tm_yday >= 0) {
+	if(rhs->tm_yday > 0) {
 		/* days since January 1 [0-365] */
 		lhs->yeardays = 1 + rhs->tm_yday;
 		lhs->has->yeardays =TRUE;
@@ -876,19 +846,19 @@ ha_set_tm_time(ha_time_t *lhs, struct tm *rhs)
 	convert_from_ordinal(lhs);
 
 	/* months since January [0-11] */
-	CRM_CHECK(rhs->tm_mon < 0  || lhs->months == (1 + rhs->tm_mon), return);
+	CRM_DEV_ASSERT(rhs->tm_mon < 0  || lhs->months == (1 + rhs->tm_mon));
 
 	/* day of the month [1-31] */
-	CRM_CHECK(rhs->tm_mday < 0 || lhs->days == rhs->tm_mday, return);
+	CRM_DEV_ASSERT(rhs->tm_mday < 0 || lhs->days == rhs->tm_mday);
 
 	/* days since Sunday [0-6] */
 	if(wday == 0) {
 		wday= 7;
 	}
+	CRM_DEV_ASSERT(rhs->tm_wday < 0 || lhs->weekdays == wday);
 	
-	CRM_CHECK(rhs->tm_wday < 0 || lhs->weekdays == wday, return);
-	CRM_CHECK(lhs->offset != NULL, return);
-	CRM_CHECK(lhs->offset->has != NULL, return);
+	CRM_DEV_ASSERT(lhs->offset != NULL);
+	CRM_DEV_ASSERT(lhs->offset->has != NULL);
 
 	/* tm_gmtoff == offset from UTC in seconds */
 	h_offset = GMTOFF(rhs) / (3600); 
@@ -916,7 +886,7 @@ ha_time_t *
 add_time(ha_time_t *lhs, ha_time_t *rhs)
 {
 	ha_time_t *answer = NULL;
-	CRM_CHECK(lhs != NULL && rhs != NULL, return NULL);
+	CRM_DEV_ASSERT(lhs != NULL && rhs != NULL);
 	
 	answer = new_ha_date(FALSE);
 	ha_set_time(answer, lhs, TRUE);	
@@ -941,8 +911,6 @@ add_time(ha_time_t *lhs, ha_time_t *rhs)
 	add_minutes(answer, rhs->minutes);
 	add_seconds(answer, rhs->seconds);
 
-	normalize_time(answer);
-	
 	return answer;
 }
 
@@ -951,7 +919,7 @@ ha_time_t *
 subtract_time(ha_time_t *lhs, ha_time_t *rhs)
 {
 	ha_time_t *answer = NULL;
-	CRM_CHECK(lhs != NULL && rhs != NULL, return NULL);
+	CRM_DEV_ASSERT(lhs != NULL && rhs != NULL);
 
 	answer = new_ha_date(FALSE);
 	ha_set_time(answer, lhs, TRUE);	
@@ -968,8 +936,6 @@ subtract_time(ha_time_t *lhs, ha_time_t *rhs)
 	sub_minutes(answer, rhs->minutes);
 	sub_seconds(answer, rhs->seconds);
 
-	normalize_time(answer);
-	
 	return answer;
 }
 
@@ -1011,9 +977,9 @@ parse_int(char **str, int field_width, int uppper_bound, int *result)
 	gboolean fraction = FALSE;
 	gboolean negate = FALSE;
 
-	CRM_CHECK(str != NULL, return FALSE);
-	CRM_CHECK(*str != NULL, return FALSE);
-	CRM_CHECK(result != NULL, return FALSE);
+	CRM_DEV_ASSERT(str != NULL);
+	CRM_DEV_ASSERT(*str != NULL);
+	CRM_DEV_ASSERT(result != NULL);
 
 	*result = 0;
 
@@ -1035,7 +1001,7 @@ parse_int(char **str, int field_width, int uppper_bound, int *result)
 		(*str)++;
 	}
 
-	for(; (fraction || lpc < field_width) && isdigit((int) (*str)[0]); lpc++) {
+	for(; (fraction || lpc < field_width) && isdigit((*str)[0]); lpc++) {
 		if(fraction) {
 			intermediate = ((*str)[0] - '0')/(10^lpc);
 		} else {
@@ -1064,11 +1030,11 @@ parse_int(char **str, int field_width, int uppper_bound, int *result)
 gboolean
 check_for_ordinal(const char *str)
 {
-	if(isdigit((int) str[2]) == FALSE) {
+	if(isdigit(str[2]) == FALSE) {
 		crm_debug_6("char 3 == %c", str[2]);
 		return FALSE;		
 	}
-	if(isspace((int) str[3])) {
+	if(isspace(str[3])) {
 		return TRUE;
 	} else if(str[3] == 0) {
 		return TRUE;
@@ -1148,7 +1114,7 @@ is_date_sane(ha_time_t *a_date)
 	int mdays = 0;
 	int weeks = 0;
 
-	CRM_CHECK(a_date != NULL, return FALSE);
+	CRM_DEV_ASSERT(a_date != NULL);
 	
 	ydays = is_leap_year(a_date->years)?366:365;
 	mdays = days_per_month(a_date->months, a_date->years);
@@ -1156,37 +1122,37 @@ is_date_sane(ha_time_t *a_date)
 	crm_debug_5("max ydays: %d, max mdays: %d, max weeks: %d",
 		    ydays, mdays, weeks);
 	
-	CRM_CHECK(a_date->has->years, return FALSE);
-	CRM_CHECK(a_date->has->weekyears, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->years);
+	CRM_DEV_ASSERT(a_date->has->weekyears);
 
-	CRM_CHECK(a_date->has->months, return FALSE);
-	CRM_CHECK(a_date->months > 0, return FALSE);
-	CRM_CHECK(a_date->months <= 12, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->months);
+	CRM_DEV_ASSERT(a_date->months > 0);
+	CRM_DEV_ASSERT(a_date->months <= 12);
 
-	CRM_CHECK(a_date->has->weeks, return FALSE);
-	CRM_CHECK(a_date->weeks > 0, return FALSE);
-	CRM_CHECK(a_date->weeks <= weeks, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->weeks);
+	CRM_DEV_ASSERT(a_date->weeks > 0);
+	CRM_DEV_ASSERT(a_date->weeks <= weeks);
 
-	CRM_CHECK(a_date->has->days, return FALSE);
-	CRM_CHECK(a_date->days > 0, return FALSE);
-	CRM_CHECK(a_date->days <= mdays, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->days);
+	CRM_DEV_ASSERT(a_date->days > 0);
+	CRM_DEV_ASSERT(a_date->days <= mdays);
 
-	CRM_CHECK(a_date->has->weekdays, return FALSE);
-	CRM_CHECK(a_date->weekdays > 0, return FALSE);
-	CRM_CHECK(a_date->weekdays <= 7, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->weekdays);
+	CRM_DEV_ASSERT(a_date->weekdays > 0);
+	CRM_DEV_ASSERT(a_date->weekdays <= 7);
 
-	CRM_CHECK(a_date->has->yeardays, return FALSE);
-	CRM_CHECK(a_date->yeardays > 0, return FALSE);
-	CRM_CHECK(a_date->yeardays <= ydays, return FALSE);
+	CRM_DEV_ASSERT(a_date->has->yeardays);
+	CRM_DEV_ASSERT(a_date->yeardays > 0);
+	CRM_DEV_ASSERT(a_date->yeardays <= ydays);
 
-	CRM_CHECK(a_date->hours >= 0, return FALSE);
-	CRM_CHECK(a_date->hours < 24, return FALSE);
+	CRM_DEV_ASSERT(a_date->hours >= 0);
+	CRM_DEV_ASSERT(a_date->hours < 24);
 
-	CRM_CHECK(a_date->minutes >= 0, return FALSE);
-	CRM_CHECK(a_date->minutes < 60, return FALSE);
+	CRM_DEV_ASSERT(a_date->minutes >= 0);
+	CRM_DEV_ASSERT(a_date->minutes < 60);
 	
-	CRM_CHECK(a_date->seconds >= 0, return FALSE);
-	CRM_CHECK(a_date->seconds <= 60, return FALSE);
+	CRM_DEV_ASSERT(a_date->seconds >= 0);
+	CRM_DEV_ASSERT(a_date->seconds <= 60);
 	
 	return TRUE;
 }
@@ -1207,14 +1173,6 @@ is_date_sane(ha_time_t *a_date)
 int
 compare_date(ha_time_t *lhs, ha_time_t *rhs)
 {
-	if(lhs == NULL && rhs == NULL) {
-		return 0;
-	} else if(lhs == NULL) {
-		return -1;
-	} else if(rhs == NULL) {
-		return 1;		
-	}
-	
 	normalize_time(lhs);
 	normalize_time(rhs);
 
@@ -1270,7 +1228,7 @@ log_tm_date(int log_level, struct tm *some_tm)
 	tzn = NULL;
 #endif
 
-	do_crm_log(log_level,
+	crm_log_maybe(log_level,
 		      "%.2d/%.2d/%.4d %.2d:%.2d:%.2d %s"
 		      " (wday=%d, yday=%d, dst=%d, offset=%ld)",
 		      some_tm->tm_mday,
