@@ -1,3 +1,4 @@
+/* $Id: client_lib.c,v 1.32 2005/05/17 18:51:22 gshi Exp $ */
 /* 
  * client_lib: heartbeat API client side code
  *
@@ -32,7 +33,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -48,9 +49,7 @@
 #include <hb_api_core.h>
 #include <hb_api.h>
 #include <glib.h>
-#include <clplumbing/cl_random.h>
 
-#define CLIENTID_MAXLEN		36
 struct sys_config *		config  = NULL;
 
 int			netstring_format = TRUE;
@@ -161,7 +160,6 @@ static int		get_iflist(llc_private_t*, const char *host);
 static void		zap_iflist(llc_private_t*);
 static void		zap_order_seq(llc_private_t* pi);
 static void		zap_order_queue(llc_private_t* pi);
-static void		zap_msg_queue(llc_private_t* pi);
 static int		enqueue_msg(llc_private_t*,struct ha_msg*);
 static struct ha_msg*	dequeue_msg(llc_private_t*);
 static gen_callback_t*	search_gen_callback(const char * type, llc_private_t*);
@@ -190,8 +188,6 @@ static int init_nodewalk (ll_cluster_t*);
 static const char * nextnode (ll_cluster_t* ci);
 static int init_ifwalk (ll_cluster_t* ci, const char * host);
 static const char *	get_nodestatus(ll_cluster_t*, const char *host);
-static int 		get_nodeweight(ll_cluster_t*, const char *host);
-static const char *	get_nodesite(ll_cluster_t*, const char *host);
 static const char *
 get_clientstatus(ll_cluster_t*, const char *host, const char *clientid
 ,	int timeout);
@@ -219,13 +215,12 @@ static int		rcvmsg(ll_cluster_t* llc, int blocking);
 
 volatile struct process_info *	curproc = NULL;
 static char		OurPid[16];
-static char		OurClientID[CLIENTID_MAXLEN];
+static const char *	OurClientID = NULL;
 static char 		OurNode[SYS_NMLN];
 static ll_cluster_t*	hb_cluster_new(void);
 
 static void ha_api_perror(const char * fmt, ...) G_GNUC_PRINTF(1,2);
 static void ha_api_log(int priority, const char * fmt, ...) G_GNUC_PRINTF(2,3);
-static int	get_num_nodes(ll_cluster_t* lcl);
 
 
 #define	ZAPMSG(m)	{ha_msg_del(m); (m) = NULL;}
@@ -313,10 +308,10 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 
 	/* Create our client id */
 	if (clientid != NULL) {
-		strncpy(OurClientID, clientid, CLIENTID_MAXLEN);
+		OurClientID = clientid;
 		iscasual = 0;
 	}else{
-		strncpy(OurClientID, OurPid, CLIENTID_MAXLEN);
+		OurClientID = OurPid;
 		iscasual = 1;
 	}
 
@@ -362,9 +357,8 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 
 	/* Connect to the heartbeat API server */
 
-	pi->chan = ipc_channel_constructor(IPC_ANYTYPE, wchanattrs);
-	g_hash_table_destroy(wchanattrs);
-	if (pi->chan == NULL) {
+	if ((pi->chan = ipc_channel_constructor(IPC_ANYTYPE, wchanattrs))
+	==	NULL) {
 		ha_api_log(LOG_ERR, "hb_api_signon: Can't connect"
 		" to heartbeat");
 		ZAPMSG(request);
@@ -372,7 +366,6 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 	}
 
 	pi->chan->should_send_block = TRUE;
-	pi->chan->refcount++;
 
         if (pi->chan->ops->initiate_connection(pi->chan) != IPC_OK) {
 		ha_api_log(LOG_ERR, "hb_api_signon: Can't initiate"
@@ -404,7 +397,7 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 	if ((result = ha_msg_value(reply, F_APIRESULT)) != NULL
 	&&	strcmp(result, API_OK) == 0) {
 		rc = HA_OK;
-		pi->SignedOn = TRUE;
+		pi->SignedOn = 1;
 
 		if ((tmpstr = ha_msg_value(reply, F_DEADTIME)) == NULL
 		||	sscanf(tmpstr, "%lx", (unsigned long*)&(pi->deadtime_ms)) != 1) {
@@ -458,44 +451,33 @@ hb_api_signoff(struct ll_cluster* cinfo,gboolean need_destroy_chan)
 		return HA_FAIL;
 	}
 	pi = (llc_private_t*)cinfo->ll_cluster_private;
-	if (debug_level > 1) {
-		cl_log(LOG_DEBUG, "%s(%d){"
-		,	__FUNCTION__, need_destroy_chan);
+
+	if (!pi->SignedOn) {
+		/* Already signed off... No value in returning an error... */
+		return HA_OK;
 	}
 
-	if (pi->SignedOn && pi->chan && IPC_ISWCONN(pi->chan)) {
-		if ((request = hb_api_boilerplate(API_SIGNOFF)) == NULL) {
-			ha_api_log(LOG_ERR, "hb_api_signoff: can't create msg");
-			return HA_FAIL;
-		}
-		
-		/* Send the message */
-		if (msg2ipcchan(request, pi->chan) != HA_OK) {
-			ZAPMSG(request);
-			ha_api_perror("can't send message to IPC");
-			return HA_FAIL;
-		}
-		pi->chan->ops->waitout(pi->chan);
-		ZAPMSG(request);
+	if ((request = hb_api_boilerplate(API_SIGNOFF)) == NULL) {
+		ha_api_log(LOG_ERR, "hb_api_signoff: can't create msg");
+		return HA_FAIL;
 	}
-	OurClientID[0] = EOS;
 	
-	if(pi->chan) {
-		if (need_destroy_chan) {
-			pi->chan->ops->destroy(pi->chan);
-			pi->chan = NULL;
-		} else if (IPC_ISRCONN(pi->chan)) {
-			pi->chan->ops->disconnect(pi->chan);
-		}
+	/* Send the message */
+	if (msg2ipcchan(request, pi->chan) != HA_OK) {
+		ZAPMSG(request);
+		ha_api_perror("can't send message to IPC");
+		return HA_FAIL;
 	}
-	pi->SignedOn = FALSE;
+	pi->chan->ops->waitout(pi->chan);
+	ZAPMSG(request);
+	OurClientID = NULL;
+	if(need_destroy_chan){
+		pi->chan->ops->destroy(pi->chan);
+	}
+	pi->SignedOn = 0;
 	zap_order_seq(pi);
 	zap_order_queue(pi);
 
-	if (debug_level > 1) {
-		cl_log(LOG_DEBUG, "}/*%s(%d)*/", __FUNCTION__
-		,	need_destroy_chan);
-	}
 	return HA_OK;
 }
 /*
@@ -518,16 +500,15 @@ hb_api_delete(struct ll_cluster* ci)
 	zap_iflist(pi);
 	zap_nodelist(pi);
 
-	/* Free up the message queue */
-	zap_msg_queue(pi);
+	/* What about our message queue? */
 
 	/* Free up the private information */
 	memset(pi, 0, sizeof(*pi));
-	cl_free(pi);
+	ha_free(pi);
 
 	/* Free up the generic (llc) information */
 	memset(ci, 0, sizeof(*ci));
-	cl_free(ci);
+	ha_free(ci);
 	return HA_OK;
 }
 
@@ -700,7 +681,10 @@ get_nodelist(llc_private_t* pi)
 
 		ZAPMSG(reply);
 	}
-
+	if (reply != NULL) {
+		zap_nodelist(pi);
+		ZAPMSG(reply);
+	}
 	if (reply == NULL) {
 		ha_api_log(LOG_ERR, "General read_api_msg() failure");
 	}else if (result == NULL) {
@@ -713,10 +697,6 @@ get_nodelist(llc_private_t* pi)
 		ha_api_log(LOG_ERR, "new_stringlist() failure.");
 	}
 
-	if (reply != NULL) {
-		zap_nodelist(pi);
-		ZAPMSG(reply);
-	}
 	return HA_FAIL;
 }
 /*
@@ -840,128 +820,6 @@ get_nodestatus(ll_cluster_t* lcl, const char *host)
 
 	return ret;
 }
-
-/*
- * Return the weight of the given node.
- */
-
-static int
-get_nodeweight(ll_cluster_t* lcl, const char *host)
-{
-	struct ha_msg*		request;
-	struct ha_msg*		reply;
-	const char *		result;
-	const char *		weight_s;
-	int			ret;
-	llc_private_t*		pi;
-
-	ClearLog();
-	if (!ISOURS(lcl)) {
-		ha_api_log(LOG_ERR, "get_nodeweight: bad cinfo");
-		return -1;
-	}
-	pi = (llc_private_t*)lcl->ll_cluster_private;
-
-	if (!pi->SignedOn) {
-		ha_api_log(LOG_ERR, "not signed on");
-		return -1;
-	}
-
-	if ((request = hb_api_boilerplate(API_NODEWEIGHT)) == NULL) {
-		return -1;
-	}
-	if (ha_msg_add(request, F_NODENAME, host) != HA_OK) {
-		ha_api_log(LOG_ERR, "get_nodeweight: cannot add field");
-		ZAPMSG(request);
-		return -1;
-	}
-
-	/* Send message */
-	if (msg2ipcchan(request, pi->chan) != HA_OK) {
-		ZAPMSG(request);
-		ha_api_perror("Can't send message to IPC Channel");
-		return -1;
-	}
-	ZAPMSG(request);
-
-	/* Read reply... */
-	if ((reply=read_api_msg(pi)) == NULL) {
-		return -1;
-	}
-	if ((result = ha_msg_value(reply, F_APIRESULT)) != NULL
-	&&	strcmp(result, API_OK) == 0
-	&&	(weight_s = ha_msg_value(reply, F_WEIGHT)) != NULL) {
-		ret = atoi(weight_s);
-	}else{
-		ret = -1;
-	}
-	ZAPMSG(reply);
-
-	return ret;
-}
-
-/*
- * Return the site of the given node.
- */
-
-static const char *
-get_nodesite(ll_cluster_t* lcl, const char *host)
-{
-	struct ha_msg*		request;
-	struct ha_msg*		reply;
-	const char *		result;
-	const char *		site;
-	static char		sitebuf[HOSTLENG];
-	const char *		ret;
-	llc_private_t*		pi;
-
-	ClearLog();
-	if (!ISOURS(lcl)) {
-		ha_api_log(LOG_ERR, "get_nodesite: bad cinfo");
-		return NULL;
-	}
-	pi = (llc_private_t*)lcl->ll_cluster_private;
-
-	if (!pi->SignedOn) {
-		ha_api_log(LOG_ERR, "not signed on");
-		return NULL;
-	}
-
-	if ((request = hb_api_boilerplate(API_NODESITE)) == NULL) {
-		return NULL;
-	}
-	if (ha_msg_add(request, F_NODENAME, host) != HA_OK) {
-		ha_api_log(LOG_ERR, "get_nodesite: cannot add field");
-		ZAPMSG(request);
-		return NULL;
-	}
-
-	/* Send message */
-	if (msg2ipcchan(request, pi->chan) != HA_OK) {
-		ZAPMSG(request);
-		ha_api_perror("Can't send message to IPC Channel");
-		return NULL;
-	}
-	ZAPMSG(request);
-
-	/* Read reply... */
-	if ((reply=read_api_msg(pi)) == NULL) {
-		return NULL;
-	}
-	if ((result = ha_msg_value(reply, F_APIRESULT)) != NULL
-	&&	strcmp(result, API_OK) == 0
-	&&	(site = ha_msg_value(reply, F_SITE)) != NULL) {
-                memset(sitebuf, 0, sizeof(sitebuf));
-		strncpy(sitebuf, site, sizeof(sitebuf) - 1);
-		ret = sitebuf;
-	}else{
-		ret = NULL;
-	}
-	ZAPMSG(reply);
-
-	return ret;
-}
-
 /*
 * Return the status of the given client.
 */
@@ -994,10 +852,7 @@ get_clientstatus(ll_cluster_t* lcl, const char *host
 	/* If host is NULL, user choose the callback method to
 	 * get the result. This also implies timeout is useless */
 	if (host == NULL) {
-		int max_delay;
 		struct ha_msg * m = NULL;
-		int delay;
-		int num_nodes;
 
 		if ((m = ha_msg_new(0)) == NULL
 		||	ha_msg_add(m, F_TYPE, T_QCSTATUS) != HA_OK
@@ -1011,21 +866,6 @@ get_clientstatus(ll_cluster_t* lcl, const char *host
 			ha_log(LOG_ERR, "%s: cannot add field", __FUNCTION__);
 			return NULL;
 		}
-		
-		/* We delay random time here to distribute requests from different nodes 
-		 * across time in a big cluster
-		 * in a 100-node cluster, the max deley is 10 seconds
-		 */
-		num_nodes = get_num_nodes(lcl);
-		max_delay =  (1.0*num_nodes /10) *1000000; /* in microsecond*/
-		srand(cl_randseed());
-		delay = (1.0* rand()/RAND_MAX)*max_delay;
-		if (ANYDEBUG){
-			cl_log(LOG_DEBUG, "Delaying cstatus request for %d ms", delay/1000);
-		}
-		
-		usleep(delay);
-		
 		if (sendclustermsg(lcl, m) != HA_OK) {
 			ha_log(LOG_ERR, "%s: sendclustermsg fail",__FUNCTION__);
 		}
@@ -1137,63 +977,6 @@ get_nodetype(ll_cluster_t* lcl, const char *host)
 
 	return ret;
 }
-
-
-
-static int
-get_num_nodes(ll_cluster_t* lcl)
-{
-	struct ha_msg*		request;
-	struct ha_msg*		reply;
-	const char *		result;
-	llc_private_t*		pi;
-	const char*		num_s;
-	int			num;
-	
-
-	ClearLog();
-	if (!ISOURS(lcl)) {
-		ha_api_log(LOG_ERR, "%s: bad cinfo", __FUNCTION__);
-		return -1;
-	}
-	pi = (llc_private_t*)lcl->ll_cluster_private;
-
-	if (!pi->SignedOn) {
-		ha_api_log(LOG_ERR, "not signed on");
-		return -1;
-	}
-	
-	if ((request = hb_api_boilerplate(API_NUMNODES)) == NULL) {
-		return -1;
-	}
-	/* Send message */
-	if (msg2ipcchan(request, pi->chan) != HA_OK) {
-		ZAPMSG(request);
-		ha_api_perror("Can't send message to IPC Channel");
-		return -1;
-	}
-	ZAPMSG(request);
-	
-	/* Read reply... */
-	if ((reply=read_api_msg(pi)) == NULL) {
-		return -1;
-	}
-	if ((result = ha_msg_value(reply, F_APIRESULT)) != NULL
-	    &&	strcmp(result, API_OK) == 0
-	    &&	(num_s = ha_msg_value(reply, F_NUMNODES)) != NULL
-	    && (num = atoi(num_s)) > 0){
-		/*everything is good, do nothing*/		
-	}else{
-		cl_log(LOG_ERR, "Wrong reply message");
-		cl_log_message(LOG_ERR, reply);
-		num = -1;
-	}
-	ZAPMSG(reply);
-
-	return num;
-}
-
-
 static char *	
 get_parameter(ll_cluster_t* lcl, const char* pname)
 {
@@ -1240,7 +1023,7 @@ get_parameter(ll_cluster_t* lcl, const char* pname)
 	if ((result = ha_msg_value(reply, F_APIRESULT)) != NULL
 	&&	strcmp(result, API_OK) == 0
 	&&	(pvalue = ha_msg_value(reply, F_PVALUE)) != NULL) {
-		ret = cl_strdup(pvalue);
+		ret = ha_strdup(pvalue);
 	}else{
 		ret = NULL;
 	}
@@ -1295,10 +1078,6 @@ get_resources(ll_cluster_t* lcl)
 		retvalue[DIMOF(retvalue)-1] = EOS;
 		ret = retvalue;
 	}else{
-		const char* failreason = ha_msg_value(reply, F_COMMENT);
-		if (failreason){
-			ha_api_log(LOG_ERR,  "%s", failreason);
-		}
 		ret = NULL;
 	}
 	ZAPMSG(reply);
@@ -1464,7 +1243,7 @@ zap_order_seq(llc_private_t* pi)
 
 	while (order_seq != NULL){
 		next = order_seq->next;
-		cl_free(order_seq);
+		ha_free(order_seq);
 		order_seq = next;	 
 	}
 	pi->order_seq_head.next = NULL;
@@ -1489,29 +1268,12 @@ zap_order_queue(llc_private_t* pi)
 				oq->cluster.orderQ[i] = NULL;
 			}
 		}
-		cl_free(oq);
+		ha_free(oq);
 		oq = next;     
 	}
 	pi->order_queue_head = NULL;
 }
 
-static void
-zap_msg_queue(llc_private_t* pi)
-{
-	struct MsgQueue* qelem = pi->firstQdmsg;
-	struct MsgQueue* next;
-
-	while (qelem != NULL){
-		next = qelem->next;
-		ZAPMSG(qelem->value);
-		cl_free(qelem);
-		qelem = next;	 
-	}
-	pi->firstQdmsg = NULL;
-	pi->lastQdmsg = NULL;
-}
-
-
 /*
  * Create a new stringlist.
  */
@@ -1525,11 +1287,11 @@ new_stringlist(const char *s)
 		return(NULL);
 	}
 
-	if ((cp = cl_strdup(s)) == NULL) {
+	if ((cp = ha_strdup(s)) == NULL) {
 		return(NULL);
 	}
 	if ((ret = MALLOCT(struct stringlist)) == NULL) {
-		cl_free(cp);
+		ha_free(cp);
 		return(NULL);
 	}
 	ret->next = NULL;
@@ -1548,9 +1310,9 @@ destroy_stringlist(struct stringlist * s)
 
 	for (this=s; this; this=next) {
 		next = this->next;
-		cl_free(this->value);
+		ha_free(this->value);
 		memset(this, 0, sizeof(*this));
-		cl_free(this);
+		ha_free(this);
 	}
 }
 
@@ -1605,7 +1367,7 @@ dequeue_msg(llc_private_t* pi)
 		 * and the prev pointer of the next element in the queue.
 		 * (or possibly lastQdmsg... See below)
 		 */
-		cl_free(qret);
+		ha_free(qret);
 	}
 	if (pi->firstQdmsg == NULL) {
 		 /* Zap lastQdmsg if it pointed at this Q element */
@@ -1645,9 +1407,9 @@ add_gen_callback(const char * msgtype, llc_private_t* lcp
 		if (gcb == NULL) {
 			return(HA_FAIL);
 		}
-		type = cl_strdup(msgtype);
+		type = ha_strdup(msgtype);
 		if (type == NULL) {
-			cl_free(gcb);
+			ha_free(gcb);
 			return(HA_FAIL);
 		}
 		gcb->msgtype = type;
@@ -1677,7 +1439,7 @@ del_gen_callback(llc_private_t* lcp, const char * msgtype)
 			}else{
 				lcp->genlist = gcb->next;
 			}
-			cl_free(gcb->msgtype);
+			ha_free(gcb->msgtype);
 			gcb->msgtype = NULL;
 			free(gcb);
 			return(HA_OK);
@@ -1784,7 +1546,7 @@ moveup_backupQ(struct orderQ* q)
 			cl_log(LOG_ERR, "moveup_backupQ:"
 			       "backupQ in backupQ is not NULL");  
 		}
-		cl_free(backup_q);
+		ha_free(backup_q);
 		q->backupQ = NULL;
 	}else {
 		/*the queue must be empty*/
@@ -1865,7 +1627,7 @@ reset_orderQ(struct orderQ* q)
 	
 	if (q->backupQ != NULL){
 		reset_orderQ(q->backupQ);
-		cl_free(q->backupQ);
+		ha_free(q->backupQ);
 		q->backupQ = NULL;
 	}
 	
@@ -1963,7 +1725,7 @@ process_ordered_msg(struct orderQ* q, struct ha_msg* msg,
 		/*client restarted*/
 		
 		if (q->backupQ == NULL){
-			if ( (q->backupQ = cl_malloc(sizeof(struct orderQ))) 
+			if ( (q->backupQ = ha_malloc(sizeof(struct orderQ))) 
 			     ==NULL  ){
 				
 				cl_log(LOG_ERR, "process_ordered_msg: "
@@ -2139,7 +1901,7 @@ process_hb_msg(llc_private_t* pi, struct ha_msg* msg)
 		}
 		if (oq == NULL){
 			oq = (order_queue_t *) 
-				cl_malloc(sizeof(order_queue_t));
+				ha_malloc(sizeof(order_queue_t));
 			if (oq == NULL){
 				ha_api_log(LOG_ERR
 				,	"%s: order_queue_t malloc failed"
@@ -2341,36 +2103,30 @@ CallbackCall(llc_private_t* p, struct ha_msg * msg)
 	if ((strcasecmp(mtype, T_STATUS) == 0
 	||	strcasecmp(mtype, T_NS_STATUS) == 0)) {
 		/* If DEADSTATUS, cleanup order queue for the node */
-		const char *mstatus = ha_msg_value(msg, F_STATUS);
-		if (mstatus && (strcmp(mstatus, DEADSTATUS) == 0)) {
+		if (strcmp(ha_msg_value(msg, F_STATUS), DEADSTATUS) == 0) {
 			order_queue_t *	oq = p->order_queue_head;
 			order_queue_t *	prev;
 			order_queue_t *	next;
 			int		i;
 
 			for (prev = NULL; oq != NULL; prev = oq, oq = oq->next){
-				const char *morig = ha_msg_value(msg, F_ORIG);
-				if (morig && (strcmp(oq->from_node,  morig) 
-							== 0)) {
+				if (strcmp(oq->from_node
+				,	ha_msg_value(msg, F_ORIG)) == 0)
 					break;
-				}
 			}
 			if (oq){
 				next = oq->next;
 				for (i = 0; i < MAXMSGHIST; i++){
-					if (oq->node.orderQ[i]) {
+					if (oq->node.orderQ[i])
 						ZAPMSG(oq->node.orderQ[i]);
-					}
-					if (oq->cluster.orderQ[i]) {
+					if (oq->cluster.orderQ[i])
 						ZAPMSG(oq->cluster.orderQ[i]);
-					}
 				}
-				cl_free(oq);
-				if (prev) {
+				ha_free(oq);
+				if (prev)
 					prev->next = next;
-				} else {
+				else
 					p->order_queue_head = next;
-				}
 			}
 		}
 		if (p->node_callback) {
@@ -2427,7 +2183,7 @@ read_msg_w_callbacks(ll_cluster_t* llc, int blocking)
 
 	if (!ISOURS(llc)) {
 		ha_api_log(LOG_ERR, "read_msg_w_callbacks: bad cinfo");
-		return NULL;
+		return HA_FAIL;
 	}
 	pi = (llc_private_t*) llc->ll_cluster_private;
 
@@ -3036,7 +2792,7 @@ add_order_seq(llc_private_t* pi, struct ha_msg* msg)
 		}
 	}
 	if (order_seq == NULL && to_node != NULL){
-		order_seq = (order_seq_t *) cl_malloc(sizeof(order_seq_t));
+		order_seq = (order_seq_t *) ha_malloc(sizeof(order_seq_t));
 		if (order_seq == NULL){
 			ha_api_log(LOG_ERR
 			,	"add_order_seq: order_seq_t malloc failed!");
@@ -3244,10 +3000,7 @@ static struct llc_ops heartbeat_ops = {
 	nextnode:		nextnode,		
 	end_nodewalk:		end_nodewalk,		
 	node_status:		get_nodestatus,		
-	node_weight:		get_nodeweight,		
-	node_site:		get_nodesite,		
-	node_type:		get_nodetype,	
-	num_nodes:		get_num_nodes,
+	node_type:		get_nodetype,		
 	init_ifwalk:		init_ifwalk,		
 	nextif:			nextif,			
 	end_ifwalk:		end_ifwalk,		
@@ -3295,7 +3048,7 @@ hb_cluster_new()
 	}
 	memset(hb, 0, sizeof(*hb));
 	if ((ret = MALLOCT(ll_cluster_t)) == NULL) {
-		cl_free(hb);
+		ha_free(hb);
 		hb = NULL;
 		return(NULL);
 	}
@@ -3320,131 +3073,3 @@ ll_cluster_new(const char * llctype)
 	}
 	return NULL;
 }
-
-#include <clplumbing/GSource_internal.h>
-
-#define	OURMAGIC	0xbeef1234
-
-struct GLLclusterSource_s {
-	GCHSource	chsrc;
-	unsigned long	magic2;
-	ll_cluster_t*	hbchan;
-	gboolean(*dispatch)(ll_cluster_t* llc, gpointer udata);
-};
-
-
-
-static gboolean G_llc_prepare_int(GSource* source, gint* timeout);
-static gboolean G_llc_check_int(GSource* source);
-static gboolean G_llc_dispatch_int(GSource* source, GSourceFunc callback
-,	gpointer user_data);
-static void G_llc_destroy_int(GSource* source);
-
-static GSourceFuncs G_llc_SourceFuncs = {
-	G_llc_prepare_int,
-	G_llc_check_int,
-	G_llc_dispatch_int,
-	G_llc_destroy_int,
-};
-
-#define	CHECKMAGIC(s,value)	{if ((s)->magic2 != OURMAGIC) {		\
-					cl_log(LOG_ERR			\
-					,	"%s: invalid magic number"\
-					,	__FUNCTION__);		\
-					return value;			\
-				}}
-
-static gboolean
-G_llc_prepare_int(GSource* source, gint* timeout)
-{
-	GLLclusterSource*	s = (GLLclusterSource*)source;
-
-	CHECKMAGIC(s, FALSE);
-	(void)G_CH_prepare_int(source, timeout);
-	
-	return s->hbchan->llc_ops->msgready(s->hbchan);
-}
-
-static gboolean
-G_llc_check_int(GSource* source)
-{
-	GLLclusterSource*	s = (GLLclusterSource*)source;
-
-	CHECKMAGIC(s, FALSE);
-	(void)G_CH_check_int(source);
-	
-	return s->hbchan->llc_ops->msgready(s->hbchan);
-}
-
-static gboolean
-G_llc_dispatch_int(GSource* source, GSourceFunc callback
-,	gpointer user_data)
-{
-	gboolean	ret1 = TRUE;
-	gboolean	ret2 = TRUE;
-	GLLclusterSource*	s = (GLLclusterSource*)source;
-
-	CHECKMAGIC(s, FALSE);
-	ret1 = G_CH_dispatch_int(source, callback, user_data);
-	
-	if (s->hbchan->llc_ops->msgready(s->hbchan) && s->dispatch) {
-		ret2 = s->dispatch(s->hbchan, s->chsrc.udata);
-	}
-	return ret1 && ret2;
-}
-
-static void
-G_llc_destroy_int(GSource* source)
-{
-	GLLclusterSource*	s = (GLLclusterSource*)source;
-	llc_private_t*		pi;
-	pi = (llc_private_t*)s->hbchan->ll_cluster_private;
-
-	CHECKMAGIC(s, );
-	s->magic2 = 0;
-	G_CH_destroy_int(source);
-	pi->chan = NULL;
-	s->hbchan->llc_ops->delete(s->hbchan);
-}
-
-
-GLLclusterSource*
-G_main_add_ll_cluster(int priority, ll_cluster_t* api
-,	gboolean can_recurse
-,	gboolean (*dispatch)(ll_cluster_t* source_data,gpointer user_data)
-,	gpointer userdata, GDestroyNotify notify)
-{
-	GSource * 		source;
-	IPC_Channel*		ch;
-	GLLclusterSource*	s;
-
-	source =  g_source_new(&G_llc_SourceFuncs , sizeof(GLLclusterSource));
-	if (source == NULL || api == NULL || api->llc_ops == NULL
-	||	(ch = api->llc_ops->ipcchan(api)) == NULL) {
-		return NULL;
-	}
-
-	s = (GLLclusterSource*)source;
-	s->magic2 = OURMAGIC;
-	s->hbchan = api;
-	s->dispatch = dispatch;
-	(void)G_main_IPC_Channel_constructor(source, ch, userdata, notify);
-
-	
-	g_source_set_priority(source, priority);
-	g_source_set_can_recurse(source, can_recurse);
-	
-	s->chsrc.description = "Heartbeat API channel";
-	s->chsrc.gsourceid = g_source_attach(source, NULL);
-
-	if (s->chsrc.gsourceid == 0) {
-		g_source_remove_poll(source, &s->chsrc.infd);
-		if (!s->chsrc.fd_fdx) {
-			g_source_remove_poll(source, &s->chsrc.outfd);
-		}
-		g_source_unref(source);
-		source = NULL;
-		s = NULL;
-	}
-	return s;
-}
