@@ -19,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <lha_internal.h>
+#include <portability.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -93,8 +93,6 @@ static int lrm_add_rsc (ll_lrm_t*, const char* id, const char* class
 			,const char* type, const char* provider
 			,GHashTable* parameter);
 static int lrm_delete_rsc (ll_lrm_t*, const char* id);
-static int lrm_fail_rsc (ll_lrm_t* lrm, const char* rsc_id, const int fail_rc
-			,const char* fail_reason);
 static IPC_Channel* lrm_ipcchan (ll_lrm_t*);
 static int lrm_msgready (ll_lrm_t*);
 static int lrm_rcvmsg (ll_lrm_t*, int blocking);
@@ -113,7 +111,6 @@ static struct lrm_ops lrm_ops_instance =
 	lrm_get_rsc,
 	lrm_add_rsc,
 	lrm_delete_rsc,
-	lrm_fail_rsc,
 	lrm_ipcchan,
 	lrm_msgready,
 	lrm_rcvmsg
@@ -477,45 +474,26 @@ lrm_get_rsc_provider_supported (ll_lrm_t* lrm, const char* class, const char* ty
 	
 /*
  * lrm_get_all_type_metadatas():
- * The key of the hash table is in the format "type:provider"
- * The value of the hash table is the metadata.
+ * For OCF RAs, they may have more than one providers so they may have more than
+ * one metadata. The hash table is not suitable for this. Fix Me
  */
 static GHashTable*
 lrm_get_all_type_metadata (ll_lrm_t* lrm, const char* rclass)
 {
-	GHashTable* metas = g_hash_table_new_full(g_str_hash, g_str_equal
-						  , g_free, g_free);
+	GHashTable* metas = g_hash_table_new(g_str_hash, g_str_equal);
 	GList* types = lrm_get_rsc_type_supported (lrm, rclass);
-	GList* providers = NULL;
-	GList* cur_type = NULL;
-	GList* cur_provider = NULL;
+	GList* node = NULL;
+        const char* meta;
 
-	cur_type = g_list_first(types);
-	while (cur_type != NULL)
-	{
-	        const char* type;
-	        char key[MAXLENGTH];
-		type = (const char*) cur_type->data;
-		providers = lrm_get_rsc_provider_supported(lrm, rclass, type);
-		cur_provider = g_list_first(providers);
-		while (cur_provider != NULL) {
-		        const char* meta;
-		        const char* provider;
-			provider = (const char*) cur_provider->data;
-			meta = lrm_get_rsc_type_metadata(lrm,rclass,type,provider);
-			if (NULL == meta) {
-				cur_provider = g_list_next(cur_provider);
-				continue;
-			}
-			snprintf(key,MAXLENGTH, "%s:%s",type,provider);
-			key[MAXLENGTH-1]='\0';
-			g_hash_table_insert(metas,g_strdup(key),g_strdup(meta));
-			cur_provider = g_list_next(cur_provider);
+	for (node = g_list_first(types); NULL!=node; node=g_list_next(node)) {
+		meta = lrm_get_rsc_type_metadata(lrm,rclass,node->data,NULL);
+		if (NULL == meta) {
+			continue;
 		}
-		lrm_free_str_list(providers);
-		cur_type=g_list_next(cur_type);
+		g_hash_table_insert(metas, node->data,g_strdup(meta));
 	}
-	lrm_free_str_list(types);
+	g_list_free(types);
+	
 	return metas;
 }
 
@@ -672,6 +650,7 @@ lrm_get_rsc (ll_lrm_t* lrm, const char* rsc_id)
 	}
 	/* get the return code of return message */
 	if (HA_OK != get_ret_from_msg(ret)) {
+		LOG_GOT_FAIL_RET(LOG_WARNING, GETRSC);
 		ha_msg_del(ret);
 		return NULL;
 	}
@@ -691,53 +670,6 @@ lrm_get_rsc (ll_lrm_t* lrm, const char* rsc_id)
 	return rsc;
 }
 
-static int
-lrm_fail_rsc (ll_lrm_t* lrm, const char* rsc_id, const int fail_rc
-,		 const char* fail_reason)
-{
-	struct ha_msg* msg;
-
-	/* check whether the rsc_id is available */
-	if (NULL == rsc_id || RID_LEN <= strlen(rsc_id))	{
-		cl_log(LOG_ERR, "%s: wrong parameter rsc_id.", __FUNCTION__);
-		return HA_FAIL;
-	}
-
-	/* check whether the channel to lrmd is available */
-	if (NULL == ch_cmd)	{
-		cl_log(LOG_ERR, "%s: ch_mod is null.", __FUNCTION__);
-		return HA_FAIL;
-	}
-
-	/* create the message */
-	msg = create_lrm_rsc_msg(rsc_id,FAILRSC);
-	if (NULL == msg) {
-		LOG_FAIL_create_lrm_rsc_msg(FAILRSC);
-		return HA_FAIL;
-	}
-	if ((fail_reason && HA_OK != ha_msg_add(msg,F_LRM_FAIL_REASON,fail_reason))
-		|| HA_OK != ha_msg_add_int(msg, F_LRM_ASYNCMON_RC, fail_rc)
-	) {
-		ha_msg_del(msg);
-		LOG_BASIC_ERROR("ha_msg_add");
-		return HA_FAIL;
-	}
-	/* send to lrmd */
-	if (HA_OK != msg2ipcchan(msg,ch_cmd)) {
-		ha_msg_del(msg);
-		LOG_FAIL_SEND_MSG(FAILRSC, "ch_cmd");
-		return HA_FAIL;
-	}
-	ha_msg_del(msg);
-	/* check the result */
-	if (HA_OK != get_ret_from_ch(ch_cmd)) {
-		LOG_GOT_FAIL_RET(LOG_ERR, FAILRSC);
-		return HA_FAIL;
-	}
-
-	return HA_OK;
-}
-
 static int
 lrm_add_rsc (ll_lrm_t* lrm, const char* rsc_id, const char* class
 , 	     const char* type, const char* provider, GHashTable* parameter)
@@ -877,7 +809,6 @@ lrm_rcvmsg (ll_lrm_t* lrm, int blocking)
 		if (NULL!=op && NULL!=op_done_callback) {
 			(*op_done_callback)(op);
 		}
-		free_op(op);
 		ha_msg_del(msg);
 	}
 
@@ -890,7 +821,6 @@ rsc_perform_op (lrm_rsc_t* rsc, lrm_op_t* op)
 {
 	int rc = 0;
 	struct ha_msg* msg = NULL;
-	char* rsc_id;
 
 	/* check whether the channel to lrmd is available */
 	if (NULL == ch_cmd
@@ -903,10 +833,8 @@ rsc_perform_op (lrm_rsc_t* rsc, lrm_op_t* op)
 		return HA_FAIL;
 	}
 	/* create the msg of perform op */
-	rsc_id = op->rsc_id;
 	op->rsc_id = rsc->id;
 	msg = op_to_msg(op);
-	op->rsc_id = rsc_id;
 	if ( NULL == msg) {
 		cl_log(LOG_ERR, "rsc_perform_op: failed to create a message "
 			"with function op_to_msg");
@@ -922,6 +850,7 @@ rsc_perform_op (lrm_rsc_t* rsc, lrm_op_t* op)
 
 	/* check return code, the return code is the call_id of the op */
 	rc = get_ret_from_ch(ch_cmd);
+	op->rsc_id = NULL;
 	return rc;
 }
 
@@ -1010,7 +939,7 @@ compare_call_id(gconstpointer a, gconstpointer b)
 static GList*
 rsc_get_cur_state (lrm_rsc_t* rsc, state_flag_t* cur_state)
 {
-	GList* op_list = NULL, * tmplist = NULL;
+	GList* op_list = NULL;
 	struct ha_msg* msg = NULL;
 	struct ha_msg* ret = NULL;
 	struct ha_msg* op_msg = NULL;
@@ -1086,40 +1015,7 @@ rsc_get_cur_state (lrm_rsc_t* rsc, state_flag_t* cur_state)
 		}
 		ha_msg_del(op_msg);
 	}
-	op_list = g_list_sort(op_list, compare_call_id);
-
-	/* Delete the duplicate op for call_id */
-#if 0	
-	cl_log(LOG_WARNING, "Before uniquing");
-	tmplist = g_list_first(op_list);
-	while (tmplist != NULL) {
-		cl_log(LOG_WARNING, "call_id=%d", ((lrm_op_t*)(tmplist->data))->call_id);
-		tmplist = g_list_next(tmplist);
-	}
-#endif
-
-	tmplist = g_list_first(op_list);
-	while (tmplist != NULL) {
-		if (NULL != g_list_previous(tmplist)) {
-			if (((lrm_op_t*)(g_list_previous(tmplist)->data))->call_id
-			     == ((lrm_op_t*)(tmplist->data))->call_id) {
-				op_list = g_list_remove_link (op_list, tmplist);
-				free_op((lrm_op_t *)tmplist->data);
-				g_list_free_1(tmplist);
-				tmplist = g_list_first(op_list);
-			}
-		}
-		tmplist = g_list_next(tmplist);
-	}
-
-#if 0
-	cl_log(LOG_WARNING, "After uniquing");
-	while (tmplist != NULL) {
-		cl_log(LOG_WARNING, "call_id=%d", ((lrm_op_t*)(tmplist->data))->call_id);
-		tmplist = g_list_next(tmplist);
-	}
-#endif
-
+	g_list_sort(op_list, compare_call_id);
 	return op_list;
 }
 
@@ -1201,7 +1097,6 @@ msg_to_op(struct ha_msg* msg)
 	const char* op_type;
 	const char* app_name;
 	const char* rsc_id;
-	const char* fail_reason;
 	const char* output;
 	const void* user_data;
 
@@ -1241,6 +1136,10 @@ msg_to_op(struct ha_msg* msg)
 		else {
 			op->output = NULL;
 		}
+		/* check the rc to revise the op_status */
+		if (0 != op->rc) {
+			op->op_status = LRM_OP_ERROR;
+		}
 	} else if(op->op_status == LRM_OP_PENDING) {
 		op->rc = EXECRA_STATUS_UNKNOWN;
 		
@@ -1275,13 +1174,6 @@ msg_to_op(struct ha_msg* msg)
 		free_op(op);
 		return NULL;
 	}
-	op->rsc_id = g_strdup(rsc_id);
-
-	/* op->fail_reason present only on async failures */
-	fail_reason = ha_msg_value(msg, F_LRM_FAIL_REASON);
-	if (fail_reason) {
-		op->fail_reason = g_strdup(fail_reason);
-	}
 
 	/* op->user_data */
 	user_data = cl_get_string(msg, F_LRM_USERDATA);
@@ -1290,17 +1182,8 @@ msg_to_op(struct ha_msg* msg)
 		op->user_data = g_strdup(user_data);
 	}
 	
-	/* time_stamps */
-	if (ha_msg_value_ul(msg, F_LRM_T_RUN, &op->t_run) != HA_OK
-	   || ha_msg_value_ul(msg, F_LRM_T_RCCHANGE, &op->t_rcchange) != HA_OK
-	   || ha_msg_value_ul(msg, F_LRM_EXEC_TIME, &op->exec_time) != HA_OK
-	   || ha_msg_value_ul(msg, F_LRM_QUEUE_TIME, &op->queue_time) != HA_OK) {
-		/* cl_log(LOG_WARNING
-		, "%s:%d: failed to get the timing information"
-		, __FUNCTION__, __LINE__);
-		*/
-	}
-	
+	op->rsc_id = g_strdup(rsc_id);
+
 	/* op->params */
 	op->params = ha_msg_value_str_table(msg, F_LRM_PARAM);
 
@@ -1310,8 +1193,8 @@ msg_to_op(struct ha_msg* msg)
 static struct ha_msg*
 op_to_msg (lrm_op_t* op)
 {
-	struct ha_msg* msg = ha_msg_new(15);
-	if (!msg) {
+	struct ha_msg* msg = ha_msg_new(5);
+	if (NULL == msg) {
 		LOG_BASIC_ERROR("ha_msg_new");
 		return NULL;
 	}
@@ -1322,20 +1205,36 @@ op_to_msg (lrm_op_t* op)
 	||  HA_OK != ha_msg_add_int(msg, F_LRM_TIMEOUT, op->timeout)
 	||  HA_OK != ha_msg_add_int(msg, F_LRM_INTERVAL, op->interval)
 	||  HA_OK != ha_msg_add_int(msg, F_LRM_DELAY, op->start_delay)
-	||  HA_OK != ha_msg_add_int(msg, F_LRM_COPYPARAMS, op->copyparams)
-	||  HA_OK != ha_msg_add_ul(msg, F_LRM_T_RUN,op->t_run)
-	||  HA_OK != ha_msg_add_ul(msg, F_LRM_T_RCCHANGE, op->t_rcchange)
-	||  HA_OK != ha_msg_add_ul(msg, F_LRM_EXEC_TIME, op->exec_time)
-	||  HA_OK != ha_msg_add_ul(msg, F_LRM_QUEUE_TIME, op->queue_time)
-	||  HA_OK != ha_msg_add_int(msg, F_LRM_TARGETRC, op->target_rc)
-	||  ( op->app_name && (HA_OK != ha_msg_add(msg, F_LRM_APP, op->app_name)))
-	||  ( op->user_data && (HA_OK != ha_msg_add(msg,F_LRM_USERDATA,op->user_data)))
-	||  ( op->params && (HA_OK != ha_msg_add_str_table(msg,F_LRM_PARAM,op->params)))) {
-		LOG_BASIC_ERROR("op_to_msg conversion failed");
+	||  HA_OK != ha_msg_add_int(msg, F_LRM_TARGETRC, op->target_rc)) {
 		ha_msg_del(msg);
+		LOG_BASIC_ERROR("ha_msg_add*");
 		return NULL;
 	}
 
+	if (NULL != op->app_name) {
+		if (HA_OK != ha_msg_add(msg, F_LRM_APP, op->app_name)){
+			ha_msg_del(msg);
+			LOG_BASIC_ERROR("ha_mag_add");
+			return NULL;
+		}
+	}
+
+	if (NULL != op->user_data) {
+		if (HA_OK != ha_msg_add(msg,F_LRM_USERDATA,op->user_data)){
+			ha_msg_del(msg);
+			LOG_BASIC_ERROR("ha_msg_add");
+			return NULL;
+		}
+	}
+
+	if (NULL != op->params) {
+		if (HA_OK != ha_msg_add_str_table(msg,F_LRM_PARAM,op->params)){
+			ha_msg_del(msg);
+			LOG_BASIC_ERROR("ha_msg_add_str_table");
+			return NULL;
+		}	
+	}
+
 	return msg;
 }
 
@@ -1396,13 +1295,9 @@ free_op (lrm_op_t* op)
 	if (NULL != op->app_name) {
 		g_free(op->app_name);
 	}
-	if (NULL != op->user_data) {
-		g_free(op->user_data);
-	}
 	if (NULL != op->params) {
 		free_str_table(op->params);
 	}
-	g_free(op);
 }
 
 void lrm_free_op(lrm_op_t* op) {
@@ -1488,10 +1383,6 @@ execra_code2string(uniform_ret_execra_t code)
 		case EXECRA_NOT_RUNNING:
 			return "not running";
 		/* For status command only */
-		case EXECRA_RUNNING_MASTER:
-			return "master";
-		case EXECRA_FAILED_MASTER:
-			return "master (failed)";
 		case EXECRA_RA_DEAMON_DEAD1:
 			return "status: deamon dead";
 		case EXECRA_RA_DEAMON_DEAD2:
