@@ -1,3 +1,4 @@
+/* $Id: pils.c,v 1.47 2005/02/17 05:49:24 alan Exp $ */
 /*
  * Copyright (C) 2001 Alan Robertson <alanr@unix.sh>
  * This software licensed under the GNU LGPL.
@@ -18,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <lha_internal.h>
+#include <portability.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -584,7 +585,7 @@ RemoveAPILPluginType(PILPluginType*Plugintype)
  *	InterfaceManager_plugin_init: Initialize the handling of
  *	"Interface Manager" interfaces.
  *
- *	There are a few potential bootstrapping problems here ;-)
+ *	There are a few potential bootstraiffng problems here ;-)
  *
  */
 static PIL_rc
@@ -795,12 +796,12 @@ NewPILInterface(PILInterfaceType*	interfacetype
 	}
 	ret = NEW(PILInterface);
 	STATNEW(interface);
+	ret->MagicNum = PIL_MAGIC_INTERFACE;
 	if (DEBUGPLUGIN) {
 		PILLog(PIL_DEBUG, "NewPILInterface(0x%x)", (unsigned long)ret);
 	}
 
 	if (ret) {
-		ret->MagicNum = PIL_MAGIC_INTERFACE;
 		ret->interfacetype = interfacetype;
 		ret->exports = exports;
 		ret->ud_interface = ud_interface;
@@ -1869,7 +1870,7 @@ so_select (const struct dirent *dire)
 	if (DEBUGPLUGIN) {
 		PILLog(PIL_DEBUG, "In so_select: %s.", dire->d_name);
 	}
-	if (end < dire->d_name) {
+	if (obj_end < dire->d_name) {
 			return 0;
 	}
 	if (strcmp(end, obj_end) == 0) {
@@ -2140,12 +2141,10 @@ PILCallLog(PILLogFun logfun, PILLogLevel priority, const char * fmt, ...)
 {
 	va_list		args;
 	char *		str;
-	int		err = errno;
 
 	va_start (args, fmt);
 	str = g_strdup_vprintf(fmt, args);
 	va_end (args);
 	logfun(priority, "%s", str);
 	g_free(str);
-	errno = err;
 }
