@@ -34,8 +34,8 @@
  *
  */
 
-#include <lha_internal.h>
 #include <time.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -238,7 +238,6 @@ hbaping_new(const char * host)
 	ret = (struct hb_media *) MALLOC(sizeof(struct hb_media));
 	if (ret != NULL) {
 		char * name;
-		memset(ret, 0, sizeof(*ret));
 		ret->pd = (void*)ipi;
 		name = MALLOC(strlen(host)+1);
 		strcpy(name, host);
@@ -252,7 +251,7 @@ hbaping_new(const char * host)
 }
 
 /*
- *	Close HBA connection
+ *	Close UDP/IP broadcast heartbeat interface
  */
 
 static int
@@ -264,10 +263,7 @@ hbaping_close(struct hb_media* mp)
 	PINGASSERT(mp);
 	ei = (struct hbaping_private *) mp->pd;
 
-	if (ei->handle >= 0) {
-		HBA_CloseAdapter(ei->handle);
-		ei->handle=-1;
-	}
+	HBA_CloseAdapter(ei->handle);
 
 	return(rc);
 }
