@@ -1,3 +1,4 @@
+/* $Id: mcast.c,v 1.24 2005/08/15 21:12:16 gshi Exp $ */
 /*
  * mcast.c: implements hearbeat API for UDP multicast communication
  *
@@ -22,7 +23,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -210,7 +211,7 @@ mcast_parse(const char *line)
 
 	if (*dev != EOS)  {
 		if (!is_valid_dev(dev)) {
-			PILCallLog(LOG, PIL_CRIT, "mcast device [%s] is invalid or not set up properly", dev);
+			PILCallLog(LOG, PIL_CRIT, "mcast bad device [%s]", dev);
 			return HA_FAIL;
 		}
 		/* Skip over white space, then grab the multicast group */
@@ -384,7 +385,7 @@ mcast_close(struct hb_media* hbm)
  * Receive a heartbeat multicast packet from UDP interface
  */
 
-char			mcast_pkt[MAXMSG];
+char			mcast_pkt[MAXLINE];
 static void *
 mcast_read(struct hb_media* hbm, int *lenp)
 {
@@ -396,8 +397,8 @@ mcast_read(struct hb_media* hbm, int *lenp)
 	MCASTASSERT(hbm);
 	mcp = (struct mcast_private *) hbm->pd;
 	
-	if ((numbytes=recvfrom(mcp->rsocket, mcast_pkt, MAXMSG-1, 0
-	,	(struct sockaddr *)&their_addr, &addr_len)) < 0) {
+	if ((numbytes=recvfrom(mcp->rsocket, mcast_pkt, MAXLINE-1, 0
+			       ,(struct sockaddr *)&their_addr, &addr_len)) < 0) {
 		if (errno != EINTR) {
 			PILCallLog(LOG, PIL_CRIT, "Error receiving from socket: %s"
 			    ,	strerror(errno));
@@ -418,6 +419,8 @@ mcast_read(struct hb_media* hbm, int *lenp)
 	*lenp = numbytes + 1 ;
 
 	return mcast_pkt;;
+	
+	
 }
 
 /*
@@ -494,7 +497,7 @@ mcast_make_send_sock(struct hb_media * hbm)
  * Set up socket for listening to heartbeats (UDP multicasts)
  */
 
-#define	MAXBINDTRIES	50
+#define	MAXBINDTRIES	10
 static int
 mcast_make_receive_sock(struct hb_media * hbm)
 {
@@ -695,56 +698,33 @@ join_mcast_group(int sockfd, struct in_addr *addr, char *ifname)
 static int
 if_getaddr(const char *ifname, struct in_addr *addr)
 {
+	int	fd;
 	struct ifreq	if_info;
-	int		j;
-	int		maxtry = 120;
-	gboolean	gotaddr = FALSE;
-	int		err = 0;
 	
-	if (!addr) {
+	if (!addr)
 		return -1;
-	}
 
 	addr->s_addr = INADDR_ANY;
 
 	memset(&if_info, 0, sizeof(if_info));
 	if (ifname) {
 		strncpy(if_info.ifr_name, ifname, IFNAMSIZ-1);
-	}else{	/* ifname is NULL, so use any address */
+	} else {	/* ifname is NULL, so use any address */
 		return 0;
 	}
 
+	if ((fd=socket(AF_INET, SOCK_DGRAM, 0)) == -1)	{
+		PILCallLog(LOG, PIL_CRIT, "Error getting socket");
+		return -1;
+	}
 	if (Debug > 0) {
 		PILCallLog(LOG, PIL_DEBUG, "looking up address for %s"
 		,	if_info.ifr_name);
 	}
-	for (j=0; j < maxtry && !gotaddr; ++j) {
-		int		fd;
-		if ((fd=socket(AF_INET, SOCK_DGRAM, 0)) == -1)	{
-			PILCallLog(LOG, PIL_CRIT, "Error getting socket");
-			return -1;
-		}
-		if (ioctl(fd, SIOCGIFADDR, &if_info) >= 0) {
-			gotaddr = TRUE;
-		}else{
-			err = errno;
-			switch(err) {
-				case EADDRNOTAVAIL:
-					sleep(1);
-					break;	
-				default:
-					close(fd);
-					goto getout;
-			}
-		}
+	if (ioctl(fd, SIOCGIFADDR, &if_info) < 0) {
+		PILCallLog(LOG, PIL_CRIT, "Error ioctl(SIOCGIFADDR): %s"
+		,	strerror(errno));
 		close(fd);
-	}
-getout:
-	if (!gotaddr) {
-		PILCallLog(LOG, PIL_CRIT
-		,	"Unable to retrieve local interface address"
-		" for interface [%s] using ioctl(SIOCGIFADDR): %s"
-		,	ifname, strerror(err));
 		return -1;
 	}
 
@@ -757,6 +737,7 @@ getout:
 	memcpy(addr, &(SOCKADDR_IN(&if_info.ifr_addr)->sin_addr)
 	,	sizeof(struct in_addr));
 
+	close(fd);
 	return 0;
 }
 
@@ -817,3 +798,138 @@ get_loop(const char *loop, u_char *l)
 	return 0;
 }
 
+/*
+ * $Log: mcast.c,v $
+ * Revision 1.24  2005/08/15 21:12:16  gshi
+ * make the media read() function returns a pointer that is a global varial
+ * This should save a malloc, free, and a memcpy for each message
+ *
+ * Revision 1.23  2005/04/10 20:10:53  lars
+ * int -> socklen_t where needed
+ *
+ * Revision 1.22  2004/10/24 13:00:13  lge
+ * -pedantic-errors fixes 2:
+ *  * error: ISO C forbids forward references to 'enum' types
+ *    error: comma at end of enumerator list
+ *    error: ISO C does not allow extra ';' outside of a function
+ *
+ * Revision 1.21  2004/10/06 10:55:17  lars
+ * - Define PIL_PLUGIN_BOILERPLATE() as it used to be, which implies a
+ *   prototype for the closepi function.
+ * - Define PIL_PLUGIN_BOILERPLATE2() which just takes two arguments and
+ *   fills in NULL for all those plugins which don't use the closepi()
+ *   functionality.
+ *
+ * Revision 1.20  2004/09/27 04:23:30  alan
+ * Put in some code to print out failure cases better, and also to
+ * better diagnose bad configurations.
+ *
+ * Revision 1.19  2004/05/11 22:04:35  alan
+ * Changed all the HBcomm plugins to use PILCallLog() for logging instead of calling
+ * the function pointer directly.
+ * Also, in the process fixed several mismatches between arguments and format strings, and
+ * a couple of format string vulnerabilities.
+ *
+ * Revision 1.18  2004/04/28 22:30:29  alan
+ * Put in some fixes for extra freeing of memory in the communications plugins.
+ *
+ * Revision 1.17  2004/03/03 05:31:50  alan
+ * Put in Gochun Shi's new netstrings on-the-wire data format code.
+ * this allows sending binary data, among many other things!
+ *
+ * Revision 1.16  2004/02/17 22:11:59  lars
+ * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
+ *
+ * Revision 1.15  2004/01/21 11:34:15  horms
+ * - Replaced numerous malloc + strcpy/strncpy invocations with strdup
+ *   * This usually makes the code a bit cleaner
+ *   * Also is easier not to make code with potential buffer over-runs
+ * - Added STRDUP to pils modules
+ * - Removed some spurious MALLOC and FREE redefinitions
+ *   _that could never be used_
+ * - Make sure the return value of strdup is honoured in error conditions
+ *
+ * Revision 1.14  2003/02/07 08:37:17  horms
+ * Removed inclusion of portability.h from .h files
+ * so that it does not need to be installed.
+ *
+ * Revision 1.13  2003/02/05 09:06:34  horms
+ * Lars put a lot of work into making sure that portability.h
+ * is included first, everywhere. However this broke a few
+ * things when building against heartbeat headers that
+ * have been installed (usually somewhere under /usr/include or
+ * /usr/local/include).
+ *
+ * This patch should resolve this problem without undoing all of
+ * Lars's hard work.
+ *
+ * As an asside: I think that portability.h is a virus that has
+ * infected all of heartbeat's code and now must also infect all
+ * code that builds against heartbeat. I wish that it didn't need
+ * to be included all over the place. Especially in headers to
+ * be installed on the system. However, I respect Lars's opinion
+ * that this is the best way to resolve some weird build problems
+ * in the current tree.
+ *
+ * Revision 1.12  2003/01/31 10:02:09  lars
+ * Various small code cleanups:
+ * - Lots of "signed vs unsigned" comparison fixes
+ * - time_t globally replaced with TIME_T
+ * - All seqnos moved to "seqno_t", which defaults to unsigned long
+ * - DIMOF() definition centralized to portability.h and typecast to int
+ * - EOS define moved to portability.h
+ * - dropped inclusion of signal.h from stonith.h, so that sigignore is
+ *   properly defined
+ *
+ * Revision 1.11  2002/10/21 10:17:19  horms
+ * hb api clients may now be built outside of the heartbeat tree
+ *
+ * Revision 1.10  2002/09/19 22:40:18  alan
+ * Changed a few error return checks to not print anything and return
+ * if an error was encountered.
+ * Changed a few debug messages to only print if a strictly positive number
+ * of chars was received.
+ *
+ * Revision 1.9  2002/09/12 03:52:07  alan
+ * Fixed up a comment :-(.
+ *
+ * Revision 1.8  2002/09/12 03:39:45  alan
+ * Fixed some logging level names in the code and also
+ * fixed an error in the license chosen for a file.
+ *
+ * Revision 1.7  2002/06/16 06:11:26  alan
+ * Put in a couple of changes to the PILS interfaces
+ *  - exported license information (name, URL)
+ *  - imported malloc/free
+ *
+ * Revision 1.6  2002/05/01 23:50:35  alan
+ * Put in some comments about how the code avoids potential buffer overruns.
+ *
+ * Revision 1.5  2002/04/13 22:35:08  alan
+ * Changed ha_msg_add_nv to take an end pointer to make it safer.
+ * Added a length parameter to string2msg so it would be safer.
+ * Changed the various networking plugins to use the new string2msg().
+ *
+ * Revision 1.4  2002/04/09 12:45:36  alan
+ * Put in changes to the bcast, mcast and serial code such that
+ * interrupted system calls in reads are ignored.
+ *
+ * Revision 1.3  2002/01/17 15:21:23  alan
+ * Put in Ram Pai's patch for the bug in the multicast code.
+ *
+ * Revision 1.2  2001/09/07 16:18:17  alan
+ * Updated ping.c to conform to the new plugin loading system.
+ * Changed log messages in bcast, mcast, ping and serial to use the
+ * new logging function.
+ *
+ * Revision 1.1  2001/08/10 17:16:44  alan
+ * New code for the new plugin loading system.
+ *
+ * Revision 1.11  2001/06/23 04:30:26  alan
+ * Changed the code to use inet_pton() when it's available, and
+ * emulate it when it's not...  Patch was from Chris Wright.
+ *
+ * Revision 1.10  2001/06/08 04:57:48  alan
+ * Changed "config.h" to <portability.h>
+ *
+ */
