@@ -1,3 +1,4 @@
+/* $Id: ucast.c,v 1.26 2005/08/15 21:12:16 gshi Exp $ */
 /*
  * Adapted from alanr's UDP broadcast heartbeat bcast.c by Stphane Billiart
  *	<stephane@reefedge.com>
@@ -26,7 +27,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <lha_internal.h>
+#include <portability.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -368,7 +372,7 @@ static int ucast_close(struct hb_media* mp)
  * Receive a heartbeat unicast packet from UDP interface
  */
 
-char ucast_pkt[MAXMSG];
+char ucast_pkt[MAXLINE];
 
 static void *
 ucast_read(struct hb_media* mp, int *lenp)
@@ -382,7 +386,7 @@ ucast_read(struct hb_media* mp, int *lenp)
 	ei = (struct ip_private*)mp->pd;
 
 	addr_len = sizeof(struct sockaddr);
-	if ((numbytes = recvfrom(ei->rsocket, ucast_pkt, MAXMSG-1, 0,
+	if ((numbytes = recvfrom(ei->rsocket, ucast_pkt, MAXLINE-1, 0,
 		(struct sockaddr *)&their_addr, &addr_len)) == -1) {
 		if (errno != EINTR) {
 			PILCallLog(LOG, PIL_CRIT, "ucast: error receiving from socket: %s",
