@@ -27,7 +27,7 @@
  * 3) Optional and additional actions analysis?
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -44,8 +44,6 @@
 #include <lrm/raexec.h>
 #include <libgen.h>
 
-#include <libxml/entities.h>
-
 #define PIL_PLUGINTYPE		RA_EXEC_TYPE
 #define PIL_PLUGIN		lsb
 #define PIL_PLUGINTYPE_S	"RAExec"
@@ -59,10 +57,11 @@
  * The reload and the try-restart options are optional. Other init script
  * actions may be defined by the init script.
  */
-#define meta_data_template  \
+#define meta_data_template  "\n"\
 "<?xml version=\"1.0\"?>\n"\
 "<!DOCTYPE resource-agent SYSTEM \"ra-api-1.dtd\">\n"\
-"<resource-agent name=\"%s\" version=\"0.1\">\n"\
+"<resource-agent name=%s"\
+"\" version=\"0.1\">\n"\
 "  <version>1.0</version>\n"\
 "  <longdesc lang=\"en\">\n"\
 "    %s"\
@@ -91,8 +90,8 @@
 "</resource-agent>\n"
 
 /* The keywords for lsb-compliant comment */
-#define LSB_INITSCRIPT_INFOBEGIN_TAG "### BEGIN INIT INFO"
-#define LSB_INITSCRIPT_INFOEND_TAG "### END INIT INFO"
+#define LSB_INITSCRIPT_BEGIN_TAG "### BEGIN INIT INFO"
+#define LSB_INITSCRIPT_END_TAG "### END INIT INFO"
 #define PROVIDES    "# Provides:" 
 #define REQ_START   "# Required-Start:"
 #define REQ_STOP    "# Required-Stop:"
@@ -103,15 +102,18 @@
 #define SHORT_DSCR  "# Short-Description:"
 #define DESCRIPTION "# Description:"
 
-#define ZAPXMLOBJ(m)				\
+#define ZAPGDOBJ(m)				\
 		if ( (m) != NULL ) {		\
-			xmlFree(m);		\
+			g_free(m);		\
 			(m) = NULL;		\
 		}
 
 #define RALSB_GET_VALUE(ptr, keyword)	\
-	if ( (ptr == NULL) & (0 == strncasecmp(buffer, keyword, strlen(keyword))) ) { \
-		(ptr) = (char *)xmlEncodeEntitiesReentrant(NULL,BAD_CAST buffer+strlen(keyword)); \
+	if ( (ptr == NULL) & (0 == strncmp(buffer, keyword, strlen(keyword))) ) { \
+		(ptr) = g_strdup(buffer+strlen(keyword)); \
+		if (*(ptr+strlen(ptr)-1) == '\n') { \
+			*(ptr+strlen(ptr)-1) = ' '; \
+		} \
 		continue; \
 	}
 /*
@@ -123,11 +125,11 @@ static const char * RA_PATH = LSB_RA_DIR;
  * specification.
  */
 static const int status_op_exitcode_map[] = { 
-	EXECRA_OK,		/* LSB_STATUS_OK */
-	EXECRA_NOT_RUNNING,	/* LSB_STATUS_VAR_PID */
-	EXECRA_NOT_RUNNING,	/* LSB_STATUS_VAR_LOCK */
-	EXECRA_NOT_RUNNING,	/* LSB_STATUS_STOPPED */
-	EXECRA_UNKNOWN_ERROR	/* LSB_STATUS_UNKNOWN */
+	EXECRA_OK,
+	EXECRA_UNKNOWN_ERROR,
+	EXECRA_UNKNOWN_ERROR,
+	EXECRA_NOT_RUNNING,
+	EXECRA_UNKNOWN_ERROR
 };
 
 /* The begin of exported function list */
@@ -260,20 +262,20 @@ execra( const char * rsc_id, const char * rsc_type, const char * provider,
 		g_string_free(debug_info, TRUE);
 	} 
 
-	closefiles(); /* don't leak open files */
 	execv(ra_pathname, params_argv);
-	cl_perror("(%s:%s:%d) execv failed for %s"
-		  , __FILE__, __FUNCTION__, __LINE__, ra_pathname);
+        cl_log(LOG_ERR, "execv error when to execute a LSB RA %s.", rsc_type);
 
         switch (errno) {
                 case ENOENT:   /* No such file or directory */
 			/* Fall down */
                 case EISDIR:   /* Is a directory */
-			exit_value = EXECRA_NOT_INSTALLED;
+                        exit_value = EXECRA_NO_RA;
+        		cl_log(LOG_ERR, "Cause: No such file or directory.");
                         break;
 
                 default:
                         exit_value = EXECRA_EXEC_UNKNOWN_ERROR;
+        		cl_log(LOG_ERR, "Cause: execv unknow error.");
         }
 
         exit(exit_value);
@@ -283,22 +285,18 @@ static uniform_ret_execra_t
 map_ra_retvalue(int ret_execra, const char * op_type, const char * std_output)
 {
 	/* Except op_type equals 'status', the UNIFORM_RET_EXECRA is compatible
-	 * with the LSB standard.
-	 */
-	if (ret_execra < 0) {
-		return EXECRA_UNKNOWN_ERROR;
-	}
-	
-	if(ret_execra == EXECRA_NOT_INSTALLED) {
-		return ret_execra;
-	}
-	
-	if (	0 == STRNCMP_CONST(op_type, "status")
-	|| 	0 == STRNCMP_CONST(op_type, "monitor")) {
-		if (ret_execra < DIMOF(status_op_exitcode_map)) {
-			ret_execra =  status_op_exitcode_map[ret_execra];
+	   with LSB standard.
+	*/
+	if ( 0 == STRNCMP_CONST(op_type, "status")) {
+		if (ret_execra < 0 || ret_execra > 4 ) {
+			ret_execra = EXECRA_UNKNOWN_ERROR;
 		}
+		return status_op_exitcode_map[ret_execra];
 	}
+	/* For none-status operation return code */
+	if ( ret_execra < 0 || ret_execra > 7 ) {
+		ret_execra = EXECRA_UNKNOWN_ERROR;
+	} 
 	return ret_execra;
 }
 
@@ -346,40 +344,30 @@ get_resource_list(GList ** rsc_info)
 			} else {
 				next_continue = FALSE;
 			}
-			/* Shorten the search time */
+			/* Shorte the search time */
 			if (buffer[0] != '#' && buffer[0] != ' '
 				&& buffer[0] != '\n') {
 				break; /* donnot find */
 			}
 	
-			if (found_begin_tag == TRUE && 0 == strncasecmp(buffer
-		    		, LSB_INITSCRIPT_INFOEND_TAG
-				, strlen(LSB_INITSCRIPT_INFOEND_TAG)) ) {
+			if (found_begin_tag == TRUE && 0 == strncmp(buffer
+		    		, LSB_INITSCRIPT_END_TAG
+				, strlen(LSB_INITSCRIPT_END_TAG)) ) {
 				is_lsb_script = TRUE;
 				break;
 			}
-			if (found_begin_tag == FALSE && 0 == strncasecmp(buffer
-				, LSB_INITSCRIPT_INFOBEGIN_TAG
-				, strlen(LSB_INITSCRIPT_INFOBEGIN_TAG)) ) {
+			if (found_begin_tag == FALSE && 0 == strncmp(buffer
+				, LSB_INITSCRIPT_BEGIN_TAG
+				, strlen(LSB_INITSCRIPT_BEGIN_TAG)) ) {
 				found_begin_tag = TRUE;	
 			}
 		}
 		fclose(fp);
 		tmp = g_list_next(cur);
-
-/*
- *  Temporarily remove the filter to the initscript, or many initscripts on
- *  many distros, such as RHEL4 and fedora5, cannot be used by management GUI.
- *  Please refer to the bug 
- *  	http://www.osdl.org/developer_bugzilla/show_bug.cgi?id=1250
- */
-
-#if 0
 		if ( is_lsb_script != TRUE ) {
 			*rsc_info = g_list_remove(*rsc_info, cur->data);
 			g_free(cur->data);
 		}
-#endif
 		cur = tmp;
 	}
 
@@ -424,7 +412,7 @@ prepare_cmd_parameters(const char * rsc_type, const char * op_type,
 	 * Add the teminating NULL pointer. 
 	 */
 	params_argv[2] = NULL;
-	if ( (ht_size != 0) && (0 != STRNCMP_CONST(op_type, "status")) ) {
+	if (ht_size != 0) {
 		cl_log(LOG_WARNING, "For LSB init script, no additional "
 			"parameters are needed.");
 	}
@@ -470,8 +458,7 @@ get_resource_meta(const char* rsc_type,  const char* provider)
 	     * shld_stop  = NULL,
 	     * dflt_start = NULL,
 	     * dflt_stop  = NULL,
-	     * s_dscrpt  = NULL,
-	     * xml_l_dscrpt  = NULL;
+	     * s_dscrpt  = NULL;
 	 GString * l_dscrpt = NULL;
 	
 	/* 
@@ -487,14 +474,6 @@ get_resource_meta(const char* rsc_type,  const char* provider)
 	meta_data = g_string_new("");
 
 	next_continue = FALSE;
-
-/*
- *  Is not stick to the rule that the description should be located in the 
- *  comment block between "### BEGIN INIT INFO" and "### END INIT INFO".
- *  Please refer to the bug 
- *  	http://www.osdl.org/developer_bugzilla/show_bug.cgi?id=1250
- */
-#if 0
 	while (NULL != fgets(buffer, BUFLEN, fp)) {
 		/* Handle the lines over BUFLEN(80) columns, only
 		 * the first part is compared.
@@ -508,12 +487,11 @@ get_resource_meta(const char* rsc_type,  const char* provider)
 			next_continue = FALSE;
 		}
 
-		if ( 0 == strncasecmp(buffer , LSB_INITSCRIPT_INFOBEGIN_TAG
-			, strlen(LSB_INITSCRIPT_INFOBEGIN_TAG)) ) {
+		if ( 0 == strncmp(buffer , LSB_INITSCRIPT_BEGIN_TAG
+			, strlen(LSB_INITSCRIPT_BEGIN_TAG)) ) {
 			break;
 		}
 	}
-#endif
 
 	/* Enter into the lsb-compliant comment block */
 	while ( NULL != fgets(buffer, BUFLEN, fp) ) {
@@ -530,12 +508,12 @@ get_resource_meta(const char* rsc_type,  const char* provider)
 		RALSB_GET_VALUE(s_dscrpt,  SHORT_DSCR)
 		
 		/* Long description may cross multiple lines */
-		if ( (l_dscrpt == NULL) && (0 == strncasecmp(buffer, DESCRIPTION
+		if ( (l_dscrpt == NULL) & (0 == strncmp(buffer, DESCRIPTION
 			, strlen(DESCRIPTION))) ) {
 			l_dscrpt = g_string_new(buffer+strlen(DESCRIPTION));
 			/* Between # and keyword, more than one space, or a tab
 			 * character, indicates the continuation line.
-			 * 	Extracted from LSB init script standard
+			 * 	Extracted from LSB init script standatd
 			 */
 			while ( NULL != fgets(buffer, BUFLEN, fp) ) {
 				if ( (0 == strncmp(buffer, "#  ", 3))
@@ -550,24 +528,17 @@ get_resource_meta(const char* rsc_type,  const char* provider)
 			}
 			continue;
 		}
-		if( l_dscrpt )
-			xml_l_dscrpt = (char *)xmlEncodeEntitiesReentrant(NULL,
-				BAD_CAST (l_dscrpt->str));
 
-		if ( 0 == strncasecmp(buffer, LSB_INITSCRIPT_INFOEND_TAG
-			, strlen(LSB_INITSCRIPT_INFOEND_TAG)) ) {
+		if ( 0 == strncmp(buffer, LSB_INITSCRIPT_END_TAG
+			, strlen(LSB_INITSCRIPT_END_TAG)) ) {
 			/* Get to the out border of LSB comment block */
 			break;
 		}
-
-		if ( buffer[0] != '#' ) {
-			break; /* Out of comment block in the beginning */
-		}
 	}
 	fclose(fp);
 	
 	g_string_sprintf( meta_data, meta_data_template, rsc_type
-			, (xml_l_dscrpt==NULL)? rsc_type : xml_l_dscrpt
+			, (l_dscrpt==NULL)? rsc_type : l_dscrpt->str
 			, (s_dscrpt==NULL)? rsc_type : s_dscrpt
 			, (provides==NULL)? "" : provides
 			, (req_start==NULL)? "" : req_start
@@ -577,38 +548,25 @@ get_resource_meta(const char* rsc_type,  const char* provider)
 			, (dflt_start==NULL)? "" : dflt_start
 			, (dflt_stop==NULL)? "" : dflt_stop );
 
-	ZAPXMLOBJ(xml_l_dscrpt);	
-	ZAPXMLOBJ(s_dscrpt);	
-	ZAPXMLOBJ(provides);	
-	ZAPXMLOBJ(req_start);	
-	ZAPXMLOBJ(req_stop);	
-	ZAPXMLOBJ(shld_start);	
-	ZAPXMLOBJ(shld_stop);	
-	ZAPXMLOBJ(dflt_start);	
-	ZAPXMLOBJ(dflt_stop);	
-
-	if( l_dscrpt )
+	if ( l_dscrpt != NULL) {
 		g_string_free(l_dscrpt, TRUE);
+		l_dscrpt = NULL;
+	}
+	ZAPGDOBJ(s_dscrpt);	
+	ZAPGDOBJ(provides);	
+	ZAPGDOBJ(req_start);	
+	ZAPGDOBJ(req_stop);	
+	ZAPGDOBJ(shld_start);	
+	ZAPGDOBJ(shld_stop);	
+	ZAPGDOBJ(dflt_start);	
+	ZAPGDOBJ(dflt_stop);	
+
 	return meta_data->str;
 }
 
 static int
 get_provider_list(const char* ra_type, GList ** providers)
 {
-	if ( providers == NULL ) {
-		cl_log(LOG_ERR, "%s:%d: Parameter error: providers==NULL"
-			, __FUNCTION__, __LINE__);
-		return -2;
-	}
-
-	if ( *providers != NULL ) {
-		cl_log(LOG_ERR, "%s:%d: Parameter error: *providers==NULL."
-			"This will cause memory leak."
-			, __FUNCTION__, __LINE__);
-	}
-
-	/* Now temporarily make it fixed */
-	*providers = g_list_append(*providers, g_strdup("heartbeat"));
-
-	return g_list_length(*providers);
+	*providers = NULL;
+	return 0;
 }
