@@ -21,7 +21,7 @@
  * It's a part of Local Resource Manager. Currently it's used by lrmd only.
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>		
 #include <string.h>
 #include <unistd.h>
@@ -33,13 +33,10 @@
 #include <errno.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
-#include <clplumbing/realtime.h>
 #include <pils/plugin.h>
 #include <dirent.h>
 #include <libgen.h>  /* Add it for compiling on OSX */
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
+#include <config.h>
 
 #include <lrm/raexec.h>
 
@@ -171,22 +168,20 @@ execra(const char * rsc_id, const char * rsc_type, const char * provider,
 		g_string_free(params_gstring, TRUE);
 	}
 
-	closefiles(); /* don't leak open files */
 	/* execute the RA */
-	execl(ra_pathname, ra_pathname, op_type, (const char *)NULL);
-	cl_perror("(%s:%s:%d) execl failed for %s" 
-		  , __FILE__, __FUNCTION__, __LINE__, ra_pathname);
+	execl(ra_pathname, ra_pathname, op_type, NULL);
 
 	switch (errno) {
 		case ENOENT:   /* No such file or directory */
 		case EISDIR:   /* Is a directory */
-			exit_value = EXECRA_NOT_INSTALLED;
+			exit_value = EXECRA_NO_RA;
 			break;
 
 		default:
 			exit_value = EXECRA_EXEC_UNKNOWN_ERROR;
 	}
 
+	cl_log(LOG_ERR, "execl error when to execute RA %s.", rsc_type);
 	exit(exit_value);
 }
 
@@ -197,7 +192,7 @@ map_ra_retvalue(int ret_execra, const char * op_type, const char * std_output)
          * no actual mapping except validating, which ensure the return code
          * will be in the range 0 to 7. Too strict?
          */
-        if (ret_execra < 0 || ret_execra > 9) {
+        if (ret_execra < 0 || ret_execra > 7) {
                 cl_log(LOG_WARNING, "mapped the invalid return code %d."
                         , ret_execra);
                 ret_execra = EXECRA_UNKNOWN_ERROR;
@@ -302,11 +297,10 @@ get_resource_meta(const char* rsc_type, const char* provider)
 	char ra_pathname[RA_MAX_NAME_LENGTH];
 	FILE* file = NULL;
 	GHashTable * tmp_for_setenv;
-	struct timespec short_sleep = {0,200000000L}; /*20ms*/
 
 	get_ra_pathname(RA_PATH, rsc_type, provider, ra_pathname);
 
-	strncat(ra_pathname, " meta-data",RA_MAX_NAME_LENGTH-strlen(ra_pathname)-1);
+	strncat(ra_pathname, " meta-data",RA_MAX_NAME_LENGTH);
 	tmp_for_setenv = g_hash_table_new(g_str_hash, g_str_equal);
 	add_OCF_env_vars(tmp_for_setenv, "DUMMY_INSTANCE", rsc_type, provider);
 	raexec_setenv(tmp_for_setenv);
@@ -315,26 +309,22 @@ get_resource_meta(const char* rsc_type, const char* provider)
 
 	file = popen(ra_pathname, "r");
 	if (NULL==file) {
-		cl_log(LOG_ERR, "%s: popen failed: %s", __FUNCTION__, strerror(errno));
 		return NULL;
 	}
 
 	g_str_tmp = g_string_new("");
 	while(!feof(file)) {
-		read_len = fread(buff, 1, BUFF_LEN - 1, file);
+		memset(buff, 0, BUFF_LEN);
+		read_len = fread(buff, 1, BUFF_LEN, file);
 		if (0<read_len) {
-			*(buff+read_len) = '\0';
 			g_string_append(g_str_tmp, buff);
 		}
 		else {
-			nanosleep(&short_sleep,NULL);
+			sleep(1);
 		}
 	}
-	if( pclose(file) ) {
-		cl_log(LOG_ERR, "%s: pclose failed: %s", __FUNCTION__, strerror(errno));
-	}
 	if (0 == g_str_tmp->len) {
-		g_string_free(g_str_tmp, TRUE);
+		pclose(file);
 		return NULL;
 	}
 	data = (char*)g_new(char, g_str_tmp->len+1);
@@ -343,6 +333,7 @@ get_resource_meta(const char* rsc_type, const char* provider)
 
 	g_string_free(g_str_tmp, TRUE);
 	
+	pclose(file);
 	return data;
 }
 
@@ -366,7 +357,7 @@ add_prefix_foreach(gpointer key, gpointer value, gpointer user_data)
 	
 	newkey = g_new(gchar, keylen);
 	strncpy(newkey, "OCF_RESKEY_", keylen);
-	strncat(newkey, key, keylen-strlen(newkey)-1);
+	strncat(newkey, key, keylen);
 	g_hash_table_insert(new_hashtable, (gpointer)newkey, g_strdup(value));
 }
 
@@ -442,10 +433,9 @@ get_providers(const char* class_path, const char* ra_type, GList ** providers)
 				free(namelist[file_num]);
 				continue;
 			}
-			snprintf(tmp_buffer,FILENAME_MAX,"%s/%s",
-				 class_path, namelist[file_num]->d_name);
-			stat(tmp_buffer, &prop);
-			if (!S_ISDIR(prop.st_mode)) {
+
+			stat(namelist[file_num]->d_name, &prop);
+			if (S_ISDIR(prop.st_mode)) {
 				free(namelist[file_num]);
 				continue;
 			}
