@@ -21,7 +21,7 @@
  * It's a part of Local Resource Manager. Currently it's used by lrmd only.
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>		
 #include <string.h>
 #include <unistd.h>
@@ -33,11 +33,10 @@
 #include <errno.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
-#include <clplumbing/uids.h>
 #include <pils/plugin.h>
 #include <dirent.h>
 #include <libgen.h>  /* Add it for compiling on OSX */
-#include <libxml/entities.h>
+#include <config.h>
 
 #include <lrm/raexec.h>
 #include <fencing/stonithd_api.h>
@@ -55,7 +54,7 @@ static PIL_rc close_stonithRA(PILInterface*, void* ud_interface);
 
 /* static const char * RA_PATH = STONITH_RA_DIR; */
 /* Temporarily use it */
-static const char * RA_PATH = HA_LIBHBDIR "/stonith/plugins/stonith/";
+static const char * RA_PATH = HA_LIBDIR "/stonith/plugins/stonith/";
 
 /* The begin of exported function list */
 static int execra(const char * rsc_id,
@@ -88,45 +87,29 @@ static struct RAExecOps raops =
 	get_resource_meta
 };
 
-static const char META_TEMPLATE[] =
+static const char * meta_data1 = "\n"
 "<?xml version=\"1.0\"?>\n"
 "<!DOCTYPE resource-agent SYSTEM \"ra-api-1.dtd\">\n"
-"<resource-agent name=\"%s\">\n"
-"<version>1.0</version>\n"
-"<longdesc lang=\"en\">\n"
-"%s\n"
-"</longdesc>\n"	
-"<shortdesc lang=\"en\">%s</shortdesc>\n"
-"%s\n"
-"<actions>\n"
-"<action name=\"start\"   timeout=\"15\" />\n"
-"<action name=\"stop\"    timeout=\"15\" />\n"
-"<action name=\"status\"  timeout=\"15\" />\n"
-"<action name=\"monitor\" timeout=\"15\" interval=\"15\" start-delay=\"15\" />\n"
-"<action name=\"meta-data\"  timeout=\"15\" />\n"
-"</actions>\n"
-"<special tag=\"heartbeat\">\n"
-"<version>2.0</version>\n"
-"</special>\n"
+"<resource-agent name=\"";
+
+static const char * meta_data2 = 
+"\" version=\"0.1\">\n"
+"  <version>1.0</version>\n";
+
+static const char * meta_data3 = 
+"  <actions>\n"
+"    <action name=\"start\"   timeout=\"15\" />\n"
+"    <action name=\"stop\"    timeout=\"15\" />\n"
+"    <action name=\"status\"  timeout=\"15\" />\n"
+"    <action name=\"monitor\" timeout=\"15\" interval=\"15\" start-delay=\"15\" />\n"
+"    <action name=\"meta-data\"  timeout=\"15\" />\n"
+"  </actions>\n"
+"  <special tag=\"heartbeat\">\n"
+"    <version>2.0</version>\n"
+"  </special>\n"
 "</resource-agent>\n";
 
-static const char * no_parameter_info = "<!-- No parameter segment -->";
-
-#define CHECKMETANULL(ret, which) \
-	if (ret == NULL) { \
-		cl_log(LOG_WARNING, "stonithRA plugin: cannot get %s " \
-			"segment of %s's metadata.", which, rsc_type); \
-		ret = no_parameter_info; \
-	}
-#define xmlize(p) \
-	( p ? (char *)xmlEncodeEntitiesReentrant(NULL, \
-				(const unsigned char *)p) \
-	 	: NULL )
-#define zapxml(p) do { \
-	if( p ) { \
-		xmlFree(p); \
-	} \
-} while(0)
+static const char * no_parameter_info = "<!-- No parameter segment --->";
 
 PIL_PLUGIN_BOILERPLATE2("1.0", Debug);
 
@@ -198,8 +181,7 @@ execra(const char * rsc_id, const char * rsc_type, const char * provider,
 	g_snprintf(buffer_tmp, sizeof(buffer_tmp), "%s_%d"
 		, 	"STONITH_RA_EXEC", getpid());
 	if (ST_OK != stonithd_signon(buffer_tmp)) {
-		cl_log(LOG_ERR, "%s:%d: Cannot sign on the stonithd."
-			, __FUNCTION__, __LINE__);
+		cl_log(LOG_ERR, "STONITH_RA_EXEC: Cannot sign on the stonithd.");
 		exit(EXECRA_UNKNOWN_ERROR);
 	}
 
@@ -209,12 +191,8 @@ execra(const char * rsc_id, const char * rsc_type, const char * provider,
 	 * variables. This is a important thing to think about and do.
 	 */
 	/* send the RA operation to stonithd to simulate a RA's actions */
-	if ( 0==STRNCMP_CONST(op_type, "start") 
-		|| 0==STRNCMP_CONST(op_type, "stop") ) {
-		cl_log(LOG_INFO
-			, "Try to %s STONITH resource <rsc_id=%s> : Device=%s"
-			, op_type, rsc_id, rsc_type);
-	}
+	cl_log(LOG_DEBUG, "Will send the stonith RA operation to stonithd: "
+		"%s %s", rsc_type, op_type);
 
 	op = g_new(stonithRA_ops_t, 1);
 	op->ra_name = g_strdup(rsc_type);
@@ -228,13 +206,14 @@ execra(const char * rsc_id, const char * rsc_type, const char * provider,
 		exit(EXECRA_EXEC_UNKNOWN_ERROR);
 	}
 
+	cl_log(LOG_DEBUG, "Waiting until the final result returned.");
 	/* May be redundant */
 	/*
 	while (stonithd_op_result_ready() != TRUE) {
 		;
 	}
 	*/
-	/* cl_log(LOG_DEBUG, "Will call stonithd_receive_ops_result."); */
+	cl_log(LOG_DEBUG, "Will call stonithd_receive_ops_result.");
 	if (ST_OK != stonithd_receive_ops_result(TRUE)) {
 		cl_log(LOG_ERR, "stonithd_receive_ops_result failed.");
 		/* Need to improve the granularity for error return code */
@@ -249,14 +228,14 @@ execra(const char * rsc_id, const char * rsc_type, const char * provider,
 	g_free(op);
 
 	stonithd_signoff();
-	/* cl_log(LOG_DEBUG, "stonithRA orignal exit code=%d", exit_value); */
+	cl_log(LOG_DEBUG, "stonithRA orignal exit code=%d", exit_value);
 	exit(map_ra_retvalue(exit_value, op_type, NULL));
 }
 
 static void
 stonithRA_ops_callback(stonithRA_ops_t * op, void * private_data)
 {
-	/* cl_log(LOG_DEBUG, "setting exit code=%d", exit_value); */
+	cl_log(LOG_DEBUG, "setting exit code=%d", exit_value);
 	exit_value = op->op_result;
 }
 
@@ -267,8 +246,7 @@ map_ra_retvalue(int ret_execra, const char * op_type, const char * std_output)
 	 * actual mapping except validating, which ensure the return code
 	 * will be in the range 0 to 7. Too strict?
 	 */
-	if (ret_execra < 0 ||
-		ret_execra > EXECRA_STATUS_UNKNOWN) {
+	if (ret_execra < 0 || ret_execra > 7) {
 		cl_log(LOG_WARNING, "mapped the invalid return code %d."
 			, ret_execra);
 		ret_execra = EXECRA_UNKNOWN_ERROR;
@@ -279,8 +257,7 @@ map_ra_retvalue(int ret_execra, const char * op_type, const char * std_output)
 static int
 get_resource_list(GList ** rsc_info)
 {
-	int rc;
-	int     needprivs = !cl_have_full_privs();
+	cl_log(LOG_ERR, "get_resource_list: begin.");
 
 	if ( rsc_info == NULL ) {
 		cl_log(LOG_ERR, "Parameter error: get_resource_list");
@@ -293,22 +270,12 @@ get_resource_list(GList ** rsc_info)
 		*rsc_info = NULL;
 	}
 
-	if (needprivs) {
-		return_to_orig_privs();
-	}
 	if (ST_OK != stonithd_signon("STONITH_RA")) {
-		cl_log(LOG_ERR, "%s:%d: Can not signon to the stonithd."
-			, __FUNCTION__, __LINE__);
-		rc = -1;
-	} else {
-		rc = stonithd_list_stonith_types(rsc_info);
-		stonithd_signoff();
+		cl_log(LOG_ERR, "Can not signon to the stonithd.");
+		return -1;
 	}
 
-	if (needprivs) {
-		return_to_dropped_privs();
-	}
-	return rc;
+	return stonithd_list_stonith_types(rsc_info);
 }
 
 static int
@@ -326,65 +293,49 @@ static char *
 get_resource_meta(const char* rsc_type, const char* provider)
 {
 	char * buffer;
-	int bufferlen = 0;
-	const char * meta_param = NULL;
-	const char * meta_longdesc = NULL;
-	const char * meta_shortdesc = NULL;
-	char *xml_meta_longdesc = NULL;
-	char *xml_meta_shortdesc = NULL;
+	const char * tmp = NULL;
 	Stonith * stonith_obj = NULL;	
 
 	if ( provider != NULL ) {
-		cl_log(LOG_DEBUG, "stonithRA plugin: provider attribute "
-			"is not needed and will be ignored.");
+		cl_log(LOG_ERR, "stonithRA plugin: now donnot take the provider"
+			" into account.");
+		return NULL;
 	}
 
 	stonith_obj = stonith_new(rsc_type);
-	meta_longdesc = stonith_get_info(stonith_obj, ST_DEVICEDESCR);
-	CHECKMETANULL(meta_longdesc, "longdesc")
-	xml_meta_longdesc = xmlize(meta_longdesc);
-	meta_shortdesc = stonith_get_info(stonith_obj, ST_DEVICENAME);
-	CHECKMETANULL(meta_shortdesc, "shortdesc") 
-	xml_meta_shortdesc = xmlize(meta_shortdesc);
-	meta_param = stonith_get_info(stonith_obj, ST_CONF_XML);
-	CHECKMETANULL(meta_param, "parameters") 
+	tmp = stonith_get_info(stonith_obj, ST_CONF_XML);
+	if (tmp == NULL) {
+		cl_log(LOG_WARNING, "stonithRA plugin: cannot get the parameter"
+			" segment of %s's metadata.", rsc_type);
+		tmp = no_parameter_info;
+	}
 
-	
-	bufferlen = STRLEN_CONST(META_TEMPLATE) + strlen(rsc_type)
-			+ strlen(xml_meta_longdesc) + strlen(xml_meta_shortdesc)
-			+ strlen(meta_param) + 1;
-	buffer = g_new(char, bufferlen);
-	buffer[bufferlen-1] = '\0';
-	snprintf(buffer, bufferlen-1, META_TEMPLATE, rsc_type
-		, xml_meta_longdesc, xml_meta_shortdesc, meta_param);
+	buffer = g_new(char, strlen(meta_data1) + strlen(meta_data2)
+				+ strlen(tmp) + strlen(meta_data3) + 1);
+
+	sprintf(buffer, "%s%s%s%s", meta_data1, meta_data2, tmp, meta_data3);
 	stonith_delete(stonith_obj);
-	zapxml(xml_meta_longdesc);
-	zapxml(xml_meta_shortdesc);
 
 	return buffer;
 }
 
 /* 
- * Currently should return *providers = NULL, but remain the old code for
+ * Currently should return *providers = NULL, but rmain the old code for
  * possible unsing in the future
  */
 static int
 get_providers(const char* class_path, const char* op_type, GList ** providers)
 {
 	if ( providers == NULL ) {
-		cl_log(LOG_ERR, "%s:%d: Parameter error: providers==NULL"
-			, __FUNCTION__, __LINE__);
+		cl_log(LOG_ERR, "Parameter error: get_providers");
 		return -2;
 	}
 
 	if ( *providers != NULL ) {
-		cl_log(LOG_ERR, "%s:%d: Parameter error: *providers==NULL."
-			"This will cause memory leak."
-			, __FUNCTION__, __LINE__);
+		cl_log(LOG_ERR, "Parameter error: get_providers."\
+			"will cause memory leak.");
+		*providers = NULL;
 	}
 
-	/* Now temporarily make it fixed */
-	*providers = g_list_append(*providers, g_strdup("heartbeat"));
-
-	return g_list_length(*providers);
+	return 0;
 }
