@@ -1,3 +1,4 @@
+/* $Id: apcmaster.c,v 1.25 2005/07/03 22:15:50 alan Exp $ */
 /*
 *
 *  Copyright 2001 Mission Critical Linux, Inc.
@@ -50,8 +51,7 @@
 /*
  * Version string that is filled in by CVS
  */
-static const char *version __attribute__ ((unused)) = "$Revision: 1.27 $"; 
-#include <lha_internal.h>
+static const char *version __attribute__ ((unused)) = "$Revision: 1.25 $"; 
 
 #define	DEVICE	"APC MasterSwitch"
 
@@ -805,7 +805,7 @@ apcmaster_destroy(StonithPlugin *s)
 static StonithPlugin *
 apcmaster_new(const char *subplugin)
 {
-	struct pluginDevice*	ms = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	ms = MALLOCT(struct pluginDevice);
 
 	if (ms == NULL) {
 		LOG(PIL_CRIT, "out of memory");
