@@ -1,3 +1,4 @@
+/* $Id: drac3.c,v 1.15 2005/04/20 20:18:16 blaschke Exp $ */
 /*
  * Stonith module for Dell DRACIII (Dell Remote Access Card)
  *
@@ -118,7 +119,7 @@ static const char *NOTpluginID = "Dell DRACIII device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_HOST_PARM \
-	XML_PARAMETER_BEGIN(ST_HOST, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_HOST, "string") \
 	  XML_HOST_SHORTDESC \
 	  XML_HOST_LONGDESC \
 	XML_PARAMETER_END
@@ -136,7 +137,7 @@ static const char *drac3XML =
 static StonithPlugin *
 drac3_new(const char *subplugin)
 {
-	struct pluginDevice *drac3d = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice *drac3d = MALLOCT(struct pluginDevice);
 
 	if (drac3d == NULL) {
 			LOG(PIL_CRIT, "out of memory");
