@@ -30,7 +30,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <dirent.h>
 
@@ -217,10 +217,6 @@ external_hostlist(StonithPlugin  *s)
 	i = 0;
 	tmp = strtok(output, WHITESPACE);
 	while (tmp != NULL) {
-		if (Debug) {
-			LOG(PIL_DEBUG, "%s: %s host %s",
-				__FUNCTION__, sd->subplugin, tmp);
-		}
 		ret[i] = STRDUP(tmp);
 		if (!ret[i]) {
 			LOG(PIL_CRIT, "%s: out of memory", __FUNCTION__);
@@ -410,6 +406,7 @@ external_set_config(StonithPlugin* s, StonithNVpair *list)
 			if (OurImports->GetValue(list, *p) == NULL) {
 				LOG(PIL_INFO, "Cannot get parameter %s from "
 					"StonithNVpair", *p);
+				return S_OOPS;
 			}
 		}
 	}
@@ -491,10 +488,6 @@ external_get_confignames(StonithPlugin* p)
 		/* now copy over confignames */
 		pch = strtok(output, WHITESPACE);		
 		for (i = 0; i < namecount; i++) {
-			if (Debug) {
-				LOG(PIL_DEBUG, "%s: %s configname %s",
-					__FUNCTION__, sd->subplugin, pch);
-			}
 			sd->confignames[i] = STRDUP(pch);
 			pch = strtok(NULL, WHITESPACE);
 		}
@@ -521,7 +514,6 @@ external_get_confignames(StonithPlugin* p)
 		for (i = 0; i < dircount; i++) {
 			sd->confignames[i] = STRDUP(files[i]->d_name);
 			free(files[i]);
-			files[i] = NULL;
 		}
 		free(files);
 		sd->confignames[dircount] = NULL;
@@ -585,7 +577,7 @@ external_getinfo(StonithPlugin * s, int reqtype)
 	}
 
 	if (rc == 0) {
-		if (sd->outputbuf != NULL) {
+		if (sd->outputbuf == NULL) {
 			FREE(sd->outputbuf);
 		}
 		sd->outputbuf =  ret;
@@ -635,7 +627,7 @@ external_destroy(StonithPlugin *s)
 static StonithPlugin *
 external_new(const char *subplugin)
 {
-	struct pluginDevice*	sd = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	sd = MALLOCT(struct pluginDevice);
 
 	if (Debug) {
 		LOG(PIL_DEBUG, "%s: called.", __FUNCTION__);
@@ -684,8 +676,8 @@ external_run_cmd(struct pluginDevice *sd, const char *op, char **output)
 	char * 			data = NULL;
 	FILE *			file;
 	char			cmd[FILENAME_MAX+64];
+	GString *		g_str_tmp = NULL;
 	struct stat		buf;
-	int			slen;
 
 	rc = snprintf(cmd, FILENAME_MAX, "%s/%s", 
 		STONITH_EXT_PLUGINDIR, sd->subplugin);
@@ -694,9 +686,8 @@ external_run_cmd(struct pluginDevice *sd, const char *op, char **output)
 		return -1;
 	}
 	
-	if (stat(cmd, &buf) != 0) {
-		LOG(PIL_CRIT, "%s: stat(2) of %s failed: %s",
-			__FUNCTION__, cmd, strerror(errno));
+        if (stat(cmd, &buf) != 0) {
+		LOG(PIL_CRIT, "%s: stating %s failed.", __FUNCTION__, cmd);
                 return -1;
         }
 
@@ -733,38 +724,28 @@ external_run_cmd(struct pluginDevice *sd, const char *op, char **output)
 		goto err;
 	}
 
-	data = NULL;
-	slen=0;
-	data = MALLOC(1);
-	while(data && !feof(file)) {
-		data[slen]=EOS;
+	g_str_tmp = g_string_new("");
+	while(!feof(file)) {
+		memset(buff, 0, BUFF_LEN);
 		read_len = fread(buff, 1, BUFF_LEN, file);
-		if (read_len > 0) {
-			data=REALLOC(data, slen+read_len+1);
-			if (data == NULL) {
-				break;
-			}
-			memcpy(data+slen, buff, read_len);
-			slen += read_len;
-			data[slen] = EOS;
-		}else{
+		if (0<read_len) {
+			g_string_append(g_str_tmp, buff);
+		}
+		else {
 			sleep(1);
 		}
 	}
+	data = (char*)MALLOC(g_str_tmp->len+1);
 	if (!data) {
 		LOG(PIL_CRIT, "%s: out of memory", __FUNCTION__);
 		goto err;
 	}
 	
+	data[0] = data[g_str_tmp->len] = 0;
+	strncpy(data, g_str_tmp->str, g_str_tmp->len);
+	g_string_free(g_str_tmp, TRUE);
 
 	rc = pclose(file);
-	if (rc != 0) {
-		LOG(PIL_INFO, "%s: Calling '%s' returned %d", __FUNCTION__, cmd, rc);
-	}
-	if (Debug && data) {
-		LOG(PIL_DEBUG, "%s: '%s' output: %s", __FUNCTION__, cmd, data);
-	}
-
 	if (output) {
 		*output = data;
 	} else {
