@@ -1,5 +1,5 @@
 /*
- * Stonith module for IBM pSeries Hardware Management Console (HMC)
+ * Stonith module for IBM Hardware Management Console (HMC)
  *
  * Author: Huang Zhen <zhenh@cn.ibm.com>
  * Support for HMC V4+ added by Dave Blaschke <debltc@us.ibm.com>
@@ -65,7 +65,7 @@
  *
  *	lssyscfg -m managed-system -r lpar -F name
  *
- * The following V3 commands provide the reset actions:
+ * The following V3 commands provide the reset/on/off actions:
  *
  *	FULL SYSTEM:
  *	  on:	chsysstate -m %1 -r sys -o on -n %1 -c full
@@ -79,7 +79,7 @@
  *
  *	where %1 is managed-system, %2 is-lpar name
  *
- * The following V4 commands provide the reset actions:
+ * The following V4 commands provide the reset/on/off actions:
  *
  *	  on:	chsysstate -m %1 -r lpar -o on -n %2 -f %3
  *	  off:	chsysstate -m %1 -r lpar -o shutdown -n %2 --immed
@@ -91,18 +91,10 @@
  * which managed system's name, and which systems on the HMC are partitioned
  * and which ones aren't...
  */
- 
-#include <portability.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-#include <errno.h>
-#include <libintl.h>
-#include <sys/wait.h>
-#include <glib.h>
-#include <stonith/stonith.h>
-#include <pils/plugin.h>
+
+#define DEVICE		"IBM HMC"
+
+#include "stonith_plugin_common.h"
 
 #ifndef	SSH_CMD
 #	define SSH_CMD	"ssh"
@@ -111,58 +103,50 @@
 #	define HMCROOT	"hscroot"
 #endif
 
-#define PIL_PLUGINTYPE          STONITH_TYPE
-#define PIL_PLUGINTYPE_S        STONITH_TYPE_S
 #define PIL_PLUGIN              ibmhmc
 #define PIL_PLUGIN_S            "ibmhmc"
 #define PIL_PLUGINLICENSE 	LICENSE_LGPL
 #define PIL_PLUGINLICENSEURL 	URL_LGPL
-
-#define LOG			PluginImports->log
-#define MALLOC			PluginImports->alloc
-#define STRDUP  		PluginImports->mstrdup
-#define FREE			PluginImports->mfree
-#define EXPECT_TOK		OurImports->ExpectToken
-#define STARTPROC		OurImports->StartProcess
+#include <pils/plugin.h>
 
 #define MAX_HOST_NAME_LEN	(256*4)
-#define MAX_CMD_LEN		1024
+#define MAX_CMD_LEN		2048
 #define FULLSYSTEMPARTITION	"FullSystemPartition"
 #define MAX_POWERON_RETRY	10
-#define WHITESPACE		" \t\n\r\f"
+#define MAX_HMC_NAME_LEN	256
+
+#define ST_MANSYSPAT		"managedsyspat"
+#define NOPASS			"nopass"
 
 #define STATE_UNKNOWN		-1
 #define STATE_OFF		0
 #define STATE_ON		1
 #define STATE_INVALID		2
 
-#define HMCURL	"http://publib-b.boulder.ibm.com/Redbooks.nsf/RedbookAbstracts/SG247038.html"
+#define HMCURL	"http://publib-b.boulder.ibm.com/redbooks.nsf/RedbookAbstracts"\
+		"/SG247038.html"
 
-static void *		ibmhmc_new(void);
-static void		ibmhmc_destroy(Stonith *);
-static int		ibmhmc_set_config_file(Stonith *, const char * cfgname);
-static int		ibmhmc_set_config_info(Stonith *, const char * info);
-static const char *	ibmhmc_getinfo(Stonith * s, int InfoType);
-static int		ibmhmc_status(Stonith * );
-static int		ibmhmc_reset_req(Stonith* s,int request,const char* host);
-static char **		ibmhmc_hostlist(Stonith  *);
-static void		ibmhmc_free_hostlist(char **);
-static void		ibmhmc_closepi(PILPlugin*pi);
-static PIL_rc		ibmhmc_closeintf(PILInterface* pi, void* pd);
+static StonithPlugin *	ibmhmc_new(const char *);
+static void		ibmhmc_destroy(StonithPlugin *);
+static const char *	ibmhmc_getinfo(StonithPlugin * s, int InfoType);
+static const char**	ibmhmc_get_confignames(StonithPlugin* p);
+static int		ibmhmc_status(StonithPlugin * );
+static int		ibmhmc_reset_req(StonithPlugin * s,int request,const char* host);
+static char **		ibmhmc_hostlist(StonithPlugin  *);
+static int		ibmhmc_set_config(StonithPlugin *, StonithNVpair*);
 
-static struct stonith_ops ibmhmcOps ={
+static struct stonith_ops ibmhmcOps = {
 	ibmhmc_new,		/* Create new STONITH object	*/
 	ibmhmc_destroy,		/* Destroy STONITH object	*/
-	ibmhmc_set_config_file,	/* set configuration from file	*/
-	ibmhmc_set_config_info,	/* Get configuration from file	*/
 	ibmhmc_getinfo,		/* Return STONITH info string	*/
+	ibmhmc_get_confignames,	/* Return configuration parameters */
+	ibmhmc_set_config,      /* Set configuration            */
 	ibmhmc_status,		/* Return STONITH device status	*/
 	ibmhmc_reset_req,	/* Request a reset */
 	ibmhmc_hostlist,	/* Return list of supported hosts */
-	ibmhmc_free_hostlist	/* free above list */
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, ibmhmc_closepi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug)
 
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
@@ -190,68 +174,57 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&ibmhmcOps
-	,	ibmhmc_closeintf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	&interfprivate); 
 }
 
-struct HMCDevice {	
-	const char *		HMCid;
-	GList*		 	hostlist;
+struct pluginDevice {	
+	StonithPlugin		sp;
+	const char *		pluginid;
+	char *			idinfo;
 	char *			hmc;
+	GList*		 	hostlist;
 	int			hmcver;
+	char *			password;
 	char **			mansyspats;
 };
 
-static const char * HMCid = 	"HMCDevice-Stonith";
-static const char * NOTibmhmcID =	"HMC device has been destroyed";
+static const char * pluginid = "HMCDevice-Stonith";
+static const char * NOTpluginID = "IBM HMC device has been destroyed";
 
-static int get_hmc_hostlist(struct HMCDevice* dev);
-static void free_hmc_hostlist(struct HMCDevice* dev);
-static int get_hmc_mansyspats(struct HMCDevice* dev, const char* mansyspats);
-static void free_hmc_mansyspats(struct HMCDevice* dev);
-static char* do_shell_cmd(const char* cmd, int* status);
-static int check_hmc_status(const char* hmc);
+#include "stonith_config_xml.h"
+
+static const char *ibmhmcXML = 
+  XML_PARAMETERS_BEGIN
+    XML_IPADDR_PARM
+  XML_PARAMETERS_END;
+
+static int get_hmc_hostlist(struct pluginDevice* dev);
+static void free_hmc_hostlist(struct pluginDevice* dev);
+static int get_hmc_mansyspats(struct pluginDevice* dev, const char* mansyspats);
+static void free_hmc_mansyspats(struct pluginDevice* dev);
+static char* do_shell_cmd(const char* cmd, int* status, const char* password);
+static int check_hmc_status(struct pluginDevice* dev);
 static int get_num_tokens(char *str);
 static gboolean pattern_match(char **patterns, char *string);
-
-#define	ISHMCDEV(i)	(((i) != NULL && (i)->pinfo != NULL)	\
-	&& ((struct HMCDevice *)(i->pinfo))->HMCid == HMCid)
-
-#ifndef MALLOCT
-	#define MALLOCT(t)	((t *)(MALLOC(sizeof(t)))) 
-#endif
-
-#define N_(text)		(text)
-#define _(text)			dgettext(ST_TEXTDOMAIN, text)
-
-
-static void
-ibmhmc_closepi(PILPlugin*pi)
-{
-}
-
-static PIL_rc
-ibmhmc_closeintf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
+/* static char* do_shell_cmd_fake(const char* cmd, int* status); */
 
 static int
-ibmhmc_status(Stonith  *s)
+ibmhmc_status(StonithPlugin  *s)
 {
-	struct HMCDevice* dev;
-
-	if (!ISHMCDEV(s)) {
-		PILCallLog(LOG, PIL_CRIT, "invalid argument to %s"
-		,	__FUNCTION__);
-		return(S_OOPS);
+	struct pluginDevice* dev = NULL;
+	
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: called\n", __FUNCTION__);
 	}
 
-	dev = (struct HMCDevice*) s->pinfo;
+	ERRIFWRONGDEV(s,S_OOPS);
+
+	dev = (struct pluginDevice*) s;
 	
-	return check_hmc_status(dev->hmc);
+	return check_hmc_status(dev);
 }
 
 
@@ -260,40 +233,40 @@ ibmhmc_status(Stonith  *s)
  */
 
 static char **
-ibmhmc_hostlist(Stonith  *s)
+ibmhmc_hostlist(StonithPlugin  *s)
 {
 	int j;
-	struct HMCDevice* dev;
+	struct pluginDevice* dev;
 	int numnames = 0;
 	char** ret = NULL;
 	GList* node = NULL;
 
-	if (!ISHMCDEV(s)) {
-		PILCallLog(LOG, PIL_CRIT, "invalid argument to %s"
-		,	__FUNCTION__);
-		return(NULL);
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: called\n", __FUNCTION__);
 	}
 
-	dev = (struct HMCDevice*) s->pinfo;
+	ERRIFWRONGDEV(s,NULL);
+
+	dev = (struct pluginDevice*) s;
 
 	/* refresh the hostlist */
 	free_hmc_hostlist(dev);
 	if (S_OK != get_hmc_hostlist(dev)){
-		PILCallLog(LOG, PIL_CRIT, "unable to obtain list of managed "
-		" systems in %s", __FUNCTION__);
+		LOG(PIL_CRIT, "unable to obtain list of managed systems in %s"
+		,	__FUNCTION__);
 		return NULL;
 	}
 
 	numnames = g_list_length(dev->hostlist);
 	if (numnames < 0) {
-		PILCallLog(LOG, PIL_CRIT, "unconfigured stonith object in %s"
+		LOG(PIL_CRIT, "unconfigured stonith object in %s"
 		,	__FUNCTION__);
 		return(NULL);
 	}
 
 	ret = (char **)MALLOC((numnames+1)*sizeof(char*));
 	if (ret == NULL) {
-		PILCallLog(LOG, PIL_CRIT, "out of memory");
+		LOG(PIL_CRIT, "out of memory");
 		return ret;
 	}
 
@@ -303,109 +276,25 @@ ibmhmc_hostlist(Stonith  *s)
 	;	j++, node = g_list_next(node))	{
 		char* host = strchr((char*)node->data, '/');
 		ret[j] = STRDUP(++host);
+		if (ret[j] == NULL) {
+			LOG(PIL_CRIT, "out of memory");
+			stonith_free_hostlist(ret);
+			return NULL;
+		}
+		g_strdown(ret[j]);
 	}
 	return ret;
 }
 
-static void
-ibmhmc_free_hostlist (char** hlist)
+
+static const char**     
+ibmhmc_get_confignames(StonithPlugin* p)
 {
-	char** hl = hlist;
-	if (hl == NULL) {
-		return;
-	}
-	while (*hl) {
-		FREE(*hl);
-		*hl = NULL;
-		++hl;
-	}
-	FREE(hlist);
-	hlist = NULL;
-}
-
-/*
- *	Parse the config information, and stash it away...
- */
-static int
-ibmhmc_parse_config_info(struct HMCDevice* dev, const char * info)
-{
-	char get_hmcver[MAX_CMD_LEN];
-	char firstchar;
-	int firstnum;
-	char* output = NULL;
-	int status;
-	char *pch;
-	char *infocopy;
-	
-	/* -p or -F option with args "ipaddr [managedsyspat]..." */
-	pch = infocopy = STRDUP(info);
-	if (pch == NULL) {
-		PILCallLog(LOG, PIL_CRIT, "out of memory");
-		return S_OOPS;
-	}
-
-	/* skip over ipaddr and null-terminate */
-	pch += strcspn(pch, WHITESPACE);
-	*pch = EOS;
-
-	/* skip over white-space up to next token */
-	pch++;
-	pch += strspn(pch, WHITESPACE);
-	if (get_hmc_mansyspats(dev, pch) != S_OK) {
-		FREE(infocopy);
-		return S_OOPS;
-	}
-
-	dev->hmc = STRDUP(infocopy);
-	FREE(infocopy);
-	
-	/* check whether the HMC has ssh command enabled */
-	if (check_hmc_status(dev->hmc) != S_OK) {
-		PILCallLog(LOG, PIL_CRIT, "HMC %s does not have remote "
-		"command execution using the ssh facility enabled", dev->hmc);
-		return S_BADCONFIG;
-	}		
-
-	/* get the HMC's version info */
-	snprintf(get_hmcver, MAX_CMD_LEN
-	,	SSH_CMD " -l " HMCROOT " %s lshmc -v | grep RM", dev->hmc);
+	static const char * names[] = {ST_IPADDR, NULL};
 	if (Debug) {
-		PILCallLog(LOG, PIL_DEBUG, "%s: get_hmcver=%s"
-		,	__FUNCTION__, get_hmcver);
+		LOG(PIL_DEBUG, "%s: called.", __FUNCTION__);
 	}
-
-	output = do_shell_cmd(get_hmcver, &status);
-	if (Debug) {
-		PILCallLog(LOG, PIL_DEBUG, "%s: output=%s\n", __FUNCTION__
-		, output ? output : "(nil)");
-	}
-	if (output == NULL) {
-		return S_BADCONFIG;
-	}		
-
-	/* parse the HMC's version info (i.e. "*RM V4R2.1" or "*RM R3V2.6") */
-	if ((sscanf(output, "*RM %c%1d", &firstchar, &firstnum) == 2)
-	&& ((firstchar == 'V') || (firstchar == 'R'))) {
-		dev->hmcver = firstnum;
-		if(Debug){
-			PILCallLog(LOG, PIL_DEBUG, "%s: HMC %s version is %d"
-			,	__FUNCTION__, dev->hmc, dev->hmcver);
-		}
-	}else{
-		PILCallLog(LOG, PIL_CRIT, "%s: unable to determine HMC %s "
-		" version", __FUNCTION__, dev->hmc);
-		FREE(output);
-		return S_BADCONFIG;
-	}
-	FREE(output);
-
-	if (S_OK != get_hmc_hostlist(dev)){
-		PILCallLog(LOG, PIL_CRIT, "unable to obtain list of managed "
-		" systems in %s", __FUNCTION__);
-		return S_BADCONFIG;
-	}
-	
-	return S_OK;
+	return names;
 }
 
 
@@ -413,11 +302,12 @@ ibmhmc_parse_config_info(struct HMCDevice* dev, const char * info)
  *	Reset the given host, and obey the request type.
  *	We should reset without power cycle for the non-partitioned case
  */
+
 static int
-ibmhmc_reset_req(Stonith * s, int request, const char * host)
+ibmhmc_reset_req(StonithPlugin * s, int request, const char * host)
 {
 	GList*			node = NULL;
-	struct HMCDevice*	dev = NULL;
+	struct pluginDevice*	dev = NULL;
 	char			off_cmd[MAX_CMD_LEN];
 	char			on_cmd[MAX_CMD_LEN];
 	char			reset_cmd[MAX_CMD_LEN];
@@ -430,22 +320,28 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 	char			state_cmd[MAX_CMD_LEN];
 	int			state = STATE_UNKNOWN;
 	
-	if (!ISHMCDEV(s) || (NULL == host)) {
-		PILCallLog(LOG, PIL_CRIT, "invalid argument to %s"
-		,	__FUNCTION__);
-		return(S_OOPS);
+	status = 0;
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: called, host=%s\n", __FUNCTION__, host);
 	}
 	
-	dev = (struct HMCDevice*) s->pinfo;
+	ERRIFWRONGDEV(s,S_OOPS);
+	
+	if (NULL == host) {
+		LOG(PIL_CRIT, "invalid argument to %s", __FUNCTION__);
+		return(S_OOPS);
+	}
+
+	dev = (struct pluginDevice*) s;
 
 	for (node = g_list_first(dev->hostlist)
 	;	NULL != node
 	;	node = g_list_next(node)) {
 		if(Debug){
-			PILCallLog(LOG, PIL_DEBUG, "%s: node->data=%s\n"
+			LOG(PIL_DEBUG, "%s: node->data=%s\n"
 			,	__FUNCTION__, (char*)node->data);
 		}
-	
+		
 		if ((pch = strchr((char*)node->data, '/')) != NULL
 		&&  0 == strcasecmp(++pch, host)) {
 			break;
@@ -453,7 +349,7 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 	}
 
 	if (!node) {
-		PILCallLog(LOG, PIL_CRIT
+		LOG(PIL_CRIT
 		,	"Host %s is not configured in this STONITH module. "
 			"Please check your configuration information.", host);
 		return (S_OOPS);
@@ -463,7 +359,7 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 	/* names[0] will be the name of managed system */
 	/* names[1] will be the name of the lpar partition */
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: names[0]=%s, names[1]=%s\n"
+		LOG(PIL_DEBUG, "%s: names[0]=%s, names[1]=%s\n"
 		,	__FUNCTION__, names[0], names[1]);
 	}
 
@@ -521,9 +417,9 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 		" --filter \"lpar_names=%s\""
 		,	dev->hmc, names[0], names[1]);
 
-		output = do_shell_cmd(on_cmd, &status);
+		output = do_shell_cmd(on_cmd, &status, dev->password);
 		if (output == NULL) {
-			PILCallLog(LOG, PIL_CRIT, "command %s failed", on_cmd);
+			LOG(PIL_CRIT, "command %s failed", on_cmd);
 			return (S_OOPS);
 		}
 		if ((pch = strchr(output, '\n')) != NULL) {
@@ -547,16 +443,16 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 		,	dev->hmc, names[0], names[1]);
 	}
 	g_strfreev(names);
-
+	
 	if(Debug){
 		LOG(PIL_DEBUG, "%s: off_cmd=%s, on_cmd=%s,"
 			"reset_cmd=%s, state_cmd=%s\n" 
 		,	__FUNCTION__, off_cmd, on_cmd, reset_cmd, state_cmd);
 	}
 
-	output = do_shell_cmd(state_cmd, &status);
+	output = do_shell_cmd(state_cmd, &status, dev->password);
 	if (output == NULL) {
-		PILCallLog(LOG, PIL_CRIT, "command %s failed", on_cmd);
+		LOG(PIL_CRIT, "command %s failed", on_cmd);
 		return S_OOPS;
 	}
 	if ((pch = strchr(output, '\n')) != NULL) {
@@ -578,26 +474,51 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 	output = NULL;
 
 	if (state == STATE_INVALID) {
-		PILCallLog(LOG, PIL_CRIT, "host %s in invalid state", host);
+		LOG(PIL_CRIT, "host %s in invalid state", host);
 		return S_OOPS;
 	}
 
 	switch (request) {
+	case ST_POWERON:
+		if (state == STATE_ON) {
+			LOG(PIL_INFO, "host %s already on", host);
+			return S_OK;
+		}
+
+		output = do_shell_cmd(on_cmd, &status, dev->password);
+		if (0 != status) {
+			LOG(PIL_CRIT, "command %s failed", on_cmd);
+			return S_OOPS;
+		}
+		break;
+	case ST_POWEROFF:
+		if (state == STATE_OFF) {
+			LOG(PIL_INFO, "host %s already off", host);
+			return S_OK;
+		}
+
+		output = do_shell_cmd(off_cmd, &status, dev->password);
+		if (0 != status) {
+			LOG(PIL_CRIT, "command %s failed", off_cmd);
+			return S_OOPS;
+		}
+		break;
 	case ST_GENERIC_RESET:
 		if (dev->hmcver < 4) {
 			if (is_lpar) {
 				if (state == STATE_ON) {
-					output = do_shell_cmd(off_cmd, &status);
+					output = do_shell_cmd(off_cmd
+					, &status, dev->password);
 					if (0 != status) {
-						PILCallLog(LOG, PIL_CRIT
-						,	"command %s failed"
-						,	off_cmd);
+						LOG(PIL_CRIT, "command %s "
+							"failed", off_cmd);
 						return S_OOPS;
 					}
 				}
 				for (i = 0; i < MAX_POWERON_RETRY; i++) {
 					char *output2;
-					output2 = do_shell_cmd(on_cmd, &status);
+					output2 = do_shell_cmd(on_cmd
+					, &status, dev->password);
 					if (output2 != NULL) {
 						FREE(output2);
 					}
@@ -608,30 +529,29 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 					}
 				}
 				if (MAX_POWERON_RETRY == i) {
-					PILCallLog(LOG, PIL_CRIT
-					,	"command %s failed"
+					LOG(PIL_CRIT, "command %s failed"
 					,	on_cmd);
 					return S_OOPS;
 				}
 			}else{
-				output = do_shell_cmd(reset_cmd, &status);
+				output = do_shell_cmd(reset_cmd
+				, &status, dev->password);
 				if (0 != status) {
-					PILCallLog(LOG, PIL_CRIT
-					,	"command %s failed"
-					,	reset_cmd);
+					LOG(PIL_CRIT, "command %s failed"						,	reset_cmd);
 					return S_OOPS;
 				}
 				break;
 			}
 		}else{
 			if (state == STATE_ON) {
-				output = do_shell_cmd(reset_cmd, &status);
+				output = do_shell_cmd(reset_cmd
+				, &status, dev->password);
 			}else{
-				output = do_shell_cmd(on_cmd, &status);
+				output = do_shell_cmd(on_cmd
+				, &status, dev->password);
 			}
 			if (0 != status) {
-				PILCallLog(LOG, PIL_CRIT
-				,	"command %s failed", reset_cmd);
+				LOG(PIL_CRIT, "command %s failed", reset_cmd);
 				return S_OOPS;
 			}
 		}
@@ -643,103 +563,179 @@ ibmhmc_reset_req(Stonith * s, int request, const char * host)
 	if (output != NULL) {
 		FREE(output);
 	}
-
-	PILCallLog(LOG, PIL_INFO, "Host %s %s %d."
-	,	host, __FUNCTION__, request);
+		
+	LOG(PIL_INFO, "Host %s %s %d.", host, __FUNCTION__, request);
 
 	return S_OK;
 }
 
+
 /*
  *	Parse the information in the given configuration file,
  *	and stash it away...
  */
+
 static int
-ibmhmc_set_config_file(Stonith* s, const char * configname)
+ibmhmc_set_config(StonithPlugin * s, StonithNVpair* list)
 {
-	FILE* cfgfile = NULL;
-	struct HMCDevice* dev = NULL;
-	char hostline[MAX_HOST_NAME_LEN];
+	struct pluginDevice* dev = NULL;
+	StonithNamesToGet	namestocopy [] =
+	{	{ST_IPADDR,	NULL}
+	,	{NULL,		NULL}
+	};
+	int rc;
+	char get_hmcver[MAX_CMD_LEN];
+	char firstchar;
+	int firstnum;
+	char* output = NULL;
+	int status;
+	const char *mansyspats;
+	int len;
 	
-	if (!ISHMCDEV(s)) {
-		PILCallLog(LOG, PIL_CRIT, "invalid argument to %s"
-		,	__FUNCTION__);
-		return(S_OOPS);
+	ERRIFWRONGDEV(s,S_OOPS);
+
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: called\n", __FUNCTION__);
+	}
+	
+	dev = (struct pluginDevice*) s;
+
+	if ((rc = OurImports->CopyAllValues(namestocopy, list)) != S_OK) {
+		return rc;
+	}
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: ipaddr=%s\n", __FUNCTION__
+		,	namestocopy[0].s_value);	
 	}
 
-	dev = (struct HMCDevice*) s->pinfo;
+	if (get_num_tokens(namestocopy[0].s_value) == 1) {
+		/* name=value pairs on command line, look for managedsyspat */
+		mansyspats = OurImports->GetValue(list, ST_MANSYSPAT);
+		if (mansyspats != NULL) {
+			if (get_hmc_mansyspats(dev, mansyspats) != S_OK) {
+				FREE(namestocopy[1].s_value);
+				return S_OOPS;
+			}
+		}
+		/* look for password */
+		dev->password = STRDUP(OurImports->GetValue(list, ST_PASSWD));
+		dev->hmc = namestocopy[0].s_value;
+	}else{
+		/* -p or -F option with args "ipaddr [managedsyspat]..." */
+		char *pch = namestocopy[0].s_value;
 
-	cfgfile = fopen(configname, "r");
-	if (cfgfile == NULL)  {
-		PILCallLog(LOG, PIL_CRIT, "Cannot open %s", configname);
-		return(S_BADCONFIG);
-	}
+		/* skip over password and null-terminate */
+		pch += strcspn(pch, WHITESPACE);
+		*pch = EOS;
 
-	while (fgets(hostline, sizeof(hostline), cfgfile) != NULL){
-		if (*hostline == '#' || *hostline == '\n' || *hostline == EOS){
-			continue;
+		/* skip over white-space up to next token */
+		pch++;
+		pch += strspn(pch, WHITESPACE);
+		if (get_hmc_mansyspats(dev, pch) != S_OK) {
+			return S_OOPS;
 		}
 
-		return ibmhmc_parse_config_info(dev, hostline);
+		dev->hmc = STRDUP(namestocopy[0].s_value);
+		FREE(namestocopy[0].s_value);
 	}
-	return S_BADCONFIG;
-}
+	
+	/* check whether the HMC has ssh command enabled */
+	if (check_hmc_status(dev) != S_OK) {
+		LOG(PIL_CRIT, "HMC %s does not have remote "
+		"command execution using the ssh facility enabled", dev->hmc);
+		return S_BADCONFIG;
+	}		
 
-/*
- *	Parse the config information in the given string, and stash it away...
- */
-static int
-ibmhmc_set_config_info(Stonith* s, const char * info)
-{
-	struct HMCDevice* dev;
-
-	if (!ISHMCDEV(s)) {
-		PILCallLog(LOG, PIL_CRIT, "%s: invalid argument", __FUNCTION__);
-		return(S_OOPS);
+	/* get the HMC's version info */
+	snprintf(get_hmcver, MAX_CMD_LEN
+	,	SSH_CMD " -l " HMCROOT " %s lshmc -v | grep RM", dev->hmc);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s: get_hmcver=%s", __FUNCTION__, get_hmcver);
 	}
 
-	dev = (struct HMCDevice *)s->pinfo;
+	output = do_shell_cmd(get_hmcver, &status, dev->password);
+	if (Debug) {
+		LOG(PIL_DEBUG, "%s: output=%s\n", __FUNCTION__
+		, output ? output : "(nil)");
+	}
+	if (output == NULL) {
+		return S_BADCONFIG;
+	}		
 
-	return(ibmhmc_parse_config_info(dev, info));
+	/* parse the HMC's version info (i.e. "*RM V4R2.1" or "*RM R3V2.6") */
+	if ((sscanf(output, "*RM %c%1d", &firstchar, &firstnum) == 2)
+	&& ((firstchar == 'V') || (firstchar == 'R'))) {
+		dev->hmcver = firstnum;
+		if(Debug){
+			LOG(PIL_DEBUG, "%s: HMC %s version is %d"
+			,	__FUNCTION__, dev->hmc, dev->hmcver);
+		}
+	}else{
+		LOG(PIL_CRIT, "%s: unable to determine HMC %s version"
+		,	__FUNCTION__, dev->hmc);
+		FREE(output);
+		return S_BADCONFIG;
+	}
+
+	len = strlen(output+4) + sizeof(DEVICE) + 1;
+	if (dev->idinfo != NULL) {
+		FREE(dev->idinfo);
+		dev->idinfo = NULL;
+	}
+	dev->idinfo = MALLOC(len * sizeof(char));
+	if (dev->idinfo == NULL) {
+		LOG(PIL_CRIT, "out of memory");
+		FREE(output);
+		return S_OOPS;
+	}
+	snprintf(dev->idinfo, len, "%s %s", DEVICE, output+4);
+	FREE(output);
+
+	if (S_OK != get_hmc_hostlist(dev)){
+		LOG(PIL_CRIT, "unable to obtain list of managed systems in %s"
+		,	__FUNCTION__);
+		return S_BADCONFIG;
+	}
+	
+	return S_OK;
 }
 
+
 static const char*
-ibmhmc_getinfo(Stonith* s, int reqtype)
+ibmhmc_getinfo(StonithPlugin* s, int reqtype)
 {
-	struct HMCDevice* dev;
+	struct pluginDevice* dev;
 	const char* ret;
 
-	if (!ISHMCDEV(s)) {
-		PILCallLog(LOG, PIL_CRIT, "HMC_idinfo: invalid argument");
-		return NULL;
-	}
+	ERRIFWRONGDEV(s,NULL);
 
-	dev = (struct HMCDevice *)s->pinfo;
+	dev = (struct pluginDevice *)s;
 
 	switch (reqtype) {
 		case ST_DEVICEID:
-			ret = _("IBM HMC Device");
+			ret = dev->idinfo;
 			break;
 
-		case ST_CONF_INFO_SYNTAX:
-			ret = _("ipaddr [managedsyspat]...");
-			break;
-
-		case ST_CONF_FILE_SYNTAX:
-			ret = _("ipaddr [managedsyspat]...\n"
-			"All items must be on one line. "
-			"Blank lines and lines beginning with # are ignored");
+		case ST_DEVICENAME:
+			ret = dev->hmc;
 			break;
 
 		case ST_DEVICEDESCR:
-			ret = _("IBM Hardware Management Console (HMC)\n"
+			ret = "IBM Hardware Management Console (HMC)\n"
 			"Use for IBM i5, p5, pSeries and OpenPower systems "
 			"managed by HMC\n"
-			"Optional parameter(s) after ipaddr is/are a "
+			"Optional parameter name " ST_MANSYSPAT " is "
 			"white-space delimited list of patterns used to match "
 			"managed system names; if last character is '*', all "
-			"names that begin with the pattern are matched\n"
-			" See " HMCURL " for more information.");
+			"names that begin with the pattern are matched\n";
+			break;
+
+		case ST_DEVICEURL:
+			ret = HMCURL;
+			break;
+
+		case ST_CONF_XML:		/* XML metadata */
+			ret = ibmhmcXML;
 			break;
 
 		default:
@@ -749,50 +745,82 @@ ibmhmc_getinfo(Stonith* s, int reqtype)
 	return ret;
 }
 
+
 /*
  *	HMC Stonith destructor...
  */
+
 static void
-ibmhmc_destroy(Stonith *s)
+ibmhmc_destroy(StonithPlugin *s)
 {
-	struct HMCDevice* dev;
+	struct pluginDevice* dev;
 
-	if (!ISHMCDEV(s)) {
-		PILCallLog(LOG, PIL_CRIT, "%s: invalid argument", __FUNCTION__);
-		return;
+	if(Debug){
+		LOG(PIL_DEBUG, "%s : called\n", __FUNCTION__);
 	}
-	dev = (struct HMCDevice *)s->pinfo;
 
-	dev->HMCid = NOTibmhmcID;
+	VOIDERRIFWRONGDEV(s);
+
+	dev = (struct pluginDevice *)s;
+
+	dev->pluginid = NOTpluginID;
 	if (dev->hmc) {
 		FREE(dev->hmc);
 		dev->hmc = NULL;
 	}
+	if (dev->password) {
+		FREE(dev->password);
+		dev->password = NULL;
+	}
+	if (dev->idinfo) {
+		FREE(dev->idinfo);
+		dev->idinfo = NULL;
+	}
 	free_hmc_hostlist(dev);
 	free_hmc_mansyspats(dev);
+	
 	FREE(dev);
 }
 
-static void *
-ibmhmc_new(void)
-{
-	struct HMCDevice* dev = MALLOCT(struct HMCDevice);
 
+static StonithPlugin *
+ibmhmc_new(const char *subplugin)
+{
+	struct pluginDevice* dev = MALLOCT(struct pluginDevice);
+	
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: called\n", __FUNCTION__);
+	}
+	
 	if (dev == NULL) {
-		PILCallLog(LOG, PIL_CRIT, "out of memory");
+		LOG(PIL_CRIT, "%s: out of memory", __FUNCTION__);
 		return(NULL);
 	}
+
 	memset(dev, 0, sizeof(*dev));
-	dev->HMCid = HMCid;
-	dev->hostlist = NULL;
+
+	dev->pluginid = pluginid;
 	dev->hmc = NULL;
+	dev->password = NULL;
+	dev->hostlist = NULL;
 	dev->mansyspats = NULL;
 	dev->hmcver = -1;
+	REPLSTR(dev->idinfo, DEVICE);
+	if (dev->idinfo == NULL) {
+		FREE(dev);
+		return(NULL);
+	}
+	dev->sp.s_ops = &ibmhmcOps;
+
+	if(Debug){
+		LOG(PIL_DEBUG, "%s: returning successfully\n", __FUNCTION__);
+	}
+
 	return((void *)dev);
 }
 
 static int
-get_hmc_hostlist(struct HMCDevice* dev)
+get_hmc_hostlist(struct pluginDevice* dev)
 {
 	int i, j, status;
 	char* output = NULL;
@@ -804,8 +832,8 @@ get_hmc_hostlist(struct HMCDevice* dev)
 	gchar** lparlist = NULL;
 
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: called, dev->hmc=%s\n"
-		,	__FUNCTION__, dev->hmc);
+		LOG(PIL_DEBUG, "%s: called, dev->hmc=%s\n", __FUNCTION__
+		,	dev->hmc);
 	}
 
 	if (dev->hmc == NULL || *dev->hmc == 0){
@@ -821,11 +849,10 @@ get_hmc_hostlist(struct HMCDevice* dev)
 			" -l " HMCROOT " %s lssyscfg -r sys -F name", dev->hmc);
 	}
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: get_syslist=%s"
-		,	__FUNCTION__, get_syslist);
+		LOG(PIL_DEBUG, "%s: get_syslist=%s", __FUNCTION__, get_syslist);
 	}
 
-	output = do_shell_cmd(get_syslist, &status);
+	output = do_shell_cmd(get_syslist, &status, dev->password);
 	if (output == NULL) {
 		return S_BADCONFIG;
 	}		
@@ -837,8 +864,7 @@ get_hmc_hostlist(struct HMCDevice* dev)
 		if (dev->hmcver < 4) {
 			name_mode = g_strsplit(syslist[i], ":", 2);
 			if(Debug){
-			PILCallLog(LOG, PIL_DEBUG
-			,	"%s: name_mode0=%s, name_mode1=%s\n"
+			LOG(PIL_DEBUG, "%s: name_mode0=%s, name_mode1=%s\n"
 			,	__FUNCTION__, name_mode[0], name_mode[1]);
 			}
 
@@ -863,12 +889,12 @@ get_hmc_hostlist(struct HMCDevice* dev)
 				" %s lssyscfg -m %s -r lpar -F name --all"
 				,	dev->hmc, name_mode[0]);
 				if(Debug){
-					PILCallLog(LOG, PIL_DEBUG
-					,	"%s: get_lpar=%s\n"
+					LOG(PIL_DEBUG, "%s: get_lpar=%s\n"
 					,	__FUNCTION__, get_lpar);
 				}
 
-				output = do_shell_cmd(get_lpar, &status);
+				output = do_shell_cmd(get_lpar
+				, &status, dev->password);
 				if (output == NULL) {
 					g_strfreev(name_mode);
 					g_strfreev(syslist);
@@ -910,11 +936,11 @@ get_hmc_hostlist(struct HMCDevice* dev)
 				 " %s lssyscfg -m %s -r lpar -F name"
 			,	dev->hmc, syslist[i]);
 			if(Debug){
-				PILCallLog(LOG, PIL_DEBUG, "%s: get_lpar=%s\n"
+				LOG(PIL_DEBUG, "%s: get_lpar=%s\n"
 				,	__FUNCTION__, get_lpar);
 			}
 
-			output = do_shell_cmd(get_lpar, &status);
+			output = do_shell_cmd(get_lpar, &status, dev->password);
 			if (output == NULL) {
 				g_strfreev(syslist);
 				return S_BADCONFIG;
@@ -941,7 +967,7 @@ get_hmc_hostlist(struct HMCDevice* dev)
 }
 
 static void
-free_hmc_hostlist(struct HMCDevice* dev)
+free_hmc_hostlist(struct pluginDevice* dev)
 {
 	if (dev->hostlist) {
 		GList* node;
@@ -955,7 +981,7 @@ free_hmc_hostlist(struct HMCDevice* dev)
 }
 
 static int
-get_hmc_mansyspats(struct HMCDevice * dev, const char *mansyspats)
+get_hmc_mansyspats(struct pluginDevice * dev, const char *mansyspats)
 {
 	char *patscopy;
 	int numpats;	
@@ -963,13 +989,13 @@ get_hmc_mansyspats(struct HMCDevice * dev, const char *mansyspats)
 	char *tmp;
 
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: called, mansyspats=%s\n"
+		LOG(PIL_DEBUG, "%s: called, mansyspats=%s\n"
 		,	__FUNCTION__, mansyspats);
 	}
 	
 	patscopy = STRDUP(mansyspats);
 	if (patscopy == NULL) {
-		PILCallLog(LOG, PIL_CRIT, "%s: out of memory", __FUNCTION__);
+		LOG(PIL_CRIT, "%s: out of memory", __FUNCTION__);
 		return S_OOPS;
 	}
 
@@ -977,7 +1003,7 @@ get_hmc_mansyspats(struct HMCDevice * dev, const char *mansyspats)
 	if (numpats > 0) {
 		dev->mansyspats = MALLOC((numpats+1)*sizeof(char *));
 		if (dev->mansyspats == NULL) {
-			PILCallLog(LOG, PIL_CRIT, "%s: out of memory"
+			LOG(PIL_CRIT, "%s: out of memory"
 			,	__FUNCTION__);
 			FREE(patscopy);
 			return S_OOPS;
@@ -991,7 +1017,7 @@ get_hmc_mansyspats(struct HMCDevice * dev, const char *mansyspats)
 		while (tmp != NULL) {
 			dev->mansyspats[i] = STRDUP(tmp);
 			if (dev->mansyspats[i] == NULL) {
-				PILCallLog(LOG, PIL_CRIT, "%s: out of memory"
+				LOG(PIL_CRIT, "%s: out of memory"
 				,	__FUNCTION__);
 				free_hmc_mansyspats(dev);
 				dev->mansyspats = NULL;
@@ -1000,14 +1026,13 @@ get_hmc_mansyspats(struct HMCDevice * dev, const char *mansyspats)
 			}
 	
 			if(Debug){
-				PILCallLog(LOG, PIL_DEBUG
-				,	"%s: adding pattern %s\n"
+				LOG(PIL_DEBUG, "%s: adding pattern %s\n"
 				,	__FUNCTION__, dev->mansyspats[i]);
 			}
 
 			/* no patterns necessary if all specified */
 			if (strcmp(dev->mansyspats[i], "*") == 0) {
-				ibmhmc_free_hostlist(dev->mansyspats);
+				stonith_free_hostlist(dev->mansyspats);
 				dev->mansyspats = NULL;
 				break;
 			}
@@ -1021,24 +1046,43 @@ get_hmc_mansyspats(struct HMCDevice * dev, const char *mansyspats)
 }
 
 static void
-free_hmc_mansyspats(struct HMCDevice* dev)
+free_hmc_mansyspats(struct pluginDevice* dev)
 {
 	if (dev->mansyspats) {
-		ibmhmc_free_hostlist(dev->mansyspats);
+		stonith_free_hostlist(dev->mansyspats);
 		dev->mansyspats = NULL;
 	}
 }
 
 static char*
-do_shell_cmd(const char* cmd, int* status)
+do_shell_cmd(const char* cmd, int* status, const char* password)
 {
 	const int BUFF_LEN=4096;
 	int read_len = 0;
 	char buff[BUFF_LEN];
+	char cmd_password[MAX_CMD_LEN];
 	char* data = NULL;
 	GString* g_str_tmp = NULL;
 
-	FILE* file = popen(cmd, "r");
+	FILE* file;
+	if (NULL == password) {
+		file = popen(cmd, "r");
+	} else {
+		snprintf(cmd_password, MAX_CMD_LEN
+		,"umask 077;"
+		 "if [ ! -d  " HA_VARRUNDIR "/heartbeat/rsctmp/ibmhmc ];"
+		 "then mkdir " HA_VARRUNDIR "/heartbeat/rsctmp/ibmhmc 2>/dev/null;"
+		 "fi;"
+		 "export hmc_tmp=`mktemp -p " HA_VARRUNDIR "/heartbeat/rsctmp/ibmhmc/`;" 
+		 "echo \"echo '%s'\">$hmc_tmp;" 
+		 "chmod +x $hmc_tmp;"
+		 "unset SSH_AGENT_SOCK SSH_AGENT_PID;"
+		 "SSH_ASKPASS=$hmc_tmp setsid %s;"
+		 "rm $hmc_tmp -f;"
+		 "unset hmc_tmp"
+		,password, cmd);
+		file = popen(cmd_password, "r"); 
+	}		
 	if (NULL == file) {
 		return NULL;
 	}
@@ -1054,16 +1098,17 @@ do_shell_cmd(const char* cmd, int* status)
 		}
 	}
 	data = (char*)MALLOC(g_str_tmp->len+1);
-	data[0] = data[g_str_tmp->len] = 0;
-	strncpy(data, g_str_tmp->str, g_str_tmp->len);
+	if (data != NULL) {
+		data[0] = data[g_str_tmp->len] = 0;
+		strncpy(data, g_str_tmp->str, g_str_tmp->len);
+	}
 	g_string_free(g_str_tmp, TRUE);
-
 	*status = pclose(file);
 	return data;
 }
 
 static int
-check_hmc_status(const char* hmc)
+check_hmc_status(struct pluginDevice* dev)
 {
 	int status;
 	char check_status[MAX_CMD_LEN];
@@ -1071,21 +1116,20 @@ check_hmc_status(const char* hmc)
 	int rc = S_OK;
 
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: called, hmc=%s\n"
-		,	__FUNCTION__, hmc);
+		LOG(PIL_DEBUG, "%s: called, hmc=%s\n", __FUNCTION__, dev->hmc);
 	}
 
 	snprintf(check_status, MAX_CMD_LEN
-	,	SSH_CMD " -l " HMCROOT " %s lshmc -r -F ssh", hmc);
+	,	SSH_CMD " -l " HMCROOT " %s lshmc -r -F ssh", dev->hmc);
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: check_status %s\n"
-		,	__FUNCTION__, check_status);
+		LOG(PIL_DEBUG, "%s: check_status %s\n", __FUNCTION__
+		,	check_status);
 	}
 
-	output = do_shell_cmd(check_status, &status);
+	output = do_shell_cmd(check_status, &status, dev->password);
 	if (Debug) {
-		PILCallLog(LOG, PIL_DEBUG, "%s: status=%d, output=%s\n"
-		,	__FUNCTION__, status, output ? output : "(nil)");
+		LOG(PIL_DEBUG, "%s: status=%d, output=%s\n", __FUNCTION__
+		,	status, output ? output : "(nil)");
 	}
 
 	if (NULL == output || strncmp(output, "enable", 6) != 0) {
@@ -1118,8 +1162,7 @@ pattern_match(char **patterns, char *string)
 	char **pattern;
 
 	if(Debug){
-		PILCallLog(LOG, PIL_DEBUG, "%s: called, string=%s\n"
-		,	__FUNCTION__, string);
+		LOG(PIL_DEBUG, "%s: called, string=%s\n", __FUNCTION__, string);
 	}
 
 	for (pattern = patterns; *pattern; pattern++) {
@@ -1140,3 +1183,14 @@ pattern_match(char **patterns, char *string)
 
 	return FALSE;
 }
+
+/*
+static char*
+do_shell_cmd_fake(const char* cmd, int* status)
+{
+	printf("%s()\n", __FUNCTION__);
+	printf("cmd:%s\n", cmd);
+	*status=0;
+	return NULL;
+}
+*/
