@@ -1,3 +1,4 @@
+/* $Id: ipmilan.c,v 1.16 2005/04/20 20:18:16 blaschke Exp $ */
 /*
  * Stonith module for ipmi lan Stonith device
  *
@@ -133,7 +134,7 @@ static const char * NOTpluginid = "IPMI-LAN device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_HOSTNAME_PARM \
-	XML_PARAMETER_BEGIN(ST_HOSTNAME, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_HOSTNAME, "string") \
 	  XML_HOSTNAME_SHORTDESC \
 	  XML_HOSTNAME_LONGDESC \
 	XML_PARAMETER_END
@@ -149,7 +150,7 @@ static const char * NOTpluginid = "IPMI-LAN device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_PORT_PARM \
-	XML_PARAMETER_BEGIN(ST_PORT, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_PORT, "string") \
 	  XML_PORT_SHORTDESC \
 	  XML_PORT_LONGDESC \
 	XML_PARAMETER_END
@@ -165,7 +166,7 @@ static const char * NOTpluginid = "IPMI-LAN device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_AUTH_PARM \
-	XML_PARAMETER_BEGIN(ST_AUTH, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_AUTH, "string") \
 	  XML_AUTH_SHORTDESC \
 	  XML_AUTH_LONGDESC \
 	XML_PARAMETER_END
@@ -181,7 +182,7 @@ static const char * NOTpluginid = "IPMI-LAN device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_PRIV_PARM \
-	XML_PARAMETER_BEGIN(ST_PRIV, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_PRIV, "string") \
 	  XML_PRIV_SHORTDESC \
 	  XML_PRIV_LONGDESC \
 	XML_PARAMETER_END
@@ -383,6 +384,9 @@ ipmilan_set_config(StonithPlugin* s, StonithNVpair * list)
 	int		rc;
 	struct ipmilanHostInfo *  tmp;
 
+	ERRIFWRONGDEV(s,S_OOPS);
+	nd = (struct pluginDevice *)s;
+
 	StonithNamesToGet	namestocopy [] =
 	{	{ST_HOSTNAME,	NULL}
 	,	{ST_IPADDR,	NULL}
@@ -394,9 +398,6 @@ ipmilan_set_config(StonithPlugin* s, StonithNVpair * list)
 	,	{NULL,		NULL}
 	};
 
-	ERRIFWRONGDEV(s,S_OOPS);
-	nd = (struct pluginDevice *)s;
-
 	ERRIFWRONGDEV(s, S_OOPS);
 	if (nd->sp.isconfigured) {
 		return S_OOPS;
@@ -406,45 +407,14 @@ ipmilan_set_config(StonithPlugin* s, StonithNVpair * list)
 		return rc;
 	}
 
-	tmp = ST_MALLOCT(struct ipmilanHostInfo);
+	tmp = MALLOCT(struct ipmilanHostInfo);
 	tmp->hostname = namestocopy[0].s_value;
 	tmp->ipaddr   = namestocopy[1].s_value;
 	tmp->portnumber = atoi(namestocopy[2].s_value);
 	FREE(namestocopy[2].s_value);
-	if (namestocopy[3].s_value == NULL) {
-		LOG(PIL_CRIT, "ipmilan auth type is NULL.  See "
-		"README.ipmilan for allowed values");
-		return S_OOPS;
-	} else if (strcmp(namestocopy[3].s_value, "none") == 0) {
-		tmp->authtype = 0;
-	} else if (strcmp(namestocopy[3].s_value, "md2") == 0) {
-		tmp->authtype = 1;
-	} else if (strcmp(namestocopy[3].s_value, "md5") == 0) {
-		tmp->authtype = 2;
-	} else if (strcmp(namestocopy[3].s_value, "key") == 0) {
-		tmp->authtype = 4;
-	} else if (strcmp(namestocopy[3].s_value, "password") == 0) {
-		tmp->authtype = 4;
-	} else {
-		LOG(PIL_CRIT, "ipmilan auth type '%s' invalid.  See "
-		"README.ipmilan for allowed values", namestocopy[3].s_value);
-		return S_OOPS;
-	}
+	tmp->authtype = atoi(namestocopy[3].s_value);
 	FREE(namestocopy[3].s_value);
-	if (namestocopy[4].s_value == NULL) {
-		LOG(PIL_CRIT, "ipmilan priv value is NULL.  See "
-		"README.ipmilan for allowed values");
-		return S_OOPS;
-	if (strcmp(namestocopy[4].s_value, "operator") == 0) {
-		tmp->privilege = 3;
-		}
-	if (strcmp(namestocopy[4].s_value, "admin") == 0) {
-		tmp->privilege = 4;
-	} else {
-		LOG(PIL_CRIT, "ipmilan priv value '%s' invalid.  See "
-		"README.ipmilan for allowed values", namestocopy[4].s_value);
-		return(S_OOPS);
-	}
+	tmp->privilege = atoi(namestocopy[4].s_value);
 	FREE(namestocopy[4].s_value);
 	tmp->username = namestocopy[5].s_value;
 	tmp->password = namestocopy[6].s_value;
@@ -545,7 +515,7 @@ ipmilan_destroy(StonithPlugin *s)
 static StonithPlugin *
 ipmilan_new(const char *subplugin)
 {
-	struct pluginDevice*	nd = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	nd = MALLOCT(struct pluginDevice);
 
 	if (nd == NULL) {
 		LOG(PIL_CRIT, "out of memory");
