@@ -1,3 +1,4 @@
+/* $Id: rcd_serial.c,v 1.29 2005/04/22 12:23:05 blaschke Exp $ */
 /*
  * Stonith module for RCD_SERIAL Stonith device
  *
@@ -32,8 +33,6 @@
  *
  */
 
-#include <lha_internal.h>
-
 #define	DEVICE	"RC Delayed Serial"
 #include "stonith_plugin_common.h"
 #include "stonith_signal.h"
@@ -268,7 +267,7 @@ static const char * NOTrcd_serialID = "RCD_Serial device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_DTRRTS_PARM \
-	XML_PARAMETER_BEGIN(ST_DTRRTS, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_DTRRTS, "string") \
 	  XML_DTRRTS_SHORTDESC \
 	  XML_DTRRTS_LONGDESC \
 	XML_PARAMETER_END
@@ -284,7 +283,7 @@ static const char * NOTrcd_serialID = "RCD_Serial device has been destroyed";
 	XML_PARM_LONGDESC_END
 
 #define XML_MSDURATION_PARM \
-	XML_PARAMETER_BEGIN(ST_MSDURATION, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_MSDURATION, "string") \
 	  XML_MSDURATION_SHORTDESC \
 	  XML_MSDURATION_LONGDESC \
 	XML_PARAMETER_END
@@ -581,7 +580,7 @@ rcd_serial_destroy(StonithPlugin *s)
 static StonithPlugin *
 rcd_serial_new(const char *subplugin)
 {
-	struct pluginDevice*	rcd = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	rcd = MALLOCT(struct pluginDevice);
 
 	if (rcd == NULL) {
 		LOG(PIL_CRIT, "out of memory");
