@@ -1,3 +1,4 @@
+/* $Id: rps10.c,v 1.24 2005/04/22 12:23:05 blaschke Exp $ */
 /*
  *	Stonith module for WTI Remote Power Controllers (RPS-10M device)
  *
@@ -34,7 +35,7 @@
 #define PIL_PLUGIN_S            "rps10"
 #define PIL_PLUGINLICENSE 	LICENSE_LGPL
 #define PIL_PLUGINLICENSEURL 	URL_LGPL
-#define	ST_RPS10		"serial_to_targets"
+#define	ST_RPS10		"<serial_device> <node> <outlet> [ <node> <outlet> ]..."
 #define MAX_PRSID		256
 #include <pils/plugin.h>
 
@@ -185,16 +186,16 @@ static const char * NOTwtiid = "OBJECT DESTROYED: (WTI RPS-10)";
 
 #define XML_RPS10_SHORTDESC \
 	XML_PARM_SHORTDESC_BEGIN("en") \
-	"Value in the format \"serial_device remotenode outlet [remotenode outlet]...\"" \
+	ST_RPS10 \
 	XML_PARM_SHORTDESC_END
 
 #define XML_RPS10_LONGDESC \
 	XML_PARM_LONGDESC_BEGIN("en") \
-	"The RPS-10 STONITH device configuration information in the format \"serial_device remotenode outlet [remotenode outlet]...\"" \
+	"The RPS-10 STONITH device configuration information in the format \"<serial_device> <remotenode> <outlet> [<remotenode> <outlet>]...\"" \
 	XML_PARM_LONGDESC_END
 
 #define XML_RPS10_PARM \
-	XML_PARAMETER_BEGIN(ST_RPS10, "string", "1") \
+	XML_PARAMETER_BEGIN(ST_RPS10, "string") \
 	  XML_RPS10_SHORTDESC \
 	  XML_RPS10_LONGDESC \
 	XML_PARAMETER_END
@@ -1046,7 +1047,7 @@ rps10_destroy(StonithPlugin *s)
 static StonithPlugin * 
 rps10_new(const char *subplugin)
 {
-	struct pluginDevice*	ctx = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	ctx = MALLOCT(struct pluginDevice);
 
 	if (Debug) {
 		LOG(PIL_DEBUG, "%s:called.", __FUNCTION__);
