@@ -1,3 +1,4 @@
+/* $Id: wti_nps.c,v 1.28 2005/07/03 22:15:50 alan Exp $ */
 /*
  *
  *  Copyright 2001 Mission Critical Linux, Inc.
@@ -59,7 +60,6 @@
  *    private subnet.
  */
 
-#include <lha_internal.h>
 #define	DEVICE	"WTI Network Power Switch"
 
 #define DOESNT_USE_STONITHKILLCOMM	1
@@ -428,11 +428,11 @@ NPSNametoOutlet(struct pluginDevice* nps, const char * name, char **outlets)
   					break;
   				}
   			}
-  			if (strncasecmp(name, sockname, 16) == 0) {
+  			if (strcasecmp(name, sockname) == 0) {
   				ret = sockno;
   				snprintf(buf, sizeof(buf), "%d ", sockno);
   				strncat(*outlets, buf, left);
-  				left = left - strlen(buf);
+  				left = left - 2;
   			}
   		}
   	} while (strlen(NameMapping) > 2 && left > 0);
@@ -787,7 +787,7 @@ wti_nps_destroy(StonithPlugin *s)
 static StonithPlugin *
 wti_nps_new(const char *subplugin)
 {
-	struct pluginDevice*	nps = ST_MALLOCT(struct pluginDevice);
+	struct pluginDevice*	nps = MALLOCT(struct pluginDevice);
 
 	if (Debug) {
 		LOG(PIL_DEBUG, "%s: called.", __FUNCTION__);
