@@ -1,3 +1,4 @@
+/* $Id: main.c,v 1.20 2005/04/06 18:58:42 blaschke Exp $ */
 /*
  * Stonith: simple test program for exercising the Stonith API code
  *
@@ -20,7 +21,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -30,7 +31,7 @@
 #include <pils/plugin.h>
 #include <glib.h>
 
-#define	OPTIONS	"c:F:p:t:T:snSlLvhd"
+#define	OPTIONS	"F:p:t:T:sSlLvhd"
 #define	EQUAL	'='
 
 extern char *	optarg;
@@ -38,8 +39,8 @@ extern int	optind, opterr, optopt;
 
 static int	debug = 0;
 
-void usage(const char * cmd, int exit_status, const char * devtype);
-void confhelp(const char * cmd, FILE* stream, const char * devtype);
+void usage(const char * cmd, int exit_status);
+void confhelp(const char * cmd, FILE* stream);
 
 /*
  * Note that we don't use the cl_log logging code because the STONITH
@@ -49,54 +50,39 @@ void confhelp(const char * cmd, FILE* stream, const char * devtype);
  */
 
 void
-usage(const char * cmd, int exit_status, const char * devtype)
+usage(const char * cmd, int exit_status)
 {
 	FILE *stream;
 
 	stream = exit_status ? stderr : stdout;
 
-	/* non-NULL devtype indicates help for specific device, so no usage */
-	if (devtype == NULL) {
-		fprintf(stream, "usage:\n");
-		fprintf(stream, "\t %s [-svh] "
-		"-L\n"
-		, cmd);
+	fprintf(stream, "usage:\n");
+	fprintf(stream, "\t %s [-svh] "
+	"-L\n"
+	, cmd);
+	fprintf(stream, "\t %s [-svh] "
+	"-t stonith-device-type "
+	"[-p stonith-device-parameters | "
+	"-F stonith-device-parameters-file] "
+	"-lS\n"
+	, cmd);
+	fprintf(stream, "\t %s [-svh] "
+	"-t stonith-device-type "
+	"[-p stonith-device-parameters | "
+	"-F stonith-device-parameters-file] "
+	"-T {reset|on|off} nodename\n"
+	, cmd);
 
-		fprintf(stream, "\t %s [-svh] "
-		"-t stonith-device-type "
-		"-n\n"
-		, cmd);
-
-		fprintf(stream, "\t %s [-svh] "
-		"-t stonith-device-type "
-		"{-p stonith-device-parameters | "
-		"-F stonith-device-parameters-file | "
-		"name=value...} "
-		"[-c count] "
-		"-lS\n"
-		, cmd);
-
-		fprintf(stream, "\t %s [-svh] "
-		"-t stonith-device-type "
-		"{-p stonith-device-parameters | "
-		"-F stonith-device-parameters-file | "
-		"name=value...} "
-		"[-c count] "
-		"-T {reset|on|off} nodename\n"
-		, cmd);
-
-		fprintf(stream, "\nwhere:\n");
-		fprintf(stream, "\t-L\tlist supported stonith device types\n");
-		fprintf(stream, "\t-l\tlist hosts controlled by this stonith device\n");
-		fprintf(stream, "\t-S\treport stonith device status\n");
-		fprintf(stream, "\t-s\tsilent\n");
-		fprintf(stream, "\t-v\tverbose\n");
-		fprintf(stream, "\t-n\toutput the config names of stonith-device-parameters\n");
-		fprintf(stream, "\t-h\tdisplay detailed help message with stonith device description(s)\n");
-	}
+	fprintf(stream, "\nwhere:\n");
+	fprintf(stream, "\t-L\tlist supported stonith device types\n");
+	fprintf(stream, "\t-l\tlist hosts controlled by this stonith device\n");
+	fprintf(stream, "\t-S\treport stonith device status\n");
+	fprintf(stream, "\t-s\tsilent\n");
+	fprintf(stream, "\t-v\tverbose\n");
+	fprintf(stream, "\t-h\tdisplay detailed help message with stonith device desriptions\n");
 
 	if (exit_status == 0) {
-		confhelp(cmd, stream, devtype);
+		confhelp(cmd, stream);
 	}
 
 	exit(exit_status);
@@ -104,20 +90,15 @@ usage(const char * cmd, int exit_status, const char * devtype)
 
 /* Thanks to Lorn Kay <lorn_kay@hotmail.com> for the confhelp code */
 void
-confhelp(const char * cmd, FILE* stream, const char * devtype)
+confhelp(const char * cmd, FILE* stream)
 {
 	char ** typelist;
 	char ** this;
 	Stonith *       s;
-	int	devfound = 0;
 
-	
-	/* non-NULL devtype indicates help for specific device, so no header */
-	if (devtype == NULL) {
-		fprintf(stream
-		,	"\nSTONITH -t device types and"
-			" associated configuration details:\n");
-	}
+	fprintf(stream
+	,	"\nSTONITH -t device types and"
+		" associated configuration details:\n");
 
 	typelist = stonith_types();
 	
@@ -126,26 +107,17 @@ confhelp(const char * cmd, FILE* stream, const char * devtype)
 			"Failed to retrieve list of STONITH modules!\n");
 		return;
 	}
-	for(this=typelist; *this && !devfound; ++this) {
+	for(this=typelist; *this; ++this) {
 		const char *    SwitchType = *this;
 		const char *	cres;
 		const char **	pnames;
 
-
 		if ((s = stonith_new(SwitchType)) == NULL) {
 			fprintf(stderr, "Invalid STONITH type %s(!)\n"
 			,	SwitchType);
 			continue;
 		}
 
-		if (devtype) {
-			if (strcmp(devtype, SwitchType)) {
-				continue;
-			} else {
-				devfound = 1;
-			}
-		}
-
 		fprintf(stream, "\n\nSTONITH Device: %s - ", SwitchType);
 
 		if ((cres = stonith_get_info(s, ST_DEVICEDESCR)) != NULL){
@@ -174,8 +146,8 @@ confhelp(const char * cmd, FILE* stream, const char * devtype)
 		,    SwitchType, stonith_get_info(s, ST_CONF_INFO_SYNTAX));
 #else
 		fprintf(stream, "For Config info [-p] syntax"
-		", give each of the above parameters in order as"
-		"\nthe -p value.\n"
+		", give each of the above parameters in order as the"
+		" -p value.\n"
 		"Arguments are separated by white space.");
 #endif
 #ifdef ST_CONFI_FILE_SYNTAX
@@ -191,9 +163,6 @@ confhelp(const char * cmd, FILE* stream, const char * devtype)
 		stonith_delete(s); s = NULL;
 	}
 	/* Note that the type list can't/shouldn't be freed */
-	if (devtype && !devfound) {
-		fprintf(stderr, "Invalid device type: '%s'\n", devtype);
-	}
 	
 }
 
@@ -215,16 +184,12 @@ main(int argc, char** argv)
 	int		silent = 0;
 	int		listhosts = 0;
 	int		listtypes = 0;
-	int 		listparanames = 0;
 
 	int		c;
 	int		errors = 0;
 	int		argcount;
 	StonithNVpair	nvargs[MAXNVARG];
 	int		nvcount=0;
-	int		j;
-	int		count = 1;
-	int		help = 0;
 
 	if ((cmdname = strrchr(argv[0], '/')) == NULL) {
 		cmdname = argv[0];
@@ -236,22 +201,13 @@ main(int argc, char** argv)
 	while ((c = getopt(argc, argv, OPTIONS)) != -1) {
 		switch(c) {
 
-		case 'c':	count = atoi(optarg);
-				if (count < 1) {
-					fprintf(stderr
-					,	"bad count [%s]\n"
-					,	optarg);
-					usage(cmdname, 1, NULL);
-				}
-				break;
-
 		case 'd':	debug++;
 				break;
 
 		case 'F':	optfile = optarg;
 				break;
 
-		case 'h':	help++;
+		case 'h':	usage(cmdname, 0);
 				break;
 
 		case 'l':	++listhosts;
@@ -282,11 +238,9 @@ main(int argc, char** argv)
 					fprintf(stderr
 					,	"bad reset type [%s]\n"
 					,	optarg);
-					usage(cmdname, 1, NULL);
+					usage(cmdname, 1);
 				}
-				break;
-
-		case 'n':	++listparanames;
+			
 				break;
 
 		case 'v':	++verbose;
@@ -297,16 +251,11 @@ main(int argc, char** argv)
 		}
 	}
 
-	if (help && !errors) {
-		usage(cmdname, 0, SwitchType);
-	}
 	if (debug) {
 		PILpisysSetDebugLevel(debug);
 	}
 	if (optfile && parameters) {
-		fprintf(stderr
-		,	"Cannot include both -F and -p options\n");
-		usage(cmdname, 1, NULL);
+		++errors;
 	}
 
 	/*
@@ -320,18 +269,17 @@ main(int argc, char** argv)
 		if (parameters)  {
 			fprintf(stderr
 			,	"Cannot include both -p and name=value "
-			"style arguments\n");
-			usage(cmdname, 1, NULL);
+			" style arguments\n");
+			usage(cmdname, 1);
 		}
 		if (optfile)  {
 			fprintf(stderr
 			,	"Cannot include both -F and name=value "
-			"style arguments\n");
-			usage(cmdname, 1, NULL);
+			" style arguments\n");
+			usage(cmdname, 1);
 		}
 		if (nvcount >= MAXNVARG) {
-			fprintf(stderr
-			,	"Too many name=value style arguments\n");
+			fprintf(stderr, "Too many n=v arguments\n");
 			exit(1);
 		}
 		nvargs[nvcount].s_name = argv[optind];
@@ -345,12 +293,12 @@ main(int argc, char** argv)
 	argcount = argc - optind;
 
 	if (!(argcount == 1 || (argcount < 1
-	&&	(status||listhosts||listtypes||listparanames)))) {
+	&&	(status||listhosts||listtypes)))) {
 		++errors;
 	}
 
 	if (errors) {
-		usage(cmdname, 1, NULL);
+		usage(cmdname, 1);
 	}
 
 	if (listtypes) {
@@ -366,38 +314,24 @@ main(int argc, char** argv)
 				printf("%s\n", *this);
 			}
 		}
-		exit(0);
+		return(0);
 	}
 
+	if (optfile == NULL && parameters == NULL && nvcount == 0) {
+		fprintf(stderr
+		,	"Must specify either -p option, -F option or "
+		" name=value style arguments\n");
+		usage(cmdname, 1);
+	}
 	if (SwitchType == NULL) {
 		fprintf(stderr,	"Must specify device type (-t option)\n");
-		usage(cmdname, 1, NULL);
+		usage(cmdname, 1);
 	}
-
-	s = stonith_new(SwitchType);
-	if (!listparanames && optfile == NULL && parameters == NULL && nvcount == 0) {
-		const char**	names;
-		int		needs_parms = 1;
-
-		if (s != NULL && (names = stonith_get_confignames(s)) != NULL && names[0] == NULL) {
-			needs_parms = 0;
-		}
-
-		if (needs_parms) {
-			fprintf(stderr
-			,	"Must specify either -p option, -F option or "
-			"name=value style arguments\n");
-			if (s != NULL) {
-				stonith_delete(s); 
-			}
-			usage(cmdname, 1, NULL);
-		}
-	}
-
 #ifndef LOG_PERROR
 #	define LOG_PERROR	0
 #endif
 	openlog(cmdname, (LOG_CONS|(silent ? 0 : LOG_PERROR)), LOG_USER);
+	s = stonith_new(SwitchType);
 	if (s == NULL) {
 		syslog(LOG_ERR, "Invalid device type: '%s'", SwitchType);
 		exit(S_OOPS);
@@ -406,22 +340,6 @@ main(int argc, char** argv)
 		stonith_set_debug(s, debug);
 	}
 
-	if (listparanames) {
-		const char**	names;
-		int		i;
-		names = stonith_get_confignames(s);
-
-		if (names != NULL) {
-			for (i=0; names[i]; ++i) {
-				printf("%s  ", names[i]);
-			}
-		}
-		printf("\n");
-		stonith_delete(s); 
-		s=NULL;
-		exit(0);
-	}
-
 	/* Old STONITH version 1 stuff... */
 	if (optfile) {
 		/* Configure the Stonith object from a file */
@@ -479,48 +397,45 @@ main(int argc, char** argv)
 		}
 	}
 
+	rc = stonith_get_status(s);
 
-	for (j=0; j < count; ++j) {
-		rc = stonith_get_status(s);
+	if ((tmp = stonith_get_info(s, ST_DEVICEID)) == NULL) {
+		SwitchType = tmp;
+	}
 
-		if ((tmp = stonith_get_info(s, ST_DEVICEID)) == NULL) {
-			SwitchType = tmp;
+	if (status && !silent) {
+		if (rc == S_OK) {
+			syslog(LOG_ERR, "%s device OK.", SwitchType);
+		}else{
+			/* Uh-Oh */
+			syslog(LOG_ERR, "%s device not accessible."
+			,	SwitchType);
 		}
+	}
 
-		if (status && !silent) {
-			if (rc == S_OK) {
-				syslog(LOG_ERR, "%s device OK.", SwitchType);
-			}else{
-				/* Uh-Oh */
-				syslog(LOG_ERR, "%s device not accessible."
-				,	SwitchType);
+	if (listhosts) {
+		char **	hostlist;
+
+		hostlist = stonith_get_hostlist(s);
+		if (hostlist == NULL) {
+			syslog(LOG_ERR, "Could not list hosts for %s."
+			,	SwitchType);
+		}else{
+			char **	this;
+
+			for(this=hostlist; *this; ++this) {
+				printf("%s\n", *this);
 			}
+			stonith_free_hostlist(hostlist);
 		}
+	}
 
-		if (listhosts) {
-			char **	hostlist;
-
-			hostlist = stonith_get_hostlist(s);
-			if (hostlist == NULL) {
-				syslog(LOG_ERR, "Could not list hosts for %s."
-				,	SwitchType);
-			}else{
-				char **	this;
-
-				for(this=hostlist; *this; ++this) {
-					printf("%s\n", *this);
-				}
-				stonith_free_hostlist(hostlist);
-			}
-		}
-
-		if (optind < argc) {
-			char *nodename;
-			nodename = g_strdup(argv[optind]);
-			g_strdown(nodename);
-			rc = stonith_req_reset(s, reset_type, nodename);
-			g_free(nodename);
-		}
+	if (optind < argc) {
+		char *nodename;
+		nodename = g_strdup(argv[optind]);
+		g_strdown(nodename);
+		rc = stonith_req_reset(s, reset_type, nodename);
+		g_free(nodename);
 	}
 	stonith_delete(s); s = NULL;
 	return(rc);
