@@ -1,4 +1,4 @@
-/* $Id: portability.h,v 1.31.2.2 2004/04/29 03:12:24 alan Exp $ */
+/* $Id: portability.h,v 1.47 2005/07/29 23:59:05 alan Exp $ */
 #ifndef PORTABILITY_H
 #  define PORTABILITY_H
 
@@ -24,10 +24,13 @@
 
 #define	EOS			'\0'
 #define	DIMOF(a)		((int) (sizeof(a)/sizeof(a[0])) )
-#define	STRLEN(conststr)	((int)(sizeof(conststr)/sizeof(char))-1)
+#define	STRLEN_CONST(conststr)  ((size_t)((sizeof(conststr)/sizeof(char))-1))
+#define	STRNCMP_CONST(varstr, conststr) strncmp((varstr), conststr, STRLEN_CONST(conststr)+1)
+#define	STRLEN(c)		STRLEN_CONST(c)
 
 /* Needs to be defined before any other includes, otherwise some system
  * headers do not behave as expected! Major black magic... */
+#undef _GNU_SOURCE  /* in case it was defined on the command line */
 #define _GNU_SOURCE
 
 #ifdef __STDC__
@@ -37,6 +40,13 @@
 #endif
 
 
+
+/* Please leave this as the first #include - Solaris needs it there */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/param.h>
 #ifdef BSD
 #	define SCANSEL_CAST	(void *)
@@ -44,15 +54,10 @@
 #	define SCANSEL_CAST	/* Nothing */
 #endif
 
-#if	__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
-#define G_GNUC_PRINTF( format_idx, arg_idx )	\
-  __attribute__((format (printf, format_idx, arg_idx)))
-#else	/* !__GNUC__ */
-#define G_GNUC_PRINTF( format_idx, arg_idx )
-#endif	/* !__GNUC__ */
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
+#if defined(ANSI_ONLY) && !defined(inline)
+#	define inline	/* nothing */
+#	undef	NETSNMP_ENABLE_INLINE
+#	define	NETSNMP_NO_INLINE 1
 #endif
 
 #ifndef HA_HAVE_SETENV
@@ -112,19 +117,10 @@ inet_pton(int af, const char *src, void *dst);
 #	define USE_GNU
 #endif
 
-#ifdef HAVE_UUID_H
-#  include <uuid.h>
-#endif /* HAVE_UUID_H */
-
-#ifdef HAVE_UUID_UUID_H
-#  include <uuid/uuid.h>
-#endif /* HAVE_UUID_UUID_H */
-
 #ifndef HA_HAVE_NFDS_T 
 	typedef unsigned int nfds_t;
 #endif
 
-
 #ifdef HAVE_STRUCT_UCRED_DARWIN
 #	include <sys/utsname.h>
 #	ifndef SYS_NMLN
@@ -132,4 +128,7 @@ inet_pton(int af, const char *src, void *dst);
 #	endif /* SYS_NMLN */
 #endif
 
+#define	POINTER_TO_SIZE_T(p)	((size_t)(p)) /*pointer cast as int*/
+#define	POINTER_TO_SSIZE_T(p)	((ssize_t)(p)) /*pointer cast as int*/
+
 #endif /* PORTABILITY_H */
