@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-#include <lha_internal.h>
+#include <portability.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/ipc.h>
@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <ha_config.h>
 #include <clplumbing/loggingdaemon.h>
 #include <netinet/in.h>
 #include <clplumbing/lsb_exitcodes.h>
@@ -42,7 +43,6 @@
 #include <clplumbing/coredumps.h>
 #include <clplumbing/setproctitle.h>
 #include <clplumbing/cl_signal.h>
-#include <clplumbing/cl_misc.h>
 #include <sys/wait.h>
 #include <clplumbing/cl_pidfile.h>
 #include <clplumbing/cl_syslog.h>
@@ -61,8 +61,12 @@
   
 
 
-#define DEFAULT_CFG_FILE	HA_SYSCONFDIR "/logd.cf"
-#define	LOGD_PIDFILE		HA_VARRUNDIR "/logd.pid"
+#ifndef DEFAULT_CFG_FILE
+#	define DEFAULT_CFG_FILE	"/etc/logd.cf"
+#endif
+#ifndef	LOGD_PIDFILE
+#	define	LOGD_PIDFILE	VAR_RUN_D "/logd.pid"
+#endif
 
 #define	FD_STDIN	0
 #define	FD_STDOUT	1
@@ -74,6 +78,7 @@
 #define READ_PROC_CHAN	1
 #define LOGD_QUEUE_LEN  128
 
+#define MAXLINE 128
 #define EOS '\0'
 #define	nullchk(a)	((a) ? (a) : "<null>")
 
@@ -83,9 +88,9 @@ int	logd_deadtime_ms = 10000;
 gboolean RegisteredWithApphbd = FALSE;
 gboolean	verbose =FALSE;
 pid_t	write_process_pid;
-IPC_Channel*	chanspair[2];
-gboolean	stop_reading = FALSE;
-gboolean	needs_shutdown = FALSE;
+IPC_Channel*		chanspair[2];
+gboolean stop_reading = FALSE;
+gboolean needs_shutdown = FALSE;
 
 struct {
 	char		debugfile[MAXLINE];
@@ -93,16 +98,12 @@ struct {
 	char		entity[MAXLINE];
 	int		log_facility;
 	gboolean	useapphbd;
-	mode_t		logmode;
-	gboolean	syslogfmtmsgs;
 } logd_config =
 	{
 		"",
 		"",
 		"logd",
-		HA_LOG_FACILITY,
-		FALSE,
-		0644,
+		LOG_LOCAL7,
 		FALSE
 	};
 
@@ -114,8 +115,6 @@ static int	set_entity(const char * option);
 static int	set_useapphbd(const char* option);
 static int	set_sendqlen(const char * option);
 static int	set_recvqlen(const char * option);
-static int	set_logmode(const char * option);
-static int	set_syslogfmtmsgs(const char * option);
 
 
 static char*			cmdname = NULL;
@@ -125,15 +124,13 @@ struct directive{
 	const char* name;
 	int (*add_func)(const char*);
 } Directives[]= {
-	{"debugfile",	set_debugfile},
-	{"logfile",	set_logfile},
-	{"logfacility",	set_facility},
-	{"entity",	set_entity},
-	{"useapphbd",	set_useapphbd},
-	{"sendqlen",	set_sendqlen},
-	{"recvqlen",	set_recvqlen},
-	{"logmode",	set_logmode},
-	{"syslogmsgfmt",set_syslogfmtmsgs}
+	{"debugfile", set_debugfile},
+	{"logfile", set_logfile},
+	{"logfacility", set_facility},
+	{"entity", set_entity},
+	{"useapphbd", set_useapphbd},
+	{"sendqlen", set_sendqlen},
+	{"recvqlen", set_recvqlen}
 };
 
 struct _syslog_code {
@@ -282,40 +279,7 @@ set_recvqlen(const char * option)
 	
 }
 
-static int
-set_logmode(const char * option)
-{
-	unsigned long	mode;
-	char *		endptr;
-	if (!option){
-		cl_log(LOG_ERR, "NULL logmode parameter");
-		return FALSE;
-	}
-	mode = strtoul(option, &endptr, 8);
-	if (*endptr != EOS) {
-		cl_log(LOG_ERR, "Invalid log mode [%s]", option);
-		return FALSE;
-	}
-	if (*option != '0') {
-		/* Whine if mode doesn't start with '0' */
-		cl_log(LOG_WARNING, "Log mode [%s] assumed to be octal"
-		,	option);
-	}
-	logd_config.logmode = (mode_t)mode;
-	return TRUE;
-}
-static int
-set_syslogfmtmsgs(const char * option)
-{
-	gboolean	dosyslogfmt;
 
-	if (cl_str_to_boolean(option, &dosyslogfmt) == HA_OK) {
-		cl_log_enable_syslog_filefmt(dosyslogfmt);
-	}else{
-		return FALSE;
-	}
-	return TRUE;
-}
 
 
 typedef struct {
@@ -555,23 +519,7 @@ logd_make_daemon(gboolean daemonize)
 	}
 
 	if (daemonize){
-		mode_t	mask;
-		/*
-		 *	Some sample umask calculations:
-		 *
-		 *	logmode		= 0644
-		 *
-		 *	(~0644)&0777	= 0133
-		 *	(0133 & ~0111)	= 0022
-		 *	=> umask will be 022 (the expected result)
-		 *
-		 *	logmode		= 0600
-		 *	(~0600)&0777	= 0177
-		 *	(0177 & ~0111)	= 0066
-		 */
-		mask = (mode_t)(((~logd_config.logmode) & 0777) & (~0111));
-		umask(mask);
-		
+		umask(022);
 		close(FD_STDIN);
 		(void)open(devnull, O_RDONLY);		/* Stdin:  fd 0 */
 		close(FD_STDOUT);
@@ -790,13 +738,17 @@ logd_term_action(int sig, gpointer userdata)
 		       (int)client->logchan->send_queue->current_qlen,
 		       client->app_name);
 		
-		client->logchan->ops->waitout(client->logchan);
+		while(client->logchan->send_queue->current_qlen > 0) {
+			sleep(1);
+		}
 	}
 
-	cl_log(LOG_DEBUG, "logd_term_action"
-	": waiting for %d messages to be read by write process"
-	,	(int)chanspair[WRITE_PROC_CHAN]->send_queue->current_qlen);
-	chanspair[WRITE_PROC_CHAN]->ops->waitout(chanspair[WRITE_PROC_CHAN]);
+	cl_log(LOG_DEBUG, "logd_term_action:"
+	       " waiting for %d messages to be read by write process",
+	       (int)chanspair[WRITE_PROC_CHAN]->send_queue->current_qlen);
+	while(chanspair[WRITE_PROC_CHAN]->send_queue->current_qlen > 0) {
+		sleep(1);
+	}
 	
         cl_log(LOG_DEBUG, "logd_term_action: sending SIGTERM to write process");
 	if (CL_KILL(write_process_pid, SIGTERM) >= 0){
@@ -875,6 +827,8 @@ direct_log(IPC_Channel* ch, gpointer user_data)
 {
 	
 	IPC_Message*		ipcmsg;
+	LogDaemonMsg*		logmsg;
+	int			priority;
 	GMainLoop*		loop;
 	
 
@@ -896,22 +850,12 @@ direct_log(IPC_Channel* ch, gpointer user_data)
 		
 		if( ipcmsg->msg_body 
 		    && ipcmsg->msg_len > 0 ){
-			LogDaemonMsg*	logmsg;
-			LogDaemonMsg	copy;
 			
 			logmsg = (LogDaemonMsg*) ipcmsg->msg_body;
-#define	COPYFIELD(copy, msg, field) memcpy(((u_char*)&copy.field), ((u_char*)&msg->field), sizeof(copy.field))
-			COPYFIELD(copy, logmsg, use_pri_str);
-			COPYFIELD(copy, logmsg, entity);
-			COPYFIELD(copy, logmsg, entity_pid);
-			COPYFIELD(copy, logmsg, timestamp);
-			COPYFIELD(copy, logmsg, priority);
-			/* Don't want to copy logmsg->message */
+			priority = logmsg->priority;
 		
-			cl_direct_log(copy.priority, logmsg->message
-			,	copy.use_pri_str
-			,	copy.entity, copy.entity_pid
-			,	copy.timestamp);
+			cl_direct_log(priority, logmsg->message, logmsg->use_pri_str,
+				      logmsg->entity, logmsg->entity_pid, logmsg->timestamp);
 		
 
 			(void)logd_log;
@@ -919,9 +863,9 @@ direct_log(IPC_Channel* ch, gpointer user_data)
 			if (verbose){
 				logd_log("%s[%d]: %s %s\n", 
 					 logmsg->entity[0]=='\0'?
-					 "unknown": copy.entity,
-					 copy.entity_pid, 
-					 ha_timestamp(copy.timestamp),
+					 "unknown": logmsg->entity,
+					 logmsg->entity_pid, 
+					 ha_timestamp(logmsg->timestamp),
 					 logmsg->message);
 				 }
  */
@@ -932,7 +876,7 @@ direct_log(IPC_Channel* ch, gpointer user_data)
 		
 	}
 	if(needs_shutdown) {
-		cl_log(LOG_INFO, "Exiting write process");
+		cl_log(LOG_INFO, "ha_logd: Exiting write process");
 		g_main_quit(loop);
 		return FALSE;
 	}
