@@ -1,3 +1,4 @@
+/* $Id: lrmadmin.c,v 1.35 2005/06/22 06:53:41 sunjd Exp $ */
 /* File: lrmadmin.c
  * Description: A adminstration tool for Local Resource Manager
  *
@@ -20,13 +21,12 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include <lha_internal.h>
+#include <portability.h>
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <time.h>
 #ifndef __USE_GNU
 #define __USE_GNU
 /* For strnlen protype */ 
@@ -41,13 +41,12 @@
 #endif /* HAVE_GETOPT_H */
 #include <clplumbing/cl_log.h>
 #include <lrm/lrm_api.h>
-#include <lrm/lrm_msg.h>
 #include <lrm/raexec.h>
 #include <clplumbing/lsb_exitcodes.h>
 #include <clplumbing/GSource.h>
 #include <clplumbing/Gmain_timeout.h>
 
-const char * optstring = "AD:X:dEF:d:sg:M:O:P:c:S:LI:CT:n:h";
+const char * optstring = "AD:dEF:d:sg:M:P:c:S:LI:CT:h";
 
 #ifdef HAVE_GETOPT_H
 static struct option long_options[] = {
@@ -55,15 +54,13 @@ static struct option long_options[] = {
 	{"executera", 1, 0, 'E'},
 	{"flush",1,0,'F'},
 	{"monitor",0,0,'M'},
-	{"state",1,0,'S'},
+	{"status",1,0,'S'},
 	{"listall",0,0,'L'},
 	{"information",1,0,'I'},
 	{"add",1,0,'A'},
 	{"delete",1,0,'D'},
-	{"fail",1,0,'X'},
 	{"raclass_supported",1,0,'C'},
 	{"ratype_supported",1,0,'T'},
-	{"all_type_metadata",1,0,'O'},
 	{"metadata",1,0,'M'},
 	{"provider",1,0,'P'},
 	{"help",0,0,'h'},
@@ -73,7 +70,6 @@ static struct option long_options[] = {
 
 GMainLoop *mainloop = NULL;
 const char * lrmadmin_name = "lrmadmin";
-const char * fake_name = NULL;
 /* 20 is the length limit for a argv[x] */
 const int ARGVI_MAX_LEN = 48;
 
@@ -88,47 +84,21 @@ typedef enum {
 	INF_RSC,
 	ADD_RSC,
 	DEL_RSC,
-	FAIL_RSC,
 	RACLASS_SUPPORTED,
 	RATYPE_SUPPORTED,
 	RA_METADATA,
 	RA_PROVIDER,
-	ALL_RA_METADATA,
 	HELP
 } lrmadmin_cmd_t;
 
-#define nullcheck(p)       ((p) ? (p) : "<null>")
-static const char * status_msg[6] = {
-	"pending",		  /* LRM_OP_PENDING	 */
+static const char * status_msg[5] = {
 	"succeed", 		  /* LRM_OP_DONE         */
         "cancelled", 		  /* LRM_OP_CANCELLED    */
         "timeout",		  /* LRM_OP_TIMEOUT 	 */
         "not Supported",	  /* LRM_OP_NOTSUPPORTED */
-        "failed due to an error"  /* LRM_OP_ERROR	 */
+        "failed due to an error"   /* LRM_OP_ERROR	 */
 };
 
-static const char * rc_msg[] = {
-        "unknown error",
-        "no ra",
-        "ok",
-        "unknown error",
-        "invalid parameter",
-        "unimplement feature",
-        "insufficient priority",
-        "not installed",
-        "not configured",
-        "not running",
-        "running master",
-        "failed master",
-	"invalid rc",
-        /* For status command only */
-        "daemon dead1",
-        "daemon dead2",
-        "daemon stopped",
-        "status unknow"
-};
-
-
 static gboolean QUIT_GETOPT = FALSE;
 static lrmadmin_cmd_t lrmadmin_cmd = NULL_OP;
 static gboolean ASYN_OPS = FALSE; 
@@ -140,16 +110,14 @@ const char * simple_help_screen =
 "         {-A|--add} <rscid> <raclass> <ratype> <provider|NULL> [<rsc_params_list>]\n"
 "         {-D|--delete} <rscid>\n"
 "         {-F|--flush} <rscid>\n"
-"         {-X|--fail} <rscid> [<fail_rc> [<fail_reason>]]\n"
 "         {-E|--execute} <rscid> <operator> <timeout> <interval> <target_rc|EVERYTIME|CHANGED> [<operator_parameters_list>]\n"
-"         {-S|--state} <rscid> [-n <fake_name>]\n"
+"         {-S|--state} <rscid>\n"
 "         {-L|--listall}\n"
 "         {-I|--information} <rsc_id>\n"
 "         {-C|--raclass_supported}\n"
-"         {-T|--ratype_supported} <raclass>\n"
-"         {-O|--all metadata of this class} <raclass>\n"
-"         {-M|--metadata} <raclass> <ratype> <provider|NULL>\n"
-"         {-P|--provider} <raclass> <ratype>\n"
+"         {-T|--ratype_supported} <raclss>\n"
+"         {-M|--metadata} <raclss> <ratype> <provider|NULL>\n"
+"         {-P|--provider} <raclss> <ratype>\n"
 "         {-h|--help}\n";
 
 #define OPTION_OBSCURE_CHECK \
@@ -162,15 +130,12 @@ const char * simple_help_screen =
 static int resource_operation(ll_lrm_t * lrmd, int argc, int optind, 
 			      char * argv[]);
 static int add_resource(ll_lrm_t * lrmd, int argc, int optind, char * argv[]);
-static int fail_resource(ll_lrm_t * lrmd, char *rsc_id, int optc, char *opts[]);
 static int transfer_cmd_params(int amount, int start, char * argv[], 
 			   const char * class, GHashTable ** params_ht);
-static void g_print_stringitem_and_free(gpointer data, gpointer user_data);
-static void g_print_rainfo_item_and_free(gpointer data, gpointer user_data);
+static void g_print_stringitem(gpointer data, gpointer user_data);
+static void g_print_rainfo_item(gpointer data, gpointer user_data);
 static void g_print_ops(gpointer data, gpointer user_data);
 static void g_get_rsc_description(gpointer data, gpointer user_data);
-static void g_print_meta(gpointer key, gpointer value, gpointer user_data);
-
 static void print_rsc_inf(lrm_rsc_t * lrmrsc);
 static char * params_hashtable_to_str(const char * class, GHashTable * ht);
 static void free_stritem_of_hashtable(gpointer key, gpointer value, 
@@ -200,9 +165,7 @@ int main(int argc, char **argv)
 	GList 	*raclass_list = 0, 
 		*ratype_list = 0,
 		*rscid_list;
-	GHashTable *all_meta = NULL;
 	char raclass[20];
-	const char * login_name = lrmadmin_name;
 
 	/* Prevent getopt_long to print error message on stderr isself */
 	/*opterr = 0; */  
@@ -213,7 +176,7 @@ int main(int argc, char **argv)
 	}
 	
         cl_log_set_entity(lrmadmin_name);
-	cl_log_enable_stderr(TRUE);
+	cl_log_enable_stderr(FALSE);
 	cl_log_set_facility(LOG_USER);
 
 	memset(rscid_arg_tmp, '\0', RID_LEN);
@@ -250,12 +213,6 @@ int main(int argc, char **argv)
 				}
 				break;
 
-			case 'X':
-				OPTION_OBSCURE_CHECK 
-				lrmadmin_cmd = FAIL_RSC;
-				strncpy(rscid_arg_tmp, optarg, RID_LEN-1);
-				break;
-
 			case 'C':
 				OPTION_OBSCURE_CHECK 
 				lrmadmin_cmd = RACLASS_SUPPORTED;
@@ -268,14 +225,6 @@ int main(int argc, char **argv)
 					strncpy(raclass, optarg, 19);
 				}
 				break;
-			
-			case 'O':
-				OPTION_OBSCURE_CHECK 
-				lrmadmin_cmd = ALL_RA_METADATA;
-				if (optarg) {
-					strncpy(raclass, optarg, 19);
-				}
-				break;
 
 			case 'F':
 				OPTION_OBSCURE_CHECK 
@@ -321,14 +270,9 @@ int main(int argc, char **argv)
 				lrmadmin_cmd = INF_RSC;
 				break;
 
-			case 'n':
-				if (optarg) {
-					fake_name = optarg;
-				}
-				break;
-
 			case 'h':
 				OPTION_OBSCURE_CHECK 
+				/* print detailed help screen? */
 				printf("%s",simple_help_screen);
 				return 0;
 
@@ -340,7 +284,7 @@ int main(int argc, char **argv)
 				return -1;
 
 			default:
-				cl_log(LOG_ERR,"getopt returned character"
+				cl_log(LOG_ERR,"Error:getopt returned character"
 					 " code %c.", option_char);
 				return -1;
                }
@@ -349,27 +293,24 @@ int main(int argc, char **argv)
         lrmd = ll_lrm_new("lrm");
 
         if (NULL == lrmd) {
-               	cl_log(LOG_ERR,"ll_lrm_new returned NULL.");
+               	cl_log(LOG_ERR,"ll_lrn_new return null.");
                	return -2;
         }
 
 	lrmd->lrm_ops->set_lrm_callback(lrmd, lrm_op_done_callback);
 
-	if (fake_name != NULL) {
-		login_name = fake_name;
-	}
-        if (lrmd->lrm_ops->signon(lrmd, login_name) != 1) { /* != HA_OK */
-		printf("lrmd is not running.\n");
+        if (lrmd->lrm_ops->signon(lrmd, lrmadmin_name) != 1) { /* != HA_OK */
+		printf("lrmd daemon is not running.\n");
 		if (lrmadmin_cmd == DAEMON_OP) { 
 			return LSB_STATUS_STOPPED;
 		} else {
-			cl_log(LOG_WARNING,"Can't connect to lrmd!");
+			cl_log(LOG_WARNING,"Can't connect to lrmd, quit!");
 			return -2;
 		}
 	}
 	
 	if (lrmadmin_cmd == DAEMON_OP) { 
-		printf("lrmd is stopped.\n");
+		printf("lrmd daemon is running.\n");
 		lrmd->lrm_ops->signoff(lrmd);
 		return 0;
 	}
@@ -385,7 +326,7 @@ int main(int argc, char **argv)
 					ret_value = -3;
 				}
 				if ( call_id == -1 ) {
-					cl_log(LOG_WARNING, "Failed! No such "
+					cl_log(LOG_WARNING, "Failed! no this "
 					   "resource %s.", argv[optind]);
 					ret_value = -2;
 				}
@@ -400,7 +341,7 @@ int main(int argc, char **argv)
 				/* Return value: HA_OK = 1 Or  HA_FAIL = 0 */
 				if ( call_id == 0 ) {
 					cl_log(LOG_ERR, "Resource operation "
-					"failed." );
+					"Failed." );
 					ret_value = -3;
 					ASYN_OPS = FALSE;
 				} else { 
@@ -431,7 +372,7 @@ int main(int argc, char **argv)
 		case DEL_RSC:
 			/* Return value: HA_OK = 1 Or  HA_FAIL = 0 */
 			if (lrmd->lrm_ops->delete_rsc(lrmd, rscid_arg_tmp)==1) {
-				printf("Succeeded in deleting this resource.\n");
+				printf("Succeeded in delete this resource.\n");
 			} else {
 				printf("Failed to delete this resource.\n");
 				ret_value = -3;
@@ -439,19 +380,6 @@ int main(int argc, char **argv)
 			ASYN_OPS = FALSE;
 			break;	
 
-		case FAIL_RSC:
-			/* Return value: HA_OK = 1 Or  HA_FAIL = 0 */
-			if (fail_resource(lrmd, rscid_arg_tmp,
-				argc-optind, argv+optind) == 1)
-			{
-				printf("Succeeded in failing the resource.\n");
-			} else {
-				printf("Failed to fail the resource.\n");
-				ret_value = -3;
-			}
-			ASYN_OPS = FALSE;
-			break;	
-
 		case FLUSH:
 			lrm_rsc = get_lrm_rsc(lrmd, rscid_arg_tmp);
 			if (!(lrm_rsc)) {
@@ -464,7 +392,6 @@ int main(int argc, char **argv)
 					printf("Failed to flush.\n");
 					ret_value = -3;
 				}
-				lrm_free_rsc(lrm_rsc);
 			}
 
 			ASYN_OPS = FALSE;
@@ -473,15 +400,15 @@ int main(int argc, char **argv)
 		case RACLASS_SUPPORTED:
 			raclass_list = lrmd->lrm_ops->
 					get_rsc_class_supported(lrmd);
-			printf("There are %d RA classes supported:\n", 
+			printf("Support %d RA classes\n", 
 					g_list_length(raclass_list));
 			if (raclass_list) {
-				g_list_foreach(raclass_list, g_print_stringitem_and_free,
+				g_list_foreach(raclass_list, g_print_stringitem,
 						NULL);
 				g_list_free(raclass_list);
 				ret_value = LSB_EXIT_OK;
 			} else {
-				printf("No RA classes found!\n");
+				printf("No any RA class is supported\n");
 				ret_value = -3;
 			}
 
@@ -491,23 +418,19 @@ int main(int argc, char **argv)
 		case RATYPE_SUPPORTED:
 		     	ratype_list = lrmd->lrm_ops->
 				get_rsc_type_supported(lrmd, raclass);
-			printf("There are %d RAs:\n", g_list_length(ratype_list));
+			printf("List size: %d\n", g_list_length(ratype_list));
 			if (ratype_list) {
-				g_list_foreach(ratype_list, g_print_rainfo_item_and_free,
+				g_list_foreach(ratype_list, g_print_rainfo_item,
 						NULL);
-				g_list_free(ratype_list);
+				/* g_list_free(ratype_list); */
+			} else {
+				printf("For this RA class, no any RA type is "
+					"supported\n");
 			}
 
 			ASYN_OPS = FALSE;
 			break;
-		case ALL_RA_METADATA:
-			all_meta = lrmd->lrm_ops->get_all_type_metadata(lrmd, raclass);
-			if (all_meta) {
-				g_hash_table_foreach(all_meta, g_print_meta, NULL);
-				g_hash_table_destroy(all_meta);
-			}
-			ASYN_OPS = FALSE;
-			break;
+
 		case LIST_ALLRSC:
 			rscid_list = lrmd->lrm_ops->get_all_rscs(lrmd);
 			if (rscid_list) {
@@ -515,7 +438,7 @@ int main(int argc, char **argv)
 						, lrmd);
 				g_list_free(rscid_list);
 			} else
-				printf("Currently no resources are managed by "
+				printf("Currently no resource is managed by "
 					 "LRM.\n");
 
 			ASYN_OPS = FALSE;
@@ -527,7 +450,7 @@ int main(int argc, char **argv)
 				ret_value = -3;
 			} else {
 				print_rsc_inf(lrm_rsc);
-				lrm_free_rsc(lrm_rsc);
+				g_free(lrm_rsc);
 			}
 
 			ASYN_OPS = FALSE;
@@ -546,16 +469,11 @@ int main(int argc, char **argv)
 					 cur_state==LRM_RSC_IDLE?
 					 "LRM_RSC_IDLE":"LRM_RSC_BUSY");
 								
-				printf("The resource %d operations' "
-					"information:\n"
-					, g_list_length(ops_queue));
 				if (ops_queue) {
-					g_list_foreach(ops_queue,
-						       g_print_ops, 
-						       NULL);
-					lrm_free_op_list(ops_queue);
+					g_list_foreach(ops_queue, g_print_ops, 
+							NULL);
+					g_list_free(ops_queue);
 				}
-				lrm_free_rsc(lrm_rsc);
 			}
 
 			ASYN_OPS = FALSE;
@@ -563,8 +481,7 @@ int main(int argc, char **argv)
 
 
 		default:
-			fprintf(stderr, "Option %c is not supported yet.\n",
-				option_char);
+			fprintf(stderr, "This option is not supported yet.\n");
 			ret_value = -1;
 			ASYN_OPS = FALSE;
 			break;	
@@ -578,7 +495,7 @@ int main(int argc, char **argv)
 		}
 
 		mainloop = g_main_new(FALSE);
-		printf( "Waiting for lrmd to callback...\n");
+		printf( "waiting for calling result from the lrmd.\n");
         	g_main_run(mainloop);
 	}
 
@@ -621,17 +538,16 @@ lrm_op_done_callback(lrm_op_t* op)
 	printf("type:%s\n", op->op_type);
 	if ( (0 == STRNCMP_CONST(op->op_type, "status") 
 		|| 0 == STRNCMP_CONST(op->op_type, "monitor")) && (op->rc == 7) ) {
-		printf("operation status:%s\n", status_msg[LRM_OP_DONE-LRM_OP_PENDING]);
-		printf("op_status: %d\n", LRM_OP_DONE);
+		printf("operation status:%s\n", status_msg[LRM_OP_DONE]);
 	} else {
 		printf("operation status:%s\n", status_msg[(op->op_status 
-			- LRM_OP_PENDING) % DIMOF(status_msg)]);
-		printf("op_status: %d\n", op->op_status);
+			- LRM_OP_DONE) % DIMOF(status_msg)]);
 	}
+	printf("op_status: %d\n", op->op_status);
 	printf("return code: %d\n", op->rc);
-	printf("output data: \n%s\n", (op->output ? op->output : "[null]"));
+	printf("output data: \n%s\n", op->output);
 	printf("---------------------------------------\n\n");
-	ret_value = op->rc;	
+	ret_value = op->op_status;	
 }
 
 static int 
@@ -644,7 +560,7 @@ resource_operation(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	int call_id;
 	
 	if ((argc - optind) < 3) {
-		cl_log(LOG_ERR,"Not enough parameters.");
+		cl_log(LOG_ERR,"No enough parameters.");
 		return -2;
 	}
 
@@ -658,7 +574,7 @@ resource_operation(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	op.op_type = argv[optind+1];
 	op.timeout = atoi(argv[optind+2]);
 
- 	/* When op.timeout!=0, plus additional 1s. Or lrmadmin may time out before
+ 	/* When op.timeout!=0, plus addtional 1s. Or lrmadmin may time out before
 	   the normal operation result returned from lrmd. This may be redudant, 
 	   but harmless. */
 	if (0 < op.timeout ) {
@@ -687,7 +603,7 @@ resource_operation(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	op.params = params_ht;
 
 	call_id = lrm_rsc->ops->perform_op(lrm_rsc, &op);
-	lrm_free_rsc(lrm_rsc);
+	/* g_free(lrm_rsc);   don't need to free it ? */
 	if (params_ht) {
 		g_hash_table_foreach(params_ht, free_stritem_of_hashtable, NULL);
 		g_hash_table_destroy(params_ht);
@@ -703,7 +619,7 @@ ra_metadata(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	char* metadata;
 
 	if(argc < 5) {
-		cl_log(LOG_ERR,"Not enough parameters.");
+		cl_log(LOG_ERR,"No enough parameters.");
 		return -2;
 	}
 
@@ -713,7 +629,7 @@ ra_metadata(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 
 	metadata = lrmd->lrm_ops->get_rsc_type_metadata(lrmd, class, type, provider);
 	if (NULL!=metadata) {
-		printf ("%s\n", metadata);
+		printf ("metadata of %s(%s) is: %s\n",type,class,metadata);
 		g_free (metadata);
 	}
 	return 0;
@@ -728,7 +644,7 @@ ra_provider(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	GList* provider = NULL;
 	
 	if(argc < 4) {
-		cl_log(LOG_ERR,"Not enough parameters.");
+		cl_log(LOG_ERR,"No enough parameters.");
 		return -2;
 	}
 
@@ -736,8 +652,8 @@ ra_provider(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	
 	while (NULL != (provider = g_list_first(providers))) {
 		printf("%s\n",(char*)provider->data);
-		g_free(provider->data);
 		providers = g_list_remove(providers, provider->data);
+		g_free(provider->data);
 	}
 	g_list_free(providers);
 	return 0;
@@ -754,7 +670,7 @@ add_resource(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	int tmp_ret;
 
 	if ((argc - optind) < 4) {
-		cl_log(LOG_ERR,"Not enough parameters.");
+		cl_log(LOG_ERR,"No enough parameters.");
 		return -2;
 	}
 	
@@ -785,25 +701,6 @@ add_resource(ll_lrm_t * lrmd, int argc, int optind, char * argv[])
 	return (tmp_ret ? 0 : -1); /* tmp_ret is HA_OK=1 or HA_FAIL=0 */
 }
 
-static int 
-fail_resource(ll_lrm_t * lrmd, char *rsc_id, int optc, char *opts[])
-{
-	int fail_rc = 0;
-	const char * reason = NULL;
-
-	if (optc > 2) {
-		cl_log(LOG_ERR,"Bad usage.");
-		return -2;
-	}
-
-	if (optc >= 1)
-		fail_rc = atoi(opts[0]);
-	if (optc == 2)
-		reason = opts[1];
-
-	return lrmd->lrm_ops->fail_rsc(lrmd, rsc_id, fail_rc, reason);
-}
-
 static int
 transfer_cmd_params(int amount, int start, char * argv[], const char * class, 
 GHashTable ** params_ht)
@@ -823,16 +720,16 @@ GHashTable ** params_ht)
 		for (i=start; i<amount; i++) {
 			delimit = strchr(argv[i], '=');
 			if (!delimit) {
-				cl_log(LOG_ERR, "Parameter %s is invalid for "
-					"the OCF standard.", argv[i]);
+				cl_log(LOG_ERR, "parameter %s is invalid for "
+					"OCF standard.", argv[i]);
 				goto error_return; /* Have to */
 			}
 
-			len_tmp = strnlen(delimit+1, MAX_PARAM_LEN) + 1;
+			len_tmp = strnlen(delimit+1, 80) + 1;
 			value = g_new(gchar, len_tmp);
 			strncpy(value, delimit+1, len_tmp);
 
-			len_tmp = strnlen(argv[i], MAX_PARAM_LEN) - strnlen(delimit, MAX_PARAM_LEN);
+			len_tmp = strnlen(argv[i], 80) - strnlen(delimit, 80);
 			key = g_new(gchar, len_tmp+1);
 			key[len_tmp] = '\0';
 			strncpy(key, argv[i], len_tmp);
@@ -853,7 +750,7 @@ GHashTable ** params_ht)
 			/* printf("index: %d  value: %s \n", i-start+1, argv[i]); */
 		}
 	} else {
-		fprintf(stderr, "Not supported resource agent class.\n");
+		fprintf(stderr, "Not supported resource agency class.\n");
 		return -1;
 	}
 
@@ -907,67 +804,36 @@ params_hashtable_to_str(const char * class, GHashTable * ht)
 		strncpy(params_str, gstr_tmp->str, gstr_tmp->len+1);
 		g_string_free(gstr_tmp, TRUE);
 	} else {
-		fprintf(stderr, "Not supported resource agent class.\n");
+		fprintf(stderr, "Not supported resource agency class.\n");
 	}
 
 	return params_str;
 }
 
 static void
-g_print_stringitem_and_free(gpointer data, gpointer user_data)
+g_print_stringitem(gpointer data, gpointer user_data)
 {
 	printf("%s\n", (char*)data);
-	g_free(data);
+	g_free(data);  /*  ?  */
 }
 
 static void
-g_print_rainfo_item_and_free(gpointer data, gpointer user_data)
+g_print_rainfo_item(gpointer data, gpointer user_data)
 {
-	printf("%s\n", (char *)data);
-	g_free(data);
+/*	rsc_info_t * rsc_info = (rsc_info_t *) data; */
+	printf("RA type name: %s\n", (char *)data);
+/*
+	printf("RA type name: %s  Version: %s\n", 
+		rsc_info->rsc_type, rsc_info->version);
+*/
+	g_free(data); /*  ?  */
 }
 
-
 static void
 g_print_ops(gpointer data, gpointer user_data)
 {
-	lrm_op_t* op = (lrm_op_t*)data;
-	GString * param_gstr;
-	time_t run_at=0, rcchange_at=0;
-
-	if (NULL == op) {
-		cl_log(LOG_ERR, "%s:%d: op==NULL"
-			, __FUNCTION__, __LINE__);
-		return;
-	}
-
-	param_gstr = g_string_new("");
-	g_hash_table_foreach(op->params, ocf_params_hash_to_str, &param_gstr);
-
-	if( op->t_run )
-		run_at=(time_t)op->t_run;
-	if( op->t_rcchange )
-		rcchange_at=(time_t)op->t_rcchange;
-	printf("   operation '%s' [call_id=%d]:\n"
-	       "      start_delay=%d, interval=%d, timeout=%d, app_name=%s\n"
-	       "      rc=%d (%s), op_status=%d (%s)\n"
-		, nullcheck(op->op_type), op->call_id
-		, op->start_delay, op->interval, op->timeout
-		, nullcheck(op->app_name), op->rc
-		, rc_msg[(op->rc-EXECRA_EXEC_UNKNOWN_ERROR) % DIMOF(rc_msg)]
-		, op->op_status
-		, status_msg[(op->op_status-LRM_OP_PENDING) % DIMOF(status_msg)]
-	);
-	if( op->t_run || op->t_rcchange )
-		printf("      run at: %s"
-			   "      last rc change at: %s"
-			   "      queue time: %lums, exec time: %lums\n"
-			, op->t_run ? ctime(&run_at) : "N/A\n"
-			, op->t_rcchange ? ctime(&rcchange_at) : "N/A\n"
-			, op->queue_time, op->exec_time
-		);
-	printf("      parameters: %s\n", param_gstr->str);
-	g_string_free(param_gstr, TRUE);
+	printf("%s  ", (char*)data);
+	g_free(data);  /*  ?  */
 }
 
 static void
@@ -987,19 +853,14 @@ g_get_rsc_description(gpointer data, gpointer user_data)
 	lrm_rsc = lrmd->lrm_ops->get_rsc(lrmd, rsc_id_tmp);
 	if (lrm_rsc) {
 		print_rsc_inf(lrm_rsc);
-		lrm_free_rsc(lrm_rsc); 
+		g_free(lrm_rsc);   /* ? */
 	} else
-		cl_log(LOG_ERR, "Invalid resource id: %s.", 
+		cl_log(LOG_ERR, "There is a invalid resource id %s.", 
 			rsc_id_tmp);
 	
-	g_free(data);
-}
-static void
-g_print_meta(gpointer key, gpointer value, gpointer user_data)
-{
-	printf("%s\n", (const char*)key);
-	printf("%s\n", (const char*)value);
+	g_free(data); /* ? */
 }
+
 static void
 print_rsc_inf(lrm_rsc_t * lrm_rsc)
 {
@@ -1013,15 +874,15 @@ print_rsc_inf(lrm_rsc_t * lrm_rsc)
 	rscid_str_tmp[RID_LEN-1] = '\0';
 	strncpy(rscid_str_tmp, lrm_rsc->id, RID_LEN-1);
 	printf("\nResource ID:%s\n", rscid_str_tmp);
-	printf("Resource agent class:%s\n", lrm_rsc->class);
-	printf("Resource agent type:%s\n", lrm_rsc->type);
-	printf("Resource agent provider:%s\n"
+	printf("Resource agency class:%s\n", lrm_rsc->class);
+	printf("Resource agency type:%s\n", lrm_rsc->type);
+	printf("Resource agency provider:%s\n"
 		, lrm_rsc->provider?lrm_rsc->provider:"default");
 
 	if (lrm_rsc->params) {
 		tmp = params_hashtable_to_str(lrm_rsc->class, 
 				lrm_rsc->params);
-		printf("Resource agent parameters:%s\n"
+		printf("Resource agency parameters:%s\n"
 			, (tmp == NULL) ? "No parameter" : tmp);
 		if (tmp != NULL) {
 			 g_free(tmp);
@@ -1058,9 +919,6 @@ normal_params_hash_to_str(gpointer key, gpointer value, gpointer user_data)
 	}
 
 	key_int = atoi((char *)key) - 1;
-	if( key_int < 0 ) {
-		return;
-	}
 	strncpy(str_tmp + key_int * ARGVI_MAX_LEN, (char*)value,
 		ARGVI_MAX_LEN - 1);
 }
@@ -1074,8 +932,90 @@ get_lrm_rsc(ll_lrm_t * lrmd, char * rscid)
 	if (!(lrm_rsc)) {
 		uuid_str_tmp[RID_LEN-1] = '\0';
 		strncpy(uuid_str_tmp, rscid, RID_LEN-1);
-		cl_log(LOG_ERR,"Resource %s does not exist.", uuid_str_tmp);
+		cl_log(LOG_ERR,"No this resource %s.", uuid_str_tmp);
 	}
 	return lrm_rsc;
 }
 
+/*
+ * $Log: lrmadmin.c,v $
+ * Revision 1.35  2005/06/22 06:53:41  sunjd
+ * handling for no parameter
+ *
+ * Revision 1.34  2005/06/15 15:06:50  davidlee
+ * Somes OSes don't have getopt_long()
+ *
+ * Revision 1.33  2005/05/09 03:20:55  alan
+ * Finished changes for bug 544 :-(
+ *
+ * Revision 1.32  2005/04/27 07:33:40  sunjd
+ * Donnot set a default timeout value when the user does not want timeout
+ *
+ * Revision 1.31  2005/04/25 05:47:54  zhenh
+ * when user did not give timeout of op, use 60s as timeout of lrmadmin, if user gave one, add 1s as timeout of lrmadmin
+ *
+ * Revision 1.30  2005/04/22 06:08:50  alan
+ * Put in a fix for an uninitialized variable -- added a new
+ * const lrm_op_t object lrm_zero_op - which can be used as an initializer for
+ * lrm_op_t objects so this doesn't happen.
+ *
+ * Revision 1.29  2005/04/18 15:47:41  alan
+ * Fixed a compile error (warning) in lrmadmin.
+ *
+ * Revision 1.28  2005/04/18 09:42:20  sunjd
+ * have its own timeout watching, now lrmadmin should not be blocked when no result from LRMd
+ *
+ * Revision 1.27  2005/02/28 10:34:36  zhenh
+ * change the log from LOG_ERR TO LOG_WARNING
+ *
+ * Revision 1.26  2005/02/28 08:52:46  zhenh
+ * no such resource should be a warning instead of error
+ *
+ * Revision 1.25  2004/12/09 07:16:58  sunjd
+ * add the support to stonith RA; some minor polish
+ *
+ * Revision 1.24  2004/12/05 13:18:32  sunjd
+ * add the support to stonith RAs
+ *
+ * Revision 1.23  2004/11/23 20:58:18  andrew
+ * Commit zhenh's patch for preserving user data across connections
+ * Only supports flat objects (ie. char* or structs without pointers in them)
+ *
+ * Revision 1.22  2004/10/24 12:38:33  lge
+ * -pedantic-errors fixes take one:
+ * * error: ISO C89 forbids mixed declarations and code
+ *
+ * Revision 1.21  2004/10/11 02:11:07  zhenh
+ * remove comment line with //
+ *
+ * Revision 1.20  2004/10/10 09:27:53  zhenh
+ * change some output information to make it more clear
+ *
+ * Revision 1.19  2004/10/08 04:47:54  zhenh
+ * fix a bug: checking the return value of get_rsc_type_metadata
+ *
+ * Revision 1.18  2004/09/16 06:16:45  sunjd
+ * BEAM bug fix: passing NULL to argument 1 of g_free
+ *
+ * Revision 1.17  2004/09/14 09:17:35  sunjd
+ * fix two pointer bugs found by BEAM
+ *
+ * Revision 1.16  2004/09/10 10:25:50  sunjd
+ * Minor polish to message format
+ *
+ * Revision 1.15  2004/09/10 00:28:17  zhenh
+ * change the usage information
+ *
+ * Revision 1.14  2004/09/09 03:32:50  zhenh
+ * fix a mis type
+ *
+ * Revision 1.13  2004/09/03 01:29:44  zhenh
+ * add provider for resource
+ *
+ * Revision 1.12  2004/08/30 03:17:40  msoffen
+ * Fixed more comments from // to standard C comments
+ *
+ * Revision 1.11  2004/08/29 04:42:03  msoffen
+ * Added missing ID and Log
+ *
+ */
