@@ -20,7 +20,7 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <sys/poll.h>
