@@ -1,4 +1,4 @@
-/* $Id: ccm.h,v 1.19.2.3 2004/12/01 00:11:28 yixiong Exp $ */
+/* $Id: ccm.h,v 1.38 2005/08/18 16:33:14 andrew Exp $ */
 /*
  * ccm.h: definitions Consensus Cluster Manager internal header
  *				file
@@ -6,19 +6,19 @@
  * Copyright (c) International Business Machines  Corp., 2002
  * Author: Ram Pai (linuxram@us.ibm.com)
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #ifndef _CLUSTER_MANAGER_H_
@@ -54,6 +54,9 @@
 #include <heartbeat.h>
 #include <hb_api.h>
 
+#include <ocf/oc_event.h>
+
+#define BitsInByte CHAR_BIT
 
 /* BEGINNING OF version request tracking interfaces */
 typedef struct ccm_version_s {
@@ -81,47 +84,58 @@ unsigned int version_get_nresp(ccm_version_t *);
 #define NODEIDSIZE 255 /* if this value is changed, change it
 			  	also in ccmlib.h */
 #define STATUSSIZE 15
-#define CCMFIFO    HA_VARLIBDIR "/heartbeat/ccm/ccm" /* if this value is
+#define CCMFIFO    HA_VARRUNDIR "/heartbeat/ccm/ccm" /* if this value is
 			changed change it also in ccmlib.h */
 
-typedef struct NodeList_s {
-		uint  NodeUuid;  /* a cluster unique id for the node */
-		char NodeID[NODEIDSIZE];
-		char Status[STATUSSIZE];
-		uint received_change_msg;
-} NodeList_t;
+typedef struct llm_node_s {
+	uint  uuid;  /* a cluster unique id for the node */
+	gboolean join_request;
+	char nodename[NODEIDSIZE];
+	char status[STATUSSIZE];
+	uint received_change_msg;
+}llm_node_t;
+
 typedef struct llm_info_s { /* information about low level membership info */
-	uint	   llm_nodeCount; /*number of nodes in the cluster  */
-	int	   llm_mynode;	 /*index of mynode */
-	NodeList_t llm_nodes[MAXNODE];  /*information of each node */
+	uint	   nodecount; /*total number of nodes in the cluster  */
+	int	   myindex;	 /*index of mynode */
+	llm_node_t nodes[MAXNODE];  /*information of each node */
 } llm_info_t;
+
 #define CLUST_INACTIVE  "inctv"
-#define LLM_GET_MYNODE(llm) llm->llm_mynode
-#define LLM_GET_NODECOUNT(llm) llm->llm_nodeCount
-#define LLM_GET_UUID(llm,i) llm->llm_nodes[i].NodeUuid
+#define LLM_GET_MYNODE(llm) llm->myindex
+#define LLM_GET_NODECOUNT(llm) llm->nodecount
+#define LLM_GET_UUID(llm,i) llm->nodes[i].uuid
 #define LLM_GET_MYUUID(llm) LLM_GET_UUID(llm, LLM_GET_MYNODE(llm))
-#define LLM_GET_NODEID(llm,i) llm->llm_nodes[i].NodeID
+#define LLM_GET_NODEID(llm,i) llm->nodes[i].nodename
 #define LLM_GET_MYNODEID(llm) LLM_GET_NODEID(llm, LLM_GET_MYNODE(llm))
-#define LLM_GET_STATUS(llm,i) llm->llm_nodes[i].Status
-#define LLM_SET_MYNODE(llm,indx) llm->llm_mynode = indx
-#define LLM_SET_NODECOUNT(llm, count) llm->llm_nodeCount = count
-#define LLM_INC_NODECOUNT(llm) (llm->llm_nodeCount)++
-#define LLM_SET_UUID(llm,i, uuid) llm->llm_nodes[i].NodeUuid = uuid
-#define LLM_SET_MYUUID(llm, uuid) LLM_SET_UUID(llm, LLM_GET_MYNODE(llm), uuid)
+#define LLM_GET_STATUS(llm,i) llm->nodes[i].status
+#define LLM_SET_MYNODE(llm,indx) llm->myindex = indx
+#define LLM_SET_NODECOUNT(llm, count) llm->nodecount = count
+#define LLM_INC_NODECOUNT(llm) (llm->nodecount)++
+#define LLM_SET_UUID(llm,i, _uuid) llm->nodes[i].uuid = _uuid
+#define LLM_SET_MYUUID(llm, _uuid) LLM_SET_UUID(llm, LLM_GET_MYNODE(llm), _uuid)
 #define LLM_SET_NODEID(llm, i, name)  \
-			(strncpy(llm->llm_nodes[i].NodeID,name,NODEIDSIZE))
+			(strncpy(llm->nodes[i].nodename,name,NODEIDSIZE))
 #define LLM_SET_MYNODEID(llm, name) \
 			LLM_SET_NODEID(llm, LLM_GET_MYNODE(llm), name)
 #define LLM_SET_STATUS(llm,i,status) \
-			(strncpy(llm->llm_nodes[i].Status,status,STATUSSIZE))
-#define LLM_COPY(llm,dst,src) (llm->llm_nodes[dst] = llm->llm_nodes[src])
+			(strncpy(llm->nodes[i].status,status,STATUSSIZE))
+#define LLM_COPY(llm,dst,src) (llm->nodes[dst] = llm->nodes[src])
 #define LLM_GET_NODEIDSIZE(llm) NODEIDSIZE
-int llm_get_active_nodecount(llm_info_t *);
+int llm_get_live_nodecount(llm_info_t *);
 gboolean llm_only_active_node(llm_info_t *);
 int llm_get_uuid(llm_info_t *, const char *);
 char *llm_get_nodeid_from_uuid(llm_info_t *, const int );
 int llm_nodeid_cmp(llm_info_t *, int , int );
-int llm_status_update(llm_info_t *, const char *, const char *);
+int llm_status_update(llm_info_t *, const char *, const char *, char*);
+
+/*	Get the number of nodes that are inactive 
+ *	inactive means this node is STONITHed
+ *	therefore we don't count that in quorum computation
+ */
+int	llm_get_inactive_node_count(llm_info_t *llm);
+
+void	display_llm(llm_info_t *llm);
 void llm_init(llm_info_t *);
 void llm_end(llm_info_t *);
 int llm_is_valid_node(llm_info_t *, const char *);
@@ -192,6 +206,7 @@ int update_strcreate(ccm_update_t *, char **, llm_info_t *);
 void update_strdelete(char *memlist);
 int update_is_member(ccm_update_t *, llm_info_t *, const char *);
 int update_get_uptime(ccm_update_t *, llm_info_t *, int );
+void	update_display(int pri,llm_info_t* llm, ccm_update_t* tab);
 /* END OF update interfaces */
 
 
@@ -242,10 +257,143 @@ void client_init(void);
 int  client_add(struct IPC_CHANNEL *);
 void client_delete(struct IPC_CHANNEL *);
 void client_delete_all(void);
-void client_new_mbrship(int n, int , int *, gboolean, void *);
 void client_llm_init(llm_info_t *);
 void client_influx(void);
 void client_evicted(void);
 /* END OF client management interfaces */
 
+
+
+
+
+/* */
+/* the various states of the CCM state machine. */
+/* */
+enum ccm_state  {
+	CCM_STATE_NONE=0,		/* is in NULL state  */
+	CCM_STATE_VERSION_REQUEST,	/* sent a request for protocol version */
+	CCM_STATE_JOINING,  		/* has initiated a join protocol  */
+	CCM_STATE_RCVD_UPDATE,	/* has recevied the updates from other nodes */
+	CCM_STATE_SENT_MEMLISTREQ,	/* CL has sent a request for member list  */
+					/* this state is applicable only on CL */
+	CCM_STATE_REQ_MEMLIST,	/* CL has requested member list */
+				  	/* this state is applicable only on non-CL */
+	CCM_STATE_MEMLIST_RES,	/* Responded member list to the Cluster  */
+				 	/* Leader */
+	CCM_STATE_JOINED,    /* PART of the CCM cluster membership! */
+	CCM_STATE_WAIT_FOR_MEM_LIST,
+	CCM_STATE_WAIT_FOR_CHANGE,
+	CCM_STATE_NEW_NODE_WAIT_FOR_MEM_LIST,
+	CCM_STATE_END
+};
+
+/* the times for repeating sending message */
+#define REPEAT_TIMES 10
+
+/* add new enums to this structure as and when new protocols are added */
+enum ccm_protocol {
+	CCM_VER_NONE = 0,
+	CCM_VER_1,
+	CCM_VER_LAST
+};
+
+typedef struct ccm_proto_s {
+	enum ccm_protocol  com_hiproto;/* highest protocol version that  */
+				/* this node can handle */
+	int	com_active_proto;/* protocol version */
+} ccm_proto_t;
+
+
+typedef struct memcomp_s {
+	graph_t		*mem_graph;  /* memlist calculation graph */
+
+	GSList 		*mem_maxt; 	    /* the maxtrans of each node */
+				    /* participating in the computation . */
+				    /* NOTE: the transition number of the */
+				    /* next transition is always 1 higher */
+				    /* than that of all transitions seen  */
+				    /* by each node participating in the  */
+				    /* membership */
+	longclock_t  	mem_inittime; /* the time got intialized */
+} memcomp_t;
+#define 	MEMCOMP_GET_GRAPH(memc)  	memc->mem_graph
+#define 	MEMCOMP_GET_MAXT(memc)  	memc->mem_maxt
+#define 	MEMCOMP_GET_INITTIME(memc)  	memc->mem_inittime
+#define 	MEMCOMP_SET_GRAPH(memc, gr)  	memc->mem_graph=gr
+#define 	MEMCOMP_SET_MAXT(memc, list)  	memc->mem_maxt=list
+#define 	MEMCOMP_SET_INITTIME(memc,time)	memc->mem_inittime=time
+
+
+typedef struct ccm_tmout_s {
+	long	iff;  /* membership_Info_From_Followers_timeout */
+	long	itf;  /* membership_Info_To_Followers_timeout */
+	long	fl;  /* membership_Final_List_timeout */
+	long	u;  /* update timeout */
+	long	lu;  /* long update timeout */
+	long	vrs;  /* version timeout */
+} ccm_tmout_t;
+
+enum change_event_type{
+    TYPE_NONE,	
+    NODE_LEAVE,
+    NEW_NODE
+};
+
+#define COOKIESIZE 15
+typedef struct ccm_info_s {
+	llm_info_t 	llm;	/*  low level membership info */
+	
+	int		ccm_nodeCount;	/*  number of nodes in the ccm cluster */
+	int		ccm_member[MAXNODE];/* members of the ccm cluster */
+	memcomp_t	ccm_memcomp;	/* the datastructure to compute the  */
+					/* final membership for each membership */
+	 				/* computation instance of the ccm protocol. */
+	 				/* used by the leader only. */
+
+	ccm_proto_t  	ccm_proto;	/* protocol version information */
+#define ccm_active_proto ccm_proto.com_active_proto
+#define ccm_hiproto	  ccm_proto.com_hiproto
+
+	char		ccm_cookie[COOKIESIZE];/* context identification string. */
+	uint32_t	ccm_transition_major;/* transition number of the cluster */
+	int		ccm_cluster_leader; /* cluster leader of the last major */
+				/* transition. index of cl in ccm_member table */
+	int		ccm_joined_transition;
+					/* this indicates the major transition  */
+					/* number during which this node became */
+					/* a member of the cluster. */
+					/* A sideeffect of this is it also */
+					/* is used to figure out if this node */
+					/* was ever a part of the cluster. */
+					/* Should be intially set to 0 */
+	uint32_t	ccm_max_transition;	/* the maximum transition number seen */
+					/* by this node ever since it was born. */
+	enum ccm_state 	ccm_node_state;	/* cluster state of this node  */
+	uint32_t	ccm_transition_minor;/* minor transition number of the  */
+					/* cluster */
+
+	ccm_update_t   ccm_update; 	/* structure that keeps track */
+					/* of uptime of each member */
+	ccm_version_t  ccm_version;     /* keeps track of version request  */
+					/* related info */
+	ccm_tmout_t	tmout;
+	uint32_t change_event_remaining_count; 		
+	enum change_event_type change_type;
+	char change_node_id[NODEIDSIZE];
+
+} ccm_info_t;
+
+/*
+ * datastructure passed to the event loop.
+ * This acts a handle, and should not be interpreted
+ * by the event loop.
+ */
+typedef struct  ccm_s {
+	ll_cluster_t    *hbfd;
+	void    	*info;
+} ccm_t;
+
+
+void client_new_mbrship(ccm_info_t*, void*);
+void ccm_reset(ccm_info_t *info);
 #endif /*  _CLUSTER_MANAGER_H_ */
