@@ -1,4 +1,4 @@
-/* $Id: ccmclient.c,v 1.11.2.4 2004/12/02 00:08:25 yixiong Exp $ */
+/* $Id: ccmclient.c,v 1.27 2005/07/29 10:32:30 sunjd Exp $ */
 /* 
  * client.c: Consensus Cluster Client tracker
  *
@@ -6,19 +6,19 @@
  * Author: Ram Pai (linuxram@us.ibm.com)
  * 
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #include <ccm.h>
@@ -72,25 +72,26 @@ static void
 send_message(ccm_client_t *ccm_client, ccm_ipc_t *msg)
 {
 	int send_rc = IPC_OK;
-	struct IPC_CHANNEL *ipc_client = ccm_client->ccm_ipc_client;
+	struct IPC_CHANNEL *chan = ccm_client->ccm_ipc_client;
 
 	++(msg->count);
 
 	do {
-		send_rc = ipc_client->ops->send(ipc_client, &(msg->ipcmsg));
-
-		if(send_rc != IPC_OK
-		   && (ipc_client->ch_status == IPC_DISCONNECT
-		       || ipc_client->ch_status == IPC_DISC_PENDING)) {
-			cl_log(LOG_ERR, "Channel is dead.  Cannot send message.");
-			break;
+		if (chan->ops->get_chan_status(chan) == IPC_CONNECT){
+			send_rc = chan->ops->send(chan, &(msg->ipcmsg));
+		}
+		if(send_rc != IPC_OK){
+			if (chan->ops->get_chan_status(chan) != IPC_CONNECT){
+				cl_log(LOG_WARNING, "Channel is dead.  Cannot send message.");
+				break;
+			}else {
+				cl_shortsleep();				
+			}
 		}
 		
-		cl_log(LOG_WARNING, "ipc channel blocked");
-		cl_shortsleep();
-
+		
 	} while(send_rc == IPC_FAIL);
-
+	
 	return;
 }
 
@@ -102,7 +103,15 @@ send_func(gpointer key, gpointer value, gpointer user_data)
 	if(evicted_flag) {
 		/*send evicted message*/
 		if(ccm_client->ccm_flags == CL_MEM) {
-			send_message(ccm_client, ipc_misc_message);
+			struct IPC_CHANNEL* chan = ccm_client->ccm_ipc_client;
+			
+			if (chan->ops->get_chan_status(chan) == IPC_CONNECT){
+				send_message(ccm_client, ipc_misc_message);
+			}else {
+				/* IPC is broken, the client is already gone
+				 * Do nothing
+				 */
+			}
 			ccm_client->ccm_flags = CL_INIT;
 		}
 		return;
@@ -194,7 +203,7 @@ create_message(GMemChunk *chk, void *data, int size)
 	ipcmsg->ipcmsg.msg_len     = size;
 	ipcmsg->ipcmsg.msg_done    = send_func_done;
 	ipcmsg->ipcmsg.msg_private = ipcmsg;
-	// ipcmsg->ipcmsg.msg_buf     = NULL;
+	ipcmsg->ipcmsg.msg_buf     = NULL;
 
 	return ipcmsg;
 }
@@ -216,7 +225,9 @@ flush_func(gpointer key, gpointer value, gpointer user_data)
 	while(ipc_client->ops->is_sending_blocked(ipc_client)) {
 		cl_log(LOG_WARNING, "ipc channel blocked");
 		cl_shortsleep();
-		if(ipc_client->ops->resume_io(ipc_client) == IPC_BROKEN) break;
+		if(ipc_client->ops->resume_io(ipc_client) == IPC_BROKEN) {
+			break;
+		}
 	}
 }
 
@@ -286,26 +297,32 @@ client_add(struct IPC_CHANNEL *ipc_client)
 	return 0;
 }
 
+static void
+client_destroy(struct IPC_CHANNEL *ipc_client)
+{
+	ccm_client_t  *ccm_client;
+	if((ccm_client = g_hash_table_lookup(ccm_hashclient, ipc_client))
+	!=	NULL){
+		g_free(ccm_client);
+	}
+	/* IPC_Channel is automatically destroyed when channel is disconnected */
+}
 
 void
 client_delete(struct IPC_CHANNEL *ipc_client)
 {
-	ccm_client_t  *ccm_client;
 
-	if((ccm_client = g_hash_table_lookup(ccm_hashclient, ipc_client))
-				!=NULL){
-		g_hash_table_remove(ccm_hashclient, ipc_client);
-		g_free(ccm_client);
-	}
+	g_hash_table_remove(ccm_hashclient, ipc_client);
+	client_destroy(ipc_client);
 	return;
 }
 
 static gboolean 
-delete_func(gpointer key, gpointer value, gpointer user_data)
+destroy_func(gpointer key, gpointer value, gpointer user_data)
 {
 	struct IPC_CHANNEL *ipc_client = (struct IPC_CHANNEL *)key;
 
-	client_delete(ipc_client);
+	client_destroy(ipc_client);
 	return TRUE;
 }
 
@@ -313,24 +330,59 @@ void
 client_delete_all(void)
 {
 	if(g_hash_table_size(ccm_hashclient)) {
-		g_hash_table_foreach_remove(ccm_hashclient, delete_func, NULL);
+		g_hash_table_foreach_remove(ccm_hashclient, destroy_func, NULL);
 	}
 	return;
 }
 
 
+/* a sophisticated quorum algorithm has to be introduced here
+ *  currently we are just using the simplest algorithm
+ */
+static gboolean
+mem_quorum(llm_info_t* llm, int member_count)
+{
+	int	inactive_count = llm_get_inactive_node_count(llm);
+	int	total_count = llm->nodecount;
+	
+	cl_log(LOG_INFO, "n_member=%d, nodecount=%d, inactive_count=%d",
+	       member_count, total_count, inactive_count); 
+	
+	/* XXX REVISIT TODO: This is a temporary WORK-AROUND for the two
+	 * node clusters. With one node missing, always assume quorum.
+	 * This will be farmed out to plugins later! */
+	if (total_count == 2) {
+		cl_log(LOG_INFO, "Asserting quorum for two node cluster!");
+		return TRUE;
+	}
+	
+	if(member_count <((total_count - inactive_count)/2 + 1)){
+		return FALSE;
+	}
+	return TRUE;
+}
+static void
+display_func(gpointer key, gpointer value, gpointer user_data)
+{
+	ccm_client_t * ccm_client = (ccm_client_t*) value;
+	cl_log(LOG_INFO, "client: pid =%d", ccm_client->ccm_ipc_client->farside_pid);	
+	
+	return;
+}
+
 void
-client_new_mbrship(int n,  int trans, int *member, 
-		gboolean q_flag, void *borndata)
+client_new_mbrship(ccm_info_t* info, void* borndata)
 {
 	/* creating enough heap memory in order to avoid allocation */
 	static struct born_s	bornbuffer[MAXNODE+10];
 	ccm_meminfo_t *ccm=(ccm_meminfo_t *)bornbuffer;
 	ccm_born_t    *born=(ccm_born_t *)bornbuffer;
-
 	struct born_s *born_arry = (struct born_s *)borndata;
-
-	assert(n<=MAXNODE);
+	int		n = info->ccm_nodeCount;
+	int		trans = info->ccm_transition_major;
+	int*		member = info->ccm_member;
+	
+	assert( n<= MAXNODE);
 
 	prim_flag=TRUE;
 	restored_flag=FALSE;
@@ -339,7 +391,7 @@ client_new_mbrship(int n,  int trans, int *member,
 	ccm->ev = CCM_NEW_MEMBERSHIP;
 	ccm->n = n;
 	ccm->trans = trans;
-	ccm->q_overide = q_flag;
+	ccm->quorum = mem_quorum(&info->llm, n);
 	memcpy(ccm->member, member, n*sizeof(int));
 
 	if(ipc_mem_message && --(ipc_mem_message->count)==0){
@@ -356,9 +408,19 @@ client_new_mbrship(int n,  int trans, int *member,
 		delete_message(ipc_born_message);
 	}
 	ipc_born_message = create_message(ipc_born_chk, born, 
-		 sizeof(ccm_born_t )+n*sizeof(struct born_s));
+					  sizeof(ccm_born_t )+n*sizeof(struct born_s));
 	ipc_born_message->count++;
 	
+#if 1
+	cl_log(LOG_INFO, "delivering new membership to %d clients: ",
+	       g_hash_table_size(ccm_hashclient));
+	if(g_hash_table_size(ccm_hashclient)){
+		g_hash_table_foreach(ccm_hashclient, display_func, NULL);	
+	}
+#else
+	(void)display_func;
+#endif 
+	
 	send_all();
 	if(global_verbose) {
 		cl_log(LOG_DEBUG, "membership state: new membership");
