@@ -1,4 +1,4 @@
-/* $Id: ccmlib.h,v 1.13.2.3 2004/12/01 00:11:28 yixiong Exp $ */
+/* $Id: ccmlib.h,v 1.19 2005/08/18 16:33:14 andrew Exp $ */
 /*
  * ccmlib.h: internal definations for ccm library files.
  *
@@ -40,7 +40,7 @@ void oc_ev_special(const oc_ev_t *, oc_ev_class_t , int );
 
 #define NODEIDSIZE 255 /* if this value is changed change it 
 			  	also in ccm.h */
-#define CCMFIFO HA_VARLIBDIR "/heartbeat/ccm/ccm" /* if this value
+#define CCMFIFO HA_VARRUNDIR "/heartbeat/ccm/ccm" /* if this value
 			is changed change it also in ccm.h */
 
 size_t strnlen(const char *, size_t); /*TOBEDONE*/
@@ -50,8 +50,7 @@ typedef struct ccm_meminfo_s {
 	int 		ev;
 	int		n;
 	int		trans;
-	int		q_overide; /* if set just assume we have quorum */
-				   /* if unset compute quorum if desired */
+	int		quorum;
 	int 		member[0];
 } ccm_meminfo_t;
 
