@@ -1,4 +1,4 @@
-/* $Id: ccmlib_memapi.c,v 1.16.2.3 2004/12/01 00:11:28 yixiong Exp $ */
+/* $Id: ccmlib_memapi.c,v 1.38 2005/07/30 02:33:08 alan Exp $ */
 /* 
  * ccmlib_memapi.c: Consensus Cluster Membership API
  *
@@ -22,11 +22,13 @@
 
 #define __CCM_LIBRARY__
 #include <ccmlib.h>
+#include <ccm.h>
 /*#include <syslog.h> */
-/*#include <clplumbing/cl_log.h> */
+#include <clplumbing/cl_log.h> 
 
 /* structure to track the membership delivered to client */
 typedef struct mbr_track_s {
+	int			quorum;
 	int			m_size;
 	oc_ev_membership_t 	m_mem;
 } mbr_track_t;
@@ -53,6 +55,15 @@ typedef struct mbr_private_s {
 } mbr_private_t;
 
 
+static char event_strings[5][32]={
+	"OC_EV_MS_INVALID",
+	"OC_EV_MS_NEW_MEMBERSHIP",
+	"OC_EV_MS_NOT_PRIMARY",
+	"OC_EV_MS_PRIMARY_RESTORED",
+	"OC_EV_MS_EVICTED"
+};
+
+#define EVENT_STRING(x) event_strings[x - OC_EV_MS_INVALID]
 #define OC_EV_SET_INSTANCE(m,trans)  m->m_mem.m_instance=trans
 #define OC_EV_SET_N_MEMBER(m,n)  m->m_mem.m_n_member=n
 #define OC_EV_SET_MEMB_IDX(m,idx)  m->m_mem.m_memb_idx=idx
@@ -169,13 +180,13 @@ reset_llm(mbr_private_t *private)
 static int
 init_llmborn(mbr_private_t *private)
 {
-	fd_set rset;
 	struct IPC_CHANNEL *ch;
 	int 	sockfd, i=0, ret;
 	struct IPC_MESSAGE *msg;
-	struct timeval tv;
 
-	if(private->llm) return 0;
+	if(private->llm) {
+		return 0;
+	}
 
 	ch 	   = private->channel;
 	sockfd = ch->ops->get_recv_select_fd(ch);
@@ -186,15 +197,8 @@ init_llmborn(mbr_private_t *private)
 	*  second iteration 
 	*/
 	while( i < 2) {
-
-		FD_ZERO(&rset);
-		FD_SET(sockfd,&rset);
-		tv.tv_sec = 1;
-		tv.tv_usec = 0;
-
-		if(!ch->ops->is_message_pending(ch) && (select(sockfd + 1, 
-				&rset, NULL,NULL,&tv)) == -1){
-			perror("select");
+		
+		if(ch->ops->waitin(ch) != IPC_OK){
 			ch->ops->destroy(ch);
 			return -1;
 		}
@@ -261,11 +265,23 @@ compare(const void *value1, const void *value2)
 {
 	const oc_node_t *t1 = (const oc_node_t *)value1;
 	const oc_node_t *t2 = (const oc_node_t *)value2;
+	
+	if (t1->node_born_on < t2->node_born_on){
+		return -1;
+	}
 
-	if (t1->node_born_on < t2->node_born_on) return -1;
-	if (t1->node_born_on > t2->node_born_on) return 1;
-	if (t1->node_id < t2->node_id) return -1;
-	if (t1->node_id > t2->node_id) return 1;
+	if (t1->node_born_on > t2->node_born_on){
+		return 1;
+	}
+
+	if (t1->node_id < t2->node_id) {
+		return -1;
+	}
+
+	if (t1->node_id > t2->node_id) {
+		return 1;
+	}
+	
 	return 0;
 }
 
@@ -274,8 +290,9 @@ llm_get_Id_from_Uuid(ccm_llm_t *stuff, uint uuid)
 {
 	uint lpc = 0;
 	for (; lpc < stuff->n; lpc++) {
-		if(stuff->node[lpc].Uuid == uuid)
+		if(stuff->node[lpc].Uuid == uuid){		       
 			return stuff->node[lpc].Id;
+		}
 	}
 	return NULL;
 }
@@ -293,12 +310,13 @@ get_new_membership(mbr_private_t *private,
 	int n_members,uuid;
 	
 	int n_nodes = CLLM_GET_NODECOUNT(private->llm);
-
+	
 	int size    = sizeof(oc_ev_membership_t) + 
-			2*n_nodes*sizeof(oc_node_t);
-
- 	newmbr = *mbr = (mbr_track_t *) g_malloc(size+sizeof(int));
+		2*n_nodes*sizeof(oc_node_t);
 
+ 	newmbr = *mbr = (mbr_track_t *)g_malloc(size +
+						sizeof(mbr_track_t)-sizeof(newmbr->m_mem));
+	
 	trans = OC_EV_SET_INSTANCE(newmbr,mbrinfo->trans);
 	n_members = OC_EV_SET_N_MEMBER(newmbr,mbrinfo->n);
 	OC_EV_SET_SIZE(newmbr, size);
@@ -318,7 +336,8 @@ get_new_membership(mbr_private_t *private,
 
 		gborn = g_hash_table_lookup(private->bornon, 
 				GINT_TO_POINTER(mbrinfo->member[i]));
-		born = GPOINTER_TO_INT(gborn);
+		/* gborn was an int to begin with - so this is safe */
+		born = POINTER_TO_SIZE_T(gborn);/*pointer cast as int*/
 
 		/* if there is already a born entry for the
 		 * node, use it. Otherwise create a born entry
@@ -381,8 +400,9 @@ mem_free_func(void *data)
 	if(mbr_track) {
 		/* free m_n_member uname, m_n_in is actually the same ptr */
 		for (lpc = 0 ; lpc < OC_EV_GET_N_MEMBER(mbr_track); lpc++ ) {
-			if ((uname = OC_EV_GET_NODE(mbr_track, lpc).node_uname))
+			if ((uname = OC_EV_GET_NODE(mbr_track, lpc).node_uname)){
 				g_free(uname);
+			}
 		}
 		/* free m_n_out uname */
 		for (lpc = OC_EV_GET_OUT_IDX(mbr_track)
@@ -390,8 +410,9 @@ mem_free_func(void *data)
 			+	OC_EV_GET_N_OUT(mbr_track)
 		;	lpc++) {
 
-			if ((uname = OC_EV_GET_NODE(mbr_track, lpc).node_uname))
+			if ((uname = OC_EV_GET_NODE(mbr_track, lpc).node_uname)){
 				g_free(uname);
+			}
 		}
 		g_free(mbr_track);
 	}
@@ -407,19 +428,7 @@ mem_callback_done(void *cookie)
 	return;
 }
 
-/* a sophisticated quorum algorithm has to be introduced here
- *  currently we are just using the simplest algorithm
- */
-static gboolean
-mem_quorum(mbr_private_t *private, mbr_track_t *mbr)
-{
-	/*cl_log(LOG_DEBUG, "n_member=%d, cllm_get_nodecount=%d\n", */
-	/*	OC_EV_GET_N_MEMBER(mbr), CLLM_GET_NODECOUNT(private->llm)); */
-	if(OC_EV_GET_N_MEMBER(mbr) <
-		(CLLM_GET_NODECOUNT(private->llm)/2+1))
-		return FALSE;
-	return TRUE;
-}
+
 
 static void
 update_bornons(mbr_private_t *private, mbr_track_t *mbr)
@@ -445,7 +454,9 @@ membership_unchanged(mbr_private_t *private, mbr_track_t *mbr)
 	mbr_track_t *oldmbr = (mbr_track_t *) 
 				cookie_get_data(private->cookie);
 
-	if(!oldmbr) return FALSE;
+	if(!oldmbr) {
+		return FALSE;
+	}
 
 	if(OC_EV_GET_N_MEMBER(mbr) != OC_EV_GET_N_MEMBER(oldmbr)){
 			return FALSE;
@@ -476,7 +487,9 @@ mem_handle_event(class_t *class)
 	int ret;
 	gboolean quorum;
 
-	if(!class_valid(class)) return FALSE;
+	if(!class_valid(class)){
+		return FALSE;
+	}
 
 	private = (mbr_private_t *)class->private;
 	ch 	   = private->channel;
@@ -484,28 +497,25 @@ mem_handle_event(class_t *class)
 	if(init_llmborn(private)){
 		return FALSE;
 	}
-
+	
+	
+	
 	while(ch->ops->is_message_pending(ch)){
 		/* receive the message and call the callback*/
 		ret=ch->ops->recv(ch,&msg);
-
-		if(ret == IPC_FAIL) {
-			return TRUE;
-		}
-
-		if(ret!=IPC_OK){
-			/* FIXIT. setting it to CCM_EVICTED is not
-			 * correct. However the draft api says nothing
-			 * about this case.
-			 * Proposal:
-			 * Set something like  CCM_LOST_SERVICE here
-			 * and return OC_EV_MS_LOST_SERVICE to the
-			 * client
+		
+		if(ret != IPC_OK){
+			/* If IPC is broken
+			 * the we return FALSE, which results in removing of 
+			 * this class in handle function
+			 * This should only happen when ccm is shutdown before the client
+			 *
 			 */
-			type = CCM_EVICTED;
-		} else {
-			type = ((ccm_meminfo_t *)msg->msg_body)->ev;
+			cl_log(LOG_INFO, "mem_handle_func:IPC broken, ccm is dead before the client!");
+			return FALSE;
 		}
+		
+		type = ((ccm_meminfo_t *)msg->msg_body)->ev;
 
 
 		cookie= mbr_track = NULL;
@@ -513,23 +523,18 @@ mem_handle_event(class_t *class)
 		oc_type = OC_EV_MS_INVALID;
 
 		switch(type) {
-		case CCM_NEW_MEMBERSHIP :
-
+		case CCM_NEW_MEMBERSHIP :{
+			
+			ccm_meminfo_t* cmi = (ccm_meminfo_t*)msg->msg_body;
+			
 			size = get_new_membership(private, 
-				(ccm_meminfo_t *)msg->msg_body, 
-				msg->msg_len, 
-				&mbr_track);
-
-
-			/* if ccm daemon informs us that we have quorum
-			 * by default, we do not have to compute quorum 
-			 */
-			if(((ccm_meminfo_t *)(msg->msg_body))->q_overide) {
-				quorum = TRUE;
-			} else {
-				quorum = mem_quorum(private, mbr_track);
-			}
+						  cmi,
+						  msg->msg_len, 
+						  &mbr_track);
 
+			
+			mbr_track->quorum = quorum = cmi->quorum;
+			
 			/* if no quorum, delete the bornon dates for lost 
 			 * nodes, add  bornon dates for the new nodes and 
 			 * return
@@ -552,15 +557,19 @@ mem_handle_event(class_t *class)
 			* Do not construct a new membership  
 			*/
 			if (membership_unchanged(private, mbr_track)){
-				/* we do not need the new mbr_track, 
-				*  the old one is the same as the new
-				*/
-				mem_free_func(mbr_track);
+				mbr_track_t* old_mbr_track;
+				
+				old_mbr_track = (mbr_track_t *)
+					cookie_get_data(private->cookie);
+				
+				if (mbr_track->quorum == old_mbr_track->quorum){
+					oc_type = OC_EV_MS_PRIMARY_RESTORED;
+				}else {
+					cl_log(LOG_ERR, "membership unchanged but quorum changed"
+					       "something is wrong!!!");
+					return FALSE;
+				}
 
-				oc_type = OC_EV_MS_PRIMARY_RESTORED;
-				cookie = private->cookie;
-				mbr_track = (mbr_track_t *)
-					cookie_get_data(cookie);
 			} else {
 				oc_type = quorum?
 					OC_EV_MS_NEW_MEMBERSHIP:
@@ -576,26 +585,39 @@ mem_handle_event(class_t *class)
 				if(!private->special) {
 				  assert(oc_type == OC_EV_MS_NEW_MEMBERSHIP);
 				}
-
-				update_bornons(private, mbr_track);
-				cookie_unref(private->cookie);
-				cookie = cookie_construct(mem_callback_done, 
-						mem_free_func, mbr_track);
-				private->cookie = cookie;
 			}
+
+			update_bornons(private, mbr_track);
+			cookie_unref(private->cookie);
+			cookie = cookie_construct(mem_callback_done, 
+						  mem_free_func, mbr_track);
+			private->cookie = cookie;
 			size = OC_EV_GET_SIZE(mbr_track);
 			break;
-
+		}
 		case CCM_EVICTED:
 			oc_type = OC_EV_MS_EVICTED;
 			private->client_report = TRUE;
-			/* FALL THROUGH */
+			
+			size = 0;
+			mbr_track = NULL;
 
+			if (private->cookie){
+				cookie_unref(private->cookie);
+			}
+			cookie= cookie_construct(mem_callback_done, NULL,NULL);
+			if ( cookie == NULL){
+				cl_log(LOG_ERR, "mem_handle_event: coookie construction failed");
+				abort();
+			}
+			private->cookie=cookie;
+			break;
+			
 		case CCM_INFLUX:
 
-			if(type==CCM_INFLUX)
+			if(type==CCM_INFLUX){
 				oc_type = OC_EV_MS_NOT_PRIMARY;
-
+			}
 
 
 			cookie = private->cookie;
@@ -617,6 +639,30 @@ mem_handle_event(class_t *class)
 			break;
 		}
 
+		
+		cl_log(LOG_INFO, "%s: Got an event %s from ccm"
+		,	__FUNCTION__
+		,	EVENT_STRING(oc_type));
+#define ALAN_DEBUG 1
+#ifdef ALAN_DEBUG
+		if (!mbr_track) {
+			cl_log(LOG_INFO, "%s: no mbr_track info"
+			,	__FUNCTION__);
+		}else{
+			cl_log(LOG_INFO
+			,	"%s: instance=%d, nodes=%d, new=%d, lost=%d"
+			", n_idx=%d, new_idx=%d, old_idx=%d"
+			,	__FUNCTION__
+			,	mbr_track->m_mem.m_instance
+			,	mbr_track->m_mem.m_n_member
+			,	mbr_track->m_mem.m_n_in
+			,	mbr_track->m_mem.m_n_out
+			,	mbr_track->m_mem.m_memb_idx
+			,	mbr_track->m_mem.m_in_idx
+			,	mbr_track->m_mem.m_out_idx);
+		}
+#endif
+
 		if(private->callback && private->client_report && cookie){
 			cookie_ref(cookie);
 			private->callback(oc_type,
@@ -652,12 +698,16 @@ mem_activate(class_t *class)
 	struct IPC_CHANNEL *ch;
 	int sockfd;
 
-	if(!class_valid(class)) return -1;
+	if(!class_valid(class)) {
+		return -1;
+	}
 
 	/* if already activated */
 
 	private = (mbr_private_t *)class->private;
-	if(private->llm)return -1;
+	if(private->llm){
+		return -1;
+	}
 
 	ch 	   = private->channel;
 
@@ -665,6 +715,7 @@ mem_activate(class_t *class)
 		return -1;
 	}
 
+	ch->ops->set_recv_qlen(ch, 0);
 	sockfd = ch->ops->get_recv_select_fd(ch);
 
 	return sockfd;
@@ -698,7 +749,9 @@ mem_set_callback(class_t *class, oc_ev_callback_t f)
 	mbr_private_t 	*private;
 	oc_ev_callback_t *ret_f;
 
-	if(!class_valid(class)) return NULL;
+	if(!class_valid(class)){
+		return NULL;
+	}
 
 	private = (mbr_private_t *)class->private;
 	
@@ -717,7 +770,9 @@ mem_set_special(class_t *class, int type)
 {
 	mbr_private_t 	*private;
 
-	if(!class_valid(class)) return;
+	if(!class_valid(class)) {
+		return;
+	}
 
 	private = (mbr_private_t *)class->private;
 	
@@ -733,13 +788,15 @@ mem_is_my_nodeid(class_t *class, const oc_node_t *node)
 {
 	mbr_private_t 	*private;
 
-	if(!class_valid(class)) return FALSE;
-
+	if(!class_valid(class)){
+		return FALSE;
+	}
 	private = (mbr_private_t *)class->private;
 
-	if (node->node_id == CLLM_GET_MYUUID(private->llm))
+	if (node->node_id == CLLM_GET_MYUUID(private->llm)){
 		return TRUE;
-
+	}
+	
 	return FALSE;
 }
 
@@ -756,7 +813,9 @@ oc_ev_memb_class(oc_ev_callback_t  *fn)
 
 	memclass = g_malloc(sizeof(class_t));
 
-	if (!memclass) return NULL;
+	if (!memclass){
+		return NULL;
+	}
 
 	private = (mbr_private_t *)g_malloc0(sizeof(mbr_private_t));
 	if (!private) {
