@@ -1,3 +1,4 @@
+/* $Id: ccmmain.c,v 1.27 2005/07/29 10:32:30 sunjd Exp $ */
 /* 
  * ccm.c: Consensus Cluster Service Program 
  *
@@ -19,16 +20,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-#include <lha_internal.h>
-#include "ccm.h"
-#include "ccmmisc.h"
+#include <ccm.h>
 #include <clplumbing/cl_malloc.h>
 #include <clplumbing/cl_signal.h>
 
 #define SECOND   1000
 #define OPTARGS  "dv"
 
-GMainLoop*	mainloop = NULL;
+int global_debug=0;
+int global_verbose=0;
 
 /*
  * hearbeat event source.
@@ -37,15 +37,21 @@ GMainLoop*	mainloop = NULL;
 static gboolean hb_input_dispatch(IPC_Channel *, gpointer);
 static void hb_input_destroy(gpointer);
 static gboolean hb_timeout_dispatch(gpointer);
+
+typedef struct hb_usrdata_s {
+	void		*ccmdata;
+	GMainLoop	*mainloop;
+} hb_usrdata_t;
+
 static gboolean
 hb_input_dispatch(IPC_Channel * channel, gpointer user_data)
 {
 	if (channel && (channel->ch_status == IPC_DISCONNECT)) {
-		ccm_log(LOG_ERR, "Lost connection to heartbeat service. Need to bail out.");
+		cl_log(LOG_ERR, "Lost connection to heartbeat service. Need to bail out.");
 		return FALSE;
 	}
 	
-	return ccm_take_control(user_data);
+	return ccm_take_control(((hb_usrdata_t *)user_data)->ccmdata);
 }
 
 static void
@@ -53,14 +59,14 @@ hb_input_destroy(gpointer user_data)
 {
 	/* close connections to all the clients */
 	client_delete_all();
-	g_main_quit(mainloop);
+	g_main_quit(((hb_usrdata_t *)user_data)->mainloop);
 	return;
 }
 
 static gboolean
 hb_timeout_dispatch(gpointer user_data)
 {	
-	if(debug_level > 0) {
+	if(global_debug) {
 		ccm_check_memoryleak();
 	}
 	return hb_input_dispatch(0, user_data);
@@ -80,7 +86,7 @@ clntCh_input_dispatch(IPC_Channel *client,
 	      gpointer        user_data)
 {
 	if(client->ch_status == IPC_DISCONNECT){
-		ccm_log(LOG_INFO, "client (pid=%d) removed from ccm", 
+		cl_log(LOG_INFO, "client (pid=%d) removed from ccm", 
 		       client->farside_pid);
 		
 		client_delete(client);
@@ -154,7 +160,7 @@ wait_channel_init(void)
 static void
 usage(const char *cmd)
 {
-	fprintf(stderr, "\nUsage: %s [-v]\n", cmd);
+	fprintf(stderr, "\nUsage: %s [-dv]\n", cmd);
 }
 
 
@@ -162,39 +168,36 @@ usage(const char *cmd)
 /* debug facilitator. */
 /* */
 static void
-usr_signal_handler(int signum) 
+ccm_debug(int signum) 
 {
 	switch(signum) {
 	case SIGUSR1:
-		debug_level++;
+		global_debug = !global_debug;
 		break;
-
 	case SIGUSR2:
-		debug_level--;
-		if (debug_level < 0) {
-			debug_level = 0;
-		}
+		global_verbose = !global_verbose;
 		break;
 	}
-	
-	if(debug_level > 0){
+
+	if(global_debug || global_verbose){
 		cl_log_enable_stderr(TRUE);
 	} else {
 		cl_log_enable_stderr(FALSE);
 	}
-	ccm_log(LOG_INFO, "set debug_level to %d", debug_level);
 
 }
 
 static gboolean
 ccm_shutdone(int sig, gpointer userdata)
 {	
-	ccm_t*		ccm = (ccm_t*)userdata;
+	hb_usrdata_t*	hb = (hb_usrdata_t*)userdata;
+	ccm_t*		ccm = (ccm_t*)hb->ccmdata;
 	ccm_info_t *	info = (ccm_info_t*)ccm->info;
-
-	ccm_log(LOG_INFO, "received SIGTERM, going to shut down");
-	if (info == NULL){
-		ccm_log(LOG_ERR, "ccm_shutdone: invalid arguments");
+	GMainLoop *	mainloop = hb->mainloop;
+	
+	cl_log(LOG_INFO, "received SIGTERM in node");
+	if (info == NULL || mainloop == NULL){
+		cl_log(LOG_ERR, "ccm_shutdone: invalid arguments");
 		return FALSE;
 	}
 	
@@ -215,8 +218,8 @@ main(int argc, char **argv)
 	char *cmdname;
 	char *tmp_cmdname;
 	int  flag;
-	ccm_t*	ccm;
-	
+	hb_usrdata_t	usrdata;
+
 #if 1
 	cl_malloc_forced_for_glib();
 #endif
@@ -227,50 +230,55 @@ main(int argc, char **argv)
 		cmdname = tmp_cmdname;
 	}
 	cl_log_set_entity(cmdname);
-	cl_log_set_facility(HA_LOG_FACILITY);
+	cl_log_set_facility(LOG_DAEMON);
 	while ((flag = getopt(argc, argv, OPTARGS)) != EOF) {
 		switch(flag) {
-		case 'v':		/* Debug mode, more logs*/
-				++debug_level;
+			case 'v':
+				global_verbose = 1;
+				break;
+			case 'd': 
+				global_debug = 1;
 				break;
 		default:
 				usage(cmdname);
 				return 1;
 		}
 	}
-	if(debug_level > 0)
+	if(global_verbose || global_debug)
 		cl_log_enable_stderr(TRUE);
 
 
-	CL_SIGNAL(SIGUSR1, usr_signal_handler);
-	CL_SIGNAL(SIGUSR2, usr_signal_handler);
+	CL_SIGNAL(SIGUSR1, ccm_debug);
+	CL_SIGNAL(SIGUSR2, ccm_debug);
 	CL_IGNORE_SIG(SIGPIPE);
 	
-	cl_inherit_logging_environment(0);
-
+	cl_inherit_use_logd(ENV_PREFIX ""KEY_LOGDAEMON, 0);
+	
 	/* initialize the client tracking system */
 	client_init();
 
+	usrdata.mainloop = g_main_new(TRUE);
+
 	/* 
 	 * heartbeat is the main source of events. 
 	 * This source must be listened 
 	 * at high priority 
 	 */
-	ccm = ccm_initialize();
-	if(ccm == NULL){
-		ccm_log(LOG_ERR, "Initialization failed. Exit");
+	usrdata.ccmdata = ccm_initialize();
+	if(!usrdata.ccmdata) {
 		exit(1);
 	}
 	
 	G_main_add_SignalHandler(G_PRIORITY_HIGH, SIGTERM, 
-				 ccm_shutdone, ccm, NULL);
+				 ccm_shutdone, &usrdata, NULL);
+	
 	/* we want hb_input_dispatch to be called when some input is
 	 * pending on the heartbeat fd, and every 1 second 
 	 */
-	G_main_add_IPC_Channel(G_PRIORITY_HIGH, ccm_get_ipcchan(ccm), 
-			FALSE, hb_input_dispatch, ccm, hb_input_destroy);
+	G_main_add_IPC_Channel(G_PRIORITY_HIGH, ccm_get_ipcchan(usrdata.ccmdata), 
+			FALSE, hb_input_dispatch, &usrdata, hb_input_destroy);
 	Gmain_timeout_add_full(G_PRIORITY_HIGH, SECOND, hb_timeout_dispatch,
-				ccm, hb_input_destroy);
+				&usrdata, hb_input_destroy);
 
 	/* the clients wait channel is the other source of events.
 	 * This source delivers the clients connection events.
@@ -281,10 +289,10 @@ main(int argc, char **argv)
 		FALSE, waitCh_input_dispatch, wait_ch,
 		waitCh_input_destroy);
 
-	mainloop = g_main_loop_new(NULL, FALSE);
-	g_main_run(mainloop);
-	g_main_destroy(mainloop);
-	
+
+	g_main_run(usrdata.mainloop);
+	g_main_destroy(usrdata.mainloop);
+
 	g_free(tmp_cmdname);
 	/*this program should never terminate,unless killed*/
 	return(1);
