@@ -81,11 +81,10 @@
  *	return 0(OCF_SUCCESS) for response correctly.
  *	return 1(OCF_NOT_RUNNING) for no response.
  *	return 2(OCF_ERR_ARGS) for invalid or excess argument(s)
+ *
  */
 
-#include <lha_internal.h>
-
-#include <stdlib.h>
+#include <config.h>
 #include <sys/types.h>
 #include <netinet/icmp6.h>
 #include <libgen.h>
@@ -110,8 +109,6 @@
 5 	program is not installed
 6 	program is not configured
 7 	program is not running
-8	resource is running in "master" mode and fully operational
-9	resource is in "master" mode but in a failed state
 */
 #define	OCF_SUCCESS		0
 #define	OCF_ERR_GENERIC		1
@@ -158,8 +155,6 @@ int write_pid_file(const char *pid_file);
 int create_pid_directory(const char *pid_file);
 static void byebye(int nsig);
 
-static char* scan_if(struct in6_addr* addr_target, int* plen_target,
-		     int use_mask);
 static char* find_if(struct in6_addr* addr_target, int* plen_target);
 static char* get_if(struct in6_addr* addr_target, int* plen_target);
 static int assign_addr6(struct in6_addr* addr6, int prefix_len, char* if_name);
@@ -189,7 +184,7 @@ main(int argc, char* argv[])
 
 	/* open system log */
 	cl_log_set_entity(APP_NAME);
-	cl_log_set_facility(HA_LOG_FACILITY);
+	cl_log_set_facility(LOG_DAEMON);
 
 	/* the meta-data dont need any parameter */
 	if (0 == strncmp(META_DATA_CMD, argv[1], strlen(META_DATA_CMD))) {
@@ -372,6 +367,7 @@ monitor_addr6(struct in6_addr* addr6, int prefix_len)
 		return OCF_SUCCESS;
 	}
 	return OCF_NOT_RUNNING;
+
 }
 
 /* Send an unsolicited advertisement packet
@@ -380,7 +376,6 @@ monitor_addr6(struct in6_addr* addr6, int prefix_len)
 int
 send_ua(struct in6_addr* src_ip, char* if_name)
 {
-	int status = -1;
 	libnet_t *l;
 	char errbuf[LIBNET_ERRBUF_SIZE];
 
@@ -391,13 +386,13 @@ send_ua(struct in6_addr* src_ip, char* if_name)
 
 	if ((l=libnet_init(LIBNET_RAW6, if_name, errbuf)) == NULL) {
 		cl_log(LOG_ERR, "libnet_init failure on %s", if_name);
-		goto err;
+		return -1;
 	}
 
 	mac_address = libnet_get_hwaddr(l);
 	if (!mac_address) {
 		cl_log(LOG_ERR, "libnet_get_hwaddr: %s", errbuf);
-		goto err;
+		return -1;
 	}
 
 	dst_ip = libnet_name2addr6(l, BCAST_ADDR, LIBNET_DONT_RESOLVE);
@@ -418,25 +413,23 @@ send_ua(struct in6_addr* src_ip, char* if_name)
         if (libnet_write(l) == -1)
         {
 		cl_log(LOG_ERR, "libnet_write: %s", libnet_geterror(l));
-		goto err;
+		return -1;
 	}
 
-	status = 0;
-err:
-	libnet_destroy(l);
-	return status;
+	return 0;
 }
 
-/* find the network interface associated with an address */
+/* find a proper network interface to assign the address */
 char*
-scan_if(struct in6_addr* addr_target, int* plen_target, int use_mask)
+find_if(struct in6_addr* addr_target, int* plen_target)
 {
 	FILE *f;
-	static char devname[21]="";
+	char addr6[40];
+	static char devname[20]="";
 	struct in6_addr addr;
 	struct in6_addr mask;
 	unsigned int plen, scope, dad_status, if_idx;
-	unsigned int addr6p[4];
+	char addr6p[8][5];
 
 	/* open /proc/net/if_inet6 file */
 	if ((f = fopen(IF_INET6, "r")) == NULL) {
@@ -444,24 +437,20 @@ scan_if(struct in6_addr* addr_target, int* plen_target, int use_mask)
 	}
 
 	/* Loop for each entry */
-	while (1) {
+	while ( fscanf(f,"%4s%4s%4s%4s%4s%4s%4s%4s %02x %02x %02x %02x %20s\n",
+			addr6p[0], addr6p[1], addr6p[2], addr6p[3],
+			addr6p[4], addr6p[5], addr6p[6], addr6p[7],
+			&if_idx, &plen, &scope, &dad_status, devname) != EOF){
+
 		int		i;
 		int		n;
 		int		s;
 		gboolean	same = TRUE;
 
-		i = fscanf(f, "%08x%08x%08x%08x %02x %02x %02x %02x %20s\n",
-		       	   &addr6p[0], &addr6p[1], &addr6p[2], &addr6p[3],
-			   &if_idx, &plen, &scope, &dad_status, devname);
-		if (i == EOF) {
-			break;
-		}
-		else if (i != 9) {
-			cl_log(LOG_INFO, "Error parsing %s, "
-			       "perhaps the format has changed\n", IF_INET6);
-			break;
-		}
-
+		sprintf(addr6, "%s:%s:%s:%s:%s:%s:%s:%s",
+			addr6p[0], addr6p[1], addr6p[2], addr6p[3],
+			addr6p[4], addr6p[5], addr6p[6], addr6p[7]);
+	
 		/* Only Global address entry would be considered.
 		 * maybe change?
 		 */
@@ -476,26 +465,26 @@ scan_if(struct in6_addr* addr_target, int* plen_target, int use_mask)
 			continue;
 		}
 		*plen_target = plen;
-
-		for (i = 0; i< 4; i++) {
-			addr.s6_addr32[i] = htonl(addr6p[i]);
-		}
+		
+		/* Convert string to sockaddr_in6 */
+		inet_pton(AF_INET6, addr6, &addr);
 
 		/* Make the mask based on prefix length */
-		memset(mask.s6_addr, 0xff, 16);
-		if (use_mask && plen < 128) {
-			n = plen / 32;
-			memset(mask.s6_addr32 + n + 1, 0, (3 - n) * 4);
-			s = 32 - plen % 32;
-			mask.s6_addr32[n] = 0xffffffff << s;
-			mask.s6_addr32[n] = htonl(mask.s6_addr32[n]);
+		for (i = 0; i < 16; i++) {
+			mask.s6_addr[i] = 0;
+		}	
+		n = plen / 8;
+		for (i = 0; i < n+1; i++) {
+			mask.s6_addr[i] = 0xFF;
 		}
+		s = 8 - plen % 8;
+		mask.s6_addr[n]<<=s;
 
 		/* compare addr and addr_target */
 		same = TRUE;
-		for (i = 0; i < 4; i++) {
-			if ((addr.s6_addr32[i]&mask.s6_addr32[i]) !=
-			    (addr_target->s6_addr32[i]&mask.s6_addr32[i])) {
+		for (i = 0; i < 16; i++) {
+			if ((addr.s6_addr[i]&mask.s6_addr[i]) !=
+			    (addr_target->s6_addr[i]&mask.s6_addr[i])) {
 				same = FALSE;
 				break;
 			}
@@ -510,17 +499,57 @@ scan_if(struct in6_addr* addr_target, int* plen_target, int use_mask)
 	fclose(f);
 	return NULL;
 }
-/* find a proper network interface to assign the address */
-char*
-find_if(struct in6_addr* addr_target, int* plen_target)
-{
-	return scan_if(addr_target, plen_target, 1);
-}
 /* get the device name and the plen_target of a special address */
 char*
 get_if(struct in6_addr* addr_target, int* plen_target)
 {
-	return scan_if(addr_target, plen_target, 0);
+	FILE *f;
+	char addr6[40];
+	static char devname[20]="";
+	struct in6_addr addr;
+	unsigned int plen, scope, dad_status, if_idx;
+	char addr6p[8][5];
+
+	/* open /proc/net/if_inet6 file */
+	if ((f = fopen(IF_INET6, "r")) == NULL) {
+		return NULL;
+	}
+	/* loop for each entry */
+	while ( fscanf(f,"%4s%4s%4s%4s%4s%4s%4s%4s %02x %02x %02x %02x %20s\n",
+		addr6p[0], addr6p[1], addr6p[2], addr6p[3],
+		addr6p[4], addr6p[5], addr6p[6], addr6p[7],
+		&if_idx, &plen, &scope, &dad_status, devname) != EOF) {
+
+		sprintf(addr6, "%s:%s:%s:%s:%s:%s:%s:%s",
+			addr6p[0], addr6p[1], addr6p[2], addr6p[3],
+			addr6p[4], addr6p[5], addr6p[6], addr6p[7]);
+
+		/* Only Global address entry would be considered.
+		 * maybe change
+		 */
+		if (0 != scope) {
+			continue;
+		}
+
+		/* "if" specified prefix, only same prefix entry
+		 * would be considered.
+		 */
+		if (*plen_target!=0 && plen != *plen_target) {
+			continue;
+		}
+		*plen_target = plen;
+
+		/* Convert to sockaddr_in6 */
+		inet_pton(AF_INET6, addr6, &addr);
+
+		/* We found it! */
+		if (0 == memcmp(&addr, addr_target,sizeof(addr))) {
+			fclose(f);
+			return devname;
+		}
+	}
+	fclose(f);
+	return NULL;
 }
 int
 assign_addr6(struct in6_addr* addr6, int prefix_len, char* if_name)
@@ -657,7 +686,6 @@ int
 create_pid_directory(const char *pid_file)
 {
 	int status;
-	int return_status = -1;
 	struct stat stat_buf;
 	char* dir;
 
@@ -675,29 +703,28 @@ create_pid_directory(const char *pid_file)
 	if (status < 0 && errno != ENOENT && errno != ENOTDIR) {
 		cl_log(LOG_INFO, "Could not stat pid-file directory "
 				"[%s]: %s", dir, strerror(errno));
-		goto err;
+		free(dir);
+		return -1;
 	}
 
 	if (!status) {
 		if (S_ISDIR(stat_buf.st_mode)) {
-			goto out;
+			return 0;
 		}
 		cl_log(LOG_INFO, "Pid-File directory exists but is "
 				"not a directory [%s]", dir);
-		goto err;
+		free(dir);
+		return -1;
         }
 
 	if (mkdir(dir, S_IRUSR|S_IWUSR|S_IXUSR | S_IRGRP|S_IXGRP) < 0) {
 		cl_log(LOG_INFO, "Could not create pid-file directory "
 				"[%s]: %s", dir, strerror(errno));
-		goto err;
+		free(dir);
+		return -1;
 	}
 
-out:
-	return_status = 0;
-err:
-	free(dir);
-	return return_status;
+	return 0;
 }
 
 int
@@ -829,7 +856,7 @@ meta_data_addr6(void)
 	"  </longdesc>\n"
 	"  <shortdesc lang=\"en\">manages IPv6 alias</shortdesc>\n"
 	"  <parameters>\n"
-	"    <parameter name=\"ipv6addr\" unique=\"0\" required=\"1\">\n"
+	"    <parameter name=\"ipv6addr\" unique=\"0\">\n"
 	"      <longdesc lang=\"en\">\n"
 	"	The IPv6 address this RA will manage \n"
 	"      </longdesc>\n"
