@@ -21,10 +21,13 @@
  *
  */
 
-#include <lha_internal.h>
+#include <portability.h>
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "hbagent.h"
-#include "hbagentv2.h"
 
 #include "hb_api.h"
 #include "heartbeat.h"
@@ -67,12 +70,13 @@
 #include <errno.h>
 
 #include "saf/ais.h"
-#include "clplumbing/cl_uuid.h" /* UU_UNPARSE_SIZEOF */
+
+#define DEFAULT_TIME_OUT 5 /* default timeout value for snmp in sec. */
+#define LHAAGENTID "lha-snmpagent"
 
 static unsigned long hbInitialized = 0;
 static ll_cluster_t * hb = NULL; /* heartbeat handle */
-char * myid = NULL; /* my node id */
-char * myuuid = NULL; /* my node uuid */
+static char * myid = NULL; /* my node id */
 static SaClmHandleT clm = 0;
 static unsigned long clmInitialized = 0;
 
@@ -82,7 +86,6 @@ static GPtrArray * gMembershipTable = NULL;
 static GPtrArray * gResourceTable = NULL;
 
 static int keep_running;
-int snmp_cache_time_out = CACHE_TIME_OUT;
 
 int init_heartbeat(void);
 int get_heartbeat_fd(void);
@@ -108,35 +111,6 @@ int hbagent_trap(int online, const char * node);
 
 int ping_membership(int * mem_fd);
 
-/* LHAHeartbeatConfigInfo partial-mode */
-#define DEFAULT_REFRESH_TIMING  (0)
-
-int  init_hbconfig(void);
-void free_hbconfig(void);
-int  hbconfig_get_str_value(const char * attr, char * * value);
-
-static int    hbconfig_partial = 0;
-static char * hbconfig[LHA_CONF_END];
-static struct hbconfig_map {
-	lha_hbconfig_t  attr_no;
-	const char *    attr;
-} hbconfig_map[] =
-{
-	{LHA_CONF_HBVERSION,   KEY_HBVERSION},
-	{LHA_CONF_KEEPALIVE,   KEY_KEEPALIVE},
-	{LHA_CONF_DEADTIME,    KEY_DEADTIME},
-	{LHA_CONF_DEADPING,    KEY_DEADPING},
-	{LHA_CONF_WARNTIME,    KEY_WARNTIME},
-	{LHA_CONF_INITDEAD,    KEY_INITDEAD},
-	{LHA_CONF_BAUDRATE,    KEY_BAUDRATE},
-	{LHA_CONF_AUTOFAIL,    KEY_AUTOFAIL},
-	{LHA_CONF_STONITH,     KEY_STONITH},
-	{LHA_CONF_STONITHHOST, KEY_STONITHHOST},
-	{LHA_CONF_RESPAWN,     KEY_CLIENT_CHILD},
-	{LHA_CONF_END,         NULL}
-};
-
-
 uint32_t get_status_value(const char * status, const char * * status_array, uint32_t * value_array);
 
 const char * NODE_STATUS [] = 
@@ -310,73 +284,6 @@ clusterinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * value)
     return HA_OK;
 }
 
-/* LHAHeartbeatConfigInfo partial-mode */
-int
-init_hbconfig(void)
-{
-    lha_hbconfig_t      attr_no;
-    char *              value;
-
-    for (attr_no = LHA_CONF_HBVERSION; attr_no < LHA_CONF_END; ++attr_no) {
-
-        hbconfig[attr_no] = NULL;
-
-        if(!hbconfig_partial)
-            continue;
-
-        if (hbconfig_get_str_value(hbconfig_map[attr_no].attr, &value)
-                                                                != HA_OK) {
-            cl_log(LOG_INFO, "LHAHeartbeatConfigInfo initialization error.  "
-                             "maybe get_parameter returned NULL.");
-            continue;
-        }
-
-        hbconfig[attr_no] = value;
-    }
-
-    return HA_OK;
-}
-
-void
-free_hbconfig(void)
-{
-    lha_hbconfig_t      attr_no;
-
-    if (!hbconfig_partial)
-        return;
-
-    for (attr_no = LHA_CONF_HBVERSION; attr_no < LHA_CONF_END; ++attr_no) {
-
-        if (hbconfig[attr_no] != NULL)
-            cl_free(hbconfig[attr_no]);
-
-    }
-}
-
-int
-hbconfig_get_str(lha_hbconfig_t attr_no, char * * value)
-{
-    static char err[] = "N/A";
-
-    if (hbconfig[attr_no] != NULL) {
-
-        *value = cl_strdup(hbconfig[attr_no]);
-
-    } else {
-
-        if (hbconfig_get_str_value(hbconfig_map[attr_no].attr, value)
-                                                                != HA_OK) {
-            cl_log(LOG_INFO, "LHAHeartbeatConfigInfo getting parameter error.  "
-                             "maybe get_parameter returned NULL.");
-            *value  = cl_strdup(err);
-        }
-
-    }
-
-    return HA_OK;
-}
-
-
 int
 hbconfig_get_str_value(const char * attr, char * * value)
 {
@@ -391,11 +298,11 @@ hbconfig_get_str_value(const char * attr, char * * value)
 
 	/* we have to return HA_OK here otherwise the 
 	   agent code would not progress */
-	*value  = cl_strdup(err);
+	*value  = ha_strdup(err);
 	return HA_OK;
     };
 
-    *value = ret;
+    *value = ha_strdup(ret);
 
     return HA_OK;
 }
@@ -429,7 +336,6 @@ NodeStatus(const char * node, const char * status, void * private)
         cl_log(LOG_NOTICE, "Status update: Node %s now has status %s"
         ,       node, status);
 	walk_nodetable();
-	walk_iftable();
 
 	nodestatus_trap(node, status);
 }
@@ -474,7 +380,7 @@ free_nodetable(void)
 		node = (struct hb_nodeinfo *) g_ptr_array_remove_index_fast(gNodeTable, 0);
 
 		free(node->name);
-		cl_free(node);
+		ha_free(node);
 	}
 
 	return;
@@ -493,7 +399,7 @@ free_iftable(void)
 
 		free(interface->name);
 		free(interface->node);
-		cl_free(interface);
+		ha_free(interface);
 	}
 
 	return;
@@ -512,7 +418,7 @@ free_resourcetable(void)
 		resource = (struct hb_rsinfo *) g_ptr_array_remove_index_fast(gResourceTable, 0);
 
 		free(resource->resource);
-		cl_free(resource);
+		ha_free(resource);
 
 	}
 
@@ -540,35 +446,31 @@ void
 free_storage(void)
 {
     	free_nodetable();
-	g_ptr_array_free(gNodeTable, 1);
+	g_ptr_array_free(gNodeTable, 0);
 	gNodeTable = NULL;
 
 	free_iftable();
-	g_ptr_array_free(gIFTable, 1);
+	g_ptr_array_free(gIFTable, 0);
 	gIFTable = NULL;
 
 	free_resourcetable();
-	g_ptr_array_free(gResourceTable, 1);
+	g_ptr_array_free(gResourceTable, 0);
 	gResourceTable = NULL;
 
 	free_membershiptable();
-	g_ptr_array_free(gMembershipTable, 1);
-	gMembershipTable = NULL;
+	g_ptr_array_free(gMembershipTable, 0);
+	gResourceTable = NULL;
 
 }
 
 int
 init_heartbeat(void)
 {
-	char * parameter;
-	cl_uuid_t uuid;
+	const char * parameter;
 	hb = NULL;
 
 	cl_log_set_entity("lha-snmpagent");
-	cl_log_set_facility(HA_LOG_FACILITY);
-
-	/* Use logd if it's enabled by heartbeat */
-	cl_inherit_logging_environment(0);
+	cl_log_set_facility(LOG_USER);
 
 	hb = ll_cluster_new("heartbeat");
 
@@ -585,28 +487,15 @@ init_heartbeat(void)
 	parameter = hb->llc_ops->get_parameter(hb, KEY_COREROOTDIR);
 	if (parameter) {
 		cl_set_corerootdir(parameter);
-		cl_free(parameter);
 	}
 	cl_cdtocoredir();
 
-	if (NULL == (myid = cl_strdup(hb->llc_ops->get_mynodeid(hb)))) {
+	if (NULL == (myid = ha_strdup(hb->llc_ops->get_mynodeid(hb)))) {
 		cl_log(LOG_ERR, "Cannot get mynodeid");
 		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
 		return HA_FAIL;
 	}
 
-	/*
-	 * get uuid for trap message.
-	 *   see: hbagentv2_update_diff() in hbagentv2.c
-	 */
-	if (hb->llc_ops->get_uuid_by_name(hb, myid, &uuid) == HA_FAIL) {
-		cl_log(LOG_ERR, "Cannot get mynodeid");
-		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
-		return HA_FAIL;
-	}
-	myuuid = cl_malloc(UU_UNPARSE_SIZEOF);
-	cl_uuid_unparse(&uuid, myuuid);
-
 	if (hb->llc_ops->set_nstatus_callback(hb, NodeStatus, NULL) !=HA_OK){
 	        cl_log(LOG_ERR, "Cannot set node status callback");
 	        cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
@@ -674,7 +563,7 @@ walk_nodetable(void)
 		cl_log(LOG_INFO, "node %ld: %s, type: %s, status: %s", (unsigned long)id, name 
 		,	type, status);
 
-		node = (struct hb_nodeinfo *) cl_malloc(sizeof(struct hb_nodeinfo));
+		node = (struct hb_nodeinfo *) ha_malloc(sizeof(struct hb_nodeinfo));
 		if (!node) {
 			cl_log(LOG_CRIT, "malloc failed for node info.");
 			return HA_FAIL;
@@ -737,7 +626,7 @@ walk_iftable(void)
 			cl_log(LOG_INFO, "node: %s, interface: %s, status: %s",
 					node->name, name, status);
 
-			interface = (struct hb_ifinfo *) cl_malloc(sizeof(struct hb_ifinfo));
+			interface = (struct hb_ifinfo *) ha_malloc(sizeof(struct hb_ifinfo));
 			if (!interface) {
 				cl_log(LOG_CRIT, "malloc failed for if info.");
 				return HA_FAIL;
@@ -864,20 +753,20 @@ init_membership(void)
 	}
 
 	if (nbuf) {
-		cl_free(nbuf);
+		ha_free(nbuf);
 	}
 
-        nbuf = (SaClmClusterNotificationT *) cl_malloc(gNodeTable->len *
+        nbuf = (SaClmClusterNotificationT *) ha_malloc(gNodeTable->len *
                                 sizeof (SaClmClusterNotificationT));
 	if (!nbuf) {
-		cl_log(LOG_ERR, "%s: cl_malloc failed for SaClmClusterNotificationT.", __FUNCTION__);
+		cl_log(LOG_ERR, "%s: ha_malloc failed for SaClmClusterNotificationT.", __FUNCTION__);
 		return HA_FAIL;
 	}
 
         if (saClmClusterTrackStart(&handle, SA_TRACK_CURRENT, nbuf,
                 gNodeTable->len) != SA_OK) {
                 cl_log(LOG_ERR, "SA_TRACK_CURRENT error, errno [%d]", ret);
-                cl_free(nbuf);
+                ha_free(nbuf);
                 return HA_FAIL;
         }
 
@@ -885,7 +774,7 @@ init_membership(void)
         if (saClmClusterTrackStart(&handle, SA_TRACK_CHANGES, nbuf,
                 gNodeTable->len) != SA_OK) {
                 cl_log(LOG_ERR, "SA_TRACK_CURRENT error, errno [%d]", ret);
-                cl_free(nbuf);
+                ha_free(nbuf);
                 return HA_FAIL;
         }
 
@@ -928,9 +817,7 @@ handle_membership_msg(void)
 			   again later. */
 			clmInitialized = 0;
 			free_membershiptable();
-                        
-                        /* finalize membership */
-                        saClmFinalize(&clm);
+
 			return HA_OK;
 		} else {
 		    cl_log(LOG_WARNING, "saClmDispatch error, ret = [%d]", ret);
@@ -1010,7 +897,7 @@ init_resource_table(void)
 		if (!found)
 		    continue;
 
-		resource = (struct hb_rsinfo *) cl_malloc(sizeof(struct hb_rsinfo));
+		resource = (struct hb_rsinfo *) ha_malloc(sizeof(struct hb_rsinfo));
 		if (!resource) {
 			cl_log(LOG_CRIT, "malloc resource info failed.");
 			fclose(rcsf);
@@ -1043,7 +930,7 @@ rsinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * value)
     switch (attr) {
 	case RESOURCE_STATUS:
 	    resource = ((struct hb_rsinfo *) g_ptr_array_index(gResourceTable, index))->resource;
-	    sprintf(getcmd, HA_NOARCHDATAHBDIR "/ResourceManager status %s", resource);
+	    sprintf(getcmd, HALIB "/ResourceManager status %s", resource);
 	    rc = system(getcmd);
 	    /* cl_log(LOG_INFO, "resource [%s] status: [%d]", resource, WEXITSTATUS(rc)); */
 
@@ -1305,15 +1192,7 @@ membership_trap(const char * node, SaClmClusterChangesT status)
 static void
 usage(void)
 {
-    	fprintf(stderr, "Usage: hbagent [Options]\n"
-    			"Options:\n"
-    			" -d         enable logging to STDERR\n"
-    			" -r SECOND  set LHAHeartbeatConfigInfo to partial-mode\n"
-    			"            SECOND -- time interval between update "
-    			"LHAHeartbeatConfigInfo\n"
-    			"                      o multiple of 5\n"
-    			"                      o '0' keeps initial value (not update)\n"
-    	);
+    	fprintf(stderr, "Usage: hbagent -d\n");
 }
 
 int
@@ -1325,7 +1204,6 @@ main(int argc, char ** argv)
 	struct timeval tv, *tvp;
 	int flag, block = 0, numfds, hb_fd = 0, mem_fd = 0, debug = 0;
 	int hb_already_dead = 0;
-	int cib_fd = 0;
 
 	/* change this if you want to be a SNMP master agent */
 	int agentx_subagent=1; 
@@ -1333,42 +1211,30 @@ main(int argc, char ** argv)
 	/* change this if you want to run in the background */
 	int background = 0; 
 
-	/* LHAHeartbeatConfigInfo partial-mode */
-	int hbconfig_refresh_timing = 0;
-	int hbconfig_refresh_cnt;
-
-	while ((flag = getopt(argc, argv, "dr:h")) != EOF) {
-	    	int i;
+	/* change this if you want to use syslog */
+	int syslog = 1; 
 
+	while ((flag = getopt(argc, argv, "d")) != EOF) {
 	    	switch (flag) {
 		    case 'd':
 			debug++ ;
 			break;
-		    case 'r': /* LHAHeartbeatConfigInfo partial-mode */
-			hbconfig_partial = 1;
-			i = atoi(optarg);
-			if (i > 0)
-				hbconfig_refresh_timing = i / DEFAULT_TIME_OUT;
-			else
-				hbconfig_refresh_timing = DEFAULT_REFRESH_TIMING;
-			snmp_cache_time_out = i;
-			break;
-		    case 'h':
-			usage();
-			exit(0);
 		    default: 
 			usage();
 			exit(1);
 		}
 	}
 
-	if (debug) {
+	if (debug) 
 		cl_log_enable_stderr(TRUE);
-		snmp_enable_stderrlog();
-	} else {
+	else 
 		cl_log_enable_stderr(FALSE);
+
+	/* print log errors to syslog or stderr */
+	if (syslog)
 		snmp_enable_calllog();
-	}
+	else
+		snmp_enable_stderrlog();
 
 	/* we're an agentx subagent? */
 	if (agentx_subagent) {
@@ -1378,7 +1244,7 @@ main(int argc, char ** argv)
 	}
 
 	/* run in background, if requested */
-	if (background && netsnmp_daemonize(1, debug))
+	if (background && netsnmp_daemonize(1, !syslog))
 		exit(1);
 
 	/* initialize the agent library */
@@ -1417,11 +1283,6 @@ main(int argc, char ** argv)
 	} 
 	*/
 
-	/* LHAHeartbeatConfigInfo partial-mode */
-	if ((ret = init_hbconfig()) != HA_OK) {
-	    	return -3;
-	}
-
 	init_LHAClusterInfo();
 	init_LHANodeTable();
 	init_LHAIFStatusTable();
@@ -1429,13 +1290,6 @@ main(int argc, char ** argv)
 	init_LHAMembershipTable();
 	init_LHAHeartbeatConfigInfo();
 
-
-	/* now implementing: hbagentv2 */
-	if ((ret = init_hbagentv2()) != HA_OK ||
-	    (cib_fd = get_cib_fd()) <= 0) {
-		return -4;
-	}
-
 	/* LHA-agent will be used to read LHA-agent.conf files. */
 	init_snmp("LHA-agent");
 
@@ -1452,11 +1306,8 @@ main(int argc, char ** argv)
 
 	hbagent_trap(1, myid);
 
-	hbconfig_refresh_cnt = 0;
-
 	/* you're main loop here... */
 	while(keep_running) {
-		int	save_errno;
 	/* if you use select(), see snmp_select_info() in snmp_api(3) */
 	/*     --- OR ---  */
 		/* 0 == don't block */
@@ -1472,10 +1323,6 @@ main(int argc, char ** argv)
 			if (mem_fd > hb_fd)
 				numfds = mem_fd + 1;
 		}
-		FD_SET(cib_fd, &fdset);
-		if (numfds < (cib_fd + 1)) {
-			numfds = cib_fd + 1;
-		}
 
 		tv.tv_sec = DEFAULT_TIME_OUT;
 		tv.tv_usec = 0;
@@ -1491,41 +1338,14 @@ main(int argc, char ** argv)
 		}
 
 		ret = select(numfds, &fdset, 0, 0, tvp);
-		save_errno=errno;
 
 		if (ret < 0) {
-			if (errno == EBADF && keep_running) {
-				struct stat	foo;
-				/* Probably the membership layer shut down for some reason. */
-				if (mem_fd >= 0 && fstat(mem_fd, &foo) < 0 && errno == EBADF) {
-					cl_log(LOG_WARNING, "membership layer shut down.");
-					FD_CLR(mem_fd, &fdset);
-					if (numfds == mem_fd + 1) {
-						numfds--;
-					}
-					mem_fd = -1;
-					break;
-				}
-			}
-				
 			/* error */
-			if (keep_running) {
-				errno = save_errno;
-				cl_perror("%s: select() returned an error, shutting down."
-				,       __FUNCTION__);
-			}
+			cl_log(LOG_ERR, "select() returned with an error. shutting down...");
 			break;
 		} else if (ret == 0) {
 			/* timeout */
 			ping_membership(&mem_fd);
-
-			/* LHAHeartbeatConfigInfo partial-mode */
-			if (hbconfig_refresh_timing
-			&& ++hbconfig_refresh_cnt >= hbconfig_refresh_timing) {
-				free_hbconfig();
-				init_hbconfig();
-				hbconfig_refresh_cnt = 0;
-			}
 			snmp_timeout();
 			goto process_pending;
 		} 
@@ -1545,12 +1365,6 @@ main(int argc, char ** argv)
 			    	cl_log(LOG_DEBUG, "unrecoverable membership error. quit now.");
 				break;
 			}
-		} else  if (FD_ISSET(cib_fd, &fdset)) {
-			/* change cib info events */
-			if ((ret = handle_cib_msg()) == HA_FAIL) {
-		  		cl_log(LOG_DEBUG, "unrecoverable CIB error. quit now.");
-				break;
-			}
 		} else {
 
 			/* snmp request */
@@ -1568,10 +1382,7 @@ process_pending:
 	hbagent_trap(0, myid);
 	snmp_shutdown("LHA-agent");
 
-	free_hbagentv2();
- 	free_hbconfig();
-	cl_free(myid);
-	cl_free(myuuid);
+	ha_free(myid);
 	free_storage();
 
         if (!hb_already_dead && hb->llc_ops->signoff(hb, TRUE) != HA_OK) {
@@ -1588,3 +1399,4 @@ process_pending:
 	return 0;
 }
 
+
