@@ -197,7 +197,7 @@ SlurpAll(int fd)
 	char *newbuf;
 	size_t ntoread;
 	ssize_t nread;
-	printf("%s%s%s\n", gAction, gTarget[0] ? " " : "", gTarget);
+	printf("%s%s%s.\n", gAction, gTarget[0] ? " " : "", gTarget);
 	fflush(stdout);
 	for(;;) {
 		if (gNBufUsed == (gNBufAllocated - 1)) {
@@ -284,14 +284,14 @@ main(int argc, char **argv)
 				strcpy(gAction, "Linking");
 				snprintf(gTarget, sizeof(gTarget), "%s", Basename(argv[i + 1]));
 			}
+		} else if (strchr("-+/", (int) argv[i][0]) != NULL) {
+			continue;
 		} else if (strncasecmp(Extension(argv[i]), ".c", 2) == 0) {
 			cc++;
 			snprintf(gTarget, sizeof(gTarget), "%s", Basename(argv[i]));
 		} else if ((strncasecmp(Extension(argv[i]), ".h", 2) == 0) && (cc == 0)) {
 			pch++;
 			snprintf(gTarget, sizeof(gTarget), "%s", Basename(argv[i]));
-		} else if (strchr("-+/", (int) argv[i][0]) != NULL) {
-			continue;
 		} else if ((i == 1) && (strcmp(Basename(argv[i]), "ar") == 0)) {
 			snprintf(gAr, sizeof(gAr), "%s", Basename(argv[i]));
 		} else if ((gArLibraryTarget[0] == '\0')
@@ -358,10 +358,8 @@ main(int argc, char **argv)
 panic:
 	gDumpCmdArgs = 1;	/* print cmd when there are errors */
 	DumpFormattedOutput();
-	while ((nread = read(fd, emerg, (size_t) sizeof(emerg))) > 0) {
-		ssize_t retval;
-		retval = write(2, emerg, (size_t) nread);
-	}
+	while ((nread = read(fd, emerg, (size_t) sizeof(emerg))) > 0)
+		(void) write(2, emerg, (size_t) nread);
 	Wait();
 	exit(gExitStatus);
 }	/* main */
